Package: periscope2
Type: Package
Title: Enterprise Streamlined 'shiny' Application Framework Using
        'bs4Dash'
Version: 0.4.0
Authors@R: c(
    person("Mohammed", "Ali",      role = c("aut", "cre"), email   = "mohammed@aggregate-genius.com"),
    person("Constance", "Brett",   role = c("ctb")),
    person("Aggregate Genius Inc", role = c("spn"))
    )
Description: A framework for building enterprise, scalable and UI-standardized 'shiny' applications.  
    It brings enhanced features such as 'bootstrap' v4 <https://getbootstrap.com/docs/4.0/getting-started/introduction/>,
    additional and enhanced 'shiny' modules, customizable UI features, as well as an enhanced application file
    organization paradigm. This update allows developers to harness the ability to build powerful applications and 
    enriches the 'shiny' developers' experience when building and  maintaining applications.
URL: https://github.com/Aggregate-Genius/periscope2,
        http://periscopeapps.org:3838
BugReports: https://github.com/Aggregate-Genius/periscope2/issues
License: GPL-3
Encoding: UTF-8
Language: en-US
Depends: R (>= 4.2)
Imports: shiny (>= 1.7), bs4Dash (>= 2.3), DT, fresh, grDevices,
        lubridate, methods, reactable, shinyFeedback, shinyWidgets,
        utils, writexl, yaml, lifecycle
Suggests: assertthat, canvasXpress, colourpicker, ggplot2, knitr,
        lattice, miniUI, openxlsx, openxlsx2, rmarkdown, shinyjs,
        spelling, testthat, waiter
VignetteBuilder: knitr
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
KeepSource: no
NeedsCompilation: no
Packaged: 2026-08-19 13:11:53 UTC; mohammedali
Author: Mohammed Ali [aut, cre],
  Constance Brett [ctb],
  Aggregate Genius Inc [spn]
Maintainer: Mohammed Ali <mohammed@aggregate-genius.com>
Repository: CRAN
Date/Publication: 2026-08-19 16:10:02 UTC

# Generated by roxygen2: do not edit by hand

export(add_ui_body)
export(add_ui_footer)
export(add_ui_header)
export(add_ui_left_sidebar)
export(add_ui_right_sidebar)
export(announcementConfigurationsAddin)
export(appReset)
export(appResetButton)
export(createPSAlert)
export(create_application)
export(create_left_sidebar)
export(create_right_sidebar)
export(downloadFile)
export(downloadFileButton)
export(downloadFile_AvailableTypes)
export(downloadFile_ValidateTypes)
export(downloadablePlot)
export(downloadablePlotUI)
export(downloadableReactTable)
export(downloadableReactTableUI)
export(downloadableTable)
export(downloadableTableUI)
export(get_url_parameters)
export(load_announcements)
export(logViewerOutput)
export(logdebug)
export(logerror)
export(loginfo)
export(logwarn)
export(set_app_parameters)
export(themeConfigurationsAddin)
export(ui_tooltip)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,deprecated)
importFrom(methods,new)

---
title:  "periscope2 Shiny Application Framework"
output: 
    html_document:
        self_contained: yes
---

<!-- badges: start -->
[![CRAN status](https://www.r-pkg.org/badges/version/periscope2)](https://CRAN.R-project.org/package=periscope2)
[![Downloads Total](https://cranlogs.r-pkg.org/badges/grand-total/periscope2)](https://cran.r-project.org/package=periscope2)
[![CircleCI build status](https://circleci.com/gh/Aggregate-Genius/periscope2/tree/cran.svg?style=svg)](https://app.circleci.com/pipelines/github/Aggregate-Genius/periscope2?branch=cran)
[![codecov](https://codecov.io/gh/Aggregate-Genius/periscope2/branch/cran/graph/badge.svg?token=5LOU2JCTP6)](https://app.codecov.io/gh/Aggregate-Genius/periscope2)
<!-- badges: end -->

### Overview

***periscope2*** is a scalable and UI-standardized 'shiny' framework including a variety of developer convenience functions with the goal of both streamlining robust application development and assisting in creating a consistent user experience regardless of application or developer.


***periscope2*** is full of developer-friendly features such as:

* Predefined but flexible templates for new Shiny applications with a default [bs4Dash](https://bs4dash.rinterface.com/) layout
* Separation by file of functionality that exists in one of the three shiny scopes: global, server-global, and server-local
* Generated applications are organized in an easy to follow and maintain folder structure based on files functionality
* Off-the-shelf and ready to be used modules ('Table/React Table Downloader', 'Plot Downloader', 'File Downloader' and 'Reset Application'
* Different methods and tools to alert users and add useful information about application UI and server operations
* Application logger with different levels and a UI tool to display and review recorded application logs
* Application look and feel can be customized easily via 'www/periscope_style.yaml' or more advanced via 'www/css/custom.css'
* Application can make use of JS power by customizing 'www/js/custom.js'

Create and run sample app for full features demo

```
periscope2::create_application(name = "demo_app", location = ".", sample_app = T, right_sidebar = T)
shiny::runApp("./demo_app/", launch.browser = T)
```

-----

### Installation

periscope2 is available for installation from CRAN

```r
devtools::install_cran("periscope2")
```

and latest development version of ***periscope2*** from GitHub as follows:

```r
devtools::install_github('Aggregate-Genius/periscope2')
```

<br/>

-----

### periscope vs periscope2
[periscope](https://cran.r-project.org/package=periscope) is widely regarded and used an enterprise application development aid framework for providing the user with the ability to build a robust shiny applications and dashboards.

[periscope](https://cran.r-project.org/package=periscope) is the direct ancestor of periscope2 in which periscope2 took most of its basic philosophy and generated applications structure.

However; as periscope2 is the next generation (iteration) of [periscope](https://cran.r-project.org/package=periscope) package, they differ in many aspects as follow:

#### Underlying frameworks
[periscope](https://cran.r-project.org/package=periscope) is dependent on bootstrap 3 supported shiny packages:

- [shinydashboard](https://cran.r-project.org/package=shinydashboard)
- [shinydashboard Plus](https://cran.r-project.org/package=shinydashboardPlus)
- [shinyBS](https://cran.r-project.org/package=shinyBS)

While periscope2 is dependent only on bootstrap 4 supported shiny package to bring in more functionality (i.e dashboard functionality, alerts, tool-tips, etc..) and a modernized look and feel:

- [bs4Dash](https://cran.r-project.org/package=bs4Dash)

----

#### Generated applications layout
[periscope](https://cran.r-project.org/package=periscope) full generated app layouts:

- Left sidebar
- Right sidebar
- Header
- Body

periscope2 full generated app layouts:

- Left sidebar
- Right sidebar
- Header
- Footer
- Body

For periscope2, each part has it own UI R file that can be customized and updated. Each part also has its own dedicated alert panel, including an announcements framework as part of the header.

User can adapt layout for both packages generated apps easily via related functions (refer to `r ?periscope` and `r ?periscope2` for more info)

-----

#### Generated applications code structure

- Regardless files contents, both packages generated applications share the same first level folder structure:

<center><img src="./vignettes/figures/App_Structure.png"></center>

- Differences in the next level of structure are:
  - periscope2 **program** folder has:
    - a dedicated UI R script for each layout part
    - **config**  - folder for storing application different configuration files
      - Initially contains announcements module configuration file
    - **modules**  - folder for application shiny modules R scripts
    <center><img src="./vignettes/figures/Program_Folder.png"></center>
  - ***www*** folder that contains the application JS, CSS and theme customization files
    - Although generated applications by both frameworks will have the same folder structure, periscope2 generated applications will have two additional files **css/custom.css** and **js/custom.js** to hold custom application JS and CSS code
      - Both of these files are already linked into the generated app and ready to be used

----

#### Periscope2 Modules

- Both packages share the following modules:
  - downloadable file
    - Use openxlsx2 to download .xlsx files, openxlsx for legacy apps and backwards compatibility, and writexl as a fallback in case openxlsx2 or openxlsx not installed.
  - downloadable plot 
  - downloadable table
- However, periscope2 has more modules (more to come with each new version) as Announcements module
- The following modules behave differently between the two packages:
  - Reset Application
  - Logger

*NOTE*: Check package documentation and vignettes for module details

----

#### Periscope2 RStudio Add-ins

Periscope2 provides multiple RStudio add-ins to help package user to configure different generated application aspects visually and correctly. Please refer to the following documentation for each add-in details:

- `vignette("announcement_addin", package = "periscope2")`
- `vignette("themeBuilder_addin", package = "periscope2")`

----

#### Old Shiny module style support

While [periscope](https://cran.r-project.org/package=periscope) modules can be called either via the old style (`callModule`) or new style (`moduleServer1`),  periscope2 only supports the new module style. Check [migrating to module server](https://shiny.posit.co/r/articles/improve/modules/#migrating-from-callmodule-to-moduleserver) article for more related technical info.

----

#### fresh package Styling Support
[fresh](https://dreamrs.github.io/fresh/) package is used in both packages to style generated applications different parts via configurations rather than code using the **'www/periscope_style.yaml'** file.

While periscope uses shiny dashboard variables for styling in code and via the **'www/periscope_style.yaml'** configuration file, periscope2 uses bs4dash related variables . The file key names use related shiny dashboard or bs4dash variables for user friendly styling in both packages.

---

### Examples

These are included to get you started. You can either start with an empty application or an application that includes samples of the components that you can use within your application.  There is extensive package documentation and full Vignettes are also available to help you with all the options.

<br/>

#### Empty application

```r
periscope2::create_application('emptyapp', location = ".")
shiny::runApp('emptyapp', launch.browser = T)
```


#### Sample application - no right sidebar

```r
periscope2::create_application("sampleapp1", location = ".", sample_app = TRUE)
shiny::runApp('sampleapp1', launch.browser = T)

```

#### Sample application - including a right sidebar

```r
periscope2::create_application("sampleapp2", location = ".", sample_app = TRUE, right_sidebar = TRUE)
shiny::runApp('sampleapp2', launch.browser = T)

```

# -------------------------------------------------
# --              FRAMEWORK server.R             --
# -------------------------------------------------
# NOTES:                                         --
# Program code goes in "program" directory files --
# -------------------------------------------------
# *****        DO NOT EDIT THIS FILE       ***** --
# -------------------------------------------------

source(paste("program", "server_global.R", sep = .Platform$file.sep),
       local = TRUE)


shinyServer(function(input, output, session) {
    ss_userAction.Log <- periscope2:::fw_get_user_log()

    source(paste("program", "server_local.R", sep = .Platform$file.sep),
           local = TRUE)

    periscope2:::fw_server_setup(input            = input,
                                 output           = output,
                                 session          = session,
                                 logger           = ss_userAction.Log,
                                 logger_viewer_id = "logViewerId")

    announce_close_time <- load_announcements(announcements_file_path = "./program/config/announce.yaml")
    if (!is.null(announce_close_time)) {
        shinyjs::delay(announce_close_time,{
            bs4Dash::closeAlert("announceAlert")
        })
    }

    loginfo("%s started with log level <%s>",
            periscope2:::fw_get_title(), periscope2:::fw_get_loglevel(),
            logger = ss_userAction.Log)

    # Display application info
    observeEvent(input$app_info, {
        show_alert(html                = TRUE,
                   showConfirmButton   = FALSE,
                   btn_labels          = NA,
                   width               = "60%",
                   showCancelButton    = FALSE,
                   showCloseButton     = TRUE,
                   animation           = "slide-from-top",
                   closeOnClickOutside = TRUE,
                   text                = periscope2:::.g_opts$app_info,
                   title               = periscope2:::.g_opts$app_title)
    })
})

# ----------------------------------------
# --       PROGRAM ui_left_sidebar.R         --
# ----------------------------------------
# USE: Create UI elements for the
#      application sidebar (left side on
#      the desktop; contains options) and
#      ATTACH them to the UI by calling
#      add_ui_left_sidebar()
#
# NOTES:
#   - All variables/functions here are
#     not available to the UI or Server
#     scopes - this is isolated
# ----------------------------------------

# -- IMPORTS --



# ----------------------------------------
# --     SIDEBAR ELEMENT CREATION       --
# ----------------------------------------

# -- Left Sidebar Configurations
# Sidebar skin. "dark" or "light"
skin <- "light"

# Sidebar status
status <- "info"

# Sidebar elevation. 4 by default (until 5).
elevation <- 4

# If TRUE, the sidebar will be collapsed on app startup.
collapsed <- FALSE

# Whether to slightly close the sidebar but still show item icons
minified <- FALSE

# Whether to expand the sidebar om hover
expand_on_hover <- FALSE

# Whether to fix the sidebar
fixed <- TRUE

# -- Create Sidebar Elements
# elements to be placed before any menu
sidebar_elements <- NULL

# sidebar menu items
sidebar_menu <-  sidebarMenu(
    sidebarHeader("Periscope2 Features"),
    menuItem(
        "Application Setup",
        tabName = "application_setup",
        icon    = icon("building")
    ),
    menuItem(
        "Periscope2 Modules",
        tabName = "periscope_modules",
        icon    = icon("cubes")
    ),
    menuItem(
        "User Notifications",
        tabName = "user_notifications",
        icon    = icon("comments")
    ),
    menuItem(
        "Style Guide",
        tabName = "style_guide",
        icon    = icon("file-code")
    )
)

# An area at the bottom of the sidebar to contain elements like buttons
custom_area <- NULL

# -- Register Sidebar Elements in the ORDER SHOWN in the UI
add_ui_left_sidebar(sidebar_elements = sidebar_elements,
                    skin             = skin,
                    status           = status,
                    elevation        = elevation,
                    collapsed        = collapsed,
                    minified         = minified,
                    expand_on_hover  = expand_on_hover,
                    fixed            = fixed,
                    sidebar_menu     = sidebar_menu,
                    custom_area      = custom_area)

## ----eval=F-------------------------------------------------------------------
# # Inside ui_body.R or ui_sidebar.R
# 
# #single download type
# downloadFileButton(id            = "object_id1",
#                    downloadtypes = c("csv"),
#                    hovertext     = "Button 1 Tooltip")
# 
# #multiple download types
# downloadFileButton(id            = "object_id2",
#                    downloadtypes = c("csv", "tsv"),
#                    hovertext     = "Button 2 Tooltip")

## ----eval = F-----------------------------------------------------------------
# # Inside server_local.R
# 
# #single download type
# downloadFile(id           = "object_id1",
#              logger       = ss_userAction.Log,
#              filenameroot = "mydownload1",
#              datafxns     = list(csv = mydatafxn1),
#              aspectratio  = 1)
# 
# #multiple download types
# downloadFile(id           = "object_id2",
#              logger       = ss_userAction.Log,
#              filenameroot = "mytype2",
#              datafxns     = list(csv = mydatafxn1, xlsx = mydatafxn2),
#              aspectratio  = 1)

## ----eval=F-------------------------------------------------------------------
# library(periscope2)
# 
# app_dir = tempdir()
# create_application(name = 'mysampleapp', location = app_dir, sample_app = TRUE)
# runApp(paste(app_dir, 'mysampleapp', sep = .Platform$file.sep))

## ----eval=F-------------------------------------------------------------------
# # Add a user action to the log
# loginfo("Your Information Message with %s, %s parameters", parm1, parm2, logger = ss_userAction.Log)
# 
# # Add a warning to the log
# logwarn("Your Warning Message!", logger = ss_userAction.Log)

## ----eval=F-------------------------------------------------------------------
# createPSAlert(id      = 'bodyAlert',
#               options = list(title    = 'alert title',
#                              status   = 'warning',
#                              closable = TRUE,
#                              content  = alert contents))
# # id is the location of the alert and can be one of:
# #   'sidebarRightAlert', 'sidebarBasicAlert', 'bodyAlert', 'footerAlert' and 'headerAlert'
# 
# # status sets the bootstrap styling and can be one of:
# #    'primary', 'success', 'warning', 'danger' or 'info'

## ----eval=F-------------------------------------------------------------------
# ui_tooltip(id        = 'tooltipID',
#            label     = 'label text (optional)',
#            text      = 'text content',
#            placement = 'top')
# 
# # placement sets the tool-tip pop-up location and can be one of:
# # 'top', 'bottom', 'left' or 'right'

## ----eval=F-------------------------------------------------------------------
# library(periscope2)
# app_dir = tempdir()
# 
# # application without a right sidebar
# create_application(name       = 'mytestapp',
#                    location   = app_dir,
#                    sample_app = TRUE)
# 
# # application without a left sidebar or a right sidebar
# create_application(name         = 'mytestapp',
#                    location     = app_dir,
#                    sample_app   = TRUE,
#                    left_sidebar = FALSE)
# 
# # application with left and right sidebars
# create_application(name          = 'mytestapp',
#                    location      = app_dir,
#                    sample_app    = TRUE,
#                    right_sidebar = TRUE)

## ----eval=F-------------------------------------------------------------------
# runApp(paste(app_dir, 'mytestapp', sep = .Platform$file.sep))

## ----eval=F-------------------------------------------------------------------
# library(periscope2)
# app_dir = tempdir()
# 
# # application without a right sidebar
# create_application(name     = 'mytestapp',
#                    location = app_dir)
# 
# # application without a left sidebar or a right sidebar
# create_application(name         = 'mytestapp',
#                    location     = app_dir,
#                    left_sidebar = FALSE)
# 
# # application with a right sidebar
# create_application(name          = 'mytestapp',
#                    location      = app_dir,
#                    right_sidebar = TRUE)

## ----eval=F-------------------------------------------------------------------
# runApp('mytestapp', appDir = app_dir)

## ----eval=F-------------------------------------------------------------------
# # Plain text title
# set_app_parameters(title = "My Application Title")
# 
# # Application Title links to an external url
# set_app_parameters(title     = "My Application Title",
#                    titleinfo = "Application Valid Website")
# 
# # Application Title links to a modal window with HTML content
# set_app_parameters(title     = "My Application Title",
#                    titleinfo = HTML("<h3>This is information about this application</h3>",
#                                     "<p><b>Author: </b>Me</p>",
#                                     "<p><b>Date: </b>", Sys.Date(), "</p>"))
# 
# # Application Title links to a modal window with HTML content, setting log level to 'DEBUG',
# # using simple loading screen, init app version and pass announcements configuration module file
# set_app_parameters(title              = "periscope Example Application",
#                    app_info           = HTML("Demonstrate periscope features and generated application layout"),
#                    log_level          = "DEBUG",
#                    app_version        = "1.0.0",
#                    loading_indicator  = list(html = tagList(spin_1(), "Loading ...")),
#                    announcements_file = "./program/config/announce.yaml")

## ----eval=F-------------------------------------------------------------------
# sidebar_menu <-  sidebarMenu(
#     sidebarHeader("Periscope2 Features"),
#     menuItem(
#         "Application Setup",
#         tabName = "application_setup",
#         icon    = icon("building")
#     ),
#     menuItem(
#         "Periscope2 Modules",
#         tabName = "periscope_modules",
#         icon    = icon("cubes")
#     ),
#     menuItem(
#         "User Notifications",
#         tabName = "user_notifications",
#         icon    = icon("comments")
#     )
# )
# 
# add_ui_left_sidebar(sidebar_menu = list(sidebar_menu))

## ----eval=F-------------------------------------------------------------------
# sidebar_element <- div(checkboxInput("hideFileOrganization", "Show Files Organization"))
# 
# add_ui_right_sidebar(sidebar_elements = list(sidebar_element))

## ----eval=F-------------------------------------------------------------------
# body1 <- box(id          = "bodyElement1",
#              title       = "Box 1",
#              width       = 8,          #2/3 of the width (for 12 columns layout)
#              status      = "primary",  #colored bar at the top
#              collapsible = TRUE,
#              collapsed   = FALSE,
#              htmlOutput("example1") )
# 
# body2 <- box(id          = "bodyElement2",
#              title       = "Box 2",
#              width       = 4,          #1/3 of the width  (for 12 columns layout)
#              status      = "danger",   #colored bar at the top
#              collapsible = FALSE,
#              p("Some great text in paragraph format"),
#              pre("A pre-formatted (e.g. code) block"),
#              actionButton("exButton", label = "Example") )
# 
# add_ui_body(body_elements = list(body1, body2))

## ----eval=F-------------------------------------------------------------------
# # Navbar skin. "dark" or "light"
# skin <- "light"
# 
# # Navbar status
# status <- "white"
# 
# # Whether to separate the navbar and body by a border.
# border <- TRUE
# 
# # Whether items should be compacted
# compact <- FALSE
# 
# # Icon of the main sidebar toggle
# left_sidebar_icon <- shiny::icon("bars")
# 
# # Icon to toggle the controlbar
# right_sidebar_icon <- shiny::icon("th")
# 
# add_ui_header(skin               = skin,
#               status             = status,
#               border             = border,
#               compact            = compact,
#               left_sidebar_icon  = left_sidebar_icon,
#               right_sidebar_icon = right_sidebar_icon)

## ----eval=F-------------------------------------------------------------------
# # Left text
# left  <- a(
#     href   = "https://periscopeapps.org/",
#     target = "_blank",
#     "periscope2"
# )
# 
# # Right text
# right <- "2022"
# 
# # Whether to fix the footer in place when scrolling
# fixed <- FALSE
# 
# add_ui_footer(left  = left,
#               right = right,
#               fixed = fixed)

## ----eval=F-------------------------------------------------------------------
# source("program/fxn/makeplot.R")
# 
# #build the deferred UI from ui_body.R
# output$example1 <- renderUI({
#     list(downloadFileButton("ex_d1", c("csv"), "Download CSV"),
#          hr(),
#          p("Some great explanatory text in my application"))
#     })
# 
# downloadFile("ex_d1", ss_userAction.Log, "mydownload", list(csv=get_ref_data))
# 
# observeEvent(input$exButton, {
#     loginfo("exButton Pressed!", logger = ss_userAction.Log)
#     createPSAlert(
#         id = 'alert place',
#         options  = list(title    = 'alert title',
#                         status   = 'alert status'',
#                                          closable = TRUE,
#                                          content  = "alert contents"))
# })

## ----eval=F-------------------------------------------------------------------
# ref_data <- read.csv("program/data/mydata.csv")
# 
# get_ref_data <- function() {
#     ref_data
# }

library(ggplot2)
library(lattice)


# -- data for plots
data(mtcars)
mtcars$cyl <- factor(mtcars$cyl,
                     levels = c(4,6,8),
                     labels = c("4cyl", "6cyl", "8cyl"))
attr(mtcars, "show_rownames") <- TRUE


# -- plotting functions

plot2ggplot <- function() {
    ggplot(data = mtcars, aes(x = wt, y = mpg)) +
        geom_point(aes(color = cyl)) +
        theme(legend.justification   = c(1, 1),
              legend.position.inside = c(1, 1),
              legend.title           = element_blank()) +
        ggtitle("GGPlot Example w/Hover") +
        xlab("wt") +
        ylab("mpg")
}


plot_data <- function() {
    mtcars
}


plot3lattice <- function() {
    xyplot(mpg ~ wt,
           data     = mtcars,
           pch      = 1,
           groups   = factor(cyl),
           auto.key = list(corner = c(1, 1)),
           main     = "Lattice Example")
}


plot_htmlwidget <- function(report_modus = FALSE) {
    venn <- data.frame(A   = 57,   B = 12, C   = 67, D   = 72, AB   = 4,
                       AC  = 67,  AD = 25, BC  = 67, BD  = 27, CD   = 38,
                       ABC = 69, ABD = 28, ACD = 52, BCD = 46, ABCD = 3)

    canvasXpress(vennData = venn,
                 graphType           = "Venn",
                 vennGroups          = 4,
                 vennLegend          = list(A = "List1",
                                            B = "List2",
                                            C = "List3",
                                            D = "List4"),
                 title               = "CanvasXpress Example",
                 disableToolbar      = report_modus,
                 disableTouchToolbar = report_modus)
}

# ----------------------------------------
# --       PROGRAM server_local.R       --
# ----------------------------------------
# USE: Session-specific variables and
#      functions for the main reactive
#      shiny server functionality.  All
#      code in this file will be put into
#      the framework inside the call to
#      shinyServer(function(input, output, session)
#      in server.R
#
# NOTES:
#   - All variables/functions here are
#     SESSION scoped and are ONLY
#     available to a single session and
#     not to the UI
#
#   - For globally scoped session items
#     put var/fxns in server_global.R
#
# FRAMEWORK VARIABLES
#     input, output, session - Shiny
#     ss_userAction.Log - Reactive Logger S4 object
# ----------------------------------------

# -- IMPORTS --
source(paste("program", "fxn", "program_helpers.R", sep = .Platform$file.sep))
source(paste("program", "fxn", "plots.R", sep = .Platform$file.sep))

# -- VARIABLES --

original_theme_settings <- list()
original_theme_settings[["primary"]]   <- "#EBCDFC"
original_theme_settings[["secondary"]] <- "#6c757d"
original_theme_settings[["success"]]   <- "#2ED610"
original_theme_settings[["info"]]      <- "#7BDFF2"
original_theme_settings[["warning"]]   <- "#FFF200"
original_theme_settings[["danger"]]    <- "#CE0900"
original_theme_settings[["bg"]]        <- "#FFFFE6"

# -- FUNCTIONS --
appReset(id     = "appResetId",
         logger = ss_userAction.Log)

downloadFile("exampleDownload1",
             ss_userAction.Log,
             "examplesingle",
             list(csv = load_data1))
downloadFile("exampleDownload2",
             ss_userAction.Log,
             "examplemulti",
             list(csv = load_data2, xlsx = load_data2, tsv = load_data2))

sketch <- htmltools::withTags(
    table(
        class = "display",
        thead(
            tr(
                th(rowspan = 2, "Location"),
                th(colspan = 2, "Statistics")),
            tr(
                th("Change"),
                th("Increase")))
    ))


downloadableTable("exampleDT1",
                  ss_userAction.Log,
                  "exampletable",
                  list(csv = load_data3, tsv = load_data3),
                  load_data3,
                  table_options = list(colnames    = c("Area", "Delta", "Increase"),
                                       filter      = "bottom",
                                       callback    = htmlwidgets::JS("table.order([1, 'asc']).draw();"),
                                       container   = sketch,
                                       formatStyle = list(columns = c("Total.Population.Change"),
                                                          color   = DT::styleInterval(0, c("red", "green"))),
                                       formatStyle = list(columns         = c("Natural.Increase"),
                                                          backgroundColor = DT::styleInterval(c(7614, 15914, 34152),
                                                                                              c("lightgray", "gray", "cadetblue", "#808000")))))
downloadableReactTable(id                 = "exampleReactTable",
                       logger             = ss_userAction.Log,
                       table_data         = load_data3,
                       file_name_root     = "exampleReacttable",
                       download_data_fxns = list(csv = load_data3, tsv = load_data3),
                       table_options      = list(
                           defaultSorted = "Total.Population.Change",
                           columnGroups  = list(colGroup(name = "Statistics", columns = c("Total.Population.Change", "Natural.Increase"))),
                           columns       = list(
                               Total.Population.Change = colDef(
                                   name       = "Change",
                                   filterable = TRUE,
                                   cell       = function(value) {
                                       if (value <= 0) {
                                           tags$span(style = "color:red", value)
                                       } else {
                                           tags$span(style = "color:green", value)
                                       }
                               }),
                               Natural.Increase        = colDef(
                                   name       = "Increase",
                                   filterable = TRUE,
                                   cell       = function(value) {
                                       if (value <= 7614) {
                                           tags$span(class = "badge bg-primary", value)
                                       } else if (value <= 15914) {
                                           tags$span(class = "badge bg-secondary", value)
                                       } else if (value <= 34152) {
                                           tags$span(class = "badge bg-info", value)
                                       } else {
                                           tags$span(class = "badge bg-success", value)
                                       }
                                   }),
                               Geographic.Area         = colDef(name = "Location", filterable = TRUE))))
downloadablePlot("examplePlot2",
                 ss_userAction.Log,
                 filenameroot = "plot2_ggplot",
                 downloadfxns = list(jpeg = plot2ggplot,
                                     csv  = plot_data),
                 aspectratio  = 1.5,
                 visibleplot  = plot2ggplot)

downloadablePlot("examplePlot3",
                 ss_userAction.Log,
                 filenameroot = "plot3_lattice",
                 aspectratio  = 2,
                 downloadfxns = list(png  = plot3lattice,
                                     tiff = plot3lattice,
                                     txt  = plot_data,
                                     tsv  = plot_data),
                 visibleplot  = plot3lattice)

# ----------------------------------------
# --          SHINY SERVER CODE         --
# ----------------------------------------

# -- Observe UI Changes
observeEvent(input$rightAlert, {
    loginfo("Right Sidebar Alert Button Pushed",
            logger = ss_userAction.Log)
    createPSAlert(id      = "sidebarRightAlert",
                  options = list(title    = "Right Side",
                                 status   = "success",
                                 closable = TRUE,
                                 content  = "Example Basic Sidebar Alert"))
})

observeEvent(input$leftAlert, {
    logwarn("Left Sidebar Alert Example Button Pushed",
            logger = ss_userAction.Log)
    createPSAlert(id      = "sidebarBasicAlert",
                  options = list(
                      title    = "Left Side",
                      status   = "warning",
                      closable = TRUE,
                      content  = "Example Advanced Sidebar Alert"))

})

observeEvent(input$bodyAlertBtn, {
    logdebug("Body Alert Example Button Pushed",
             logger = ss_userAction.Log)
    createPSAlert(id      = "bodyAlert",
                  options = list(
                      title    = "Body",
                      status   = "info",
                      closable = TRUE,
                      content  = paste("Example Body Alert")))
})

observeEvent(input$footerAlertbtn, {
    logerror("Footer Alert Example Button Pushed",
             logger = ss_userAction.Log)
    createPSAlert(id      = "footerAlert",
                  options = list(title    = "Footer",
                                 status   = "danger",
                                 closable = TRUE,
                                 content  = paste("Example Footer Alert")))
})

observeEvent(input$headerAlertbtn, {
    loginfo("Header Alert Example Button Pushed",
            logger = ss_userAction.Log)
    createPSAlert(id      = "headerAlert",
                  options = list(title    = "Header",
                                 status   = "primary",
                                 closable = TRUE,
                                 content  = "Example Header Alert"))
})

observeEvent(input$showWorking, {
    loginfo("Show Busy Indicator Button Pushed",
            logger = ss_userAction.Log)
    Sys.sleep(5)
})


observeEvent(input$showAppLoading, {
    loginfo("Show App Loading Page Button Pushed",
            logger = ss_userAction.Log)
    loginfo(getwd())
    source("./ui.R", local = T)
    session$reload()
})


output$hover_info <- renderUI({
    output_panel <- NULL
    hover        <- input$examplePlot2_hover
    point        <- nearPoints(mtcars, hover,
                               xvar = "wt", yvar = "mpg",
                               maxpoints = 1)

    if (NROW(point) > 0) {
        left_pct <- (hover$x - hover$domain$left) / (hover$domain$right - hover$domain$left)
        left_px  <- hover$range$left + left_pct * (hover$range$right - hover$range$left)
        top_pct  <- (hover$domain$top - hover$y) / (hover$domain$top - hover$domain$bottom)
        top_px   <- hover$range$top + top_pct * (hover$range$bottom - hover$range$top)

        style <- paste0("position:absolute;",
                        "z-index:100;",
                        "background-color: rgba(245, 245, 245, 0.85); ",
                        "left:", left_px - 20, "px; top:", top_px + 300, "px;")

        output_panel <- wellPanel(class = "well-sm",
                                  style = style,
                                  HTML("<b> Car: </b>", rownames(point)))
    }

    output_panel
})

output$file_structure_plot <- renderCanvasXpress({
    canvasXpress(
        data             = files_idx,
        smpAnnot         = app_files,
        graphType        = "Tree",
        hierarchy        = list("App_Root", "L1"),
        title            = "Empty Application Files",
        graphOrientation = "horizontal",
        xAxis            = list("order"),
        colorBy          = list("App_Root"),
        showLegend       = FALSE,
        events           = node_event
    )
})

observeEvent(input$node_name, {
    node_name <- gsub(pattern     = "Tree:| \\(\\d*\\)",
                      replacement = "",
                      x           = input$node_name) %>%
        trimws()
    output$file_description <- renderText({
        node_description[[node_name]]
    })
})


observeEvent(input$hideFileOrganization, {
    updateBox("files_organization", action = "toggle")
})


observeEvent(TRUE,{
    output$app_theme <- renderUI(fresh::use_theme(periscope2:::create_theme()))
    theme_settings   <- read_themes()

    updateColourInput(session,
                      inputId = "primary_picker",
                      value   = ifelse(is.null(theme_settings[["primary"]]), "#00000000", theme_settings[["primary"]]))
    updateColourInput(session,
                      inputId = "secondary_picker",
                      value   = ifelse(is.null(theme_settings[["secondary"]]), "#00000000", theme_settings[["secondary"]]))
    updateColourInput(session,
                      inputId = "success_picker",
                      value   = ifelse(is.null(theme_settings[["success"]]), "#00000000", theme_settings[["success"]]))
    updateColourInput(session,
                      inputId = "info_picker",
                      value   = ifelse(is.null(theme_settings[["info"]]), "#00000000", theme_settings[["info"]]))
    updateColourInput(session,
                      inputId = "warning_picker",
                      value   = ifelse(is.null(theme_settings[["warning"]]), "#00000000", theme_settings[["warning"]]))
    updateColourInput(session,
                      inputId = "danger_picker",
                      value   = ifelse(is.null(theme_settings[["danger"]]), "#00000000", theme_settings[["danger"]]))
    updateNumericInput(session,
                       inputId = "left_width",
                       value   = ifelse(is.null(theme_settings[["sidebar_width"]]), NA, theme_settings[["sidebar_width"]]))
    updateNumericInput(session,
                       inputId = "right_width",
                       value   = ifelse(is.null(theme_settings[["control_sidebar_width"]]), NA, theme_settings[["right_sidebar_width"]]))
    updateColourInput(session,
                      inputId = "background_color_picker",
                      value   = ifelse(is.null(theme_settings[["main-bg"]]), "#00000000", theme_settings[["main-bg"]]))
    updateColourInput(session,
                      inputId = "sidebar_background_color_picker",
                      value   = ifelse(is.null(theme_settings[["bg"]]), "#00000000", theme_settings[["bg"]]))
    updateColourInput(session,
                      inputId = "sidebar_background_hover_color_picker",
                      value   = ifelse(is.null(theme_settings[["hover_bg"]]), "#00000000", theme_settings[["hover_bg"]]))
    updateColourInput(session,
                      inputId = "sidebar_hover_color_picker",
                      value   = ifelse(is.null(theme_settings[["hover_color"]]), "#00000000", theme_settings[["hover_color"]]))
    updateColourInput(session,
                      inputId = "sidebar_color_picker",
                      value   = ifelse(is.null(theme_settings[["color"]]), "#00000000", theme_settings[["color"]]))
    updateColourInput(session,
                      inputId = "sidebar_active_color_picker",
                      value   = ifelse(is.null(theme_settings[["active_color"]]), "#00000000", theme_settings[["active_color"]]))
}, once = TRUE)


observeEvent(input$update_app_theme, {
    theme_settings <- list()
    theme_settings[["primary"]]   <- input$primary_picker
    theme_settings[["secondary"]] <- input$secondary_picker
    theme_settings[["success"]]   <- input$success_picker
    theme_settings[["info"]]      <- input$info_picker
    theme_settings[["warning"]]   <- input$warning_picker
    theme_settings[["danger"]]    <- input$danger_picker

    left_width  <- input$left_width
    right_width <- input$right_width

    if (is.na(left_width) ||
        is.null(left_width) ||
        (left_width <= 0)) {
        theme_settings[["sidebar_width"]] <- NULL
    } else {
        theme_settings[["sidebar_width"]] <- left_width
    }

    if (is.na(right_width) ||
        is.null(right_width) ||
        (right_width <= 0)) {
        theme_settings[["control_sidebar_width"]] <- NULL
    } else {
        theme_settings[["control_sidebar_width"]] <- right_width
    }

    background_color_picker               <- NULL
    sidebar_background_color_picker       <- NULL
    sidebar_background_hover_color_picker <- NULL
    sidebar_hover_color_picker            <- NULL
    sidebar_color_picker                  <- NULL
    sidebar_active_color_picker           <- NULL

    if (input$background_color_picker != "#00000000") {
        background_color_picker <- input$background_color_picker
    }

    if (input$sidebar_background_color_picker != "#00000000") {
        sidebar_background_color_picker <- input$sidebar_background_color_picker
    }

    if (input$sidebar_background_hover_color_picker != "#00000000") {
        sidebar_background_hover_color_picker <- input$sidebar_background_hover_color_picker
    }

    if (input$sidebar_hover_color_picker != "#00000000") {
        sidebar_hover_color_picker <- input$sidebar_hover_color_picker
    }

    if (input$sidebar_color_picker != "#00000000") {
        sidebar_color_picker <- input$sidebar_color_picker
    }

    if (input$sidebar_active_color_picker != "#00000000") {
        sidebar_active_color_picker <- input$sidebar_active_color_picker
    }

    theme_settings[["main-bg"]]      <- background_color_picker
    theme_settings[["bg"]]           <- sidebar_background_color_picker
    theme_settings[["hover_bg"]]     <- sidebar_background_hover_color_picker
    theme_settings[["hover_color"]]  <- sidebar_hover_color_picker
    theme_settings[["color"]]        <- sidebar_color_picker
    theme_settings[["active_color"]] <- sidebar_active_color_picker

    write_yaml(theme_settings, "www/periscope_style.yaml")
    session$reload()
})


observeEvent(input$restore_app_theme, {
    write_yaml(original_theme_settings, "www/periscope_style.yaml")
    session$reload()
})

## ----eval=F-------------------------------------------------------------------
# # Inside program\global.R
# 
# load_announcements(announcement_location_id = "./program/config/announce.yaml")

## ----eval=F-------------------------------------------------------------------
# library(periscope2)
# 
# app_dir = tempdir()
# create_application(name = 'mysampleapp', location = app_dir, sample_app = TRUE)
# runApp(paste(app_dir, 'mysampleapp', sep = .Platform$file.sep))

# ----------------------------------------
# --     PROGRAM ui_footer.R     --
# ----------------------------------------
# USE: Create UI elements for the
#      application footer and
#      ATTACH them to the UI by calling
#      add_ui_footer()
#
# NOTES:
#   - All variables/functions here are
#     not available to the UI or Server
#     scopes - this is isolated
# ----------------------------------------

# -- IMPORTS --



# ----------------------------------------
# --   FOOTER ELEMENT CREATION   --
# ----------------------------------------

# -- Create Elements
# Left text
left <- div("Visit the ", a(
    href   = "https://periscopeapps.org/",
    target = "_blank",
    "hosted gallery"
), "for additional example applications using periscope and periscope2.")

# Right text
right <- div("To contribute, visit package ",  a(
    href   = "https://github.com/Aggregate-Genius/periscope2",
    target = "_blank",
    tags$i(class = "fa fa-github")))

# Whether to fix the footer
fixed <- TRUE

# -- Register Elements in the ORDER SHOWN in the UI
add_ui_footer(left  = left,
              right = right,
              fixed = fixed)

# ----------------------------------------
# --      PROGRAM server_global.R       --
# ----------------------------------------
# USE: Server-specific variables and
#      functions for the main reactive
#      shiny server functionality.  All
#      code in this file will be put into
#      the framework outside the call to
#      shinyServer(function(input, output, session)
#      in server.R
#
# NOTES:
#   - All variables/functions here are
#     SERVER scoped and are available
#     across all user sessions, but not to
#     the UI.
#
#   - For user session-scoped items
#     put var/fxns in server_local.R
#
# FRAMEWORK VARIABLES
#     none
# ----------------------------------------

# -- IMPORTS --


# -- VARIABLES --


# -- FUNCTIONS --

# ----------------------------------------
# --          PROGRAM ui_body.R         --
# ----------------------------------------
# USE: Create UI elements for the
#      application body (right side on the
#      desktop; contains output) and
#      ATTACH them to the UI by calling
#      add_ui_body()
#
# NOTES:
#   - All variables/functions here are
#     not available to the UI or Server
#     scopes - this is isolated
# ----------------------------------------

# -- IMPORTS --



# ----------------------------------------
# --      BODY ELEMENT CREATION         --
# ----------------------------------------

# -- Create Elements

### Application Setup Elements
about_box <- box(
    id          = "about_box",
    width       = 12,
    status      = "info",
    solidHeader = TRUE,
    title       = "About periscope2",
    tags$dl(tags$p("periscope2 is a scalable and UI-standardized 'shiny' framework including a variety of developer convenience",
                   "functions with the goal of both streamlining robust application development and assisting in creating a consistent",
                   " user experience regardless of application or developer."),
            tags$dt("Features"),
            tags$ul(tags$li("Predefined but flexible template for new Shiny applications with a default dashboard layout"),
                    tags$li("Separation by file of functionality that exists in one of the three shiny scopes: global, server-global, and server-local."),
                    tags$li("Off-the-shelf and ready to be used modules ('Table Downloader', 'Plot Downloader', 'File Downloader' and 'Reset Application'"),
                    tags$li("Different methods and tools to alert users and add useful information about application UI and server operations"),
                    tags$li("Application logger with different levels and a UI tool to display and review recorded application logs"),
                    tags$li("Application look and feel can be customized easily via 'www/periscope_style.yaml' or more advanced via 'www/css/custom.cs'"),
                    tags$li("Application can make use of JS power by customizing 'www/js/custom.js'")))
)


announcements_box <- box(
    id          = "announcements",
    title       = "Announcements",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$li("'Announcement' feature is a user friendly method to announce a recently added feature, initiate a shutdown warning, or ",
                                    "broadcast any news to your users at once"),
                            tags$li("Announcements feature aim to alert app user with customized admin messages upon application start for a predefined time"),
                            tags$li("Announcement can be either a simple text or a fully rich HTML text."),
                            tags$li("Announcement location is on the application header"),
                            tags$li("Announcement status (body color) can be: 'primary', 'success', 'warning', 'danger' or 'info'"),
                            tags$li("Application admin can control when an 'Announcement' should start to be seen and when it should be expire"),
                            tags$li("Application admin can control how long an 'Announcement' stay visible for users or let each user close it manually"))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$li("No development is needed to launch or control 'Announcements' feature, only configurations"),
                            tags$li("Default feature generated configuration file is called ", tags$b("announce.yaml")),
                            tags$li("Default configuration file path is: ", tags$i("'program/config/announce.yaml'")),
                            tags$li("Configuration file path can be changed based on admin preference. Only update ",
                                    tags$b("'announcements_file_path'"), " parameter in ", tags$b("'load_announcements'"), " method in ",
                                    tags$b("'server.R'"), " file with the new config file path"),
                            tags$li("Check generated configuration file self-documentation for each config variable usage details"),
                            tags$li("Review current application example configuration for more details on how to configure/disable announcements`"))))
)


logger_box <- box(
    id          = "logger",
    title       = "Logger",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$li("Logger feature presents a developer and an admin friendly method to record different user or internal actions ",
                                    "in their recoded time and using different logging levels"),
                            tags$li("There are 4 different available log levels: 'logdebug', 'loginfo', 'logwarn' and 'logerror' in that order"),
                            tags$li("Each log level is displayed in different text color in R console"),
                            tags$li("User can control which log levels to be excluded from being displayed or recorded"),
                            tags$ul(style = "list-style-type: circle;",
                                    tags$li("The log rolls over for each session"),
                                    tags$li("The log files are kept in the ", tags$b("/log"), "directory and named ",  tags$b("'actions.log'")),
                                    tags$li("ONE old copy of the log is kept as ", tags$b("'actions.log.last'"))),
                            tags$li("Beside reviewing the log files directly, admins can review logs in application via ", tags$b("logViewer UI"), " module"))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$ul(tags$li("To create actions to the framework call one of the logging ",
                                            "functions like: "),
                                    blockQuote(p(pre("S: logXXXX('Your Log Message with %s, %s parameters', parm1, parm2, logger = ss_userAction.Log)")),
                                               color = "info"),
                                    tags$li(p("The XXXX should be replaced by an actual log level like ", tags$b("'debug'"), ", ",
                                              tags$b("'info'"), ", ", tags$b("'warn'"), ", ",  "or ", tags$b("'error'"),
                                              " The framework will handle updating LogViewer UI module every time the log is added to.")),
                                    tags$li("'ss_userAction.Log' is a periscope framework logger that is defined automatically under server.R and",
                                            tags$b(" should not be altered")),
                                    tags$li("Log level can be changed based on admin preference. Only update ",
                                            tags$b("'log_level'"), " parameter in ", tags$b("'set_app_parameters'"), " method in ",
                                            tags$b("'program/global.R'"), " file with the required log level ",
                                            tags$b("'DEBUG'"), ", ", tags$b("'INFO'"), ", ", tags$b("'WARN'"), " or ", tags$b("'ERROR'")),
                                    tags$li("To display application logs like below portlet add the following logViewer table to any of 'ui_body.R' boxes",
                                            blockQuote(p(pre("U: logViewerOutput('logViewerId')")),
                                                       color = "info")))))),
    fluidRow(column(width = 12,
                    tags$dl(tags$dt("Example"),
                            tags$p("Click different button in the example application and compare console with below output"),
                            br(),
                            br(),
                            div(logViewerOutput("logViewerId"), style = "min-width: 'auto';"))))
)


files_organization_box <- box(
    id          = "files_organization",
    title       = "Files Organization",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    canvasXpressOutput("file_structure_plot", height = "88vh"),
    htmlOutput("file_description")
)

### Application Modules Elements
plot2_hover <- hoverOpts(id = "examplePlot2_hover")


react_table_box <- box(
    id          = "react_downloader",
    title       = "React Table Downloader",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$ul(tags$li("React table downloader module displays tabular data in rich formatted tables using the", tags$b("`reactable`"), "package"),
                                    tags$li("downloadableReactTable returns a named reactive list that can be used in the application:",
                                            tags$ul(tags$li(tags$b("selected_rows"), ": data.frame of current selected rows"),
                                                    tags$li(tags$b("table_state"), ": a list of the current table state"))),
                                    tags$li("Table data can be downloaded in different formats such as: ", tags$b("'csv'"),
                                            ", ", tags$b("'tsv'"), ", ", tags$b("'txt'"), "and/or ", tags$b("'xlsx'")),
                                    tags$li("User can customize downloadableReactTable modules using reactable package options."),
                                    tags$li("For more information about table options please visit the",
                                            tags$a("Reactable documentation", target = "_blank", href = "https://glin.github.io/reactable/"),
                                            "site")
                            ))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$li("Module should be configured in both UI and Server code"),
                            tags$li("In the 'body_ui.R' add the following lines:",
                                    blockQuote("downloadableReactTableUI('exampleReactTable',
                                                                         'Download react table data'))", color = "info")),
                            tags$li("In the 'server_local.R' add the following lines (make sure the two use the same ID):",
                                    blockQuote("downloadableReactTable('exampleReactTable',
                                                                       ss_userAction.Log,
                                                                       'exampletable',
                                                                        list(csv = load_data3, tsv = load_data3))", color = "info")),
                            tags$li("Review ", tags$b("?downloadableReactTableUI"), " and ", tags$b("?downloadableReactTable"), " for more information"),
                            tags$li("Review below table for detailed example code")))),
    fluidRow(column(width = 12,
                    downloadableReactTableUI(id            = "exampleReactTable",
                                             downloadtypes = list("csv", "tsv"),
                                             hovertext     = "Download react table data")))

)



table_downloader_box <- box(
    id          = "table_downloader",
    title       = "Table Downloader",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$ul(tags$li("Table downloader module display tabular data in rich formatted tables using `DT` package"),
                                    tags$li("Selected rows in the table are returned as a reactive variable that can be used in application different areas"),
                                    tags$li("Table data can be downloaded in different formats such as: ", tags$b("'csv'"),
                                            ", ", tags$b("'tsv'"), ", ", tags$b("'txt'"), "and/or ", tags$b("'xlsx'")),
                                    tags$li("User can customize downloadableTable modules using DT options such as:",
                                            tags$ul(tags$li("labels:", HTML("&nbsp;"),
                                                            tags$b(tags$i("i.e. 'colnames', 'caption', ..."))),
                                                    tags$li("layout and columns styles:", HTML("&nbsp;"),
                                                            tags$b(tags$i("i.e. 'container', 'formatStyle', ..."))),
                                                    tags$li("other addons:", HTML("&nbsp;"),
                                                            tags$b(tags$i("i.e. 'filter', 'callback', ..."))))),
                                    tags$li("For more information about table options please visit the",
                                            tags$a("DT documentation", target = "_blank", href = "https://rstudio.github.io/DT/"),
                                            "site")
                            ))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$li("Module should be configured in both UI and Server code"),
                            tags$li("In the 'body_ui.R' add the following lines:",
                                    blockQuote("downloadableTableUI('exampleDT1',
                                                                    list('csv', 'tsv'),
                                                                    'Download table data'))", color = "info")),
                            tags$li("In the 'server_local.R' add the following lines (make sure the two use the same ID):",
                                    blockQuote("downloadableTable('exampleDT1',
                                                                  ss_userAction.Log,
                                                                  'exampletable',
                                                                  list(csv = load_data3, tsv = load_data3))", color = "info")),
                            tags$li("Review ", tags$b("?downloadableTableUI"), " and ", tags$b("?downloadableTable"), " for more information"),
                            tags$li("Review below table for detailed example code")))),
    fluidRow(column(width = 12,
                    downloadableTableUI("exampleDT1",
                                        list("csv", "tsv"),
                                        "Download table data")))

)

file_downloader_box <- box(
    id          = "file_downloader",
    title       = "File Downloader",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$ul(tags$li("File downloader module provides the user with the ability to download any server generated data"),
                                    tags$li("Data can be downloaded in any format available for both 'Table Downloader' and 'Plot Downloader' modules"),
                                    tags$li("Data can be downloaded in single format or in multiple formats, based on module configurations")))),
             column(width = 6,
                    tags$dl(
                        tags$dt("Setup"),
                        tags$ul(tags$li("Module should be configured in both UI and Server code",
                                        blockQuote(p(pre("U: downloadFileButton('uiID', list(extensions))"),
                                                     pre("S: downloadFile('uiID', logger, 'filenameroot', list(datafxns)")),
                                                   color = "info")),
                                tags$li("Review ", tags$b("?downloadFileButton"), " and ", tags$b("?downloadFile"), " for more information"),
                                tags$li("Review below file downloaders for detailed example code"))))),
    fluidRow(column(width = 12,
                    tags$dl(tags$dt("Example"),
                            "Single Download: ",
                            downloadFileButton("exampleDownload1", c("csv"), "csv"),
                            "Multiple-choice Download: ",
                            downloadFileButton("exampleDownload2",
                                               c("csv", "xlsx", "tsv"), "Download options"))))
)

plot_downloader_box <- box(
    id          = "plot_downloader",
    title       = "Plot Downloader - ggplot2 & lattice",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$ul(tags$li("This module creates a custom plot output that provide download capability",
                                            "to plot tools that have no internal download ability like ggplot2 & lattice."),
                                    tags$li("Plot can be downloaded in different formats such as: ", tags$b("'png'"),
                                            ", ", tags$b("'jpeg'"), ", ", tags$b("'tiff'"), "and/or ", tags$b("'bmp'")),
                                    tags$li("Plot data itself can be downloaded in different formats such as: ", tags$b("'csv'"),
                                            ", ", tags$b("'tsv'"), ", ", tags$b("'txt'"), "and/or ", tags$b("'xlsx'")),
                                    tags$li("To enrich user experience, plot downloader module also can take advanced shiny parameters as 'clickOpts', 'hoverOpts' or 'brushOpts'"),
                                    tags$li("Plot dimensions can be easily adapted"),
                                    tags$li("plot download button location can be easily controlled")))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$ul(tags$li("Module should be configured in both the UI and Server code"),
                                    tags$li("In the 'body_ui.R' add the following lines:",
                                            blockQuote("downloadablePlotUI('myplotID', c('png', 'csv'), 'Download Plot or Data', '300px'))", color = "info")),
                                    tags$li("In the 'server_local.R' add the following lines (make sure the two use the same ID):",
                                            blockQuote("downloadablePlot('myplotID',
                                                                  ss_userAction.Log,
                                                                  filenameroot = 'mydownload1',
                                                                  downloadfxns = list(png = myplotfxn, tsv = mydatafxn))", color = "info")),
                                    tags$li(blockQuote("Run '?periscope2::downloadablePlot' for more info", color = "info")),
                                    tags$li("Review below table for detailed example code"))))),
    fluidRow(column(width = 12,
                    tags$h3("Example")),
             tags$br(),
             column(width = 6, downloadablePlotUI("examplePlot2",
                                                  list("jpeg", "csv"),
                                                  "Download plot or data",
                                                  btn_halign  = "left",
                                                  btn_valign  = "top",
                                                  btn_overlap = FALSE,
                                                  hoverOpts   = plot2_hover)),
             column(width = 6, downloadablePlotUI("examplePlot3",
                                                  list("png", "tiff",
                                                       "txt", "tsv"),
                                                  btn_overlap = FALSE,
                                                  "Download plot or data"))),
    uiOutput("hover_info")
)

reset_application_box <- box(
    id          = "reset_application",
    title       = "Reset Application",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$ul(tags$li("appReset module provides the ability to:"),
                                    tags$ul(tags$li("Resets a user's session"),
                                            tags$li("Rolls over their log.")),
                                    tags$li("It creates a toggle button to reset application session"),
                                    tags$li("Upon pressing on the button, its state is flipped to 'cancel application reload'",
                                            " with application and console warning messages indicating that the application will be reloaded "),
                                    tags$li("User can either resume reloading application session or cancel reloading process,",
                                            tags$em(" which will also generate application and console messages to indicate reloading status and result.")),
                                    tags$li("Waiting time before actual session reset is configurable in milliseconds"),
                                    tags$li("Generated module alerts location can be customized to be displayed in any place, default is application body")))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$ul(tags$li("Functionality can be added with the following code:",
                                            blockQuote(p(pre("U: appResetButton('appResetId')"),
                                                         pre("S: appReset(id = 'appResetId', reset_wait = 5000, logger = logger)")),
                                                       color = "info")),
                                    tags$li(blockQuote("Run '?periscope2::appResetButton' and '?periscope2::appReset' for more info", color = "info")),
                                    tags$li("Review example below for detailed code"))))),
    fluidRow(tags$dl(tags$dt("Example"),
                     appResetButton("appResetId")))
)

### User Notifications Elements
busy_indicator_box <- box(
    id          = "busy_indicator",
    title       = "Busy Indicators",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    p("Periscope2 has two application busy indicators"),
    tags$dl(tags$dt("Application Loading"),
            tags$li("Any 'waiter' package loading screen indicator that displayed upon application start up or reload."),
            tags$li("Loading screen can be changed by updating ",
                    tags$b("'loading_indicator'"), " parameter in ", tags$b("'set_app_parameters'"), " method in ",
                    tags$b("'program/global.R'"), " 'waiter' package loading screen"),
            tags$li("For more information about loadnig screens options please visit the",
                    tags$a("waiter documentation", target = "_blank", href = "https://waiter.john-coene.com/"),
                    "site"),
            tags$li("Check example below:"),
            br(),
            br(),
            tags$dd(actionButton("showAppLoading",
                                 label  = "Show application Loading",
                                 status = "danger")),
            hr(),
            tags$dt("Busy Session"),
            tags$li("An automatic wait indicator in the navbar when the shiny server session is busy."),
            tags$dd(actionButton("showWorking",
                                 label  = "Show application busy indicator for 5 seconds",
                                 status = "info")))
)

alerts_box <- box(
    id          = "alerts",
    title       = "Alerts",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    p("Alerts can be added with the following code in the server:"),
    blockQuote(tags$pre("S:periscope2::createPSAlert(id = 'alert place',
                         options  = list(title    = 'alert title',
                         status   = 'alert status'',
                         closable = TRUE,
                         content  = alert contents))"), color = "info"),
    hr(),
    tags$h5("Examples"),
    p("LOCATION can be: 'sidebarRightAlert', 'sidebarBasicAlert', 'bodyAlert', 'footerAlert' and 'headerAlert'"),
    conditionalPanel("$('.main-sidebar').length > 0",
                     actionButton("leftAlert",
                                  label  = "Left Alert",
                                  status = "warning",
                                  width  = "15%"),
                     style = "display: inline;"),
    actionButton("bodyAlertBtn",
                 label  = "Body",
                 status = "info",
                 width  = "15%"),
    conditionalPanel("$('.control-sidebar').length > 0",
                     actionButton("rightAlert",
                                  label  = "Right Alert",
                                  status = "success",
                                  width  = "15%"),
                     style = "display: inline;"),
    conditionalPanel("$('.main-footer').length > 0",
                     actionButton("footerAlertbtn",
                                  label  = "Footer Alert",
                                  status = "danger",
                                  width  = "15%"),
                     style = "display: inline;"),
    actionButton("headerAlertbtn",
                 label  = "Header Alert",
                 status = "primary",
                 width  = "15%")
)

tooltips_box <- box(
    id          = "tooltips",
    title       = "Tooltips",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    p("Tooltips can be added with the following code in the UI:"),
    blockQuote("U: ui_tooltip('tooltipID', 'label text (optional)', 'text content', 'placement' (optional))", color = "info"),
    tags$h5("Examples"),
    ui_tooltip(id    = "top_tip",
               label = "Top Tooltips",
               text  = "Top tooltip"),
    ui_tooltip(id        = "left_tip",
               label     = "Left Tooltips",
               text      = "Left tooltip",
               placement = "left"),
    ui_tooltip(id        = "bottom_tip",
               label     = "Bottom Tooltips",
               text      = "Bottom tooltip",
               placement = "bottom"),
    ui_tooltip(id        = "right_tip",
               label     = "Right Tooltips",
               text      = "Right tooltip",
               placement = "right")
)


style_guide_box <- box(
    id          = "style_guide",
    title       = "Style Guide",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    tags$dt("periscope2 App Theme and CSS"),
    tags$li("periscope2 depends on", tags$a("fresh package", target = "_blank", href = "https://github.com/dreamRs/fresh"),
            " for styling application theme"),
    tags$li("There are many theme variables are available for customization. Review ",
            tags$a("this article", target = "_blank", href = "https://dreamrs.github.io/fresh/articles/vars-bs4dash.html"),
            " for more info"),
    tags$li("Fortunately, periscope2 provides a user-friendly method to update fresh theme variables via configurations,
            rather than by updating them programatically.",
            tags$ul(tags$li("The theme can be updated by editing 'www/periscope_style.yaml' and relaunching the app"))),
    tags$li("'periscope_style.yaml' contains the most important variables with documentation explaining what each variable affects and possible values for it"),
    tags$li("Another method to update application themes or any controls in CSS is by updating 'www/css/custom.css' file"),
    tags$li("The below widget explains some of the variables and how they affect the application"),
    br(),
    br(),
    tags$b(tags$i("** Note: Updating app theme might take a few seconds")),
    br(),
    br(),
    hr(),
    tags$dt("Status Colors"),
    tags$li("Sets the status (used by bootstrap 4) colors that affects the color of the header, valueBox, infoBox and box"),
    tags$li("Valid values are names of the color or hex-decimal value of the color (i.e,: 'blue', '#086A87')"),
    tags$li("Blank, empty or invalid values will use the default values"),
    br(),
    fluidRow(column(width = 6,
                    colourInput(inputId          = "primary_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "primary_tip",
                                                              label = "Primary",
                                                              text  = "Set app 'Primary' color"))),
             column(width = 6,
                    colourInput(inputId          = "secondary_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "secondary_tip",
                                                              label = "Secondary",
                                                              text  = "Set app 'Secondary' color")))),
    fluidRow(column(width = 6,
                    colourInput(inputId          = "success_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "success_tip",
                                                              label = "Success",
                                                              text  = "Set app 'Success' color"))),
             column(width = 6,
                    colourInput(inputId          = "info_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "info_tip",
                                                              label = "Info",
                                                              text  = "Set app 'Info' color")))),
    fluidRow(column(width = 6,
                    colourInput(inputId          = "warning_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "warning_tip",
                                                              label = "Warning",
                                                              text  = "Set app 'Warning' color"))),
             column(width = 6,
                    colourInput(inputId          = "danger_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "danger_tip",
                                                              label = "Danger",
                                                              text  = "Set app 'Danger' color")))),
    hr(),
    tags$dt("Sidebars Width"),
    tags$li("Width or padding is to be specified as a numeric value in pixels. Must be greater than 0 and include numbers only"),
    tags$li("Valid possible value are 200, 350, 425, ..."),
    tags$li("Blank, empty or invalid values will use the default values"),
    br(),
    fluidRow(column(width = 6,
                    numericInput(inputId = "left_width",
                                 value   = 0,
                                 label   = ui_tooltip(id    = "left_tip",
                                                      label = "Left Sidebar Width",
                                                      text  = "Set Left Sidebar Width"))),
             column(width = 6,
                    numericInput(inputId = "right_width",
                                 value   = 0,
                                 label   = ui_tooltip(id    = "right_tip",
                                                      label = "Right Sidebar Width",
                                                      text  = "Set Right Sidebar Width")))),
    hr(),
    tags$dt("Layout Parts Colors"),
    tags$li("Sets application different layout parts (sidebars, background, ...) colors"),
    tags$li("Valid values are names of the color or hex-decimal value of the color (i.e,: 'blue', '#086A87')"),
    tags$li("Blank, empty or invalid values will the use the default values"),
    br(),
    fluidRow(column(width = 6,
                    colourInput(inputId          = "background_color_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "background_tip",
                                                              label = "Background Color",
                                                              text  = "Set app background color"))),
             column(width = 6,
                    colourInput(inputId          = "sidebar_background_color_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "sidebars_background_tip",
                                                              label = "Sidebars Background",
                                                              text  = "Set app sidebars background color")))),
    fluidRow(column(width = 6,
                    colourInput(inputId          = "sidebar_background_hover_color_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "sidebar_background_hover_tip",
                                                              label = "Sidebars Background Hover",
                                                              text  = "Set app sidebars background hover color"))),
             column(width = 6,
                    colourInput(inputId          = "sidebar_hover_color_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "sidebar_hover_tip",
                                                              label = "Sidebars Hover",
                                                              text  = "Set app sidebars hover color")))),
    fluidRow(column(width = 6,
                    colourInput(inputId          = "sidebar_color_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "sidebar_tip",
                                                              label = "Sidebars",
                                                              text  = "Set app sidebars  color"))),
             column(width = 6,
                    colourInput(inputId          = "sidebar_active_color_picker",
                                allowTransparent = TRUE,
                                label            = ui_tooltip(id    = "sidebar_active_tip",
                                                              label = "Sidebars Active",
                                                              text  = "Set app sidebars active color")))),
    hr(),
    fluidRow(column(width = 6,
                    actionButton(inputId = "update_app_theme",
                                 label   = "Update App Theme",
                                 status  = "info")),
             column(width = 6,
                    actionButton(inputId = "restore_app_theme",
                                 label   = "Restore Default Theme",
                                 status  = "secondary")))

)
# -- Register Elements in the ORDER SHOWN in the UI
add_ui_body(list(uiOutput("app_theme"),
                 about_box,
                 files_organization_box,
                 announcements_box,
                 fluidRow(file_downloader_box,
                          reset_application_box),
                 react_table_box,
                 table_downloader_box,
                 plot_downloader_box,
                 logger_box,
                 busy_indicator_box,
                 alerts_box,
                 tooltips_box,
                 style_guide_box))

## ----eval=F-------------------------------------------------------------------
# # Inside ui_body.R or ui_sidebar.R
# 
# downloadableTableUI(id            = "object_id1",
#                     downloadtypes = c("csv", "tsv"),
#                     hovertext     = "Download the data here!",
#                     contentHeight = "300px",
#                     singleSelect  = FALSE)

## ----eval = F-----------------------------------------------------------------
# # Inside server_local.R
# sketch <- htmltools::withTags(
#     table(
#         class = "display",
#         thead(
#             tr(
#                 th(rowspan = 2, "Location"),
#                 th(colspan = 2, "Statistics")
#             ),
#             tr(
#                 th("Change"),
#                 th("Increase")
#             )
#         )
#     )
# )
# 
# selectedrows <- downloadableTable(
#     id               = "exampleDT1",
#     logger           = ss_userAction.Log,
#     filenameroot     = "exampletable",
#     downloaddatafxns = list(csv = load_data3, tsv = load_data3),
#     tabledata        = load_data3,
#     colnames         = c("Area", "Delta", "Increase"),
#     filter           = "bottom",
#     callback         = htmlwidgets::JS("table.order([1, 'asc']).draw();"),
#     container        = sketch,
#     formatStyle      = list(columns = c("Total.Population.Change"),
#                             color   = DT::styleInterval(0, c("red", "green"))),
#     formatStyle      = list(columns = c("Natural.Increase"),
#                             backgroundColor = DT::styleInterval(
#                                 c(7614, 15914, 34152),
#                                 c("blue", "lightblue", "#FF7F7F", "red")))
# )
# 
# # NOTE: selectedrows is the reactive return value, captured for later use

## ----eval=F-------------------------------------------------------------------
# library(periscope2)
# 
# app_dir = tempdir()
# create_new_application(name = 'mysampleapp', location = app_dir, sample_app = TRUE)
# runApp(paste(app_dir, 'mysampleapp', sep = .Platform$file.sep))

# ----------------------------------------
# --       PROGRAM ui_right_sidebar.R         --
# ----------------------------------------
# USE: Create UI elements for the
#      application right sidebar and
#      ATTACH them to the UI by calling
#      add_ui_right_sidebar()
#
# NOTES:
#   - All variables/functions here are
#     not available to the UI or Server
#     scopes - this is isolated
# ----------------------------------------

# -- IMPORTS --



# ----------------------------------------
# --     SIDEBAR ELEMENT CREATION       --
# ----------------------------------------

# -- Configure Right Sidebar

# Whether the control bar on the right side is collapsed or not at start.
collapsed <- TRUE

# Whether the sidebar covers the content when expanded
overlay <- TRUE

# Control bar skin. "dark" or "light".
skin <- "light"

# Whether to block the control bar state
pinned <- FALSE


# -- Create Right Sidebar Elements
# elements to be placed before any menu
sidebar_elements <- list(div(checkboxInput("hideFileOrganization", "Show Files Organization"), style = "margin-left:20px"))
sidebar_menu     <- NULL

# -- Register Right Sidebar Elements in the ORDER SHOWN in the UI
add_ui_right_sidebar(sidebar_elements = sidebar_elements,
                     collapsed        = collapsed,
                     overlay          = overlay,
                     skin             = skin,
                     pinned           = pinned,
                     sidebar_menu     = sidebar_menu)

## ----eval=F-------------------------------------------------------------------
# # Inside ui_body.R or ui_sidebar.R
# 
# downloadablePlotUI(id                 = "object_id1",
#                    downloadtypes      = c("png", "csv"),
#                    download_hovertext = "Download the plot and data here!",
#                    height             = "500px",
#                    btn_halign         = "left")

## ----eval = F-----------------------------------------------------------------
# # Inside server_local.R
# 
# downloadablePlot(id           = "object_id1",
#                  logger       = ss_userAction.Log,
#                  filenameroot = "mydownload1",
#                  aspectratio  = 1.33,
#                  downloadfxns = list(png = myplotfxn,
#                                      tsv = mydatafxn),
#                  visibleplot  = myplotfxn)
# 

## ----eval=F-------------------------------------------------------------------
# # Inside ui_body.R or ui_sidebar.R
# 
#  downloadableReactTableUI(
#              id            = "object_id1",
#              downloadtypes = c("csv", "tsv"),
#              hovertext     = "Download the data here!")

## ----eval = F-----------------------------------------------------------------
# # Inside server_local.R
#  library(shiny)
#  library(periscope2)
#  library(reactable)
# 
# table_state <- downloadableReactTable(
#              id                 = "object_id1",
#              table_data         = reactiveVal(iris),
#              download_data_fxns = list(csv = reactiveVal(iris), tsv = reactiveVal(iris)),
#              selection_mode     = "multiple",
#              pre_selected_rows  = function() {c(1, 3, 5)},
#              table_options      = list(columns = list(
#                  Sepal.Length = colDef(name = "Sepal Length"),
#                  Sepal.Width  = colDef(filterable = TRUE),
#                  Petal.Length = colDef(show = FALSE),
#                  Petal.Width  = colDef(defaultSortOrder = "desc")),
#                  showSortable = TRUE,
#                  theme = reactableTheme(
#                      borderColor = "#dfe2e5",
#                      stripedColor = "#f6f8fa",
#                      highlightColor = "#f0f5f9",
#                      cellPadding = "8px 12px")))
# 
#         observeEvent(table_state(), { print(table_state()) })
# 
# # NOTE: table_state is the reactive return value, captured for later use

## ----eval=F-------------------------------------------------------------------
# library(periscope2)
# 
# app_dir = tempdir()
# create_new_application(name = 'mysampleapp', location = app_dir, sample_app = TRUE)
# runApp(paste(app_dir, 'mysampleapp', sep = .Platform$file.sep))

library(dplyr)

df <- read.csv("program/data/example.csv",
               strip.white = T,
               comment.char = "#")

app_files <- read.csv("program/data/structure.csv")
files_idx <- read.csv("program/data/struc_indx.csv")

rownames(app_files) <- app_files$X
app_files$X         <- NULL
app_files           <- app_files %>% mutate(across(where(is.character), ~na_if(., "")))

rownames(files_idx) <- files_idx$X
files_idx$X         <- NULL


load_data1 <- function() {
    ldf <- df %>%
        filter(substr(Geographic.Area, 1, 1) == ".") %>%
        mutate(Geographic.Area = substring(Geographic.Area, 2))

    as.data.frame(ldf)
}


load_data2 <- function() {
    ldf <- df %>%
        filter(substr(Geographic.Area, 1, 1) != ".")

    as.data.frame(ldf)
}


load_data3 <- function() {
    ldf <- df %>%
        select(1:3) %>%
        mutate(Total.Population.Change = as.numeric(gsub(",", "", Total.Population.Change)),
               Natural.Increase = as.numeric(gsub(",", "", Natural.Increase)))

    as.data.frame(ldf)
}


read_themes <- function() {
    theme_settings <- NULL

    if (file.exists("www/periscope_style.yaml")) {
        tryCatch({
            theme_settings <- yaml::read_yaml("www/periscope_style.yaml")
        },
        error = function(e){
            warning("Could not parse 'periscope_style.yaml' due to: ", e$message)
        })
    }
    theme_settings[sapply(theme_settings, is.null)] <- NA

    theme_settings
}

# ----------------------------------------
# --          PROGRAM global.R          --
# ----------------------------------------
# USE: Global variables and functions
#
# NOTES:
#   - All variables/functions here are
#     globally scoped and will be available
#     to server, UI and session scopes
# ----------------------------------------

library(shiny)
library(periscope2)
library(waiter)
library(shinyWidgets)
library(shinyjs)


# -- Setup your Application --
set_app_parameters(app_info          = NULL,
                   log_level         = "DEBUG",
                   app_version       = "1.0.0",
                   loading_indicator = list(html = tagList(spin_1(), "Loading ..."), color = "rgba(22, 65, 124, 0.3)"))

# -- PROGRAM --

# ----------------------------------------
# --       PROGRAM ui_header.R          --
# ----------------------------------------
# USE: Create UI elements for the
#      application header and
#      ATTACH them to the UI by calling
#      add_ui_header()
#
# NOTES:
#   - All variables/functions here are
#     not available to the UI or Server
#     scopes - this is isolated
# ----------------------------------------

# -- IMPORTS --



# ----------------------------------------
# --     HEADER ELEMENT CREATION       --
# ----------------------------------------

# -- Header Configurations

# Navbar skin. "dark" or "light"
skin <- "light"

# Navbar status
status <- "white"

# Whether to separate the navbar and body by a border.
border <- TRUE

# Whether items should be compacted
compact <- FALSE

# Icon of the main sidebar toggle
left_sidebar_icon <- shiny::icon("bars")

# Icon to toggle the right sidebar
right_sidebar_icon <- shiny::icon("th")

# Whether to fix the navbar to the top
fixed <- FALSE

# -- Create Header Elements

# Custom left menu content
left_menu <- NULL

# Custom right menu content
right_menu <- NULL

# -- Register Header Elements in the ORDER SHOWN in the UI
add_ui_header(title              = "Set using add_ui_header() in program/ui_header.R",
              left_menu          = left_menu,
              right_menu         = right_menu,
              skin               = skin,
              status             = status,
              border             = border,
              compact            = compact,
              left_sidebar_icon  = left_sidebar_icon,
              right_sidebar_icon = right_sidebar_icon,
              fixed              = fixed)

# ----------------------------------------
# --      PROGRAM server_global.R       --
# ----------------------------------------
# USE: Server-specific variables and
#      functions for the main reactive
#      shiny server functionality.  All
#      code in this file will be put into
#      the framework outside the call to
#      shinyServer(function(input, output, session)
#      in server.R
#
# NOTES:
#   - All variables/functions here are
#     SERVER scoped and are available
#     across all user sessions, but not to
#     the UI.
#
#   - For user session-scoped items
#     put var/fxns in server_local.R
#
# FRAMEWORK VARIABLES
#     none
# ----------------------------------------

# -- IMPORTS --


# -- VARIABLES --
node_event <- JS("{'mousemove':function(o, e, t) {
                                   if (o != null &&
                                       o != false &&
                                       o.objectType == 'Tree') {
                                       Shiny.setInputValue('node_name', o.display, {priority: 'event'});
                                   }
                               }}")
node_description <- list("global.R"             = "Handles FRAMEWORK global work, do not edit this file",
                         "server.R"             = "Handles FRAMEWORK server work, do not edit this file",
                         "ui.R"                 = "Handles FRAMEWORK UI work",
                         "log"                  = "Stores application log files",
                         "actions.log"          = "Application logs",
                         "actions.log.last"     = "Application logs history",
                         "program"              = "All logic goes here",
                         "config"               = "Where user can store application configuration files",
                         "announce.yaml"        = "Announcement module configuration file",
                         "data"                 = "Where user can store related data files (rds, csv, ...)",
                         ".gitignore"           = "Keep files extensions to be ignored by Git",
                         "fxns"                 = "Where application specialized logic goes (plot helpers, data helpers, ...)",
                         "modules"              = "Application modules",
                         "program.global.R"     = "All variables or/and functions that should be globally scoped to server, UI and session scopes should be here",
                         "server_global.R"      = "All variables or/and functions that should be SERVER scoped and are available across all user sessions, but not to UI, should be here",
                         "server_local.R"       = "All variables or/and functions here are SESSION scoped and are ONLY available to a single session and not to the UI",
                         "ui_body.R"            = "Application body UI goes here",
                         "ui_footer.R"          = "Application footer UI goes here",
                         "ui_header.R"          = "Application header UI goes here",
                         "ui_left_sidebar.R"    = "Application left sidebar UI goes here",
                         "ui_right_sidebar.R"   = "Application right sidebar UI goes here",
                         "www"                  = "store client side related code (js, css, icons, ..)",
                         "periscope_style.yaml" = "Application theme variables (i.e primary colors, sidebar width, ...)",
                         "css"                  = "Store application css files",
                         "custom.css"           = "Application main css file",
                         "js"                   = "Store application js files",
                         "custom.js"            = "Application main js file",
                         "img"                  = "Application related images and icons",
                         "loader.gif"           = "Busy indicator used image",
                         "tooltip.png"          = "Tooltips used image",
                         "App_Root"             = "Application main root")
# -- FUNCTIONS --

# ----------------------------------------
# --       PROGRAM server_local.R       --
# ----------------------------------------
# USE: Session-specific variables and
#      functions for the main reactive
#      shiny server functionality.  All
#      code in this file will be put into
#      the framework inside the call to
#      shinyServer(function(input, output, session)
#      in server.R
#
# NOTES:
#   - All variables/functions here are
#     SESSION scoped and are ONLY
#     available to a single session and
#     not to the UI
#
#   - For globally scoped session items
#     put var/fxns in server_global.R
#
# FRAMEWORK VARIABLES
#     input, output, session - Shiny
#     ss_userAction.Log - Reactive Logger S4 object
# ----------------------------------------

# -- IMPORTS --


# -- VARIABLES --


# -- FUNCTIONS --


# ----------------------------------------
# --          SHINY SERVER CODE         --
# ----------------------------------------

if (requireNamespace('spelling', quietly = TRUE)) {
    spelling::spell_check_test(vignettes    = TRUE,
                               error        = FALSE,
                               skip_on_cran = TRUE)
}

# ----------------------------------------
# --       PROGRAM ui_header.R          --
# ----------------------------------------
# USE: Create UI elements for the
#      application header and
#      ATTACH them to the UI by calling
#      add_ui_header()
#
# NOTES:
#   - All variables/functions here are
#     not available to the UI or Server
#     scopes - this is isolated
# ----------------------------------------

# -- IMPORTS --



# ----------------------------------------
# --     HEADER ELEMENT CREATION       --
# ----------------------------------------

# -- Header Configurations

# Navbar skin. "dark" or "light"
skin <- "light"

# Navbar status
status <- "white"

# Whether to separate the navbar and body by a border.
border <- TRUE

# Whether items should be compacted
compact <- FALSE

# Icon of the main sidebar toggle
left_sidebar_icon <- shiny::icon("bars")

# Icon to toggle the right sidebar
right_sidebar_icon <- shiny::icon("th")

# Whether to fix the navbar to the top
fixed <- FALSE

# -- Create Header Elements

# Custom left menu content
left_menu <- NULL

# Custom right menu content
right_menu <- NULL

# -- Register Header Elements in the ORDER SHOWN in the UI
add_ui_header(title              = "periscope2 Example Application",
              left_menu          = left_menu,
              right_menu         = right_menu,
              skin               = skin,
              status             = status,
              border             = border,
              compact            = compact,
              left_sidebar_icon  = left_sidebar_icon,
              right_sidebar_icon = right_sidebar_icon,
              fixed              = fixed)

#' Add UI elements to dashboard left sidebar section
#'
#' This function adds left sidebar configurations and UI elements. It is called within "ui_left_sidebar.R".
#' Check example application for detailed example
#'
#' @param sidebar_elements List of regular shiny UI elements (inputText, textArea, etc..)
#' @param sidebar_menu     \code{?bs4Dash::bs4SidebarMenu()} object to created a menu inside left sidebar
#' @param collapsed        If TRUE, the sidebar will be collapsed on app start up
#' @param custom_area      List of regular shiny UI elements but for sidebar bottom space area only.
#'                         Only works if sidebar is fixed
#' @param elevation        A number between 0 and 5, which applies a shadow to the sidebar to add a shadow effect.
#' @param expand_on_hover  When \code{minified} is TRUE, if this property is TRUE, the sidebar opens when hovering but re-collapses as soon as the focus is lost (default = TRUE)
#' @param fixed            Whether to see all menus at once without scrolling up and down.(default = TRUE)
#' @param minified         Whether to slightly close the sidebar but still show item icons (default = FALSE)
#' @param skin             Sidebar skin. "dark" or "light" (default = "light")
#' @param status           Determines which color menu items (if exist) will have Check \code{?bs4Dash::dashboardSidebar()} for list of valid values
#'
#' @return list of both shiny UI elements and named left sidebar properties
#'
#' @section Shiny Usage:
#' Call this function from \code{program/ui_left_sidebar.R} to set left sidebar parameters
#'
#' @examples
#'   library(shiny)
#'   library(bs4Dash)
#'   # Inside ui_left_sidebar.R
#'   # sidebar menu items
#'   sidebar_elements <- textInput("text_id", "Test", "Test Data")
#'   sidebar_menu     <- sidebarMenu(sidebarHeader("Main Menu"),
#'                                   menuItem("menu item 1",
#'                                            tabName = "item_1 page"),
#'                                   menuItem("menu item 2",
#'                                            tabName = "item_2 page"))
#'   add_ui_left_sidebar(sidebar_elements = sidebar_elements,
#'                       sidebar_menu     = sidebar_menu)
#' @seealso \link[bs4Dash:bs4DashSidebar]{bs4Dash:bs4DashSidebar()}
#' @seealso \link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}
#' @seealso \link[periscope2:add_ui_header]{periscope2:add_ui_header()}
#' @seealso \link[periscope2:add_ui_body]{periscope2:add_ui_body()}
#' @seealso \link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}
#' @seealso \link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}
#' @seealso \link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
#'
#' @export
add_ui_left_sidebar <- function(sidebar_elements = NULL,
                                sidebar_menu     = NULL,
                                collapsed        = FALSE,
                                custom_area      = NULL,
                                elevation        = 4,
                                expand_on_hover  = TRUE,
                                fixed            = TRUE,
                                minified         = FALSE,
                                status           = "primary",
                                skin             = "light") {
    .g_opts$left_sidebar <- list(list(shiny::div(id = "sidebarBasicAlert"),
                                      sidebar_elements),
                                 skin            = skin,
                                 status          = status,
                                 elevation       = elevation,
                                 collapsed       = collapsed,
                                 minified        = minified,
                                 expand_on_hover = expand_on_hover,
                                 fixed           = fixed,
                                 custom_area     = custom_area,
                                 sidebar_menu)
}


#' Add UI elements to dashboard header section
#'
#' Builds application header with given configurations and elements. It is called within "ui_header.R".
#' These elements will be displayed in the header beside application title and application busy indicator.
#'
#' \subsection{Application header consists of three elements:}{
#'    \describe{
#'      \item{busy indicator}{An automatic wait indicator that are shown when the shiny server session is busy}
#'      \item{application title}{Display application title}
#'      \item{heade menu}{Optional header menu to switch between application different tabs}
#'    }
#' }
#'
#' Header elements can be arranged via \code{ui_position} and \code{title_position} parameters.
#' \cr
#' Header elements look and feel can also be configured in \bold{"www\\css\\custom.css"} file under
#' \bold{"Application Header"} section.
#' \cr
#' Check example application for detailed example
#'
#' @param ui_elements        It can be any UI element but mostly used for navbarMenu. NULL by default.
#'                           Check \code{?bs4Dash::navbarMenu()}
#' @param ui_position        Location of UI elements in the header. Must be either of 'center', 'left' or 'right'
#'                           Default value is 'right'.
#' @param title              Sets application title. If it is not NULL, it will override "title" value that is set in
#'                           \code{?periscope2::set_app_parameters()} (default = NULL)
#' @param title_position     Location of the title in the header. Must be either of 'center', 'left' or 'right'
#'                           Default value is 'Center'. If there are no UI elements, this param will be ignored.
#' @param left_menu          Left menu. bs4DropdownMenu object (or similar dropdown menu).
#'                           Check \code{?bs4Dash::bs4DropdownMenu()}
#' @param right_menu         Right menu. bs4DropdownMenu object (or similar dropdown menu).
#'                           Check \code{?bs4Dash::bs4DropdownMenu()}
#' @param border             Whether to separate the navbar and body by a border. TRUE by default
#' @param compact            Whether items should be compacted. FALSE by default
#' @param fixed              Whether to fix the navbar to the top. FALSE by default
#' @param left_sidebar_icon  Left sidebar toggle icon
#' @param right_sidebar_icon Right sidebar toggle icon
#' @param skin               Sidebar skin. "dark" or "light"
#' @param status             Sidebar status. Check \code{?bs4Dash::bs4DashNavbar()} for list of valid values
#'
#' @return list of both shiny UI elements and named header properties
#'
#' @section Shiny Usage:
#' Call this function from \code{program/ui_header.R} to set header parameters
#'
#' @examples
#'   library(shiny)
#'   library(bs4Dash)
#'
#'   # Inside ui_header.R
#'   # Custom left UI menu
#'   left_menu <- tagList(dropdownMenu(badgeStatus = "info",
#'                                   type        = "notifications",
#'                                    notificationItem(inputId = "triggerAction2",
#'                                                     text    = "Error!",
#'                                                     status  = "danger")),
#'                       dropdownMenu(badgeStatus = "info",
#'                                    type        = "tasks",
#'                                    taskItem(inputId = "triggerAction3",
#'                                             text    = "My progress",
#'                                             color   = "orange",
#'                                             value   = 10)))
#'
#'   # Custom right UI menu
#'   right_menu <- dropdownMenu(badgeStatus = "danger",
#'                           type        = "messages",
#'                           messageItem(inputId = "triggerAction1",
#'                                       message = "message 1",
#'                                       from    = "Divad Nojnarg",
#'                                       time    = "today",
#'                                       color   = "lime"))
#'   # -- Register Header Elements in the ORDER SHOWN in the UI
#'   add_ui_header(left_menu = left_menu, right_menu = right_menu)
#'
#' @seealso \link[bs4Dash:bs4DashNavbar]{bs4Dash:bs4DashNavbar()}
#' @seealso \link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}
#' @seealso \link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}
#' @seealso \link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}
#' @seealso \link[periscope2:add_ui_body]{periscope2:add_ui_body()}
#' @seealso \link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}
#' @seealso \link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}
#' @seealso \link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
#'
#' @export
add_ui_header <- function(ui_elements        = NULL,
                          ui_position        = "right",
                          title              = NULL,
                          title_position     = "center",
                          left_menu          = NULL,
                          right_menu         = NULL,
                          border             = TRUE,
                          compact            = FALSE,
                          right_sidebar_icon = shiny::icon("bars"),
                          fixed              = FALSE,
                          left_sidebar_icon  = shiny::icon("th"),
                          skin               = "light",
                          status             = "white") {
    if (!is.null(title)) {
        .g_opts$app_title <- title
    }

    app_title <- shiny::isolate(.g_opts$app_title)
    title     <- shiny::div(id = "app_header", app_title)
    app_info  <- shiny::isolate(.g_opts$app_info)

    if (!is.null(app_info)) {
        if (class(app_info)[1] == "html") {
            title <- shiny::div(id = "app_header",
                                shiny::actionLink("app_info", app_title))
        } else {
            title <- shiny::div(id = "app_header",
                                shiny::a(id = "app_info", href = app_info, target = "_blank", app_title))
        }

    }

    busy_indicator <- shiny::div(class = "periscope-busy-ind",
                                 "Working",
                                 shiny::img(alt = "Working...",
                                            hspace = "5px",
                                            src = "img/loader.gif"))
    if (length(ui_elements) > 0) {
        ui_elements <- shiny::div(id = "header_menu", ui_elements)
    }

    header_left   <- busy_indicator
    header_center <- title
    header_right  <- ui_elements

    if (!is.null(ui_elements)) {
        if (!is.null(title_position)) {
            title_position <- tolower(title_position)
        }

        if (!is.null(ui_position)) {
            ui_position <- tolower(ui_position)
        }

        if ((is.null(title_position)) ||
            !(title_position %in% c("left", "center", "right"))) {
            warning("title_position must be on of 'left', 'center'or 'right' values. Setting default value 'center'")
            title_position <- "center"
        }

        if ((is.null(ui_position)) ||
            !(ui_position %in% c("left", "center", "right"))) {
            warning("ui_position must be on of 'left', 'center'or 'right' values. Setting default value 'right'")
            ui_position <- "right"
        }

        if (title_position == ui_position) {
            warning("title_position cannot be equal to ui_position. Setting default values")
            title_position <- "center"
            ui_position    <- "right"
        }

        if (title_position == "center") {
            if (ui_position == "left") {
                header_right <- busy_indicator
                header_left  <- ui_elements
            }
        } else if (title_position == "left") {
            header_left <- title

            if (ui_position == "right") {
                header_right  <- ui_elements
                header_center <- busy_indicator
            } else{
                header_right  <- busy_indicator
                header_center <- ui_elements
            }
        } else if (title_position == "right") {
            header_right <- title

            if (ui_position == "left") {
                header_left   <- ui_elements
                header_center <- busy_indicator
            } else{
                header_left   <- busy_indicator
                header_center <- ui_elements
            }
        }


    }

    header <- shiny::div(class = "app_header_container",
                         header_left,
                         header_center,
                         header_right)

    .g_opts$header <- bs4Dash::bs4DashNavbar(header,
                                             skin           = skin,
                                             status         = status,
                                             border         = border,
                                             compact        = compact,
                                             sidebarIcon    = left_sidebar_icon,
                                             controlbarIcon = right_sidebar_icon,
                                             fixed          = fixed,
                                             leftUi         = left_menu,
                                             rightUi        = right_menu)
}


#' Add UI elements to dashboard body section
#'
#' Builds application body with given configurations and elements. It is called within "ui_body.R".
#' Check example application for detailed example
#'
#' @param body_elements List of UI elements to be displayed in application body
#' @param append        Add elements to current body elements or remove previous body elements (default = FALSE)
#'
#' @return list of both shiny UI elements and html div tags for alert and linking app JS and CSS files
#'
#' @section Shiny Usage:
#' Call this function from \code{program/ui_body.R} to set body parameters
#'
#' @examples
#'   library(shiny)
#'   library(bs4Dash)
#'   # Inside ui_body.R
#'   about_box <- jumbotron(title  = "periscope2: Test Example",
#'                          lead   = p("periscope2 is a scalable and UI-standardized 'shiny' framework
#'                          	       including a variety of developer convenience functions"),
#'                          status = "info",
#'                          href   = "https://periscopeapps.org/")
#'   # -- Register Elements in the ORDER SHOWN in the UI
#'   add_ui_body(list(about_box))
#'
#' @seealso \link[bs4Dash:bs4DashBody]{bs4Dash:bs4DashBody()}
#' @seealso \link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}
#' @seealso \link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}
#' @seealso \link[periscope2:add_ui_header]{periscope2:add_ui_header()}
#' @seealso \link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}
#' @seealso \link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}
#' @seealso \link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
#'
#' @export
add_ui_body <- function(body_elements = NULL, append = FALSE) {
    if (append) {
        .g_opts$body_elements <- append(
            shiny::isolate(.g_opts$body_elements),
            body_elements,
            shiny::isolate(length(.g_opts$body_elements)) - 1)
    } else {
        .g_opts$body_elements <- list(shiny::div(id = "bodyAlert"),
                                      shiny::div(id = "head",
                                                 shinyjs::useShinyjs(),
                                                 shiny::tags$head(shiny::tags$link(rel  = "stylesheet",
                                                                                   type = "text/css",
                                                                                   href = "css/custom.css"),
                                                                  shiny::tags$head(shiny::tags$script(src = "js/custom.js")))),
                                      body_elements)
    }
}


#' Display alert panel at specified location
#'
#' Create an alert panel in server code to be displayed in the specified UI selector location
#'
#' @param session  Shiny session object
#' @param id       Anchor id (either id or selector only should be set)
#' @param selector Character vector represents jQuery selector to add the alert to is
#'                 (i.e ".alertClass", div.badge-danger.navbar-badge). If 'id' is specified, this parameter will be neglected
#' @param options  List of options to pass to the alert
#'
#' @return html div and inserts it in the app DOM
#'
#' @section Shiny Usage:
#' Call this function from \code{program/server_local.R} or any other server file to setup the needed alert
#'
#' @examples
#'   library(shiny)
#'   library(bs4Dash)
#'
#'   # Inside server_local.R
#'   createPSAlert(id       = "sidebarRightAlert",
#'                 options  = list(title    = "Right Side",
#'                                 status   = "success",
#'                                 closable = TRUE,
#'                                 content  = "Example Basic Sidebar Alert"))
#'   # Test se
#'   ## a div with class "badge-danger.navbar-badge" must be exist in UI to display alert
#'   selector <- "div.badge-danger.navbar-badge"
#'   createPSAlert(selector = selector,
#'                 options  = list(title    = "Selector Title",
#'                                 status   = "danger",
#'                                 closable = TRUE,
#'                                 content  = "Selector Alert"))
#'
#'
#' @seealso \link[bs4Dash:closeAlert]{bs4Dash:closeAlert()}
#' @seealso \link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}
#' @seealso \link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}
#' @seealso \link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
#'
#' @export
createPSAlert <- function(session  = shiny::getDefaultReactiveDomain(),
                          id       = NULL,
                          selector = NULL,
                          options) {
    if (!is.null(id) && !is.null(selector)) {
        stop("Please choose either target or selector!")
    }

    if (!is.null(id) && !is.null(session)) {
        id = session$ns(id)
    }

    message_params <- list(id       = id,
                           selector = selector,
                           options  = options)

    message_params <- message_params[!vapply(message_params, is.null, FUN.VALUE = logical(1))]

    if (!is.null(session)) {
        session$sendCustomMessage("pcreate-alert", message_params)
    }
}


closeResetAlert <- function(id, session = shiny::getDefaultReactiveDomain()) {
    session$sendCustomMessage("close-alert", id)
}


#' Add UI elements to dashboard right sidebar section
#'
#' Builds application right sidebar with given configurations and elements. It is called within "ui_right_sidebar.R".
#' Check example application for detailed example
#'
#' @param sidebar_elements List of regular shiny UI elements (inputText, textArea, etc..)
#' @param sidebar_menu     \code{?bs4Dash::controlbarMenu()} object to created a menu inside right sidebar
#' @param collapsed        If TRUE, the sidebar will be collapsed on app startup (default = TRUE)
#' @param overlay          Whether the sidebar covers the content when expanded (default = TRUE)
#' @param pinned           If TRUE, allows right sidebar to remain open even after a click outside (default = FALSE)
#' @param skin             Sidebar skin. "dark" or "light" (default = "light")
#'
#' @return list of both shiny UI elements and named right sidebar properties
#'
#' @section Shiny Usage:
#' Call this function from \code{program/ui_right_sidebar.R} to set right sidebar parameters
#'
#' @examples
#'   library(shiny)
#'   library(bs4Dash)
#'
#'   # Inside ui_right_sidebar.R
#'   sidebar_elements <- list(div(checkboxInput("checkMe", "Example Check")))
#'   sidebar_menu     <- controlbarMenu(id = "controlbarmenu",
#'                                      controlbarItem("Item 2", "Simple text"))
#'  # -- Register Right Sidebar Elements in the ORDER SHOWN in the UI
#'   add_ui_right_sidebar(sidebar_elements = sidebar_elements,
#'                        sidebar_menu     = sidebar_menu)
#'
#' @seealso \link[bs4Dash:bs4DashControlbar]{bs4Dash:bs4DashControlbar()}
#' @seealso \link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}
#' @seealso \link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}
#' @seealso \link[periscope2:add_ui_header]{periscope2:add_ui_header()}
#' @seealso \link[periscope2:add_ui_body]{periscope2:add_ui_body()}
#' @seealso \link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}
#' @seealso \link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}
#' @seealso \link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
#'
#' @export
add_ui_right_sidebar <- function(sidebar_elements = NULL,
                                 sidebar_menu     = NULL,
                                 collapsed        = TRUE,
                                 overlay          = TRUE,
                                 pinned           = FALSE,
                                 skin             = "light") {
    .g_opts$right_sidebar <- bs4Dash::bs4DashControlbar(list(shiny::div(id = "sidebarRightAlert"),
                                                             sidebar_elements,
                                                             sidebar_menu),
                                                        collapsed = collapsed,
                                                        overlay   = overlay,
                                                        skin      = skin,
                                                        pinned    = pinned)
}


#' Add UI elements to dashboard footer section
#'
#' Builds application footer with given configurations and elements. It is called within "ui_footer.R".
#' Check example application for detailed example
#'
#' @param left  Left side UI elements
#' @param right Right side UI elements
#' @param fixed Always show footer at page bottom regardless page scroll location (default = FALSE).
#'
#' @return list of both shiny UI elements and named footer properties
#'
#' @section Shiny Usage:
#' Call this function from \code{program/ui_footer.R} to set footer parameters
#'
#' @examples
#'   library(shiny)
#'   library(bs4Dash)
#'
#'   # Inside ui_footer.R
#'   # Left text
#'   left <- a(href   = "https://periscopeapps.org/",
#'             target = "_blank",
#'             "periscope2")
#'   # Right text
#'   right <- "2022"
#'
#'   # -- Register Elements in the ORDER SHOWN in the UI
#'   add_ui_footer(left, right)
#'
#' @seealso \link[bs4Dash:bs4DashFooter]{bs4Dash:bs4DashFooter()}
#' @seealso \link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}
#' @seealso \link[periscope2:add_ui_header]{periscope2:add_ui_header()}
#' @seealso \link[periscope2:add_ui_body]{periscope2:add_ui_body()}
#' @seealso \link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}
#' @seealso \link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}
#' @seealso \link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}
#' @seealso \link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
#'
#' @export
add_ui_footer <- function(left  = NULL,
                          right = NULL,
                          fixed = FALSE) {
    .g_opts$footer <- bs4Dash::bs4DashFooter(left  = list(shiny::div(id = "footerAlert"),
                                                          left),
                                             right = right,
                                             fixed = fixed)
}


#' Add tooltip icon and text to UI elements labels
#'
#' This function inserts a standardized tooltip image, label (optional),
#' and hovertext into the application UI
#'
#' @param id        The id for the tooltip object
#' @param label     Text label to appear to the left of the tooltip image
#' @param text      Tooltip text shown when the user hovers over the image
#' @param placement Where to display tooltip label. Available places are "top", "bottom", "left", "right" (default is "top")
#'
#' @return html span with the label, tooltip image and tooltip text
#'
#' @section Shiny Usage:
#' Call this function from \code{program/ui_body.R} to set tooltip parameters
#'
#' @examples
#'   library(shiny)
#'   library(periscope2)
#'
#'   # Inside ui_body.R or similar UI file
#'    ui_tooltip(id   = "top_tip",
#'              label = "Top Tooltips",
#'              text  = "Top tooltip")
#'
#' @seealso \link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}
#' @seealso \link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}
#' @seealso \link[periscope2:add_ui_header]{periscope2:add_ui_header()}
#' @seealso \link[periscope2:add_ui_body]{periscope2:add_ui_body()}
#' @seealso \link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}
#' @seealso \link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}
#' @seealso \link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}
#' @seealso \link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
#'
#' @export
ui_tooltip <- function(id,
                       label     = "",
                       text      = "",
                       placement = "top") {
    if (is.null(text) || is.na(text) || (text == "")) {
        warning("ui_tooltip() called without tooltip text.")
    }

   shiny::span(class = "periscope-input-label-with-tt",
               label,
               tooltip(shiny::img(id     = id,
                                  src    = shiny::isolate(.g_opts$tt_image),
                                  height = shiny::isolate(.g_opts$tt_height),
                                  width  = shiny::isolate(.g_opts$tt_width)),
                       title     = text,
                       placement = placement))
}


#' Set Application Parameters
#'
#' This function sets global parameters customizing the shiny application.
#'
#' @param title              `r lifecycle::badge("deprecated")` Use \link[periscope2]{add_ui_header} to configure application title text
#' @param app_info           Application detailed information. It can be character string, HTML value or NULL
#'                             \itemize{
#'                                      \item{A \strong{character} string will be used to set a link target.
#'                                            This means the user will be able to click on the application title and be
#'                                            redirected in a new window to whatever value is given in the string.
#'                                            Any valid URL, File, or other script functionality that would normally be
#'                                            accepted in an \code{<a href=...></a>}  tag is allowed.}
#'                                      \item{An \strong{HTML} value will be used to as the HTML content for a modal pop-up
#'                                            window that will appear on-top of the application when the user clicks on the
#'                                            application title.}
#'                                      \item{Supplying \strong{NULL} will disable the title link functionality.}
#'                             }
#' @param log_level          Designating the log level to use for the user log as 'DEBUG','INFO', 'WARNING' or 'ERROR' (default = 'DEBUG')
#' @param app_version        Character string designating the application version (default = '1.0.0')
#' @param loading_indicator  It uses waiter (see https://waiter.john-coene.com/#/).\cr
#'                             Pass a list like list(html = spin_1(), color = "#333e48") to \cr configure
#'                             waiterShowOnLoad (refer to the package help for all styles).
#' @param announcements_file `r lifecycle::badge("deprecated")`. Use \link[periscope2]{load_announcements} to configure announcement.
#'
#' @return no return value, called for setting new application global properties
#'
#' @section Shiny Usage:
#' Call this function from \code{program/global.R} to set the application
#' parameters.
#'
#' @examples
#'   library(shiny)
#'   library(waiter)
#'   library(periscope2)
#'
#'   # Inside program/global.R
#'   set_app_parameters(app_info          = HTML("Example info"),
#'                      log_level         = "DEBUG",
#'                      app_version       = "1.0.0",
#'                      loading_indicator = list(html = tagList(spin_1(), "Loading ...")))
#'
#'
#' @seealso \link[periscope2:announcementConfigurationsAddin]{periscope2:announcementConfigurationsAddin()}
#' @seealso \link[periscope2:load_announcements]{periscope2:load_announcements()}
#' @seealso \link[waiter:waiter]{waiter:waiter_show()}
#' @seealso \link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}
#' @seealso \link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}
#' @seealso \link[periscope2:add_ui_header]{periscope2:add_ui_header()}
#' @seealso \link[periscope2:add_ui_body]{periscope2:add_ui_body()}
#' @seealso \link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}
#' @seealso \link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}
#' @seealso \link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
#'
#' @export
set_app_parameters <- function(title              = NULL,
                               app_info           = NULL,
                               log_level          = "DEBUG",
                               app_version        = "1.0.0",
                               loading_indicator  = NULL,
                               announcements_file = NULL) {
    if (!is.null(announcements_file)) {
        lifecycle::deprecate_warn(
            when    = "0.2.3",
            what    = "set_app_parameters(announcements_file)",
            details = "Please use `periscope2::load_announcements` instead"
        )
    }

    if (!is.null(title)) {
        lifecycle::deprecate_warn(
            when    = "0.2.3",
            what    = "set_app_parameters(title)",
            details = "Please use `periscope2::add_ui_header(title)` instead"
        )
    }

    .g_opts$app_title          <- title
    .g_opts$app_info           <- app_info
    .g_opts$loglevel           <- log_level
    .g_opts$app_version        <- app_version
    .g_opts$loading_indicator  <- loading_indicator
    .g_opts$announcements_file <- announcements_file
}


#' Parse application passed URL parameters
#'
#' This function returns any url parameters passed to the application as
#' a named list.  Keep in mind url parameters are always user-session scoped
#'
#' @param session shiny session object
#'
#' @return named list of url parameters and values.  List may be empty if
#' no URL parameters were passed when the application instance was launched
#'
#' @section Shiny Usage:
#' Call this function from \code{program/server_local.R} or any other server file
#'
#' @examples
#'   library(shiny)
#'   library(periscope2)
#'
#'   # Display application info
#'   observeEvent(input$app_info, {
#'                url_params <- get_url_parameters(session)
#'                show_alert(html                = TRUE,
#'                           showCloseButton     = FALSE,
#'                           animation           = "slide-from-top",
#'                           closeOnClickOutside = TRUE,
#'                           text                = url_params[["passed_paramter"]],
#'                           title               = "alert title")
#'   })
#'
#'
#' @seealso \link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}
#' @seealso \link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}
#' @seealso \link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}
#' @seealso \link[periscope2:add_ui_header]{periscope2:add_ui_header()}
#' @seealso \link[periscope2:add_ui_body]{periscope2:add_ui_body()}
#' @seealso \link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}
#' @seealso \link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}
#'
#' @export
get_url_parameters <- function(session) {
    parameters <- list()

    if (!is.null(session)) {
        raw <- shiny::isolate(session$clientData$url_search)
        parameters <- shiny::parseQueryString(raw)
    }

    parameters
}


# Override bs4Dash tooltip function to allow html handling in tooltips
tooltip <- function(tag, title, placement = c("top", "bottom", "left", "right")) {
    placement <- match.arg(placement)

    tag <- shiny::tagAppendAttributes(
        tag,
        `data-toggle`    = "tooltip",
        `data-placement` = placement,
        title            = title,
        `data-html`      = "true"
    )

    tagId <- tag$attribs$id

    shiny::tagList(
        shiny::singleton(
            shiny::tags$head(
                shiny::tags$script(
                    sprintf(
                        "$(function () {
              // enable tooltip
              $('#%s').tooltip();
            });
            ",
                        tagId
                    )
                )
            )
        ),
        tag
    )
}

context("periscope2 - UI functionality")
local_edition(3)
loglevels          <- periscope2:::loglevels
set_app_parameters <- periscope2:::set_app_parameters

# helper functions
create_announcements <- function(start_date        = NULL,
                                 end_date          = NULL,
                                 start_date_format = NULL,
                                 end_date_format   = NULL,
                                 style             = NULL,
                                 text              = NULL,
                                 auto_close        = NULL) {
    appTemp_dir        <- tempdir()
    appTemp            <- tempfile(pattern = "TestThatApp", tmpdir = appTemp_dir)
    announcements_file <- paste0(gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE))), ".yaml")
    yaml::write_yaml(list("start_date"        = start_date,
                          "end_date"          = end_date,
                          "start_date_format" = start_date_format,
                          "end_date_format"   = end_date_format,
                          "style"             = style,
                          "text"              = text,
                          "auto_close"        = auto_close),
                     announcements_file)
    yaml::read_yaml(announcements_file)

    announce_output <- load_announcements(announcements_file_path = announcements_file)
    unlink(announcements_file, TRUE)
    announce_output
}

#########################################

test_that("add_ui_header - no header", {
    expect_null(shiny::isolate(periscope2:::.g_opts$header))
})

test_that("set_app_parameters default values", {
    reset_g_opts()
    expect_equal(shiny::isolate(periscope2:::.g_opts$app_title), "Set using add_ui_header() in program/ui_header.R")
    expect_null(shiny::isolate(periscope2:::.g_opts$app_info), NULL)

    expect_equal(shiny::isolate(periscope2:::.g_opts$loglevel), "DEBUG")
    expect_equal(shiny::isolate(periscope2:::.g_opts$app_version), "1.0.0")
    expect_null(shiny::isolate(periscope2:::.g_opts$loading_indicator))
    expect_null(shiny::isolate(periscope2:::.g_opts$announcements_file))
    expect_null(load_announcements())
})

test_that("add_ui_header - ui element", {
    menu <-   navbarMenu(
        id = "navmenu",
        navbarTab(tabName = "Tab1", text = "Tab 1"),
        navbarTab(tabName = "Tab2", text = "Tab 2"),
        navbarTab(
            text = "Menu",
            dropdownHeader("Dropdown header"),
            navbarTab(tabName = "Tab3", text = "Tab 3")
        )
    )

    # busy indicator - title - UI elements
    periscope2::add_ui_header(ui_elements = menu)
    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_equal(length(header), 2)
    expect_equal(length(header[[1]]), 3)
    expect_equal(length(header[[1]]$children), 3)
    expect_true(grepl('periscope-busy-ind.*Set using add_ui_header().*Tab1', header[[1]]$children[[2]]))

    # busy indicator - UI elements - title
    periscope2::add_ui_header(ui_elements    = menu,
                              ui_position    = "center",
                              title_position = "right")
    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('periscope-busy-ind.*Tab1.*Set using add_ui_header', header[[1]]$children[[2]]))

    # UI elements - busy indicator - title
    periscope2::add_ui_header(ui_elements    = menu,
                              ui_position    = "left",
                              title_position = "right")
    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('Tab1.*periscope-busy-ind.*Set using add_ui_header', header[[1]]$children[[2]]))

    # UI elements - title - busy indicator
    periscope2::add_ui_header(ui_elements    = menu,
                              ui_position    = "left",
                              title_position = "center")
    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('Tab1.*Set using add_ui_header.*periscope-busy-ind', header[[1]]$children[[2]]))

    # title - UI elements - busy indicator
    periscope2::add_ui_header(ui_elements    = menu,
                              ui_position    = "center",
                              title_position = "left")
    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('Set using add_ui_header.*Tab1.*periscope-busy-ind', header[[1]]$children[[2]]))

    # title - busy indicator - UI elements
    periscope2::add_ui_header(ui_elements    = menu,
                              ui_position    = "right",
                              title_position = "left")
    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('Set using add_ui_header.*periscope-busy-ind.*Tab1', header[[1]]$children[[2]]))

    # busy indicator - title - UI elements (center as well)
    expect_warning(periscope2::add_ui_header(ui_elements = menu,
                                             ui_position = "center"),
                   regexp = "title_position cannot be equal to ui_position")

    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('periscope-busy-ind.*Set using add_ui_header.*Tab1', header[[1]]$children[[2]]))

    # busy indicator - title - UI elements positions is NULL
    expect_warning(periscope2::add_ui_header(ui_elements = menu,
                                             ui_position = NULL),
                   regexp = "ui_position must be on of 'left', 'center'or 'right' values. Setting default value 'right'")

    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('periscope-busy-ind.*Set using add_ui_header.*Tab1', header[[1]]$children[[2]]))

    # busy indicator - title - UI elements positions is wrong
    expect_warning(periscope2::add_ui_header(ui_elements = menu,
                                             ui_position = "abc"),
                   regexp = "ui_position must be on of 'left', 'center'or 'right' values. Setting default value 'right'")

    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('periscope-busy-ind.*Set using add_ui_header.*Tab1', header[[1]]$children[[2]]))

    # busy indicator - title positions is wrong- UI elements
    expect_warning(periscope2::add_ui_header(ui_elements    = menu,
                                             title_position = "abc"),
                   regexp = "title_position must be on of 'left', 'center'or 'right' values. Setting default value 'center'")

    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('periscope-busy-ind.*Set using add_ui_header.*Tab1', header[[1]]$children[[2]]))

    # busy indicator - title positions is NULL- UI elements
    expect_warning(periscope2::add_ui_header(ui_elements    = menu,
                                             title_position = NULL),
                   regexp = "title_position must be on of 'left', 'center'or 'right' values. Setting default value 'center'")

    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('periscope-busy-ind.*Set using add_ui_header.*Tab1', header[[1]]$children[[2]]))

    # busy indicator - title positions is NULL- UI elements position is center
    warn_msgs <- capture_warnings(periscope2::add_ui_header(ui_elements    = menu,
                                                            ui_position    = "center",
                                                            title_position = NULL))
    expect_equal("title_position must be on of 'left', 'center'or 'right' values. Setting default value 'center'",
                 warn_msgs[1])
    expect_equal("title_position cannot be equal to ui_position. Setting default values",
                 warn_msgs[2])
    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('periscope-busy-ind.*Set using add_ui_header.*Tab1', header[[1]]$children[[2]]))

    # busy indicator - title positions is right- UI elements position is NULL
    warn_msgs <- capture_warnings(periscope2::add_ui_header(ui_elements    = menu,
                                                            ui_position    = NULL,
                                                            title          = "Header Title",
                                                            title_position = "right"))
    expect_equal(shiny::isolate(periscope2:::.g_opts$app_title), "Header Title")
    expect_equal("ui_position must be on of 'left', 'center'or 'right' values. Setting default value 'right'",
                 warn_msgs[1])
    expect_equal("title_position cannot be equal to ui_position. Setting default values",
                 warn_msgs[2])
    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_true(grepl('periscope-busy-ind.*Header Title.*Tab1', header[[1]]$children[[2]]))
})


test_that("add_ui_header - no ui element", {
    skin               <- "light"
    status             <- "white"
    border             <- TRUE
    compact            <- FALSE
    left_sidebar_icon  <- shiny::icon("bars")
    right_sidebar_icon <- shiny::icon("th")
    fixed              <- FALSE
    left_menu          <- NULL
    right_menu         <- NULL

    periscope2::add_ui_header(left_menu          = left_menu,
                              right_menu         = right_menu,
                              skin               = skin,
                              status             = status,
                              border             = border,
                              compact            = compact,
                              left_sidebar_icon  = left_sidebar_icon,
                              right_sidebar_icon = right_sidebar_icon,
                              fixed              = fixed,
                              title              = "good title")


    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_equal(length(header), 2)
    expect_true(grepl("good title", header[[1]], fixed = TRUE))
    expect_null(header[[2]])
})


test_that("add_ui_left_sidebar no left sidebar", {
    expect_snapshot_output(shiny::isolate(periscope2:::.g_opts$left_sidebar))
})


test_that("add_ui_left_sidebar empty left sidebar", {
    skin             <- "light"
    status           <- "primary"
    elevation        <- 4
    collapsed        <- FALSE
    minified         <- FALSE
    expand_on_hover  <- FALSE
    fixed            <- TRUE
    sidebar_elements <- NULL
    sidebar_menu     <- NULL
    custom_area      <- NULL
    add_ui_left_sidebar(sidebar_elements = sidebar_elements,
                        skin             = skin,
                        status           = status,
                        elevation        = elevation,
                        collapsed        = collapsed,
                        minified         = minified,
                        expand_on_hover  = expand_on_hover,
                        fixed            = fixed,
                        sidebar_menu     = sidebar_menu,
                        custom_area      = custom_area)
    expect_snapshot_output(shiny::isolate(periscope2:::.g_opts$left_sidebar))
})


test_that("add_ui_left_sidebar example left sidebar", {
    skin             <- "light"
    status           <- "primary"
    elevation        <- 4
    collapsed        <- FALSE
    minified         <- FALSE
    expand_on_hover  <- FALSE
    fixed            <- TRUE
    sidebar_elements <- NULL
    sidebar_menu     <-  sidebarMenu(
        id = "features_id",
        sidebarHeader("Periscope2 Features"),
        menuItem(
            text    = "Application Setup",
            tabName = "application_setup",
            icon    = icon("building")
        ),
        menuItem(
            text    = "Periscope2 Modules",
            tabName = "periscope_modules",
            icon    = icon("cubes")
        ),
        menuItem(
            text    = "User Notifications",
            tabName = "user_notifications",
            icon    = icon("comments")
        )
    )
    custom_area      <- NULL
    add_ui_left_sidebar(sidebar_elements = sidebar_elements,
                        skin             = skin,
                        status           = status,
                        elevation        = elevation,
                        collapsed        = collapsed,
                        minified         = minified,
                        expand_on_hover  = expand_on_hover,
                        fixed            = fixed,
                        sidebar_menu     = sidebar_menu,
                        custom_area      = custom_area)
    left_sidebar <- shiny::isolate(periscope2:::.g_opts$left_sidebar)
    expect_equal(length(left_sidebar), 10)
    expect_snapshot_output(left_sidebar[1:9])
    expect_true(grepl('Application Setup', left_sidebar[[10]], fixed = TRUE))
    expect_true(grepl('Periscope2 Modules', left_sidebar[[10]], fixed = TRUE))
    expect_true(grepl('User Notifications', left_sidebar[[10]], fixed = TRUE))
})


test_that("add_ui_right_sidebar no right sidebar", {
    expect_null(shiny::isolate(periscope2:::.g_opts$right_sidebar))
})


test_that("add_ui_right_sidebar empty right sidebar", {
    collapsed        <- TRUE
    overlay          <- TRUE
    skin             <- "light"
    pinned           <- FALSE
    sidebar_elements <- NULL
    sidebar_menu     <- NULL

    add_ui_right_sidebar(sidebar_elements = sidebar_elements,
                         collapsed        = collapsed,
                         overlay          = overlay,
                         skin             = skin,
                         pinned           = pinned,
                         sidebar_menu     = sidebar_menu)
    right_sidebar <- shiny::isolate(periscope2:::.g_opts$right_sidebar)
    expect_true(grepl('id="controlbarId"', right_sidebar, fixed = TRUE))
    expect_true(grepl('id="sidebarRightAlert"', right_sidebar, fixed = TRUE))
})


test_that("add_ui_right_sidebar example right sidebar", {
    collapsed        <- TRUE
    overlay          <- TRUE
    skin             <- "light"
    pinned           <- FALSE
    sidebar_elements <-  list(div(checkboxInput("hideFileOrganization", "Show Files Organization"), style = "margin-left:20px"))
    sidebar_menu     <- NULL

    add_ui_right_sidebar(sidebar_elements = sidebar_elements,
                         collapsed        = collapsed,
                         overlay          = overlay,
                         skin             = skin,
                         pinned           = pinned,
                         sidebar_menu     = sidebar_menu)
    right_sidebar <- shiny::isolate(periscope2:::.g_opts$right_sidebar)
    expect_true(grepl('id="controlbarId"', right_sidebar, fixed = TRUE))
    expect_true(grepl('id="sidebarRightAlert"', right_sidebar, fixed = TRUE))
    expect_true(grepl('id="hideFileOrganization"', right_sidebar, fixed = TRUE))
    expect_true(grepl('Show Files Organization', right_sidebar, fixed = TRUE))
})


test_that("add_ui_footer no footer", {
    expect_null(shiny::isolate(periscope2:::.g_opts$footer))
})


test_that("add_ui_footer empty footer", {
    right <- NULL
    fixed <- FALSE
    left  <- NULL

    add_ui_footer(left, right, fixed)
    expect_snapshot_output(shiny::isolate(periscope2:::.g_opts$footer))
})


test_that("add_ui_footer example footer", {
    right <- "2022"
    fixed <- FALSE
    left  <- a(
        href   = "https://periscopeapps.org/",
        target = "_blank",
        "periscope2")

    add_ui_footer(left, right, fixed)
    expect_snapshot_output(shiny::isolate(periscope2:::.g_opts$footer))
})


test_that("add_ui_body empty body", {
    expect_equal(shiny::isolate(periscope2:::.g_opts$body_elements), c())
    add_ui_body()
    expect_snapshot(shiny::isolate(periscope2:::.g_opts$body_elements))
})


test_that("add_ui_body example body", {
    about_box <- jumbotron(
        title = "periscope2: Enterprise Streamlined 'Shiny' Application Framework",
        lead  = p("periscope2 is a scalable and UI-standardized 'shiny' framework including a variety of developer convenience",
                  "functions with the goal of both streamlining robust application development and assisting in creating a consistent",
                  " user experience regardless of application or developer."),
        tags$dl(tags$dt("Features"),
                tags$ul(tags$li("Predefined but flexible template for new Shiny applications with a default dashboard layout"),
                        tags$li("Separation by file of functionality that exists in one of the three shiny scopes: global, server-global, and server-local."),
                        tags$li("Off-the-shelf and ready to be used modules ('Announcements', 'Table Downloader', 'Plot Downloader', 'File Downloader', 'Application Logger' and 'Reset Application'"),
                        tags$li("Different methods to notify user and add useful information about application UI and server operations"))),
        status = "info",
        href   = "https://periscopeapps.org/"
    )

    add_ui_body(list(about_box))
    expect_snapshot_output(shiny::isolate(periscope2:::.g_opts$body_elements))

    add_ui_body(list(div("more elements")), append = TRUE)
    expect_snapshot_output(shiny::isolate(periscope2:::.g_opts$body_elements))
    dashboard_ui <- periscope2:::create_application_dashboard()
    expect_true(grepl('id="announceAlert"' , dashboard_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="headerAlert"' , dashboard_ui[[2]], fixed = TRUE))
    expect_true(grepl('Periscope2 Features' , dashboard_ui[[3]], fixed = TRUE))
    expect_true(grepl('id="sidebarRightAlert"' , dashboard_ui[[3]], fixed = TRUE))
    expect_true(grepl('id="footerAlert"' , dashboard_ui[[3]], fixed = TRUE))
})

test_that("add_ui_body append", {
    add_ui_body(list(div("append div")), append = TRUE)
    expect_snapshot_output(shiny::isolate(periscope2:::.g_opts$body_elements))
    dashboard_ui <- periscope2:::create_application_dashboard()
    expect_true(grepl('id="announceAlert"' , dashboard_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="headerAlert"' , dashboard_ui[[2]], fixed = TRUE))
    expect_true(grepl('Periscope2 Features' , dashboard_ui[[3]], fixed = TRUE))
    expect_true(grepl('id="sidebarRightAlert"' , dashboard_ui[[3]], fixed = TRUE))
    expect_true(grepl('id="footerAlert"' , dashboard_ui[[3]], fixed = TRUE))
})


test_that("load_announcements function params", {
    expect_equal(load_announcements(announcements_file_path = system.file("fw_templ", "announce.yaml", package = "periscope2")), 30000)
})


test_that("load_announcements empty file", {
    # test empty announcement
    appTemp_dir        <- tempdir()
    appTemp            <- tempfile(pattern = "TestThatApp", tmpdir = appTemp_dir)
    announcements_file <- paste0(gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE))), ".yaml")
    yaml::write_yaml("", announcements_file)

    output_msg <- capture_output(expect_null(load_announcements(announcements_file_path = announcements_file)))
    expect_true(grepl('Announcements will be ignored', output_msg, fixed = TRUE))
    unlink(announcements_file, TRUE)
})

test_that("load_announcements - parsing error", {
    # test empty announcement
    appTemp_dir        <- tempdir()
    appTemp            <- tempfile(pattern = "TestThatApp", tmpdir = appTemp_dir)
    announcements_file <- paste0(gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE))), ".yaml")
    cat(":", file = (con <- file(announcements_file, "w", encoding = "UTF-8")))
    close(con)

    output_msg <- capture_output(expect_warning(load_announcements(announcements_file_path = announcements_file),
                                                 regexp                                    = "[(Could not parse TestThatApp)]"))
    expect_true(grepl('Parser error', output_msg, fixed = TRUE))
    unlink(announcements_file, TRUE)
})

test_that("load_announcements function parameters", {
    expect_null(create_announcements(start_date = "2222-11-26",
                                     end_date   = "2222-12-26"))
    output_msg <- capture_output(expect_null(
        create_announcements(start_date = "2022-11-26",
                             end_date   = "2222-12-26",
                             style      = "not-style")))
    expect_true(grepl("Announcement 'style' must be one of  info, danger, success, warning, primary",
                      output_msg, fixed = TRUE))
    expect_null(
        create_announcements(start_date        = "11-26-2222",
                             end_date          = "12-26-2222",
                             start_date_format = "%m-%d-%Y",
                             end_date_format   = "%m-%d-%Y"))

    output_msg <- capture_output(expect_null(
        create_announcements(start_date        = "11-26-2222",
                             end_date          = "12-26-2222",
                             end_date_format   = "%m-%d-%Y")))
    expect_true(grepl("All formats failed to parse. No formats found",
                      output_msg, fixed = TRUE))

    output_msg <- capture_output(expect_null(
        create_announcements(start_date        = "11-26-2222",
                                     end_date          = "12-26-2222",
                                     start_date_format = "%m-%d-%Y")))
    expect_true(grepl("All formats failed to parse. No formats found",
                      output_msg, fixed = TRUE))

    expect_null(create_announcements(start_date        = "11-26-2222",
                                     start_date_format = "%m-%d-%Y"))
    expect_true(grepl("All formats failed to parse. No formats found",
                      output_msg, fixed = TRUE))

    output_msg <- capture_output(expect_null(create_announcements(style = "info")))
    expect_true(grepl("Announcement 'text' value is empty. It must contain non empty text value",
                      output_msg, fixed = TRUE))

    output_msg <- capture_output(expect_null(
        create_announcements(style      = "info",
                             text       = "text",
                             auto_close = "abc")))
    expect_true(grepl("Announcement 'auto_close' value ' abc ' is invalid",
                      output_msg, fixed = TRUE))
})

test_that("load_theme_settings - null settings", {
    expect_snapshot(load_theme_settings())
})


test_that("ui_tooltip", {
    expect_snapshot_output(ui_tooltip(id = "id", label = "mylabel", text = "mytext"))
    expect_snapshot_output(ui_tooltip(id = "id2", label = "mylabel2", text = "mytext2", placement = "left"))
    expect_snapshot_error(ui_tooltip(id = "id2", label = "mylabel2", text = "mytext2", placement = "nowhere"))
})

test_that("ui_tooltip no text", {
    expect_warning(ui_tooltip(id = "id", label = "mylabel", text = ""), "ui_tooltip\\() called without tooltip text.")
})


test_that("theme - valid theme", {
    theme_settings <- yaml::read_yaml(system.file("fw_templ", "p_example", "periscope_style.yaml", package = "periscope2"))
    dir.create("www")
    yaml::write_yaml(theme_settings, "www/periscope_style.yaml")
    periscope_theme <- create_theme()
    expect_true(!is.null(periscope_theme))
    expect_true(nchar(periscope_theme) > 0)
    unlink("www/periscope_style.yaml")
    unlink("www", recursive = TRUE)
})


test_that("theme - parsing error", {
    dir.create("www")
    theme_file           <- "www/periscope_style.yaml"
    cat(":", file = (con <- file(theme_file, "w", encoding = "UTF-8")))
    close(con)
    periscope_theme <- suppressWarnings(periscope2:::create_theme())
    expect_true(!is.null(periscope_theme))
    expect_true(nchar(periscope_theme) > 0)
    unlink("www/periscope_style.yaml")
    unlink("www", recursive = TRUE)
})


test_that("theme - invalid color", {
    local_edition(3)
    theme_settings <- yaml::read_yaml(system.file("fw_templ", "p_example", "periscope_style.yaml", package = "periscope2"))
    dir.create("www")
    theme_settings[["primary"]]               <- "not color"
    theme_settings[["sidebar_width"]]         <- "300"
    theme_settings[["control_sidebar_width"]] <- "-300"

    yaml::write_yaml(theme_settings, "www/periscope_style.yaml")
    theme_warnings <- capture_warnings(periscope2:::create_theme())
    expect_snapshot(theme_warnings)
    unlink("www/periscope_style.yaml")
    unlink("www", recursive = TRUE)
})


test_that("dashboard - create default dashboard", {
    expect_snapshot(periscope2:::create_application_dashboard())
})


test_that("add_ui_header - html title", {
    title              <- "periscope Example Application"
    app_info           <- HTML("Demonstrate periscope features and generated application layout")
    log_level          <- "INFO"
    app_version        <- "2.3.1"
    loading_indicator  <- list(html = tagList(div("Loading ...")))

    periscope2::set_app_parameters(app_info           = app_info,
                                   log_level          = log_level,
                                   app_version        = app_version,
                                   loading_indicator  = loading_indicator)
    # normal header
    skin               <- "light"
    status             <- "white"
    border             <- TRUE
    compact            <- FALSE
    left_sidebar_icon  <- shiny::icon("bars")
    right_sidebar_icon <- shiny::icon("th")
    fixed              <- FALSE
    left_menu          <- NULL
    right_menu         <- NULL

    periscope2::add_ui_header(title              = title,
                              left_menu          = left_menu,
                              right_menu         = right_menu,
                              skin               = skin,
                              status             = status,
                              border             = border,
                              compact            = compact,
                              left_sidebar_icon  = left_sidebar_icon,
                              right_sidebar_icon = right_sidebar_icon,
                              fixed              = fixed)


    expect_snapshot(shiny::isolate(periscope2:::.g_opts$app_info))
    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_snapshot(header[[1]])
})


test_that("add_ui_header - url title", {
    announcements_file <- system.file("fw_templ", "announce.yaml", package = "periscope2")
    title              <- "periscope Example Application"
    app_info           <- "https://cran.r-project.org/web/packages/periscope2/index.html"
    log_level          <- "DEBUG"
    app_version        <- "2.3.1"
    loading_indicator  <- list(html = tagList(div("Loading ...")))

    set_app_parameters(app_info          = app_info,
                       log_level         = log_level,
                       app_version       = app_version,
                       loading_indicator = loading_indicator)
    # normal header
    skin               <- "light"
    status             <- "white"
    border             <- TRUE
    compact            <- FALSE
    left_sidebar_icon  <- shiny::icon("bars")
    right_sidebar_icon <- shiny::icon("th")
    fixed              <- FALSE
    left_menu          <- NULL
    right_menu         <- NULL

    periscope2::add_ui_header(title              = title,
                              left_menu          = left_menu,
                              right_menu         = right_menu,
                              skin               = skin,
                              status             = status,
                              border             = border,
                              compact            = compact,
                              left_sidebar_icon  = left_sidebar_icon,
                              right_sidebar_icon = right_sidebar_icon,
                              fixed              = fixed)

    expect_snapshot(shiny::isolate(periscope2:::.g_opts$app_info))
    header <- shiny::isolate(periscope2:::.g_opts$header)
    expect_snapshot(header[[1]])
})


test_that("create alert - id and target error", {
    expect_error(createPSAlert(id = "test_id", selector = "test_selector", options = NULL),
                 regexp = "Please choose either target or selector!")
})


test_that("create alert - id", {
    expect_snapshot_output(createPSAlert(id = "test_id", session = MockShinySession$new(), options = NULL))
})


test_that("set_app_parameters update values", {
    announcements_file <- system.file("fw_templ", "announce.yaml", package = "periscope2")
    title              <- "periscope Example Application"
    app_info           <- HTML("Demonstrate periscope features and generated application layout")
    log_level          <- "INFO"
    app_version        <- "2.3.1"
    loading_indicator  <- list(html = tagList(div("Loading ...")))

    deprecated_flds_warn <- capture_warnings(set_app_parameters(title              = title,
                                                                app_info           = app_info,
                                                                log_level          = log_level,
                                                                app_version        = app_version,
                                                                loading_indicator  = loading_indicator,
                                                                announcements_file = announcements_file))
    expect_snapshot(deprecated_flds_warn)
    expect_equal(shiny::isolate(periscope2:::.g_opts$app_title), title)
    expect_snapshot(shiny::isolate(periscope2:::.g_opts$app_info))
    expect_equal(shiny::isolate(periscope2:::.g_opts$loglevel), log_level)
    expect_equal(shiny::isolate(periscope2:::.g_opts$app_version), app_version)
    expect_snapshot(shiny::isolate(periscope2:::.g_opts$loading_indicator))
    expect_equal(shiny::isolate(periscope2:::.g_opts$announcements_file), announcements_file)
    expect_equal(load_announcements(), 30000)
    expect_equal(periscope2:::fw_get_loglevel(), log_level)
    expect_equal(periscope2:::fw_get_title(), title)
    expect_equal(periscope2:::fw_get_version(), app_version)
})

# ----------------------------------------
# --          PROGRAM global.R          --
# ----------------------------------------
# USE: Global variables and functions
#
# NOTES:
#   - All variables/functions here are
#     globally scoped and will be available
#     to server, UI and session scopes
# ----------------------------------------
library(DT)
library(reactable)
library(shiny)
library(periscope2)
library(shinyWidgets)
library(waiter)
library(shinyjs)
library(canvasXpress)
library(glue)
library(colourpicker)
library(yaml)


# -- Setup your Application --
set_app_parameters(app_info           = HTML("Demonstrate periscope features and generated application layout"),
                   log_level          = "DEBUG",
                   app_version        = "1.0.0",
                   loading_indicator  = list(html = tagList(spin_1(), "Loading ..."), color = "rgba(22, 65, 124, 0.3)"))

# -- PROGRAM --

# FRAMEWORK HELPER FUNCTIONS -------
# -- (INTERNAL ONLY - No Exports) --
# .g_opts --------------------
# holds the app options values
.g_opts <- shiny::reactiveValues(
    tt_image           = "img/tooltip.png",
    tt_height          = "16px",
    tt_width           = "16px",
    datetime.fmt       = "%m-%d-%Y %H:%M",
    log.formatter      = function(record) { paste0(record$logger,
                                                   " [", record$timestamp, "] ",
                                                   record$msg) },
    loglevel            = "DEBUG",
    app_title           = "Set using add_ui_header() in program/ui_header.R",
    app_info            = NULL,
    app_version         = "1.0.0",
    loading_indicator   = NULL,
    announcements_file  = NULL,
    data_download_types = c("csv", "xlsx", "tsv", "txt"),
    plot_download_types = c("png", "jpeg", "tiff", "bmp"),
    left_sidebar        = list(disable = TRUE),
    body_elements       = c(),
    header              = NULL,
    right_sidebar       = NULL,
    footer              = NULL)


# Server ----------------------------

# Sets up the logging functionality including archiving out any existing log
# and attaching the file handler.  DEBUG will also attach a console handler.
# NOTE: only one previous log is kept (as <name>.loglast)
.setupUserLogging <- function(logger) {
    logdir    <- "log"
    logfile   <- paste0(paste(logdir, logger$name, sep = .Platform$file.sep),
                        ".log")
    formatter <- .g_opts$log.formatter
    loglevel  <- .g_opts$loglevel

    if (!dir.exists(logdir)) {
        dir.create(logdir)
    }

    if (file.exists(logfile)) {
        # archive out for now, keeping only one for now
        file.rename(logfile, paste(logfile, "last", sep = "."))
    }

    file.create(logfile)

    addHandler(writeToFile,
               file      = logfile,
               level     = loglevel,
               logger    = logger,
               formatter = formatter)

    if (loglevel == "DEBUG") {
        addHandler(writeToConsole,
                   level     = loglevel,
                   logger    = logger,
                   formatter = formatter)
    }

    logfile
}


.setup_logging <- function(session, logger) {
    shiny::reactiveFileReader(
        500, #milliseconds,
        session,
        .setupUserLogging(logger),
        readLines)
}

library(ggplot2)
library(lattice)


# -- data for plots
data(mtcars)
mtcars$cyl <- factor(mtcars$cyl, levels = c(4,6,8),
                     labels = c("4cyl", "6cyl", "8cyl"))
attr(mtcars, "show_rownames") <- TRUE


# -- plotting functions

plot2ggplot <- function() {
    ggplot(data = mtcars, aes(x = wt, y = mpg)) +
        geom_point(aes(color = cyl)) +
        theme(legend.justification   = c(1, 1),
              legend.position.inside = c(1, 1),
              legend.title           = element_blank()) +
        ggtitle("GGPlot Example w/Hover") +
        xlab("wt") +
        ylab("mpg")
}


plot_data <- function() {
    mtcars
}


plot3lattice <- function() {
    xyplot(mpg ~ wt ,
           data     = mtcars,
           pch      = 1,
           groups   = factor(cyl),
           auto.key = list(corner = c(1, 1)),
           main     = "Lattice Example")
}



plot_htmlwidget <- function(report_modus = FALSE) {
    venn <- data.frame(A   = 57,   B = 12, C   = 67, D   = 72, AB   = 4,
                       AC  = 67,  AD = 25, BC  = 67, BD  = 27, CD   = 38,
                       ABC = 69, ABD = 28, ACD = 52, BCD = 46, ABCD = 3)

    canvasXpress(vennData            = venn,
                 graphType           = "Venn",
                 vennGroups          = 4,
                 vennLegend          = list(A = "List1", B = "List2", C = "List3", D = "List4"),
                 title               = "CanvasXpress Example",
                 disableToolbar      = report_modus,
                 disableTouchToolbar = report_modus)
}

# -------------------------------------------
# -- Application Downloadable Table Module --
# -------------------------------------------


#' downloadableTable module UI function
#'
#' Creates a custom high-functionality table paired with a linked downloadFile
#' button.  The table has search and highlight functionality, infinite scrolling,
#' sorting by columns and returns a reactive dataset of selected items.
#'
#' downloadFile button will be hidden if \code{downloadableTable} parameter \code{downloaddatafxn} or
#' \code{downloadableTableUI} parameter \code{downloadtypes} is empty
#'
#' @param id character id for the object
#' @param downloadtypes vector of values for data download types
#' @param hovertext download button tooltip hover text
#' @param contentHeight viewable height of the table (any valid css size value)
#' @param singleSelect whether the table should only allow a single row to be
#' selected at a time (FALSE by default allows multi-select).
#'
#' @return list of downloadFileButton UI and DT datatable
#'
#' @section Table Features:
#' \itemize{
#'     \item Consistent styling of the table
#'     \item downloadFile module button functionality built-in to the table (it will be shown only if downloadtypes is defined)
#'     \item Ability to show different data from the download data
#'     \item Table is automatically fit to the window size with infinite
#'     y-scrolling
#'     \item Table search functionality including highlighting built-in
#'     \item Multi-select built in, including reactive feedback on which table
#'     items are selected
#' }
#'
#' @section Example:
#' \code{downloadableTableUI("mytableID", c("csv", "tsv"),
#' "Click Here", "300px")}
#'
#' @section Notes:
#' When there are no rows to download in any of the linked downloaddatafxns the
#' button will be hidden as there is nothing to download.
#'
#' @section Shiny Usage:
#' Call this function at the place in ui.R where the table should be placed.
#'
#' Paired with a call to \code{downloadableTable(id, ...)}
#' in server.R
#'
#' @seealso \link[periscope2]{downloadableTable}
#' @seealso \link[periscope2]{downloadFileButton}
#' @seealso \link[periscope2]{logViewerOutput}
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{downloadablePlot}
#'
#' @examples
#' if (interactive()) {
#'  library(shiny)
#'  library(periscope2)
#'  shinyApp(ui = fluidPage(fluidRow(column(width = 12,
#'    downloadableTableUI("object_id1",
#'                        downloadtypes = c("csv", "tsv"),
#'                        hovertext     = "Download the data here!",
#'                        contentHeight = "300px",
#'                        singleSelect  = FALSE)))),
#'    server = function(input, output) {
#'      mydataRowIds <- function(){
#'        rownames(head(mtcars))[c(2, 5)]
#'      }
#'      selectedrows <- downloadableTable(
#'        id               = "object_id1",
#'        logger           = "",
#'        filenameroot     = "mydownload1",
#'        downloaddatafxns = list(csv = reactiveVal(mtcars), tsv = reactiveVal(mtcars)),
#'        tabledata        = reactiveVal(mtcars),
#'        selection        = mydataRowIds,
#'        table_options    = list(rownames = TRUE,
#'                             caption  = "This is a great table!"))
#'      observeEvent(selectedrows(), {
#'        print(selectedrows())
#'      })})
#'}
#'
#' @export
downloadableTableUI <- function(id,
                                downloadtypes = NULL,
                                hovertext     = NULL,
                                contentHeight = "200px",
                                singleSelect  = FALSE) {
    ns <- shiny::NS(id)

    list(
        shiny::conditionalPanel(
            condition = "output.displayButton",
            ns        = ns,
            shiny::span(
                id    = ns("dtableButtonDiv"),
                class = "periscope-downloadable-table-button",
                style = ifelse(length(downloadtypes) > 0, "", "display:none"),
                downloadFileButton(ns("dtableButtonID"),
                                   downloadtypes,
                                   hovertext))),
        DT::dataTableOutput(ns("dtableOutputID")),
        shiny::tags$input(
            id    = ns("dtableOutputHeight"),
            type  = "text",
            class = "shiny-input-container hidden",
            value = contentHeight),
        shiny::tags$input(
            id    = ns("dtableSingleSelect"),
            type  = "text",
            class = "shiny-input-container hidden",
            value = singleSelect)
    )
}


#' downloadableTable module server function
#'
#' Server-side function for the downloadableTableUI.  This is a custom
#' high-functionality table paired with a linked downloadFile
#' button.
#'
#' downloadFile button will be hidden if \code{downloadableTable} parameter \code{downloaddatafxn} or
#' \code{downloadableTableUI} parameter \code{downloadtypes} is empty
#'
#' Generated table can highly customized using function \code{?DT::datatable} same arguments
#'  except for `options` and `selection` parameters.
#'
#' For `options` user can pass the same \code{?DT::datatable} options using the same names and
#' values one by one separated by comma.
#'
#' For `selection` parameter it can be either a function or reactive expression providing the row_ids of the
#' rows that should be selected.
#'
#' Also, user can apply the same provided \code{?DT::formatCurrency} columns formats on passed
#' dataset using format functions names as keys and their options as a list.
#'
#'
#' @param id  the ID of the Module's UI element
#' @param logger logger to use
#' @param filenameroot the text used for user-downloaded file - can be
#' either a character string, a reactive expression or a function returning a character
#' string
#' @param downloaddatafxns a \strong{named} list of functions providing the data as
#' return values.  The names for the list should be the same names that were used
#' when the table UI was created.
#' @param tabledata function or reactive expression providing the table display
#' data as a return value. This function should require no input parameters.
#' @param selection function or reactive expression providing the row_ids of the
#' rows that should be selected
#' @param table_options optional table formatting parameters check \code{?DT::datatable} for options full list.
#' Also see example below to see how to pass options
#'
#' @return Reactive expression containing the currently selected rows in the
#' display table
#'
#' @section Notes:
#'  \itemize{
#'   \item When there are no rows to download in any of the linked downloaddatafxns
#'   the button will be hidden as there is nothing to download.
#'   \item \code{selection} parameter has different usage than DT::datatable \code{selection} option.
#'   See parameters usage section.
#'   \item DT::datatable options \code{editable}, \code{width} and \code{height} are not supported
#' }
#'
#' @section Shiny Usage:
#' This function is not called directly by consumers - it is accessed in
#' server.R using the same id provided in \code{downloadableTableUI}:
#'
#' \strong{\code{downloadableTable(id, logger, filenameroot,
#' downloaddatafxns, tabledata, rownames, caption, selection)}}
#'
#' \emph{Note}: calling module server returns the reactive expression containing the
#' currently selected rows in the display table.
#'
#' @seealso \link[periscope2]{downloadableTableUI}
#' @seealso \link[periscope2]{downloadFileButton}
#' @seealso \link[periscope2]{logViewerOutput}
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{downloadablePlot}
#'
#' @examples
#' if (interactive()) {
#'  library(shiny)
#'  library(periscope2)
#'  shinyApp(ui = fluidPage(fluidRow(column(width = 12,
#'    downloadableTableUI("object_id1",
#'                        downloadtypes = c("csv", "tsv"),
#'                        hovertext     = "Download the data here!",
#'                        contentHeight = "300px",
#'                        singleSelect  = FALSE)))),
#'    server = function(input, output) {
#'      mydataRowIds <- function(){
#'        rownames(head(mtcars))[c(2, 5)]
#'      }
#'      selectedrows <- downloadableTable(
#'        id               = "object_id1",
#'        logger           = "",
#'        filenameroot     = "mydownload1",
#'        downloaddatafxns = list(csv = reactiveVal(mtcars), tsv = reactiveVal(mtcars)),
#'        tabledata        = reactiveVal(mtcars),
#'        selection        = mydataRowIds,
#'        table_options    = list(rownames = TRUE,
#'                             caption  = "This is a great table!"))
#'      observeEvent(selectedrows(), {
#'        print(selectedrows())
#'      })})
#'}
#'
#' @export
downloadableTable <- function(id,
                              logger           = NULL,
                              filenameroot     = "download",
                              downloaddatafxns = NULL,
                              tabledata,
                              selection        = NULL,
                              table_options    = list()) {
    shiny::moduleServer(id,
                        function(input, output, session) {

                            if (all(!is.null(selection),
                                    is.character(selection))) {
                                message("'selection' parameter must be a function or reactive expression. Setting default value NULL.")
                                selection <- NULL
                            }

                            if (is.null(filenameroot)) {
                                filenameroot <- ""
                            } else if (shiny::is.reactive(filenameroot) || is.function(filenameroot)) {
                                filenameroot <- shiny::isolate(filenameroot())
                            }

                            downloadFile(id           = "dtableButtonID",
                                         logger       = logger,
                                         filenameroot = filenameroot,
                                         datafxns     = downloaddatafxns,
                                         row_names    = table_options$rownames)

                            dtInfo <- shiny::reactiveValues(selection = NULL,
                                                            selected  = NULL,
                                                            tabledata = NULL)

                            shiny::observe({
                                result <- list(mode = ifelse(input$dtableSingleSelect == "TRUE", "single", "multiple"))
                                if (!is.null(selection)) {
                                    selection_value <- selection()
                                    if (result[["mode"]] == "single" && length(selection_value) > 1) {
                                        selection_value <- selection_value[1]
                                    }
                                    result[["selected"]] <- selection_value
                                    dtInfo$selection <- NULL
                                }
                                dtInfo$selection <- result
                                dtInfo$selected  <- rownames(result)
                            })

                            shiny::observe({
                                dtInfo$selected  <- input$dtableOutputID_rows_selected
                            })

                            shiny::observe({
                                dtInfo$tabledata <- tabledata()
                            })

                            shiny::observe({
                                output$displayButton <- shiny::reactive(length(downloaddatafxns) > 0)
                                shiny::outputOptions(output, "displayButton", suspendWhenHidden = FALSE)
                            })


                            output$dtableOutputID <- DT::renderDataTable({
                                sourcedata <- dtInfo$tabledata

                                if (NROW(sourcedata) > 0) {
                                    row.names <- rownames(sourcedata)
                                    row.ids   <- as.character(seq(1:NROW(sourcedata)))
                                    if (is.null(row.names) || identical(row.names, row.ids)) {
                                        DT_RowId <- paste0("rowid_", row.ids)
                                        rownames(sourcedata) <- DT_RowId
                                    }
                                }

                                if (is.null(table_options[["scrollY"]])) {
                                    table_options[["scrollY"]] <- input$dtableOutputHeight
                                }

                                table_options[["selection"]] <- dtInfo$selection

                                if (is.null(table_options[["escape"]])) {
                                    table_options[["escape"]] <- FALSE
                                }

                                if (is.null(table_options[["rownames"]])) {
                                    table_options[["rownames"]] <- FALSE
                                }

                                # get format functions
                                format_options_idx <- which(startsWith(names(table_options), "format"))
                                format_options <- table_options[format_options_idx]
                                if (length(format_options_idx) > 0) {
                                    dt_args <- build_datatable_arguments(table_options[-format_options_idx])
                                } else {
                                    dt_args <- build_datatable_arguments(table_options)
                                }

                                if (is.null(sourcedata)) {
                                    sourcedata <- data.frame()
                                }

                                dt_args[["data"]] <- sourcedata

                                tryCatch({
                                    dt <- do.call(DT::datatable, dt_args)

                                    if ((length(format_options) > 0) && (NROW(dt_args$data) > 0)) {
                                        dt <- format_columns(dt, format_options)
                                    }
                                    dt
                                },
                                error = function(e) {
                                    message("Could not apply DT options due to: ", e$message)
                                    DT::datatable(sourcedata)
                                })
                            })


                            shiny::reactive({
                                shiny::isolate(dtInfo$tabledata)[dtInfo$selected, ]
                            })

                        })
}


build_datatable_arguments <- function(table_options) {
    dt_args <- list()
    formal_dt_args <- methods::formalArgs(DT::datatable)
    dt_args[["rownames"]] <- TRUE
    dt_args[["class"]] <- paste("periscope-downloadable-table table-condensed",
                               "table-striped table-responsive")
    options <- list()
    for (option in names(table_options)) {
        if (option %in% c("editable", "width", "height")) {
            message("DT option '", option ,"' is not supported. Ignoring it.")
            next
        }

        if (option %in% formal_dt_args) {
            dt_args[[option]] <- table_options[[option]]
        } else {
            options[[option]] <- table_options[[option]]
        }
    }

    if (is.null(options[["deferRender"]])) {
        options[["deferRender"]] <- FALSE
    }

    if (is.null(options[["paging"]]) && is.null(table_options[["pageLength"]])) {
        options[["paging"]] <- FALSE
    }

    if (is.null(options[["scrollX"]])) {
        options[["scrollX"]] <- TRUE
    }

    if (is.null(options[["dom"]]) && is.null(table_options[["pageLength"]])) {
        options[["dom"]] <- '<"periscope-downloadable-table-header"f>tr'
    }

    if (is.null(options[["processing"]])) {
        options[["processing"]] <- TRUE
    }

    if (is.null(options[["rowId"]])) {
        options[["rowId"]] <- 1
    }

    if (is.null(options[["searchHighlight"]])) {
        options[["searchHighlight"]] <- TRUE
    }
    dt_args[["options"]] <- options
    dt_args
}


format_columns <- function(dt, format_options) {
    for (format_idx in 1:length(format_options)) {
        format_args <- format_options[[format_idx]]
        format_args[["table"]] <- dt
        format <- tolower(names(format_options)[format_idx])
        dt <- switch(format,
                     "formatstyle" = do.call(DT::formatStyle, format_args),
                     "formatdate" = do.call(DT::formatDate, format_args),
                     "formatsignif" = do.call(DT::formatSignif, format_args),
                     "formatround" = do.call(DT::formatRound, format_args),
                     "formatpercentage" = do.call(DT::formatPercentage, format_args),
                     "formatstring" = do.call(DT::formatString, format_args),
                     "formatcurrency" = do.call(DT::formatCurrency, format_args))
    }
    dt
}

context("periscope2 - downloadable table")
local_edition(3)

test_that("downloadableTableUI", {
    table_ui <- downloadableTableUI(id            = "myid",
                                    downloadtypes = c("csv"),
                                    hovertext     = "myHoverText")
    expect_equal(length(table_ui), 4)
    expect_true(grepl('id="myid-dtableButtonDiv"', table_ui[[1]], fixed = TRUE))
    expect_true(grepl('title="myHoverText"', table_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid-dtableButtonID-csv"', table_ui[[1]], fixed = TRUE))
    expect_false(grepl('style="display:none">', table_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid-dtableOutputID"', table_ui[[2]], fixed = TRUE))
})

test_that("downloadableTableUI - no download types", {
    table_ui <- downloadableTableUI(id = "myid2")
    expect_equal(length(table_ui), 4)
    expect_true(grepl('id="myid2-dtableButtonDiv"', table_ui[[1]], fixed = TRUE))
    expect_true(grepl('style="display:none">', table_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid2-dtableOutputID"', table_ui[[2]], fixed = TRUE))
})

# helper functions
data <- reactive({
    head(mtcars)
})

data_without_rownames <- reactive({
    sub_data           <- head(mtcars)
    rownames(sub_data) <- NULL
    sub_data
})

mydataRowIds <- function(){
    rownames(head(mtcars))
}

test_that("downloadableTable - singleSelect_FALSE_selection_enabled", {
        testServer(downloadableTable,
                   args = list(logger           = periscope2:::fw_get_user_log(),
                               filenameroot     = "mydownload1",
                               downloaddatafxns = list(csv = data, tsv = data),
                               tabledata        = data,
                               selection        = mydataRowIds),
                   expr = {
                       session$setInputs(dtableSingleSelect = "FALSE")
                       selected <- paste0('"selection":{"mode":"multiple","selected":["', paste(mydataRowIds(), collapse = '","'), '"')
                       expect_true(grepl(selected, output$dtableOutputID, fixed = TRUE))
                   })
})

test_that("downloadableTable - null data", {
    testServer(downloadableTable,
               args = list(logger           = periscope2:::fw_get_user_log(),
                           filenameroot     = "mydownload1",
                           tabledata        = function() { NULL }),
               expr = {
                   session$setInputs(dtableSingleSelect = "FALSE")
                   expect_true(grepl('"x":null', output$dtableOutputID, fixed = TRUE))
               })
})

test_that("downloadableTable - invalid download option", {
    testServer(downloadableTable,
               args = list(logger           = periscope2:::fw_get_user_log(),
                           filenameroot     = "mydownload1",
                           downloaddatafxns = list(sv = data),
                           tabledata        = data,
                           selection        = mydataRowIds),
               expr = {
                   session$setInputs(dtableSingleSelect = "FALSE")
                   selected <- paste0('"selection":{"mode":"multiple","selected":["', paste(mydataRowIds(), collapse = '","'), '"')
                   expect_true(grepl(selected, output$dtableOutputID, fixed = TRUE))
               })
})

test_that("downloadableTable - singleSelect_TRUE_selection_enabled", {
    testServer(downloadableTable,
               args = list(logger           = periscope2:::fw_get_user_log(),
                           filenameroot     = "mydownload1",
                           downloaddatafxns = list(csv = data, tsv = data),
                           tabledata        = data,
                           selection        = mydataRowIds),
               expr = {
                   session$setInputs(dtableSingleSelect = "TRUE")
                   selected <- paste0('"selection":{"mode":"single","selected":"', mydataRowIds()[1], '"')
                   expect_true(grepl(selected, output$dtableOutputID, fixed = TRUE))
               })
})

test_that("downloadableTable - null rownames", {

    testServer(downloadableTable,
               args = list(logger           = periscope2:::fw_get_user_log(),
                           filenameroot     = "mydownload1",
                           downloaddatafxns = list(csv = data_without_rownames, tsv = data_without_rownames),
                           tabledata        = data_without_rownames,
                           selection        = NULL),
               expr = {
                   session$setInputs(dtableSingleSelect = "FALSE")
                   selected <- '"selection":{"mode":"multiple","selected":null,"target":"row","selectable":null}'
                   expect_true(grepl(selected, output$dtableOutputID, fixed = TRUE))
               })
})

test_that("downloadableTable - singleSelect and selection disabled", {
  expect_silent(testServer(downloadableTable,
                             args = list(logger           = periscope2:::fw_get_user_log(),
                                         filenameroot     = "mydownload1",
                                         downloaddatafxns = list(csv = data, tsv = data),
                                         tabledata        = data),
                             expr = {}))
})

test_that("downloadableTable - invalid_selection", {
    expect_message(testServer(downloadableTable,
                              args = list(logger           = periscope2:::fw_get_user_log(),
                                          filenameroot     = "mydownload1",
                                          downloaddatafxns = list(csv = data, tsv = data),
                                          tabledata        = data,
                                          selection        = "single"),
                              expr = {}), "")
})


test_that("downloadableTable - filenameroot", {
     expect_message(
        testServer(downloadableTable,
                   args = list(filenameroot     = "test",
                               downloaddatafxns = list(csv = data),
                               tabledata        = data),
                   expr = {
                       selected <- '"selection":{"mode":"multiple","selected":null,"target":"row","selectable":null}'
                       expect_true(grepl(selected, output$dtableOutputID, fixed = TRUE))
                  }),
        regexp = "Could not apply DT options due to.*selection.*argument is incorrect",
        class  = "message"
    )

    expect_message(
        testServer(downloadableTable,
                   args = list(filenameroot     = reactiveVal("test"),
                               downloaddatafxns = list(csv = data),
                               tabledata        = data),
                   expr = {
                       selected <- '"selection":{"mode":"multiple","selected":null,"target":"row","selectable":null}'
                       expect_true(grepl(selected, output$dtableOutputID, fixed = TRUE))
                  }),
        regexp = "Could not apply DT options due to.*selection.*argument is incorrect",
        class  = "message"
    )

    expect_message(
        testServer(downloadableTable,
                   args = list(filenameroot     = function(){"test"},
                               downloaddatafxns = list(csv = data),
                               tabledata        = data),
                   expr = {
                       selected <- '"selection":{"mode":"multiple","selected":null,"target":"row","selectable":null}'
                       expect_true(grepl(selected, output$dtableOutputID, fixed = TRUE))
                  }),
        regexp = "Could not apply DT options due to.*selection.*argument is incorrect",
        class  = "message"
    )

    expect_message(
        testServer(downloadableTable,
                   args = list(filenameroot     = NULL,
                               downloaddatafxns = list(csv = data),
                               tabledata        = data),
                   expr = {
                       selected <- '"selection":{"mode":"multiple","selected":null,"target":"row","selectable":null}'
                       expect_true(grepl(selected, output$dtableOutputID, fixed = TRUE))
                  }),
        regexp = "Could not apply DT options due to.*selection.*argument is incorrect",
        class  = "message"
    )
})

test_that("downloadableTable - no downloads", {
    expect_message(testServer(downloadableTable,
                              args = list(tabledata = data,
                                          selection = "single"),
                              expr = {}), "")
})

test_that("build_datatable_arguments", {
    table_options <- list(rownames = FALSE,
                          callback   = "table.order([2, 'asc']).draw();",
                          caption    = " Very Important Information",
                          colnames   = c("Area", "Delta", "Increase"),
                          filter     = "bottom",
                          width      = "150px",
                          height     = "50px",
                          extensions = "Buttons",
                          plugins    = "natural",
                          editable   = TRUE,
                          order      = list(list(2, "asc"), list(3, "desc")))
    expect_snapshot(build_datatable_arguments(table_options))
})

test_that("format_columns", {
    set.seed(123)
    dt           <- cbind(matrix(rnorm(60, 1e5, 1e6), 20), runif(20), rnorm(20, 100))
    dt[, 1:3]    <- round(dt[, 1:3])
    dt[, 4:5]    <- round(dt[, 4:5], 7)
    colnames(dt) <- head(LETTERS, NCOL(dt))
    expect_snapshot(format_columns(DT::datatable(dt),
                                   list(formatCurrency   = list(columns = c("A", "C")),
                                        formatPercentage = list(columns = c("D"), 2))))
})

# ----------------------------------------
# --          PROGRAM ui_body.R         --
# ----------------------------------------
# USE: Create UI elements for the
#      application body (right side on the
#      desktop; contains output) and
#      ATTACH them to the UI by calling
#      add_ui_body()
#
# NOTES:
#   - All variables/functions here are
#     not available to the UI or Server
#     scopes - this is isolated
# ----------------------------------------

# -- IMPORTS --



# ----------------------------------------
# --      BODY ELEMENT CREATION         --
# ----------------------------------------

# -- Create Elements


# -- Register Elements in the ORDER SHOWN in the UI
add_ui_body()

#' Periscope2 Shiny Application Framework
#'
#' Periscope2 is the next-generation package following the paradigm of the 'periscope' package to
#' support a UI-standardized and rail-guarded enterprise quality application environment.  This package
#' also includes a variety of convenience functions for 'shiny' applications in a more modernized way. Base
#' reusable functionality as well as UI paradigms are included to ensure a consistent user experience
#' regardless of application or developer.
#'
#' 'periscope2' differs from the 'periscope' package as follows:
#' \itemize{
#'  \item Upgraded dependency on bootstrap v4 instead of bootstrap v3
#'  \item New user modules (i.e. announcements)
#'  \item More functionality and finer control over existing modules such as
#'  \link[periscope2:createPSAlert]{alert} and \link[periscope2:appResetButton]{reset}
#'  \item More control over customizing different application parts (header, footer, left sidebar, right sidebar and body)
#'  \item Enhanced file structure to organize application UI, shiny modules, app configuration, .. etc
#' }
#'
#'
#' A gallery of 'periscope' and 'periscope2' example apps is hosted at \href{http://periscopeapps.org:3838}{http://periscopeapps.org}
#'
#' @section Function Overview:
#'
#' \emph{Create a new framework application instance:\cr}
#' \link[periscope2]{create_application}\cr
#'
#' \emph{Set application parameters in program/global.R:\cr}
#' \link[periscope2]{set_app_parameters}\cr
#'
#' \emph{Get any url parameters passed to the application:\cr}
#' \link[periscope2]{get_url_parameters}\cr
#'
#' \emph{Update an existing application with a needed sidebar:\cr}
#' \link[periscope2]{create_left_sidebar}\cr
#' \link[periscope2]{create_right_sidebar}\cr
#'
#' \emph{Register user-created UI objects to the requisite application locations:\cr}
#' \link[periscope2]{add_ui_body}\cr
#' \link[periscope2]{add_ui_footer}\cr
#' \link[periscope2]{add_ui_header}\cr
#' \link[periscope2]{add_ui_left_sidebar}\cr
#' \link[periscope2]{add_ui_right_sidebar}
#'
#' \emph{Included shiny modules with a customized UI:\cr}
#' \link[periscope2]{downloadFileButton}\cr
#' \link[periscope2]{downloadableTableUI}\cr
#' \link[periscope2]{downloadablePlotUI}\cr
#' \link[periscope2]{appResetButton}\cr
#' \link[periscope2]{logViewerOutput}\cr
#'
#' \emph{High-functionality standardized tooltips:\cr}
#' \link[periscope2]{ui_tooltip}
#'
#' @section More Information:
#' \code{browseVignettes(package = 'periscope2')}
#'
#' @importFrom lifecycle deprecate_warn deprecated
#' @name periscope2
"_PACKAGE"


.onAttach <- function(libname, pkgname) {
    current_location <- getwd()
    server_filename  <- "server.R"
    if (interactive() && file.exists(file.path(current_location, c(server_filename)))) {
        server_file    <- file(paste(current_location, server_filename, sep = .Platform$file.sep))
        server_content <- readLines(con = server_file)
        close(server_file)
        if (any(grepl("library\\(logging\\)", server_content))) {
            packageStartupMessage(paste("The logging package is not supported anymore. Please remove the line 'library(logging)' in", server_filename))
        }
    }


}


.onLoad <- function(libname, pkgname) {
    tryCatch({
        if ("package:periscope" %in% search()) {
            detach(name = package:periscope, unload = TRUE)
        }
    },
    warning = function(w) {
        logwarn(paste("Unable to cleanly unload the periscope package, due to:", w$message))
    },
    error = function(e) {
        logwarn(paste("Unable to cleanly unload the periscope package, due to:", e$message))
    })
}

#' Build Announcement Module Configuration YAML File
#'
#' Call this as an addin to build valid yaml file that is needed for running
#' announcements module. The generated file can be used in periscope2 app using \link[periscope2]{load_announcements}.
#'
#' The method can be called directly via `R` console or via RStudio addins menu
#'
#' @return launch gadget window
#'
#' @examples
#' if (interactive()) {
#'    periscope2:::announcementConfigurationsAddin()
#' }
#'
#' @seealso \link[periscope2:load_announcements]{periscope2:load_announcements()}
#'
#' @export
announcementConfigurationsAddin <- function() {
    if (!requireNamespace("miniUI", quietly = TRUE)) {
        stop("You must have package 'miniUI' installed to use this add-in",
             call. = FALSE)
    }

    shiny::runGadget(app    = announcement_addin_UI(),
                     server = function(input, output, session){
                         announcement_addin_server()
                     },
                     viewer = shiny::browserViewer())
}


announcement_addin_UI <- function() {
        shiny::addResourcePath(prefix        = "img",
                               directoryPath = system.file("fw_templ/www", package = "periscope2"))
    miniUI::miniPage(
        shiny::includeCSS(system.file("fw_templ/www/widgets/widgets.css", package = "periscope2")),
        shinyFeedback::useShinyFeedback(),
        shinyjs::useShinyjs(),
        miniUI::gadgetTitleBar("Announcement Configuration YAML File Builder"),
        miniUI::miniContentPanel(
            stableColumnLayout(
                shinyWidgets::airDatepickerInput(
                    width   = "100%",
                    inputId = "startPicker",
                    label   = periscope2::ui_tooltip(id        = "startPickerTip",
                                                     label     = "Start Date",
                                                     text      = paste("First date the announcement will be shown in the application.",
                                                                       "Missing or blank value indicates that the announcement will show immediately.",
                                                                       "Both missing or blank start and end values indicates that the announcement will be always be on.",
                                                                       sep = "<br/><br/>"),
                                                     placement = "bottom"),
                    minDate = Sys.Date()),
                shinyWidgets::airDatepickerInput(
                    width   = "100%",
                    inputId = "endPicker",
                    label   = periscope2::ui_tooltip(id        = "endPickerTip",
                                                     label     = "End Date",
                                                     text      = paste("Last date the announcement will be shown in the application.",
                                                                       "Missing or blank value indicates that the announcement will be shown indefinitely",
                                                                       "Both missing or blank start and end values indicates that the announcement will be always be on.",
                                                                       sep = "<br/><br/>"),
                                                     placement = "bottom"),
                    minDate = Sys.Date())),
            stableColumnLayout(
                shiny::numericInput(
                    inputId = "auto_close",
                    width   = "100%",
                    label   = periscope2::ui_tooltip(id        = "autoCloseTip",
                                                     label     = "Close after (sec)",
                                                     text      = paste("Time, in seconds, to auto close announcement banner after that time elapsed",
                                                                       "Leave value blank or zero to leave announcement bar open until user closes it manually.",
                                                                       sep = "<br/><br/>"),
                                                     placement = "bottom"),
                    value   = 30,
                    min     = 0,
                    max     = 100),
                shiny::selectInput(
                    inputId   = "style",
                    width     = "100%",
                    selectize = FALSE,
                    choices   = c("primary", "success", "warning", "danger", "info"),
                    label     = periscope2::ui_tooltip(id        = "styleTip",
                                                       label     = shiny::div("Style", shiny::div(id = "star", "*")),
                                                       text      = paste("Color for the announcement banner, possible values are {'primary', 'success', 'warning', 'danger' or 'info'}.",
                                                                         "It is a mandatory value."),
                                                       placement = "bottom"))),
            stableColumnLayout(
                shiny::textInput(
                    inputId     = "title",
                    label       = periscope2::ui_tooltip(id        = "titleTip",
                                                         label     = "Title",
                                                         text      = "Optional banner title. Leave it empty to disable it",
                                                         placement = "bottom"),
                    width       = "100%",
                    placeholder = "Announcement Banner Title"
                ),
                shiny::textAreaInput(
                    inputId     = "announcement_text",
                    label       = periscope2::ui_tooltip(id        = "textTip",
                                                         label     = shiny::div("Announcement Text", shiny::div(id = "star", "*")),
                                                         text      = "The announcement text. Text can contain html tags and is a mandatory value",
                                                         placement = "bottom"),
                    width       = "100%",
                    height      = "100%",
                    placeholder = "Announcement Text")
            ),
            stableColumnLayout(
                miniUI::miniButtonBlock(shiny::downloadButton(outputId = "downloadConfig",
                                                              disabled = TRUE,
                                                              label    = periscope2::ui_tooltip(id    = "downloadTip",
                                                                                                label = "Download",
                                                                                                text  = "Download announcement configuration file")))
            )
        )
    )
}


announcement_addin_server <- function(id = NULL) {
    shiny::moduleServer(
        id,
        function(input, output, session) {
            shiny::observeEvent(c(input$startPicker,
                                input$endPicker), {
                if (!is.null(input$startPicker) &&
                    !is.null(input$endPicker) &&
                    (input$startPicker > input$endPicker)) {
                    shinyWidgets::updateAirDateInput(session = session,
                                                     inputId = "endPicker",
                                                     value   = input$startPicker,
                                                     options = list(minDate = input$startPicker))
                } else if (!is.null(input$startPicker)) {
                    shinyWidgets::updateAirDateInput(session = session,
                                                     inputId = "endPicker",
                                                     options = list(minDate = input$startPicker))
                }
            })

            shiny::observeEvent(c(input$auto_close,
                                  input$announcement_text), {
                                      auto_close <- as.integer(input$auto_close)
                                      text       <- input$announcement_text
                                      valid      <- TRUE

                                      shinyFeedback::hideFeedback("auto_close")
                                      shinyFeedback::hideFeedback("announcement_text")
                                      shinyFeedback::hideFeedback("startPicker")
                                      shinyFeedback::hideFeedback("endPicker")

                                      if (!is.na(auto_close) && (auto_close < 0)) {
                                          shinyFeedback::showFeedbackDanger(inputId = "auto_close", text = "'auto_close' must be 0, positive or blank")
                                          valid <- FALSE
                                      }

                                      if (is.na(text) || (nchar(text) == 0)) {
                                          shinyFeedback::showFeedbackDanger(inputId = "announcement_text", text = "announcement text is a mandatory value")
                                          valid <- FALSE
                                      }


                                      if (valid) {
                                          shinyjs::enable("downloadConfig")
                                      } else {
                                          shinyjs::disable("downloadConfig")
                                      }

                                  }, ignoreInit = TRUE)

            output$downloadConfig <- shiny::downloadHandler(
                filename = function() {
                    "announce.yaml"
                },
                content = function(announcements_file) {
                    start_date <- ""
                    end_date   <- ""
                    title      <- ""
                    auto_close <- ""

                    if (!is.null(input$startPicker)) {
                        start_date <- as.character(input$startPicker)
                    }

                    if (!is.null(input$endPicker)) {
                        end_date <- as.character(input$endPicker)
                    }

                    lines <- c("### start_date",
                               "# First date the announcement will be shown in the application",
                               "# Missing or blank value indicates that the announcement will show immediately.",
                               "# Both missing or blank start and end values indicates that the announcement will be always be on.",
                               paste0("start_date: ", start_date, "\n"),

                               "### start_date_format",
                               "# Format symbols are:",
                               "#                    %Y: for year (1999),",
                               "#                    %y: for year format (99),",
                               "#                    %m: for month (1-12),",
                               "#                    %d: for day (1-31)",
                               "# All formats must be inside double quotation.",
                               "# Leave this field blank to use default format which is \"%Y-%m-%d\"",
                               "start_date_format:\n",

                               "### end_date",
                               "# Last date the announcement will be shown in the application.",
                               "# Missing or blank value indicates that the announcement will be shown indefinitely",
                               "# Both missing or blank start and end values indicates that the announcement will be always be on.",
                               paste0("end_date: ", end_date, "\n"),

                               "### end_date_format",
                               "# Format symbols are:",
                               "#                    %Y: for year (1999),",
                               "#                    %y: for year format (99),",
                               "#                    %m: for month (1-12),",
                               "#                    %d: for day (1-31)",
                               "# All formats must be inside double quotation.",
                               "# Leave this field blank to use default format which is \"%Y-%m-%d\"",
                               "end_date_format:\n",

                               "### auto_close",
                               "# Time, in seconds, to auto close announcement banner after that time elapsed",
                               "# Leave value blank or zero to leave announcement bar open until user closes it manually.",
                               paste0("auto_close: ", input$auto_close, "\n"),

                               "### style",
                               "# Color for the announcement banner, possible values are { \"primary\", \"success\", \"warning\", \"danger\" or \"info\"}.",
                               "# It is a mandatory value",
                               paste0("style: \"", input$style, "\"\n"),

                               "### title",
                               "# Optional banner title. Leave it empty to disable it.",
                               paste0("title: \"", input$title, "\"\n"),

                               "### text",
                               "# The announcement text. Text can contain html tags and is a mandatory value",
                               paste0("text: \"", input$announcement_text, "\"\n"))

                    writeLines(lines, announcements_file)
                }
            )

            shiny::observeEvent(input$done, {
                shiny::removeResourcePath(prefix = "img")
                invisible(shiny::stopApp())
            })

            shiny::observeEvent(input$cancel, {
                shiny::removeResourcePath(prefix = "img")
                invisible(shiny::stopApp())
            })

        })
}

stableColumnLayout <- function(...) {
    dots  <- list(...)
    n     <- length(dots)
    width <- 12 / n
    class <- sprintf("col-xs-%s col-md-%s", width, width)
    shiny::fluidRow(
        lapply(dots, function(el) {
            shiny::tags$div(class = class, el)
        })
    )
}

context("periscope2 - package addins")
local_edition(3)

test_that("theme builder addin - UI", {
    ui_output <- periscope2:::themeBuilder_addin_UI()
    expect_true(grepl("Status Colors", ui_output))
    expect_true(grepl("Sidebar Colors", ui_output))
    expect_true(grepl("Sidebar Layout", ui_output))
    expect_true(grepl("Main Colors", ui_output))
    expect_true(grepl("Color Contrast", ui_output))
    expect_true(grepl("Other Variables", ui_output))
    expect_true(grepl("Add Variable", ui_output))
    expect_true(grepl("Cancel", ui_output))
    expect_true(grepl("Done", ui_output))
    expect_true(grepl("Download", ui_output))
})


test_that("theme builder addin - server", {
    testServer(themeBuilder_addin_server,
               expr = {
                   session$setInputs(primary               = "#B221DD",
                                     secondary             = "#6C757D",
                                     success               = "#2ED610",
                                     info                  = "#7BDFF2",
                                     warning               = "#FFF200",
                                     danger                = "#CE0900",
                                     light                 = "#F8F9FA",
                                     dark                  = "#343A40",
                                     bg                    = "#D67E7CD1",
                                     hover_bg              = "#B0B818F7",
                                     color                 = "#CA39D4ED",
                                     hover_color           = "#000000E4",
                                     active_color          = "#BF8DF0EE",
                                     submenu_bg            = "#084D66BF",
                                     submenu_color         = "#B33634D3",
                                     submenu_hover_color   = "#0303038F",
                                     submenu_hover_bg      = "#BE25CFF2",
                                     submenu_active_color  = "#1FAB9DC2",
                                     submenu_active_bg     = "#401487E8",
                                     header_color          = "#D91F1CAF",
                                     sidebar_width         = "350",
                                     control_sidebar_width = "250",
                                     sidebar_padding_x     = "15",
                                     sidebar_padding_y     = "20",
                                     sidebar_mini_width    = "25",
                                     blue                  = "#007BFF",
                                     lightblue             = "#3C8DBC",
                                     navy                  = "#001F3F",
                                     cyan                  = "#17A2B8",
                                     teal                  = "#39CCCC",
                                     olive                 = "#3D9970",
                                     green                 = "#28A745",
                                     lime                  = "#01FF70",
                                     orange                = "#FF851B",
                                     yellow                = "#FFC107",
                                     fuchsia               = "#F012BE",
                                     purple                = "#605CA8",
                                     maroon                = "#D81B60",
                                     red                   = "#DC3545",
                                     black                 = "#111111",
                                     gray_x_light          = "#D2D6DE",
                                     gray_600              = "#6C757D",
                                     white                 = "#FFFFFF",
                                     contrasted_threshold  = "12",
                                     text_dark             = "#701110D0",
                                     text_light            = "#2F515EBA")
                   ids(2)
                   session$setInputs(addVariable = 1)
                   session$setInputs("1-variableName"  = "main-bg",
                                     "1-variableValue" = "#FFC107",
                                     "2-variableName"  = "navbar-light-color",
                                     "2-variableValue" = "#bec5cb",
                                     `1-removeVariableBtn` = 1,
                                     downloadConfig = 1)
                   expect_snapshot(yaml::read_yaml(output$downloadConfig))
                   session$setInputs(done = 1)
               }
    )

    testServer(themeBuilder_addin_server,
               expr = {
                   expect_warning(session$setInputs(cancel = 1))
               }
    )
})


test_that("announcement addin - UI", {
    ui_output <- announcement_addin_UI()
    expect_true(grepl(Sys.Date(), ui_output, fixed = TRUE))
    expect_true(grepl("startPicker", ui_output, fixed = TRUE))
    expect_true(grepl("endPicker", ui_output, fixed = TRUE))
})


test_that("announcement addin - server", {
    testServer(announcement_addin_server,
               expr = {
                   session$setInputs(startPicker       = "2023-10-27",
                                     endPicker         = "2023-10-28",
                                     announcement_text = "test body",
                                     auto_close        = 30,
                                     style             = "info",
                                     title             = "test title")
                   session$setInputs(downloadConfig = 0)
                   announce <- yaml::read_yaml(output$downloadConfig)
                   expect_equal(announce[["start_date"]], "2023-10-27")
                   expect_equal(announce[["end_date"]], "2023-10-28")
                   expect_null(announce[["start_date_format"]])
                   expect_null(announce[["end_date_format"]])
                   expect_equal(announce[["auto_close"]], 30)
                   expect_equal(announce[["style"]], "info")
                   expect_equal(announce[["title"]], "test title")
                   expect_equal(announce[["text"]], "test body")

                   session$setInputs(announcement_text = "test body2",
                                     auto_close        = 10)
                   session$setInputs(downloadConfig = 1)

                   announce <- yaml::read_yaml(output$downloadConfig)
                   expect_equal(announce[["start_date"]], "2023-10-27")
                   expect_equal(announce[["end_date"]], "2023-10-28")
                   expect_null(announce[["start_date_format"]])
                   expect_null(announce[["end_date_format"]])
                   expect_equal(announce[["auto_close"]], 10)
                   expect_equal(announce[["style"]], "info")
                   expect_equal(announce[["title"]], "test title")
                   expect_equal(announce[["text"]], "test body2")

                   session$setInputs(announcement_text = NA,
                                     auto_close        = -10)
                   session$setInputs(downloadConfig = 2)
                   announce <- yaml::read_yaml(output$downloadConfig)
                   expect_equal(announce[["text"]], "NA")
                   expect_equal(announce[["auto_close"]], -10)

                   session$setInputs(startPicker       = "2023-10-29",
                                     endPicker         = "2023-10-28")
                   announce <- yaml::read_yaml(output$downloadConfig)
                   expect_equal(announce[["start_date"]], "2023-10-29")
                   # the value is updated in live app to "2023-10-29",
                   # but it has this value for limited unit test capabilities
                   expect_equal(announce[["end_date"]], "2023-10-28")

                   session$setInputs(done = 1)
               })

    testServer(announcement_addin_server,
               expr = {
                   expect_warning(session$setInputs(cancel = 1))
               })
})

# -------------------------------------
# --      logViewer Module     --
# -- Display application log   --
# -------------------------------------

#' Display app logs
#'
#' Display app log data in downloadableReactTable table containing logged user actions. Table contents are auto updated
#' whenever a user action is logged. User can search for logs, sort them by time and download them in CSV or TSV format.
#' The id must match the same id configured in \bold{server.R} file upon calling \code{fw_server_setup} method
#'
#'
#' @param id character id for the object(default = "logViewer")
#'
#' @return downloadableReactTableUI instance
#'
#' @section Table columns:
#' \itemize{
#'     \item action - the action that id logged in any place in app
#'     \item time   - action time
#' }
#'
#' @section Example:
#' \code{logViewerOutput('logViewer')}
#'
#' @section Shiny Usage:
#' Add the log viewer box to your box list
#'
#' It is paired with a call to \code{fw_server_setup} method in \bold{server.R} file
#'
#'
#' @examples
#' # Inside ui_body add the log viewer box to your box list
#'
#' logViewerOutput('logViewerId')
#'
#'
#' @export
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{downloadablePlot}
#' @seealso \link[periscope2]{downloadFileButton}
#' @seealso \link[periscope2]{downloadableTableUI}
#' @seealso \link[periscope2]{downloadableTable}
logViewerOutput <- function(id = "logViewer") {
    ns <- shiny::NS(id)
    downloadableReactTableUI(id            = ns(id),
                             downloadtypes = c("csv", "tsv"),
                             hovertext     = "Download application logs")
}


#' logViewer module server function
#'
#' Server-side function for the logViewerOutput  This is box with table displaying application logs.
#' The server function is used to provide module configurations.
#' The function is used internally only by \code{fw_server_setup} method
#'
#' @param id     - the ID of the Module's UI element
#' @param logger - action logs to be displayed
#'
#' @return downloadableReactTable instance with logged lines
#'
#'
#' @section Shiny Usage:
#' This function is private and called by \code{fw_server_setup} method only
#'
#' @noRd
logViewer <- function(id = "logViewer", logger) {
    shiny::moduleServer(
        id,
        function(input, output, session) {
            get_log_data <- function() {
                log_data <- data.frame()
                lines    <- logger()

                if (length(lines) > 0) {
                    out1 <- data.frame(orig = lines, stringsAsFactors = F)
                    loc1 <- regexpr("\\[", out1$orig)
                    loc2 <- regexpr("\\]", out1$orig)

                    out1$logname   <- substr(out1$orig, 1, loc1 - 1)

                    out1$timestamp <- substr(out1$orig, loc1 + 1, loc2 - 1)
                    out1$timestamp <- lubridate::parse_date_time(out1$timestamp, "YmdHMS")

                    out1$action <- substring(out1$orig, loc2 + 1)
                    out1$action <- trimws(out1$action, "both")

                    log_data <- data.frame(action = out1$action,
                                           time   = format(out1$timestamp,
                                                           format = .g_opts$datetime.fmt))
                }
                log_data
            }

            downloadableReactTable(id                 = id,
                                   table_data         = get_log_data,
                                   download_data_fxns = list(csv = get_log_data, tsv = get_log_data),
                                   file_name_root     = "log_data")
        })
}

context("periscope2 - Log Viewer")
local_edition(3)
# Helper functions
sample_log <- function(){
    c("actions [2022-02-19 14:03:26] Be Sure to Remember to Log ALL user actions",
      "actions [2022-02-19 14:03:27] Sample Title (click for an info pop-up) started with log level <DEBUG>",
      "actions [2022-02-19 14:04:32] Application Reset requested by user.  Resetting in  5 seconds")
}

null_log <- function() {
    NULL
}

# UI unit tests
test_that("logViewerOutput", {
    expect_snapshot_output(logViewerOutput("myid"))
})


# Server unit tests
test_that("logViewer - valid sample log", {
    testServer(logViewer,
               args = list(id = "myid", logger = sample_log),
               expr = {
                   expect_true(grepl("Be Sure to Remember to Log ALL user actions", output$"myid-reactTableOutputID", fixed = TRUE))
                   expect_true(grepl("Resetting in  5 seconds", output$"myid-reactTableOutputID", fixed = TRUE))
               })
})


test_that("logViewer - null log", {
    testServer(logViewer,
               args = list(id = "nullLogger", logger = null_log),
               expr = {
                   expect_true(grepl('"x":null', output$"nullLogger-reactTableOutputID", fixed = TRUE))
               })
})

# -------------------------------------------------
# --              FRAMEWORK server.R             --
# -------------------------------------------------
# NOTES:                                         --
# Program code goes in "program" directory files --
# -------------------------------------------------
# *****        DO NOT EDIT THIS FILE       ***** --
# -------------------------------------------------

source(paste("program", "server_global.R", sep = .Platform$file.sep),
       local = TRUE)


shinyServer(function(input, output, session) {
    ss_userAction.Log <- periscope2:::fw_get_user_log()

    source(paste("program", "server_local.R", sep = .Platform$file.sep),
           local = TRUE)

    periscope2:::fw_server_setup(input            = input,
                                 output           = output,
                                 session          = session,
                                 logger           = ss_userAction.Log,
                                 logger_viewer_id = "logViewerId")

    announce_close_time <- periscope2:::load_announcements()
    if (!is.null(announce_close_time)) {
        shinyjs::delay(announce_close_time,{
            bs4Dash::closeAlert("announceAlert")
        })
    }

    loginfo("%s started with log level <%s>",
            periscope2:::fw_get_title(), periscope2:::fw_get_loglevel(),
            logger = ss_userAction.Log)

    # Display application info
    observeEvent(input$app_info, {
        show_alert(html                = TRUE,
                   showConfirmButton   = FALSE,
                   btn_labels          = NA,
                   width               = "60%",
                   showCancelButton    = FALSE,
                   showCloseButton     = TRUE,
                   animation           = "slide-from-top",
                   closeOnClickOutside = TRUE,
                   text                = periscope2:::.g_opts$app_info,
                   title               = periscope2:::.g_opts$app_title)
    })
})

# -------------------------------------------
# -- Application Downloadable File Module --
# -------------------------------------------


#' downloadFileButton module UI function
#'
#' Creates a custom high-functionality button for file downloads with two
#' states - single download type or multiple-download types.  The button image
#' and pop-up menu (if needed) are set accordingly.  A tooltip can also be set
#' for the button.
#'
#' @param id character id for the object
#' @param downloadtypes vector of values for data download types
#' @param hovertext tooltip hover text
#'
#' @return html span with tooltip and either shiny downloadButton in case of single download or shiny actionButton otherwise
#'
#' @section Button Features:
#' \itemize{
#'     \item Consistent styling of the button, including a hover tooltip
#'     \item Single or multiple types of downloads
#'     \item Ability to download different data for each type of download
#' }
#'
#' @section Example:
#' \code{downloadFileUI("mybuttonID1", c("csv", "tsv"), "Click Here")}
#' \code{downloadFileUI("mybuttonID2", "csv", "Click to download")}
#'
#' @section Shiny Usage:
#' Call this function at the place in ui.R where the button should be placed.
#'
#' It is paired with a call to \code{downloadFile(id, ...)}
#' in server.R
#'
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{logViewerOutput}
#' @seealso \link[periscope2]{downloadablePlot}
#' @seealso \link[periscope2]{downloadableTableUI}
#' @seealso \link[periscope2]{downloadableTable}
#'
#' @examples
#' if (interactive()) {
#'    library(shiny)
#'    library(periscope2)
#'    shinyApp(ui = fluidPage(fluidRow(column(width = 6,
#'      # single download type
#'      downloadFileButton("object_id1",
#'                         downloadtypes = c("csv"),
#'                         hovertext     = "Button 1 Tooltip")),
#'       column(width = 6,
#'       # multiple download types
#'       downloadFileButton("object_id2",
#'                          downloadtypes = c("csv", "tsv"),
#'                          hovertext     = "Button 2 Tooltip")))),
#'      server = function(input, output) {
#'        # single download type
#'        downloadFile(id           = "object_id1",
#'                     logger       = "",
#'                     filenameroot = "mydownload1",
#'                     datafxns     = list(csv = reactiveVal(iris)),
#'                     row_names    = FALSE)
#'        # multiple download types
#'        downloadFile(id           = "object_id2",
#'                     logger       = "",
#'                     filenameroot = "mydownload2",
#'                     datafxns     = list(csv = reactiveVal(mtcars),
#'                     tsv = reactiveVal(mtcars)))
#'    })
#'}
#'
#' @export
downloadFileButton <- function(id,
                               downloadtypes = c("csv"),
                               hovertext     = NULL) {
    ns             <- shiny::NS(id)
    file_button_ui <- ""

    if (length(downloadtypes) > 1) {
        # create dropdown list
        dropdown <- list()
        for (item in downloadtypes) {
            dropdown <- list(dropdown,
                             shiny::tags$li(
                                 shiny::downloadLink(
                                     ns(item),
                                     label = item,
                                     class = "periscope-download-choice")))
        }
        dropdown <- shiny::tagList(dropdown)

        # button with dropdown list
        file_button_ui <- bs4Dash::tooltip(shiny::span(class = "btn-group",
                                                       bs4Dash::actionButton(
                                                           inputId          = ns("downloadFileList"),
                                                           label            = NULL,
                                                           icon             = shiny::icon("copy", lib = "font-awesome"),
                                                           type             = "action",
                                                           class            = "dropdown-toggle periscope-download-btn",
                                                           `data-toggle`    = "dropdown",
                                                           `aria-haspopup`  = "true",
                                                           `aria-expanded`  = "false"),
                                                       shiny::tags$ul(class = "dropdown-menu",
                                                                      id = ns("testList"),
                                                                      dropdown)),
                                           title     = hovertext,
                                           placement = "top")

    } else if (length(downloadtypes) == 1) {
        # single button - no dropdown
        file_button_ui <- bs4Dash::tooltip(shiny::span(shiny::downloadButton(ns(downloadtypes[1]),
                                                                             label = NULL,
                                                                             class = "periscope-download-btn")),
                                           title     = hovertext,
                                           placement = "top")
    }
    file_button_ui
}

check_openxlsx2_availability <- function() {
    length(find.package("openxlsx2", quiet = TRUE)) > 0
}

check_openxlsx_availability <- function() {
    length(find.package("openxlsx", quiet = TRUE)) > 0
}


#' downloadFile module server function
#'
#' Server-side function for the downloadFileButton.  This is a custom
#' high-functionality button for file downloads supporting single or multiple
#' download types.  The server function is used to provide the data for download.
#'
#' @param id ID of the Module's UI element
#' @param logger logger to use
#' @param filenameroot the base text used for user-downloaded file - can be
#' either a character string or a reactive expression that returns a character
#' string
#' @param datafxns a \strong{named} list of functions providing the data as
#' return values.  The names for the list should be the same names that were used
#' when the button UI was created.
#' @param aspectratio the downloaded chart image width:height ratio (ex:
#' 1 = square, 1.3 = 4:3, 0.5 = 1:2). Where not applicable for a download type
#' it is ignored (e.g. data downloads).
#' @param row_names logical value indicating whether row names are to be written
#' for tabular data. Where not applicable for a download type it is ignored.
#'
#' @return no return value, called for downloading selected file type
#'
#' @section Shiny Usage:
#' This function is not called directly by consumers - it is accessed in
#' server.R using the same id provided in \code{downloadFileButton}:
#'
#' \strong{\code{downloadFile(id, logger, filenameroot, datafxns)}}
#'
#' @seealso \link[periscope2]{downloadFileButton}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{logViewerOutput}
#' @seealso \link[periscope2]{downloadablePlot}
#' @seealso \link[periscope2]{downloadableTableUI}
#' @seealso \link[periscope2]{downloadableTable}
#'
#' @examples
#' if (interactive()) {
#'    library(shiny)
#'    library(periscope2)
#'    shinyApp(ui = fluidPage(fluidRow(column(width = 6,
#'      # single download type
#'      downloadFileButton("object_id1",
#'                         downloadtypes = c("csv"),
#'                         hovertext     = "Button 1 Tooltip")),
#'       column(width = 6,
#'       # multiple download types
#'       downloadFileButton("object_id2",
#'                          downloadtypes = c("csv", "tsv"),
#'                          hovertext     = "Button 2 Tooltip")))),
#'      server = function(input, output) {
#'        # single download type
#'        downloadFile(id           = "object_id1",
#'                     logger       = "",
#'                     filenameroot = "mydownload1",
#'                     datafxns     = list(csv = reactiveVal(iris)),
#'                     row_names    = FALSE)
#'        # multiple download types
#'        downloadFile(id           = "object_id2",
#'                     logger       = "",
#'                     filenameroot = "mydownload2",
#'                     datafxns     = list(csv = reactiveVal(mtcars),
#'                     tsv = reactiveVal(mtcars)))
#'    })
#'}
#'
#' @export
downloadFile <- function(id,
                         logger       = NULL,
                         filenameroot = "download",
                         datafxns     = NULL,
                         aspectratio  = 1,
                         row_names    = TRUE) {
    shiny::moduleServer(
        id,
        function(input, output, session) {
            rootname <- filenameroot
            if ("character" %in% class(filenameroot)) {
                rootname <- shiny::reactive({filenameroot})
            }

            # --- DATA processing

            output$csv  <- shiny::downloadHandler(
                filename = shiny::reactive({paste(rootname(), "csv", sep = ".")}),
                content  = function(file) {
                    if (!is.null(datafxns)) {
                        writeFile(type          = "csv",
                                  data          = datafxns$csv(),
                                  file          = file,
                                  show_rownames = row_names,
                                  logger        = logger,
                                  filename      = shiny::reactive({paste(rootname(), "csv", sep = ".")}))
                    }
                })

            output$xlsx <- shiny::downloadHandler(
                filename = shiny::reactive({paste(rootname(), "xlsx", sep = ".")}),
                content  = function(file) {
                    if (!is.null(datafxns)) {
                        writeFile(type          = "xlsx",
                                  data          = datafxns$xlsx(),
                                  file          = file,
                                  show_rownames = row_names,
                                  logger        = logger,
                                  filename      = shiny::reactive({paste(rootname(), "xlsx", sep = ".")}))
                    }
                })

            output$tsv  <- shiny::downloadHandler(
                filename = shiny::reactive({paste(rootname(), "tsv", sep = ".")}),
                content = function(file) {
                    if (!is.null(datafxns)) {
                        writeFile(type          = "tsv",
                                  data          = datafxns$tsv(),
                                  file          = file,
                                  show_rownames = row_names,
                                  logger        = logger,
                                  filename      = shiny::reactive({paste(rootname(), "tsv", sep = ".")}))
                    }
                })

            output$txt  <- shiny::downloadHandler(
                filename = shiny::reactive({paste(rootname(), "txt", sep = ".")}),
                content = function(file) {
                    if (!is.null(datafxns)) {
                        writeFile(type          = "txt",
                                  data          = datafxns$txt(),
                                  file          = file,
                                  show_rownames = row_names,
                                  logger        = logger,
                                  filename      = shiny::reactive({paste(rootname(), "txt", sep = ".")}))
                    }
                })


            ## writeFile
            ##
            ## @param type type of file to write
            ## @param data data to write
            ## @param file file path to write to
            ## @param show_rownames if TRUE, row names are written for tabular data
            ## @param logger logger to use
            ## @param filename name of downloaded file, expected to be a reactive expression
            ##
            ## @returns no return value
            writeFile <- function(type, data, file, show_rownames, logger, filename) {
                show_rownames <- isTRUE(show_rownames)

                # tabular values
                if ((type == "csv") || (type == "tsv")) {
                    show_colnames <- TRUE
                    if (show_rownames) {
                        show_colnames <- NA
                    }

                    utils::write.table(data, file,
                                       sep = ifelse(type == "tsv", "\t", ","),
                                       dec = ".",
                                       qmethod = "double",
                                       col.names = show_colnames,
                                       row.names = show_rownames)
                }
                # excel file
                else if (type == "xlsx") {

                    if (inherits(data, "wbWorkbook") && check_openxlsx2_availability()) {
                        openxlsx2::wb_save(data, file)
                    } else if (inherits(data, "Workbook") &&
                               ("openxlsx" %in% attributes(class(data))) &&
                               check_openxlsx_availability()) {
                        openxlsx::saveWorkbook(data, file)
                    } else {
                        # tabular data
                        if (check_openxlsx2_availability()) {
                            openxlsx2::write_xlsx(data,
                                                  file,
                                                  as_table  = TRUE,
                                                  row_names = show_rownames)
                        } else if (check_openxlsx_availability()) {
                            openxlsx::write.xlsx(data,
                                                 file,
                                                 asTable  = TRUE,
                                                 rowNames = show_rownames)
                        } else {
                            if (show_rownames) {
                                logwarn("Downloading with 'writexl', row names are not included",
                                        logger = logger)
                            }
                            writexl::write_xlsx(data, file)
                        }
                    }
                }
                # text file processing
                else if (type == "txt") {
                    if (inherits(data, "character")) {
                        writeLines(data, file)
                    } else if (is.data.frame(data) || is.matrix(data)) {
                        utils::write.table(data, file, row.names = show_rownames)
                    } else {
                        msg <- paste(type, "could not be processed")
                        logwarn(msg)
                        warning(msg)
                        writeLines(msg, file)
                    }
                }
                # error - type not handled
                else {
                    msg <- paste(type, "not implemented as a download type")
                    logwarn(msg)
                    warning(msg)
                }
                loginfo(paste("File downloaded in browser: <",
                              filename(), ">"), logger = logger)
            }

            # --- IMAGE processing

            output$png <- shiny::downloadHandler(
                filename = shiny::reactive({paste(rootname(), "png", sep = ".")}),
                content = function(file) {
                    writeImage("png", datafxns$png(), file, aspectratio, logger,
                               shiny::reactive({paste(rootname(), "png", sep = ".")}))
                })

            output$jpeg <- shiny::downloadHandler(
                filename = shiny::reactive({paste(rootname(), "jpeg", sep = ".")}),
                content = function(file) {
                    writeImage("jpeg", datafxns$jpeg(), file, aspectratio, logger,
                               shiny::reactive({paste(rootname(), "jpeg", sep = ".")}))
                })

            output$tiff <- shiny::downloadHandler(
                filename = shiny::reactive({paste(rootname(), "tiff", sep = ".")}),
                content = function(file) {
                    writeImage("tiff", datafxns$tiff(), file, aspectratio, logger,
                               shiny::reactive({paste(rootname(), "tiff", sep = ".")}))
                })

            output$bmp <- shiny::downloadHandler(
                filename = shiny::reactive({paste(rootname(), "bmp", sep = ".")}),
                content = function(file) {
                    writeImage("bmp", datafxns$bmp(), file, aspectratio, logger,
                               shiny::reactive({paste(rootname(), "bmp", sep = ".")}))
                })

            writeImage <- function(type, data, file, aspectratio, logger, filename) {
                dim <- list(width = 7, height = 7/aspectratio, units = "in")

                #ggplot processing
                if (inherits(data, c("ggplot", "ggmatrix", "grob"))) {
                    if (type %in% c("png", "jpeg", "tiff", "bmp")) {
                        ggplot2::ggsave(filename = file,
                                        plot     = data,
                                        width    = dim$width,
                                        height   = dim$height,
                                        units    = dim$units,
                                        scale    = 2)
                    } else {
                        msg <- paste("Unsupported plot type for ggplot download - ",
                                     "must be in: <png, jpeg, tiff, bmp>")
                        logwarn(msg)
                        warning(msg)
                    }
                } else if (inherits(data, "trellis")) {
                    #lattice processing
                    if (type %in% c("png", "jpeg", "tiff", "bmp")) {
                        do.call(type, list(filename = file,
                                           width    = dim$width,
                                           height   = dim$height,
                                           units    = dim$units,
                                           res      = 600))
                        print(data)
                        grDevices::dev.off()
                    }
                    else {
                        msg <- paste("Unsupported plot type for lattice download - ",
                                     "must be in: <png, jpeg, tiff, bmp>")
                        logwarn(msg)
                        warning(msg)
                    }
                } else {
                    # error - type not handled
                    # ------- should really never be hit
                    msg <- paste(type, "not implemented as a download type")
                    logwarn(msg)
                    warning(msg)
                }
                loginfo(paste("File downloaded in browser: <",
                              filename(), ">"), logger = logger)
            }
        })
}


#' Check passed file types against downloadFile module allowed file types list
#'
#' It is a downloadFile module helper to return periscope2 defined file types list and warns user if an invalid type is included
#'
#' @param types list of types to test
#'
#' @return the list input given in types
#'
#' @examples
#'   #inside console
#'   ## Check valid types
#'   result <- periscope2::downloadFile_AvailableTypes()
#'   identical(result, c("csv", "xlsx", "tsv", "txt", "png", "jpeg", "tiff", "bmp"))
#'
#'   ## check invalid type
#'   testthat::expect_warning(downloadFile_ValidateTypes(types = "csv_invalid"),
#'                            "file download list contains an invalid type <csv_invalid>")
#'
#'
#' @seealso \link[periscope2]{downloadFileButton}
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{logViewerOutput}
#' @seealso \link[periscope2]{downloadablePlot}
#' @seealso \link[periscope2]{downloadableTableUI}
#' @seealso \link[periscope2]{downloadableTable}
#'
#' @export
downloadFile_ValidateTypes <- function(types) {
    for (type in types) {
        if (!(type %in% shiny::isolate(.g_opts$data_download_types)) &&
            !(type %in% shiny::isolate(.g_opts$plot_download_types)) ) {
            warning(paste0("file download list contains an invalid type <",
                           type, ">"))
        }
    }
    types
}


#' downloadFile module list of allowed file types
#'
#' Returns a list of all supported types
#'
#' @return a vector of all supported types
#'
#' @seealso \link[periscope2]{downloadFileButton}
#' @seealso \link[periscope2]{downloadFile}
#'
#' @export
downloadFile_AvailableTypes <- function() {
    c(shiny::isolate(.g_opts$data_download_types),
      shiny::isolate(.g_opts$plot_download_types))
}

context("periscope2 - download file")
local_edition(3)

# helper functions
download_plot <- function() {
    plot <- ggplot2::ggplot(data = mtcars, aes(x = wt, y = mpg)) +
        geom_point(aes(color = cyl)) +
        ggtitle("GGPlot Example w/Hover") +
        xlab("wt") +
        ylab("mpg")
    if (packageVersion("ggplot2") >= "3.5.0") {
        plot <- plot +
            theme(legend.justification   = c(1, 1),
                  legend.position.inside = c(1, 1),
                  legend.title           = element_blank())
    } else {
        plot <- plot +
            theme(legend.justification = c(1, 1),
                  legend.position      = c(1, 1),
                  legend.title         = element_blank())
    }
}

download_lattice_plot <- function() {
    lattice::xyplot(Sepal.Length ~ Petal.Length, data = head(iris))
}

download_data <- function() {
    head(mtcars)
}

download_string_list <- function() {
    c("test1", "test2", "tests")
}

download_char_data <- function() {
    "A123B"
}

create_openxlsx2_wb <- function() {
    openxlsx2::wb_workbook()$add_worksheet("openxlsx2_workbook")$add_data(x = download_data())
}

create_openxlsx_wb <- function() {
    wb   <- openxlsx::createWorkbook()
    openxlsx::addWorksheet(wb, "openxlsx_workbook")
    data <- as.data.frame(download_data())
    openxlsx::writeData(wb, "openxlsx_workbook", data)
    wb
}

# UI Testing
test_that("downloadFileButton", {
    file_btn <- downloadFileButton(id            = "myid",
                                   downloadtypes = c("csv"),
                                   hovertext     = "myhovertext")
    expect_true(grepl('title="myhovertext"', file_btn, fixed = TRUE))
    expect_true(grepl('id="myid-csv"', file_btn, fixed = TRUE))
})

test_that("downloadFileButton - no download type", {
    file_btn <- downloadFileButton(id            = "myid2",
                                   downloadtypes = NULL,
                                   hovertext     = "myhovertext")
    expect_equal(file_btn, "")
})

test_that("downloadFileButton multiple types", {
    file_btn <- downloadFileButton(id            = "myid",
                                   downloadtypes = c("csv", "tsv"),
                                   hovertext     = "myhovertext")
    expect_true(grepl('class="btn-group"', file_btn, fixed = TRUE))
    expect_true(grepl('myid-downloadFileList"', file_btn, fixed = TRUE))
    expect_true(grepl('id="myid-csv"', file_btn, fixed = TRUE))
    expect_true(grepl('id="myid-tsv"', file_btn, fixed = TRUE))
})

test_that("downloadFileButton invalid type", {
    file_btn <- downloadFileButton(id            = "myid",
                                   downloadtypes = c("sv"),
                                   hovertext     = "myhovertext")
    expect_true(grepl('title="myhovertext"', file_btn, fixed = TRUE))
    expect_true(grepl('id="myid-sv"', file_btn, fixed = TRUE))
})

# Server Testing
test_that("downloadFile_ValidateTypes valid", {
    result <- downloadFile_ValidateTypes(types = "csv")

    expect_equal(result, "csv")
})

test_that("downloadFile_ValidateTypes invalid", {
    expect_warning(downloadFile_ValidateTypes(types = "csv_invalid"),
                   "file download list contains an invalid type <csv_invalid>")
})

test_that("downloadFile_AvailableTypes", {
    result <- downloadFile_AvailableTypes()

    expect_equal(result, c("csv", "xlsx", "tsv", "txt", "png", "jpeg", "tiff", "bmp"))
})

test_that("downloadFile - all download types", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    testServer(downloadFile,
               args = list(logger       = periscope2:::fw_get_user_log(),
                           filenameroot = "mydownload1",
                           row_names    = FALSE,
                           datafxns     = list(csv   = download_data,
                                               xlsx  = download_data,
                                               tsv   = download_data,
                                               txt   = download_data,
                                               png   = download_plot,
                                               jpeg  = download_plot,
                                               tiff  = download_plot,
                                               bmp   = download_plot)),
               expr = {
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.csv >",
                                     x       = capture_output(expect_snapshot_file(output$csv))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.tsv >",
                                     x       = capture_output(expect_snapshot_file(output$tsv))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.txt >",
                                     x       = capture_output(expect_snapshot_file(output$txt))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.xlsx >",
                                     x       = capture_output(file.exists(output$xlsx))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.png >",
                                     x       = capture_output(file.exists(output$png))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.jpeg >",
                                     x       = capture_output(file.exists(output$jpeg))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.tiff >",
                                     x       = capture_output(file.exists(output$tiff))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.bmp >",
                                     x       = capture_output(file.exists(output$bmp))))
               })

})

test_that("downloadFile - lattice plot", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    testServer(downloadFile,
               args = list(logger       = periscope2:::fw_get_user_log(),
                           filenameroot = "mydownload1",
                           datafxns     = list(png   = download_lattice_plot,
                                               jpeg  = download_lattice_plot,
                                               tiff  = download_plot,
                                               bmp   = download_lattice_plot)),
               expr = {
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.png >",
                                     x       = capture_output(expect_true(file.exists(output$png)))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.jpeg >",
                                     x       = capture_output(expect_true(file.exists(output$jpeg)))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.tiff >",
                                     x       = capture_output(expect_true(file.exists(output$tiff)))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < mydownload1.bmp >",
                                     x       = capture_output(expect_true(file.exists(output$bmp)))))
               })

})


test_that("downloadFile - show rownames", {
    testServer(downloadFile,
               args = list(logger       = periscope2:::fw_get_user_log(),
                           filenameroot = "show_row_names_download",
                           row_names    = TRUE,
                           datafxns     = list(csv  = download_data,
                                               tsv  = download_data,
                                               txt  = download_data,
                                               xlsx = download_data)),
               expr = {
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < show_row_names_download.csv >",
                                     x       = capture_output(expect_snapshot_file(output$csv))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < show_row_names_download.tsv >",
                                     x       = capture_output(expect_snapshot_file(output$tsv))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < show_row_names_download.txt >",
                                     x       = capture_output(expect_snapshot_file(output$txt))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < show_row_names_download.xlsx >",
                                     x       = capture_output(file.exists(output$xlsx))))
               })
})

test_that("downloadFile - download char data", {
    testServer(downloadFile,
               args = list(logger       = periscope2:::fw_get_user_log(),
                           filenameroot = "my_char_download",
                           row_names    = FALSE,
                           datafxns     = list(txt = download_char_data,
                                               tsv = download_char_data,
                                               csv = download_char_data)),
               expr = {
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < my_char_download.csv >",
                                     x       = capture_output(expect_snapshot_file(output$csv))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < my_char_download.tsv >",
                                     x       = capture_output(expect_snapshot_file(output$tsv))))
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < my_char_download.txt >",
                                     x       = capture_output(expect_snapshot_file(output$txt))))
               })
})

test_that("downloadFile - download numeric data", {
    testServer(downloadFile,
               args = list(logger       = periscope2:::fw_get_user_log(),
                           filenameroot = "my_numeric_data",
                           datafxns     = list(txt = function() {123},
                                               csv = function() {123},
                                               tsv = function() {123})),
               expr = {
                   expect_warning(expect_true(grepl(
                       pattern = "INFO:actions:File downloaded in browser: < my_numeric_data.txt >",
                       x       = capture_output(output$txt))), "txt could not be processed")
                    expect_true(grepl(
                       pattern = "INFO:actions:File downloaded in browser: < my_numeric_data.csv >",
                       x       = capture_output(output$csv)))
                    expect_true(grepl(
                       pattern = "INFO:actions:File downloaded in browser: < my_numeric_data.tsv >",
                       x       = capture_output(output$tsv)))

               })
})

test_that("downloadFile - default values", {
    testServer(downloadFile,
               args = list(datafxns = list(txt = function() {"123"},
                                           csv = function() {"123"},
                                           tsv = function() {"123"})),
               expr = {
                   expect_true(grepl(pattern = "INFO::File downloaded in browser: < download.txt >",
                                     x       = capture_output(expect_snapshot_file(output$txt))))
                   expect_true(grepl(pattern = "INFO::File downloaded in browser: < download.csv >",
                                     x       = capture_output(expect_snapshot_file(output$csv))))
                   expect_true(grepl(pattern = "INFO::File downloaded in browser: < download.tsv >",
                                     x       = capture_output(expect_snapshot_file(output$tsv))))
               })
})

test_that("downloadFile - invalid type", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    testServer(downloadFile,
               args = list(datafxns = list(ttt = function() {"123"},
                                           jeg = download_lattice_plot,
                                           tff = download_plot)),
               expr = {
                   expect_error(output$ttt)
                   expect_error(output$jeg)
                   expect_error(output$tff)
               })
})

# Testing for xlsx downloads
test_that("Testing workbook openxlsx2", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    skip_if_not_installed("openxlsx2")
    testServer(downloadFile,
               args = list(logger       = periscope2:::fw_get_user_log(),
                           filenameroot = "excel_test_openxlsx2_wb",
                           datafxns     = list(xlsx = create_openxlsx2_wb)),
               expr = {
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < excel_test_openxlsx2_wb.xlsx >",
                                     x       = capture_output(file.exists(output$xlsx))))
               })
})

test_that("Testing workbook openxlsx", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    skip_if_not_installed("openxlsx")
    testServer(downloadFile,
               args = list(logger       = periscope2:::fw_get_user_log(),
                           filenameroot = "excel_test_openxlsx_wb",
                           datafxns     = list(xlsx = create_openxlsx_wb)),
               expr = {
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < excel_test_openxlsx_wb.xlsx >",
                                     x       = capture_output(file.exists(output$xlsx))))
               })
})

test_that("Dataframe xlsx download works with openxlsx2", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    skip_if_not_installed("openxlsx2")
    testServer(downloadFile,
               args = list(logger       = periscope2:::fw_get_user_log(),
                           filenameroot = "excel_test_dataframe",
                           datafxns     = list(xlsx = download_data)),
               expr = {
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < excel_test_dataframe.xlsx >",
                                     x       = capture_output(file.exists(output$xlsx))))
               })
})

test_that("Dataframe xlsx download works with openxlsx", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    skip_if_not_installed("openxlsx")
    local_mocked_bindings(check_openxlsx2_availability = function() FALSE)
    testServer(downloadFile,
               args = list(logger       = periscope2:::fw_get_user_log(),
                           filenameroot = "excel_test_dataframe",
                           datafxns     = list(xlsx = download_data)),
               expr = {
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < excel_test_dataframe.xlsx >",
                                     x       = capture_output(file.exists(output$xlsx))))
               })
})

test_that("Dataframe xlsx download works with writexl", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    local_mocked_bindings(check_openxlsx2_availability = function() FALSE)
    local_mocked_bindings(check_openxlsx_availability  = function() FALSE)
    testServer(downloadFile,
               args = list(logger       = periscope2:::fw_get_user_log(),
                           filenameroot = "excel_test_dataframe",
                           datafxns     = list(xlsx = download_data)),
               expr = {
                   expect_true(grepl(pattern = "INFO:actions:File downloaded in browser: < excel_test_dataframe.xlsx >",
                                     x       = capture_output(file.exists(output$xlsx))))
               })
})

##
## The code in this file is part of the logging package. The logging package is free
## software: you can redistribute it as well as modify it under the terms of
## the GNU General Public License as published by the Free Software
## Foundation, either version 3 of the License, or (at your option) any later
## version.
##
## this program is distributed in the hope that it will be useful, but
## WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## General Public License for more details.
##
## Copyright (c) 2009..2013 by Mario Frasca
##


## create the logging environment
logging.options <- new.env()


## The logging levels, names and values
##
## This list associates names to values and vice versa.\cr
## Names and values are the same as in the python standard logging module.
##
loglevels <- c(NOTSET = 0,
               FINEST = 1,
               FINER = 4,
               FINE = 7,
               DEBUG = 10,
               INFO = 20,
               WARNING = 30,
               WARN = 30,
               ERROR = 40,
               CRITICAL = 50,
               FATAL = 50)

namedLevel <- function(value) {
    UseMethod("namedLevel")
}

#' @keywords internal
namedLevel.default <- function(value) {
    loglevels[1]
}

#' @keywords internal
namedLevel.character <- function(value) {
    position <- which(names(loglevels) == value)
    if (length(position) == 0) {
        position <- 1
    }
    loglevels[position][1]
}

#' @keywords internal
namedLevel.numeric <- function(value) {
    position <- which(loglevels == value)
    if (length(position) == 0) {
        position <- 1
    }
    loglevels[position][1]
}

Logger <- setRefClass(
    "Logger",
    fields = list(
        name = "character",
        handlers = "list",
        level = "numeric",
        msg_composer = "function"),
    methods = list(
        getParent = function() {
            # split the name on the '.'
            parts <- unlist(strsplit(name, ".", fixed = TRUE))
            removed <- parts[-length(parts)] # except the last item
            parent_name <- paste(removed, collapse = ".")
            return(getLogger(parent_name))
        },

        getMsgComposer = function() {
            if (!is.null(msg_composer) && !is.null(functionBody(msg_composer))) {
                return(msg_composer)
            }
            if (name != "") {
                parent_logger <- getParent()
                return(parent_logger$getMsgComposer())
            }
            return(defaultMsgCompose)
        },

        setMsgComposer = function(composer_f) {
            if (!is.function(composer_f)
                || paste(formalArgs(composer_f), collapse = ", ") != "msg, ...") {
                stop(paste("message composer(passed as composer_f) must be function",
                           " with signature function(msg, ...)"))
            }
            assign("msg_composer", composer_f, inherits = TRUE)
        },

        .deducelevel = function(initial_level = loglevels[["NOTSET"]]) {
            if (initial_level != loglevels[["NOTSET"]]) {
                # it's proper level (set: Not for inheritance)
                return(initial_level)
            }

            if (level != loglevels[["NOTSET"]]) {
                return(level)
            }

            if (name == "") {
                # assume it's FINEST, as root logger cannot inherit
                return(loglevels[["FINEST"]])
            }

            # ask parent for level
            parent_logger <- getParent()
            return(parent_logger$.deducelevel())
        },

        .logrecord = function(record) {
            logger_level <- .deducelevel(level)
            if (record$level >= logger_level) {
                for (handler in handlers) {
                    handler_level <- .deducelevel(with(handler, level))
                    if (record$level >= handler_level) {
                        action <- with(handler, action)
                        formatter <- with(handler, formatter)
                        action(formatter(record), handler, record)
                    }
                }
            }

            if (name != "") {
                parent_logger <- getParent()
                parent_logger$.logrecord(record)
            }
            invisible(TRUE)
        },

        log = function(msglevel, msg, ...) {
            msglevel <- namedLevel(msglevel)
            if (msglevel < level) {
                return(invisible(FALSE))
            }
            ## fine, we create the record and pass it to all handlers attached to the
            ## loggers from here up to the root.
            record <- list()

            composer_f <- getMsgComposer()
            record$msg <- composer_f(msg, ...)
            record$timestamp <- sprintf("%s", Sys.time())
            record$logger <- name
            record$level <- msglevel
            record$levelname <- names(which(loglevels == record$level)[1])

            ## cascade action in private method.
            .logrecord(record)
        },

        setLevel = function(new_level) {
            assign("level", namedLevel(new_level), inherits = TRUE)
        },

        getLevel = function() level,

        getHandler = function(handler) {
            if (!is.character(handler))
                handler <- deparse(substitute(handler))
            handlers[[handler]]
        },

        removeHandler = function(handler) {
            if (!is.character(handler)) { # handler was passed as its action
                handler <- deparse(substitute(handler))
            }

            logger_handlers <- get("handlers", inherits = TRUE)
            logger_handlers <- handlers[!(names(logger_handlers) == handler)]
            assign("handlers", logger_handlers, inherits = TRUE)
        },

        addHandler = function(handler, ..., level = 0, formatter = defaultFormat) {
            handler_env <- new.env()
            if (is.character(handler)) {
                ## first parameter is handler name
                handler_name <- handler
                ## and hopefully action is in the dots
                params <- list(...)
                if ("action" %in% names(params)) {
                    the_action <- params[["action"]]
                } else if (length(params) > 0 && is.null(names(params)[[1]])) {
                    the_action <- params[[1]]
                } else {
                    stop("No action for the handler provided")
                }

                assign("action", the_action, handler_env)
            } else {
                ## first parameter is handler action, from which we extract the name
                updateOptions.environment(handler_env, action = handler)
                handler_name <- deparse(substitute(handler))
            }
            updateOptions.environment(handler_env, ...)
            assign("level", namedLevel(level), handler_env)
            assign("formatter", formatter, handler_env)
            removeHandler(handler_name)

            if (with(handler_env, action)(NA, handler_env, dry = TRUE) == TRUE) {
                logger_handlers                 <- get("handlers", inherits = TRUE)
                logger_handlers[[handler_name]] <- handler_env
                assign("handlers", logger_handlers, inherits = TRUE)
            }
        },

        finest = function(...) log(loglevels["FINEST"], ...),
        finer = function(...) log(loglevels["FINER"], ...),
        fine = function(...) log(loglevels["FINE"], ...),
        debug = function(...) log(loglevels["DEBUG"], ...),
        info = function(...) log(loglevels["INFO"], ...),
        warn = function(...) log(loglevels["WARN"], ...),
        error = function(...) log(loglevels["ERROR"], ...)
    ) # methods
) # setRefClass



#' Entry points for logging actions
#'
#' Generate a log record and pass it to the logging system.\cr
#'
#' A log record gets timestamped and will be independently formatted by each
#' of the handlers handling it.\cr
#'
#' Leading and trailing whitespace is stripped from the final message.
#'
#' @param msg the textual message to be output, or the format for the \dots
#'  arguments
#' @param ... if present, msg is interpreted as a format and the \dots values
#'  are passed to it to form the actual message.
#' @param logger the name of the logger to which we pass the record
#'
#'
#' @name logging-entrypoints
#' @aliases logdebug
#' @aliases loginfo
#' @aliases logwarn
#' @aliases logerror
#'
#' @importFrom methods new
#'
NULL


#' @rdname logging-entrypoints
#' @export
logdebug <- function(msg, ..., logger = "") {
    .levellog(loglevels["DEBUG"], msg, ..., logger = logger)
}

## @rdname logging-entrypoints
logfinest <- function(msg, ..., logger = "") {
    .levellog(loglevels["FINEST"], msg, ..., logger = logger)
}

## @rdname logging-entrypoints
logfiner <- function(msg, ..., logger = "") {
    .levellog(loglevels["FINER"], msg, ..., logger = logger)
}

## @rdname logging-entrypoints
logfine <- function(msg, ..., logger = "") {
    .levellog(loglevels["FINE"], msg, ..., logger = logger)
}

#' @rdname logging-entrypoints
#' @export
loginfo <- function(msg, ..., logger = "") {
    .levellog(loglevels["INFO"], msg, ..., logger = logger)
}

#' @rdname logging-entrypoints
#' @export
logwarn <- function(msg, ..., logger = "") {
    .levellog(loglevels["WARN"], msg, ..., logger = logger)
}

#' @rdname logging-entrypoints
#'
#' @return no return value, prints log contents into R console and app log file
#'
#' @export
logerror <- function(msg, ..., logger = "") {
    .levellog(loglevels["ERROR"], msg, ..., logger = logger)
}

## @rdname logging-entrypoints
## @param level The logging level
levellog <- function(level, msg, ..., logger = "") {
    # just calling .levellog
    # do not simplify it as call sequence should be same
    #   as for other logX functions
    .levellog(level, msg, ..., logger = logger)
}

.levellog <- function(level, msg, ..., logger = "") {
    if ((is.null(logger)) || (is.character(logger))) {
        logger <- getLogger(logger)
    }
    logger$log(level, msg, ...)
}



##
## Set defaults and get the named logger.
##
## Make sure a logger with a specific name exists and return it as a
## \var{Logger} S4 object.  if not yet present, the logger will be created and
## given the values specified in the \dots arguments.
## @importFrom methods new
## @param name The name of the logger
## @param ... Any properties you may want to set in the newly created
##    logger. These have no effect if the logger is already present.
##
## @return The logger retrieved or registered.
getLogger <- function(name = "", ...) {
    if ((is.null(name)) || (name == "")) {
        fullname <- "logging.ROOT"
        name     <- ""
    } else {
        fullname <- paste("logging.ROOT", name, sep = ".")
    }

    if (!exists(fullname, envir = logging.options)) {
        logger <- Logger$new(name = name,
                             handlers = list(),
                             level = namedLevel("NOTSET"))
        updateOptions.environment(logger, ...)
        logging.options[[fullname]] <- logger

        if (fullname == "logging.ROOT") {
            .basic_config(logger)
        }
    }
    logging.options[[fullname]]
}



##
## Bootstrapping the logging package.
##
## \code{basicConfig} and \code{logReset} provide a way to put the logging package
## in a know initial state.
##
## @name bootstrapping
## NULL

## @rdname bootstrapping
##
## @details
## \code{basicConfig} creates the root logger, attaches a console handler(by
## \var{basic.stdout} name) to it and sets the level of the handler to
## \code{level}. You must not call \code{basicConfig} to for logger to work any more:
## then root logger is created it gets initialized by default the same way as
## \code{basicConfig} does. If you need clear logger to fill with you own handlers
## use \code{logReset} to remove all default handlers.
##
## @param level The logging level of the root logger. Defaults to INFO. Please do notice that
##   this has no effect on the handling level of the handler that basicConfig attaches to the
##   root logger.
##
##
basicConfig <- function(level = 20) {
    root_logger <- getLogger()

    updateOptions(root_logger, level = namedLevel(level))
    .basic_config(root_logger)

    invisible()
}

## Called from basicConfig and while creating rootLogger.
## @noRd
.basic_config <- function(root_logger) {
    stopifnot(root_logger$name == "")
    root_logger$addHandler("basic.stdout", writeToConsole)
}


## @rdname bootstrapping
##
## @details
## \code{logReset} reinitializes the whole logging system as if the package had just been
## loaded except it also removes all default handlers. Typically, you would want to call
## \code{basicConfig} immediately after a call to \code{logReset}.
##
##
##
logReset <- function() {
    ## reinitialize the whole logging system

    ## remove all content from the logging environment
    rm(list = ls(logging.options), envir = logging.options)

    root_logger <- getLogger(level = "NOTSET")
    root_logger$removeHandler("basic.stdout")

    invisible()
}



##
## Add a handler to or remove one from a logger.
##
## Use this function to maintain the list of handlers attached to a logger.\cr
## \cr
## \code{addHandler} and \code{removeHandler} are also offered as methods of the
## \var{Logger} S4 class.
##
## @details
## Handlers are implemented as environments. Within a logger a handler is
## identified by its \var{name} and all handlers define at least the
## three variables:
## \describe{
##   \item{level}{all records at level lower than this are skipped.}
##   \item{formatter}{a function getting a record and returning a string}
##   \item{\code{action(msg, handler)}}{a function accepting two parameters: a
##      formatted log record and the handler itself. making the handler a
##      parameter of the action allows us to have reusable action functions.}
## }
##
## Being an environment, a handler may define as many variables as you
## think you need.  keep in mind the handler is passed to the action
## function, which can check for existence and can use all variables that
## the handler defines.
##
## @param handler The name of the handler, or its action
## @param logger the name of the logger to which to attach the new handler,
##   defaults to the root logger.
##
## @name handlers-management
## NULL


## @rdname handlers-management
##
## @param ... Extra parameters, to be stored in the handler list
##
## \dots may contain extra parameters that will be passed to the handler
## action. Some elements in the \dots will be interpreted here.
##
##
##
addHandler <- function(handler, ..., logger = "") {
    if (is.character(logger)) {
        logger <- getLogger(logger)
    }

    ## this part has to be repeated here otherwise the called function
    ## will deparse the argument to 'handler', the formal name given
    ## here to the parameter
    if (is.character(handler)) {
        logger$addHandler(handler, ...)
    } else {
        handler_name <- deparse(substitute(handler))
        logger$addHandler(handler = handler_name, action = handler, ...)
    }
}

## @rdname handlers-management
##
##
removeHandler <- function(handler, logger = "") {
    if (is.character(logger)) {
        logger <- getLogger(logger)
    }
    if (!is.character(handler)) {
        # handler was passed as its action
        handler <- deparse(substitute(handler))
    }
    logger$removeHandler(handler)
}

##
## Retrieves a handler from a logger.
##
## @description
## Handlers are not uniquely identified by their name. Only within the logger to which
## they are attached is their name unique. This function is here to allow you grab a
## handler from a logger so you can examine and alter it.
##
## @description
## Typical use of this function is in \code{setLevel(newLevel, getHandler(...))}.
##
## @param handler The name of the handler, or its action.
## @param logger Optional: the name of the logger. Defaults to the root logger.
##
## @return The retrieved handler object. It returns NULL if handler is not registered.
getHandler <- function(handler, logger = "") {
    if (is.character(logger)) {
        logger <- getLogger(logger)
    }
    if (!is.character(handler)) {
        # handler was passed as its action
        handler <- deparse(substitute(handler))
    }
    logger$getHandler(handler)
}



##
## Set \var{logging.level} for the object.
##
## Alter an existing logger or handler, setting its \var{logging.level} to a new
## value. You can access loggers by name, while you must use \code{getHandler} to
## get a handler.
##
## @param level The new level for this object. Can be numeric or character.
## @param container a logger, its name or a handler. Default is root logger.
##
setLevel <- function(level, container = "") {
    if (is.null(container)) {
        stop("NULL container provided: cannot set level for NULL container")
    }

    if (is.character(container)) {
        container <- getLogger(container)
    }
    assign("level", namedLevel(level), container)
}

##
## Sets message composer for logger.
##
## Message composer is used to compose log message out of formatting string and arguments.
## It is function with signature \code{function(msg, ...)}. Formatting message is passed under msg
## and formatting arguments are passed as \code{...}.
##
## If message composer is not set default is in use (realized with \code{sprintf}). If message
## composer is not set for sub-logger, parent's message composer will be used.
##
## @param composer_f message composer function (type: function(msg, ...))
## @param container name of logger to reset message composer for (type: character)
##
setMsgComposer <- function(composer_f, container = "") {
    if (is.null(container)) {
        stop("NULL container provided: cannot set message composer for NULL container")
    }

    if (is.character(container)) {
        container <- getLogger(container)
    }
    container$setMsgComposer(composer_f)
    assign("msg_composer", composer_f, container)
}

##
## Resets previously set message composer.
##
## @param container name of logger to reset message composer for (type: character)
##
resetMsgComposer <- function(container = "") {
    if (is.null(container)) {
        stop("NULL container provided: cannot reset message composer for NULL container")
    }

    if (is.character(container)) {
        container <- getLogger(container)
    }
    assign("msg_composer", function() NULL, container)
}

##
## Changes settings of logger or handler.
##
## @param container a logger, its name or a handler.
## @param ... options to set for the container.
##
updateOptions <- function(container, ...) {
    UseMethod("updateOptions")
}

## @describeIn updateOptions Update options for logger identified by name.
##
#' @keywords internal
updateOptions.character <- function(container, ...) {
    ## container is really just the name of the container
    updateOptions(getLogger(container), ...)
}

## @describeIn updateOptions Update options of logger or handler passed by reference.
##
#' @keywords internal
updateOptions.environment <- function(container, ...) {
    ## the container is a logger
    config <- list(...)
    if ("level" %in% names(config)) {
        config$level <- namedLevel(config$level)
    } else if (get("name", container) == "") {
        # root logger
        config$level <- loglevels["NOTSET"]
    } else {
        config$level <- loglevels["NOTSET"]
    }

    for (key in names(config)) {
        if (key != "") {
            assign(key, config[[key]], container)
        }
    }
    invisible()
}

## @describeIn updateOptions Update options of logger or handler passed by reference.
##
#' @keywords internal
updateOptions.Logger <- function(container, ...) {
    updateOptions.environment(container, ...)
}

## Filtering log messages according to chosen level
##
logging_level <- function(level_name, current_log_level) {
    switch(current_log_level,
           "INFO"     = level_name %in% c("INFO", "WARNING", "ERROR"),
           "WARNING"  = level_name %in% c("WARNING", "ERROR"),
           "ERROR"    = level_name == "ERROR",
           TRUE
    )
}

##
## Predefined(sample) handler actions
##
## When you define a handler, you specify its name and the associated action.
## A few predefined actions described below are provided.
##
## A handler action is a function that accepts a formatted message and handler
## configuration.
##
## Messages passed are filtered already regarding loglevel.
##
## \dots parameters are used by logging system to interact with the action. \dots can
## contain \var{dry} key to inform action that it meant to initialize itself. In the case
## action should return TRUE if initialization succeeded.
##
## If it's not a dry run \dots contain the whole preformatted \var{logging.record}.
## A \var{logging.record} is a named list and has following structure:
## \describe{
##   \item{msg}{contains the real formatted message}
##   \item{level}{message level as numeric}
##   \item{levelname}{message level name}
##   \item{logger}{name of the logger that generated it}
##   \item{timestamp}{formatted message timestamp}
## }
##
## @param msg A formatted message to handle.
## @param handler The handler environment containing its options. You can
##   register the same action to handlers with different properties.
## @param ... parameters provided by logger system to interact with the action.
##
## @examples
## ## define your own function and register it with a handler.
## ## author is planning a sentry client function.  please send
## ## any interesting function you may have written!
##
## @name inbuilt-actions
## NULL

## @rdname inbuilt-actions
##
## @details
## \code{writeToConsole} detects if crayon package is available and uses it
## to color messages. The coloring can be switched off by means of configuring
## the handler with \var{color_output} option set to FALSE.
##
writeToConsole <- function(msg, handler, ...) {
    if (length(list(...)) && "dry" %in% names(list(...))) {
        if (!is.null(handler$color_output) && handler$color_output == FALSE) {
            handler$color_msg <- function(msg, level_name) msg
        } else {
            handler$color_msg <- .build_msg_coloring()
        }
        return(TRUE)
    }

    stopifnot(length(list(...)) > 0)

    current_log_level <- fw_get_loglevel()
    level_name        <- list(...)[[1]]$levelname

    if (logging_level(level_name, current_log_level)) {
        msg <- handler$color_msg(msg, level_name)
        cat(paste0(msg, "\n"))
    }
}

.build_msg_coloring <- function() {
    crayon_env <- tryCatch(asNamespace("crayon"),
                           error = function(e) NULL)

    default_color_msg <- function(msg, level_name) msg
    if (is.null(crayon_env)) {
        return(default_color_msg)
    }

    if (is.null(crayon_env$make_style) ||
        is.null(crayon_env$combine_styles) ||
        is.null(crayon_env$reset)) {
        return(default_color_msg)
    }

    color_msg <- function(msg, level_name) {
        style <- switch(level_name,
                        "FINEST" = crayon_env$make_style("gray80"),
                        "FINER" = crayon_env$make_style("gray60"),
                        "FINE" = crayon_env$make_style("gray60"),
                        "DEBUG" = crayon_env$make_style("deepskyblue4"),
                        "INFO" = crayon_env$reset,
                        "WARNING" = crayon_env$make_style("darkorange"),
                        "ERROR" = crayon_env$make_style("red4"),
                        "CRITICAL" =
                            crayon_env$combine_styles(crayon_env$bold,
                                                      crayon_env$make_style("red1")),
                        crayon_env$make_style("gray100"))
        res <- paste0(style(msg), crayon_env$reset(""))
        return(res)
    }
    return(color_msg)
}


## @rdname inbuilt-actions
##
## @details \code{writeToFile} action expects file path to write to under
##  \var{file} key in handler options.
##
writeToFile <- function(msg, handler, ...) {
    if (length(list(...)) && "dry" %in% names(list(...)))
        return(exists("file", envir = handler))

    current_log_level <- fw_get_loglevel()
    level_name        <- list(...)[[1]]$levelname

    if (logging_level(level_name, current_log_level)) {
        cat(paste0(msg, "\n"), file = with(handler, file), append = TRUE)
    }
}

## the single predefined formatter
defaultFormat <- function(record) {
    ## strip leading and trailing whitespace from the final message.
    msg <- trimws(record$msg)
    text <- paste(record$timestamp,
                  paste(record$levelname, record$logger, msg, sep = ":"))
    return(text)
}

## default way of composing msg with parameters
defaultMsgCompose <- function(msg, ...) {
    optargs <- list(...)
    if (is.character(msg)) {
        ## invoked as ("printf format", arguments_for_format)
        if (length(optargs) > 0) {
            optargs <- lapply(optargs,
                              function(x) {
                                  if (length(x) != 1)
                                      x <- paste(x, collapse = ",")
                                  x
                              })
        }

        # 8192 is limitation on fmt in sprintf
        if (any(nchar(msg) > 8192)) {
            if (length(optargs) > 0) {
                stop("'msg' length exceeds maximal format length 8192")
            }

            # else msg must not change in any way
            return(msg)
        }
        if (length(optargs) > 0) {
            msg <- do.call("sprintf", c(msg, optargs))
        }
        return(msg)
    }

    ## invoked as list of expressions
    ## this assumes that the function the user calls is two levels up, e.g.:
    ## loginfo -> .levellog -> logger$log -> .default_msg_composer
    ## levellog -> .levellog -> logger$log -> .default_msg_composer
    external_call <- sys.call(-3)
    external_fn <- eval(external_call[[1]])
    matched_call <- match.call(external_fn, external_call)
    matched_call <- matched_call[-1]
    matched_call_names <- names(matched_call)

    ## We are interested only in the msg and ... parameters,
    ## i.e. in msg and all parameters not explicitly declared
    ## with the function
    formal_names <- names(formals(external_fn))
    is_output_param <-
        matched_call_names == "msg" |
        !(matched_call_names %in% c(setdiff(formal_names, "...")))

    label <- lapply(matched_call[is_output_param], deparse)
    msg <- sprintf("%s: %s", label, c(msg, optargs))
    return(msg)
}

context("periscope2 - logging functionality")


writeToConsole     <- periscope2:::writeToConsole
writeToFile        <- periscope2:::writeToFile
loglevels          <- periscope2:::loglevels
set_app_parameters <- periscope2:::set_app_parameters
test_file_name     <- file.path(tempdir(), c("1", "2", "3"))
test_file_log      <- file.path(tempdir(), "test_log.txt")

env_setup <- function() {
    test_env        <- new.env(parent = emptyenv())
    test_env$logged <- NULL

    mock_action <- function(msg, handler, ...) {
        if (length(list(...)) && "dry" %in% names(list(...)))
            return(TRUE)
        test_env$logged <- c(test_env$logged, msg)
    }

    mock_formatter <- function(record) {
        paste(record$levelname, record$logger, record$msg, sep = ":")
    }

    periscope2:::logReset()
    periscope2:::addHandler(mock_action,
                           formatter = mock_formatter)
    test_env
}

test_that("Handlers - addHandler(), getHandler() and removeHandler()", {
    periscope2:::logReset()
    periscope2:::basicConfig()

    # looking for handler in RootLogger
    expect_identical(periscope2:::getHandler("basic.stdout"),
                     periscope2:::getLogger()[["handlers"]][[1]])
    # looking for handler in object
    expect_identical(periscope2:::getLogger()$getHandler("basic.stdout"),
                     periscope2:::getLogger()[["handlers"]][[1]])

    # add handler
    periscope2:::addHandler(writeToConsole)

    expect_equal(length(with(periscope2:::getLogger(), names(handlers))), 2)
    expect_true("writeToConsole" %in% with(periscope2:::getLogger(), names(handlers)))

    # get handler
    expect_true(!is.null(periscope2:::getHandler("writeToConsole")))
    expect_true(!is.null(periscope2:::getHandler(writeToConsole)))

    # remove handler
    periscope2:::removeHandler("writeToConsole")
    expect_equal(length(with(periscope2:::getLogger(), names(handlers))), 1)
    expect_false("writeToConsole" %in% with(periscope2:::getLogger(), names(handlers)))

    expect_error(periscope2:::addHandler("handlerName"),
                 regexp = "No action for the handler provided",
                 fixed  = TRUE)
})

test_that("Handlers in Logger object - addHandler(), getHandler() and removeHandler()", {
    periscope2:::logReset()
    periscope2:::basicConfig()

    # add handler
    log <- periscope2:::getLogger("testLogger")
    log$addHandler(writeToConsole)
    expect_equal(length(with(log, names(handlers))), 1)

    # get handler
    expect_true(!is.null(log$getHandler("writeToConsole")))
    expect_true(!is.null(log$getHandler(writeToConsole)))

    # remove handler
    log$removeHandler(writeToConsole)
    expect_equal(length(with(log, names(handlers))), 0)
    expect_false("writeToConsole" %in% with(log, names(handlers)))
})

test_that("Levels - setLevel()", {
    periscope2:::logReset()
    periscope2:::basicConfig()

    expect_error(periscope2:::setLevel("INFO", NULL),
                 regexp = "NULL container provided: cannot set level for NULL container",
                 fixed  = TRUE)

    periscope2:::logReset()
    expect_equal(periscope2:::setLevel(TRUE), loglevels["NOTSET"]) # invalid level

    periscope2:::logReset()
    expect_equal(periscope2:::setLevel("INVALID"), loglevels["NOTSET"]) # invalid level
})

test_that("Levels in Logger object- setLevel() and getLevel()", {
    periscope2:::logReset()
    log <- periscope2:::getLogger("testLogger")

    # invalid level
    log$setLevel(150)
    expect_equal(log$getLevel(), loglevels["NOTSET"])

    log$setLevel(TRUE)
    expect_equal(log$getLevel(), loglevels["NOTSET"])

    log$setLevel("INVALID")
    expect_equal(log$getLevel(), loglevels["NOTSET"])
})

test_that("UpdateOptions - updateOptions()", {
    periscope2:::logReset()
    periscope2:::basicConfig()

    periscope2:::updateOptions.character("", level = "WARNING")
    expect_equal(periscope2:::getLogger()$getLevel(), loglevels["WARNING"])
})

test_that("LoggingToConsole", {
    periscope2:::logReset()
    periscope2:::basicConfig()

    periscope2:::getLogger()$setLevel("FINEST")
    periscope2:::addHandler(writeToConsole, level = "DEBUG")

    expect_equal(with(periscope2:::getLogger(), names(handlers)), c("basic.stdout", "writeToConsole"))
    expect_true(grepl(pattern = "DEBUG::log generated for testing",
                      x       = capture_output(logdebug("log generated for testing"), print = TRUE)))
    expect_true(grepl(pattern = "INFO::log generated for testing",
                      x       = capture_output(loginfo("log generated for testing"), print = TRUE)))

    succeed()
})

test_that("LoggingToFile", {
    periscope2:::logReset()
    unlink(test_file_name, force = TRUE)

    periscope2:::getLogger()$setLevel("FINEST")
    periscope2:::addHandler(writeToFile, file = test_file_name[[1]], level = "DEBUG")

    expect_equal(with(periscope2:::getLogger(), names(handlers)), c("writeToFile"))
    logerror("log generated for testing")
    logwarn("log generated for testing")
    loginfo("log generated for testing")
    logdebug("log generated for testing")
    periscope2:::logfinest("log generated for testing")
    periscope2:::logfiner("log generated for testing")
    periscope2:::logfine("log generated for testing")
    periscope2:::levellog("log generated for testing")

    succeed()
})

test_that("LoggingToFile in Logger object", {
    periscope2:::logReset()
    unlink(test_file_name, force = TRUE)

    log <- periscope2:::getLogger()
    log$setLevel("FINEST")
    log$addHandler(writeToFile, file = test_file_name[[1]], level = "DEBUG")
    expect_equal(with(log, names(handlers)), c("writeToFile"))

    log$error("log generated for testing")
    log$warn("log generated for testing")
    log$info("log generated for testing")
    log$debug("log generated for testing")
    log$finest("log generated for testing")
    log$finer("log generated for testing")
    log$fine("log generated for testing")

    succeed()
})

test_that("Msgcomposer - setMsgComposer(), resetMsgComposer()", {
    env <- env_setup()

    periscope2:::setMsgComposer(function(msg, ...) { paste(msg, "comp") })
    loginfo("test")

    expect_equal(env$logged, "INFO::test comp")
    expect_error(periscope2:::setMsgComposer(function(msgX, ...) { paste(msg, "comp") }),
                 regexp = "message composer(passed as composer_f) must be function  with signature function(msg, ...)",
                 fixed = TRUE)
    expect_error(periscope2:::setMsgComposer(container = NULL),
                 regexp = "NULL container provided: cannot set message composer for NULL container")

    # reset_composer
    env <- env_setup()
    periscope2:::resetMsgComposer()
    loginfo("test")

    expect_equal(env$logged, "INFO::test")
    expect_error(periscope2:::resetMsgComposer(NULL),
                 regexp = "NULL container provided: cannot reset message composer for NULL container")

    # set_sublogger_composer
    env <- env_setup()

    periscope2:::setMsgComposer(function(msg, ...) { paste(msg, "comp") }, container = "named")
    loginfo("test")
    loginfo("test", logger = "named")

    expect_equal(env$logged, c("INFO::test", "INFO:named:test comp"))
})

test_that("MsgComposer function - defaultMsgCompose()",{
    expect_equal(periscope2:::defaultMsgCompose(msg = "Message"), "Message")
    expect_error(periscope2:::defaultMsgCompose(msg = paste(rep(LETTERS, 316), collapse = ""), param = 1),
                 "'msg' length exceeds maximal format length 8192")
    expect_equal(periscope2:::defaultMsgCompose(msg = paste(rep(LETTERS, 316), collapse = "")),
                 paste(rep(LETTERS, 316), collapse = ""))
})

# Testing logging levels
test_that("DEBUG level shows all messages", {
    set_app_parameters(log_level = "DEBUG")
    periscope2:::addHandler(writeToConsole)
    expect_output(logdebug("debug message"), "DEBUG::debug message")
    expect_output(loginfo("info message"),   "INFO::info message")
    expect_output(logwarn("warn message"),   "WARNING::warn message")
    expect_output(logerror("error message"), "ERROR::error message")
})
I
test_that("INFO level filters DEBUG", {
    set_app_parameters(log_level = "INFO")
    periscope2:::addHandler(writeToConsole)
    expect_silent(logdebug("debug message"))
    expect_output(loginfo("info message"),   "INFO::info message")
    expect_output(logwarn("warn message"),   "WARNING::warn message")
    expect_output(logerror("error message"), "ERROR::error message")
})

test_that("WARNING level filters DEBUG and INFO", {
    set_app_parameters(log_level = "WARNING")
    periscope2:::addHandler(writeToConsole)
    expect_silent(logdebug("debug message"))
    expect_silent(loginfo("info message"))
    expect_output(logwarn("warn message"),   "WARNING::warn message")
    expect_output(logerror("error message"), "ERROR::error message")
})

test_that("ERROR level filters all but ERROR", {
    set_app_parameters(log_level = "ERROR")
    periscope2:::addHandler(writeToConsole)
    expect_silent(logdebug("debug message"))
    expect_silent(loginfo("info message"))
    expect_silent(logwarn("warn message"))
    expect_output(logerror("error message"), "ERROR::error message")
})

test_that("File logging level DEBUG", {
    unlink(test_file_log, force  = TRUE)
    set_app_parameters(log_level = "DEBUG")
    periscope2:::addHandler(writeToFile, file = test_file_log)

    expect_output(logdebug("debug message"), "debug message")
    expect_output(loginfo("info message"), "info message")
    expect_output(logwarn("warn message"), "warn message")
    expect_output(logerror("error message"), "error message")

    log_content <- readLines(test_file_log)
    expect_true(any(grepl("debug message", log_content)))
    expect_true(any(grepl("info message",  log_content)))
    expect_true(any(grepl("warn message",  log_content)))
    expect_true(any(grepl("error message", log_content)))
})

test_that("File logging level INFO", {
    unlink(test_file_log, force  = TRUE)
    set_app_parameters(log_level = "INFO")
    periscope2:::addHandler(writeToFile, file = test_file_log)

    logdebug("debug message")
    expect_output(loginfo("info message"), "info message")
    expect_output(logwarn("warn message"), "warn message")
    expect_output(logerror("error message"), "error message")

    log_content <- readLines(test_file_log)
    expect_false(any(grepl("debug message", log_content)))
    expect_true(any(grepl("info message",   log_content)))
    expect_true(any(grepl("warn message",   log_content)))
    expect_true(any(grepl("error message",  log_content)))
})

test_that("File logging level WARNING", {
    unlink(test_file_log, force  = TRUE)
    set_app_parameters(log_level = "WARNING")
    periscope2:::addHandler(writeToFile, file = test_file_log)

    logdebug("debug message")
    loginfo("info message")
    expect_output(logwarn("warn message"), "warn message")
    expect_output(logerror("error message"), "error message")

    log_content <- readLines(test_file_log)
    expect_false(any(grepl("debug message", log_content)))
    expect_false(any(grepl("info message",  log_content)))
    expect_true(any(grepl("warn message",   log_content)))
    expect_true(any(grepl("error message",  log_content)))
})

test_that("File logging level ERROR", {
    unlink(test_file_log, force  = TRUE)
    set_app_parameters(log_level = "ERROR")
    periscope2:::addHandler(writeToFile, file = test_file_log)

    logdebug("debug message")
    loginfo("info message")
    logwarn("warn message")
    expect_output(logerror("error message"), "error message")

    log_content <- readLines(test_file_log)
    expect_false(any(grepl("debug message", log_content)))
    expect_false(any(grepl("info message",  log_content)))
    expect_false(any(grepl("warn message",  log_content)))
    expect_true(any(grepl("error message",  log_content)))
})

# ----------------------------------------
# --          PROGRAM ui_body.R         --
# ----------------------------------------
# USE: Create UI elements for the
#      application body (right side on the
#      desktop; contains output) and
#      ATTACH them to the UI by calling
#      add_ui_body()
#
# NOTES:
#   - All variables/functions here are
#     not available to the UI or Server
#     scopes - this is isolated
# ----------------------------------------

# -- IMPORTS --



# ----------------------------------------
# --      BODY ELEMENT CREATION         --
# ----------------------------------------

# -- Create Elements

### Application Setup Elements
about_box <- box(
    id          = "about_box",
    width       = 12,
    status      = "info",
    solidHeader = TRUE,
    title       = "About periscope2",
    tags$dl(tags$p("periscope2 is a scalable and UI-standardized 'shiny' framework including a variety of developer convenience",
                   "functions with the goal of both streamlining robust application development and assisting in creating a consistent",
                   " user experience regardless of application or developer."),
            tags$dt("Features"),
            tags$ul(tags$li("Predefined but flexible template for new Shiny applications with a default dashboard layout"),
                    tags$li("Separation by file of functionality that exists in one of the three shiny scopes: global, server-global, and server-local."),
                    tags$li("Off-the-shelf and ready to be used modules ('Table Downloader', 'Plot Downloader', 'File Downloader' and 'Reset Application'"),
                    tags$li("Different methods and tools to alert users and add useful information about application UI and server operations"),
                    tags$li("Application logger with different levels and a UI tool to display and review recorded application logs"),
                    tags$li("Application look and feel can be customized easily via 'www/periscope_style.yaml' or more advanced via 'www/css/custom.cs'"),
                    tags$li("Application can make use of JS power by customizing 'www/js/custom.js'")))
)


announcements_box <- box(
    id          = "announcements",
    title       = "Announcements",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$li("'Announcement' feature is a user friendly method to announce a recently added feature, initiate a shutdown warning, or ",
                                    "broadcast any news to your users at once"),
                            tags$li("Announcements feature aim to alert app user with customized admin messages upon application start for a predefined time"),
                            tags$li("Announcement can be either a simple text or a fully rich HTML text."),
                            tags$li("Announcement location is on the application header"),
                            tags$li("Announcement status (body color) can be: 'primary', 'success', 'warning', 'danger' or 'info'"),
                            tags$li("Application admin can control when an 'Announcement' should start to be seen and when it should be expire"),
                            tags$li("Application admin can control how long an 'Announcement' stay visible for users or let each user close it manually"))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$li("No development is needed to launch or control 'Announcements' feature, only configurations"),
                            tags$li("Default feature generated configuration file is called ", tags$b("announce.yaml")),
                            tags$li("Default configuration file path is: ", tags$i("'program/config/announce.yaml'")),
                            tags$li("Configuration file path can be changed based on admin preference. Only update ",
                                    tags$b("'announcements_file_path'"), " parameter in ", tags$b("'load_announcements'"), " method in ",
                                    tags$b("'server.R'"), " file with the new config file path"),
                            tags$li("Check generated configuration file self-documentation for each config variable usage details"),
                            tags$li("Review current application example configuration for more details on how to configure/disable announcements`"))))
)


logger_box <- box(
    id          = "logger",
    title       = "Logger",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$li("Logger feature presents a developer and an admin friendly method to record different user or internal actions ",
                                    "in their recoded time and using different logging levels"),
                            tags$li("There are 4 different available log levels: 'logdebug', 'loginfo', 'logwarn' and 'logerror' in that order"),
                            tags$li("Each log level is displayed in different text color in R console"),
                            tags$li("User can control which log levels to be excluded from being displayed or recorded"),
                            tags$ul(style = "list-style-type: circle;",
                                    tags$li("The log rolls over for each session"),
                                    tags$li("The log files are kept in the ", tags$b("/log"), "directory and named ",  tags$b("'actions.log'")),
                                    tags$li("ONE old copy of the log is kept as ", tags$b("'actions.log.last'"))),
                            tags$li("Beside reviewing the log files directly, admins can review logs in application via ", tags$b("logViewer UI"), " module"))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$ul(tags$li("To create actions to the framework call one of the logging ",
                                            "functions like: "),
                                    blockQuote(p(pre("S: logXXXX('Your Log Message with %s, %s parameters', parm1, parm2, logger = ss_userAction.Log)")),
                                               color = "info"),
                                    tags$li(p("The XXXX should be replaced by an actual log level like ", tags$b("'debug'"), ", ",
                                              tags$b("'info'"), ", ", tags$b("'warn'"), ", ",  "or ", tags$b("'error'"),
                                              " The framework will handle updating LogViewer UI module every time the log is added to.")),
                                    tags$li("'ss_userAction.Log' is a periscope framework logger that is defined automatically under server.R and",
                                            tags$b(" should not be altered")),
                                    tags$li("Log level can be changed based on admin preference. Only update ",
                                            tags$b("'log_level'"), " parameter in ", tags$b("'set_app_parameters'"), " method in ",
                                            tags$b("'program/global.R'"), " file with the required log level ",
                                            tags$b("'DEBUG'"), ", ", tags$b("'INFO'"), ", ", tags$b("'WARN'"), " or ", tags$b("'ERROR'")),
                                    tags$li("To display application logs like below portlet add the following logViewer table to any of 'ui_body.R' boxes",
                                            blockQuote(p(pre("U: logViewerOutput('logViewerId')")),
                                                       color = "info")))))),
    fluidRow(column(width = 12,
                    tags$dl(tags$dt("Example"),
                            tags$p("Click different button in the example application and compare console with below output"),
                            br(),
                            br(),
                            div(logViewerOutput("logViewerId2"), style = "min-width: 'auto';"))))
)


files_organization_box <- box(
    id          = "files_organization",
    title       = "Files Organization",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    canvasXpressOutput("file_structure_plot", height = "88vh"),
    htmlOutput("file_description")
)

### Application Modules Elements
plot2_hover <- hoverOpts(id = "examplePlot2_hover")


table_downloader_box <- box(
    id          = "table_downloader",
    title       = "Table Downloader",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$ul(tags$li("Table downloader module display tabular data in rich formatted tables using `DT` package"),
                                    tags$li("Selected rows in the table are returned as a reactive variable that can be used in application different areas"),
                                    tags$li("Table data can be downloaded in different formats such as: ", tags$b("'csv'"),
                                            ", ", tags$b("'tsv'"), ", ", tags$b("'txt'"), "and/or ", tags$b("'xlsx'")),
                                    tags$li("User can customize downloadableTable modules using DT options such as:",
                                            tags$ul(tags$li("labels:", HTML("&nbsp;"),
                                                            tags$b(tags$i("i.e. 'colnames', 'caption', ..."))),
                                                    tags$li("layout and columns styles:", HTML("&nbsp;"),
                                                            tags$b(tags$i("i.e. 'container', 'formatStyle', ..."))),
                                                    tags$li("other addons:", HTML("&nbsp;"),
                                                            tags$b(tags$i("i.e. 'filter', 'callback', ..."))))),
                                    tags$li("For more information about table options please visit the",
                                            tags$a("DT documentation", target = "_blank", href = "https://rstudio.github.io/DT/"),
                                            "site")
                            ))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$li("Module should be configured in both UI and Server code"),
                            tags$li("In the 'body_ui.R' add the following lines:",
                                    blockQuote("downloadableTableUI('exampleDT1',
                                                                    list('csv', 'tsv'),
                                                                    'Download table data'))", color = "info")),
                            tags$li("In the 'server_local.R' add the following lines (make sure the two use the same ID):",
                                    blockQuote("downloadableTable('exampleDT1',
                                                                  ss_userAction.Log,
                                                                  'exampletable',
                                                                  list(csv = load_data3, tsv = load_data3))", color = "info")),
                            tags$li("Review ", tags$b("?downloadableTableUI"), " and ", tags$b("?downloadableTable"), " for more information"),
                            tags$li("Review below table for detailed example code")))),
    fluidRow(column(width = 12,
                    downloadableTableUI("exampleDT1",
                                        list("csv", "tsv"),
                                        "Download table data")))

)


react_table_box <- box(
    id          = "react_downloader",
    title       = "React Table Downloader",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$ul(tags$li("React table downloader module displays tabular data in rich formatted tables using the", tags$b("`reactable`"), "package"),
                                    tags$li("downloadableReactTable returns a named reactive list that can be used in the application:",
                                            tags$ul(tags$li(tags$b("selected_rows"), ": data.frame of current selected rows"),
                                                    tags$li(tags$b("table_state"), ": a list of the current table state"))),
                                    tags$li("Table data can be downloaded in different formats such as: ", tags$b("'csv'"),
                                            ", ", tags$b("'tsv'"), ", ", tags$b("'txt'"), "and/or ", tags$b("'xlsx'")),
                                    tags$li("User can customize downloadableReactTable modules using reactable package options."),
                                    tags$li("For more information about table options please visit the",
                                            tags$a("Reactable documentation", target = "_blank", href = "https://glin.github.io/reactable/"),
                                            "site")
                            ))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$li("Module should be configured in both UI and Server code"),
                            tags$li("In the 'body_ui.R' add the following lines:",
                                    blockQuote("downloadableReactTableUI('exampleReactTable',
                                                                         'Download react table data'))", color = "info")),
                            tags$li("In the 'server_local.R' add the following lines (make sure the two use the same ID):",
                                    blockQuote("downloadableReactTable('exampleReactTable',
                                                                       ss_userAction.Log,
                                                                       'exampletable',
                                                                        list(csv = load_data3, tsv = load_data3))", color = "info")),
                            tags$li("Review ", tags$b("?downloadableReactTableUI"), " and ", tags$b("?downloadableReactTable"), " for more information"),
                            tags$li("Review below table for detailed example code")))),
    fluidRow(column(width = 12,
                    downloadableReactTableUI(id            = "exampleReactTable",
                                             downloadtypes = list("csv", "tsv"),
                                             hovertext     = "Download react table data")))

)


file_downloader_box <- box(
    id          = "file_downloader",
    title       = "File Downloader",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$ul(tags$li("File downloader module provides the user with the ability to download any server generated data"),
                                    tags$li("Data can be downloaded in any format available for both 'Table Downloader' and 'Plot Downloader' modules"),
                                    tags$li("Data can be downloaded in single format or in multiple formats, based on module configurations")))),
             column(width = 6,
                    tags$dl(
                        tags$dt("Setup"),
                        tags$ul(tags$li("Module should be configured in both UI and Server code",
                                        blockQuote(p(pre("U: downloadFileButton('uiID', list(extensions))"),
                                                     pre("S: downloadFile('uiID', logger, 'filenameroot', list(datafxns)")),
                                                   color = "info")),
                                tags$li("Review ", tags$b("?downloadFileButton"), " and ", tags$b("?downloadFile"), " for more information"),
                                tags$li("Review below file downloaders for detailed example code"))))),
    fluidRow(column(width = 12,
                    tags$dl(tags$dt("Example"),
                            "Single Download: ",
                            downloadFileButton("exampleDownload1", c("csv"), "csv"),
                            "Multiple-choice Download: ",
                            downloadFileButton("exampleDownload2",
                                               c("csv", "xlsx", "tsv"), "Download options"))))
)

plot_downloader_box <- box(
    id          = "plot_downloader",
    title       = "Plot Downloader - ggplot2 & lattice",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$ul(tags$li("This module creates a custom plot output that provide download capability",
                                            "to plot tools that have no internal download ability like ggplot2 & lattice."),
                                    tags$li("Plot can be downloaded in different formats such as: ", tags$b("'png'"),
                                            ", ", tags$b("'jpeg'"), ", ", tags$b("'tiff'"), "and/or ", tags$b("'bmp'")),
                                    tags$li("Plot data itself can be downloaded in different formats such as: ", tags$b("'csv'"),
                                            ", ", tags$b("'tsv'"), ", ", tags$b("'txt'"), "and/or ", tags$b("'xlsx'")),
                                    tags$li("To enrich user experience, plot downloader module also can take advanced shiny parameters as 'clickOpts', 'hoverOpts' or 'brushOpts'"),
                                    tags$li("Plot dimensions can be easily adapted"),
                                    tags$li("plot download button location can be easily controlled")))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$ul(tags$li("Module should be configured in both the UI and Server code"),
                                    tags$li("In the 'body_ui.R' add the following lines:",
                                            blockQuote("downloadablePlotUI('myplotID', c('png', 'csv'), 'Download Plot or Data', '300px'))", color = "info")),
                                    tags$li("In the 'server_local.R' add the following lines (make sure the two use the same ID):",
                                            blockQuote("downloadablePlot('myplotID',
                                                                  ss_userAction.Log,
                                                                  filenameroot = 'mydownload1',
                                                                  downloadfxns = list(png = myplotfxn, tsv = mydatafxn))", color = "info")),
                                    tags$li(blockQuote("Run '?periscope2::downloadablePlot' for more info", color = "info")),
                                    tags$li("Review below table for detailed example code"))))),
    fluidRow(column(width = 12,
                    tags$h3("Example")),
             tags$br(),
             column(width = 6, downloadablePlotUI("examplePlot2",
                                                  list("jpeg", "csv"),
                                                  "Download plot or data",
                                                  btn_halign  = "left",
                                                  btn_valign  = "top",
                                                  btn_overlap = FALSE,
                                                  hoverOpts   = plot2_hover)),
             column(width = 6, downloadablePlotUI("examplePlot3",
                                                  list("png", "tiff",
                                                       "txt", "tsv"),
                                                  btn_overlap = FALSE,
                                                  "Download plot or data"))),
    uiOutput("hover_info")
)

reset_application_box <- box(
    id          = "reset_application",
    title       = "Reset Application",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    fluidRow(column(width = 6,
                    tags$dl(tags$dt("Features"),
                            tags$ul(tags$li("appReset module provides the ability to:"),
                                    tags$ul(tags$li("Resets a user's session"),
                                            tags$li("Rolls over their log.")),
                                    tags$li("It creates a toggle button to reset application session"),
                                    tags$li("Upon pressing on the button, its state is flipped to 'cancel application reload'",
                                            " with application and console warning messages indicating that the application will be reloaded "),
                                    tags$li("User can either resume reloading application session or cancel reloading process,",
                                            tags$em(" which will also generate application and console messages to indicate reloading status and result.")),
                                    tags$li("Waiting time before actual session reset is configurable in milliseconds"),
                                    tags$li("Generated module alerts location can be customized to be displayed in any place, default is application body")))),
             column(width = 6,
                    tags$dl(tags$dt("Setup"),
                            tags$ul(tags$li("Functionality can be added with the following code:",
                                            blockQuote(p(pre("U: appResetButton('appResetId')"),
                                                         pre("S: appReset(id = 'appResetId', reset_wait = 5000, logger = logger)")),
                                                       color = "info")),
                                    tags$li(blockQuote("Run '?periscope2::appResetButton' and '?periscope2::appReset' for more info", color = "info")),
                                    tags$li("Review example below for detailed code"))))),
    fluidRow(tags$dl(tags$dt("Example"),
                     appResetButton("appResetId")))
)


periscope_modules <- tabItem(tabName = "periscope_modules",
                             react_table_box,
                             table_downloader_box,
                             plot_downloader_box,
                             file_downloader_box,
                             reset_application_box)

### User Notifications Elements
busy_indicator_box <- box(
    id          = "busy_indicator",
    title       = "Busy Indicators",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    p("Periscope2 has two application busy indicators"),
    tags$dl(tags$dt("Application Loading"),
            tags$li("Any 'waiter' package loading screen indicator that displayed upon application start up or reload."),
            tags$li("Loading screen can be changed by updating ",
                    tags$b("'loading_indicator'"), " parameter in ", tags$b("'set_app_parameters'"), " method in ",
                    tags$b("'program/global.R'"), " 'waiter' package loading screen"),
            tags$li("For more information about loadnig screens options please visit the",
                    tags$a("waiter documentation", target = "_blank", href = "https://waiter.john-coene.com/"),
                    "site"),
            tags$li("Check example below:"),
            br(),
            br(),
            tags$dd(actionButton("showAppLoading",
                                 label  = "Show application Loading",
                                 status = "danger")),
            hr(),
            tags$dt("Busy Session"),
            tags$li("An automatic wait indicator in the navbar when the shiny server session is busy."),
            tags$dd(actionButton("showWorking",
                                 label  = "Show application busy indicator for 5 seconds",
                                 status = "info")))
)

alerts_box <- box(
    id          = "alerts",
    title       = "Alerts",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    p("Alerts can be added with the following code in the server:"),
    blockQuote(tags$pre("S:periscope2::createPSAlert(id = 'alert place',
                         options  = list(title    = 'alert title',
                         status   = 'alert status'',
                         closable = TRUE,
                         content  = alert contents))"), color = "info"),
    hr(),
    tags$h5("Examples"),
    p("LOCATION can be: 'sidebarRightAlert', 'sidebarBasicAlert', 'bodyAlert', 'footerAlert' and 'headerAlert'"),
    conditionalPanel("$('.main-sidebar').length > 0",
                     actionButton("leftAlert",
                                  label  = "Left Alert",
                                  status = "warning",
                                  width  = "15%"),
                     style = "display: inline;"),
    actionButton("bodyAlertBtn",
                 label  = "Body",
                 status = "info",
                 width  = "15%"),
    conditionalPanel("$('.control-sidebar').length > 0",
                     actionButton("rightAlert",
                                  label  = "Right Alert",
                                  status = "success",
                                  width  = "15%"),
                     style = "display: inline;"),
    conditionalPanel("$('.main-footer').length > 0",
                     actionButton("footerAlertbtn",
                                  label  = "Footer Alert",
                                  status = "danger",
                                  width  = "15%"),
                     style = "display: inline;"),
    actionButton("headerAlertbtn",
                 label  = "Header Alert",
                 status = "primary",
                 width  = "15%")
)

tooltips_box <- box(
    id          = "tooltips",
    title       = "Tooltips",
    status      = "info",
    solidHeader = TRUE,
    collapsible = TRUE,
    width       = 12,
    p("Tooltips can be added with the following code in the UI:"),
    blockQuote("U: ui_tooltip('tooltipID', 'label text (optional)', 'text content', 'placement' (optional))", color = "info"),
    tags$h5("Examples"),
    ui_tooltip(id    = "top_tip",
               label = "Top Tooltips",
               text  = "Top tooltip"),
    ui_tooltip(id        = "left_tip",
               label     = "Left Tooltips",
               text      = "Left tooltip",
               placement = "left"),
    ui_tooltip(id        = "bottom_tip",
               label     = "Bottom Tooltips",
               text      = "Bottom tooltip",
               placement = "bottom"),
    ui_tooltip(id        = "right_tip",
               label     = "Right Tooltips",
               text      = "Right tooltip",
               placement = "right")
)


# -- Register Elements in the ORDER SHOWN in the UI
add_ui_body(list(about_box,
                 files_organization_box,
                 announcements_box,
                 table_downloader_box,
                 plot_downloader_box,
                 fluidRow(file_downloader_box,
                          reset_application_box),
                 logger_box,
                 busy_indicator_box,
                 alerts_box,
                 tooltips_box))

# FRAMEWORK HELPER FUNCTIONS ---------------------------------
# -- (Used in shiny with ::: but not exported as user fxns) --


# Framework Server Setup
fw_server_setup <- function(input, output, session, logger, logger_viewer_id = "logViewer") {
    logfile <- shiny::isolate(.setup_logging(session, logger))
    logViewer(logger_viewer_id, logfile)
}


# Get Logging Level
fw_get_loglevel <- function() {
    shiny::isolate(.g_opts$loglevel)
}

# Get Application Title
fw_get_title <- function() {
    shiny::isolate(.g_opts$app_title)
}

# Get Application Version
fw_get_version <- function() {
    shiny::isolate(.g_opts$app_version)
}

# Get User Action Log
fw_get_user_log <- function() {
    getLogger(name = "actions")
}


#' create_application_dashboard
#'
#' Creates application final dashboard from application different settings and configurations.
#' It is called once in the application in "ui.R". It should not be modified or updated by user
#'
#' @keywords internal
#' @noRd
create_application_dashboard <- function() {
    list(
        shiny::fluidRow(shiny::column(width = 12, shiny::div(id = "announceAlert"))),
        shiny::fluidRow(shiny::column(width = 12, shiny::div(id = "headerAlert"))),
        shiny::fluidRow(shiny::column(width = 12,
                        bs4Dash::bs4DashPage(title      = shiny::isolate(.g_opts$app_title),
                                             header     = shiny::isolate(.g_opts$header),
                                             body       = bs4Dash::bs4DashBody(shiny::isolate(.g_opts$body_elements)),
                                             sidebar    = do.call(bs4Dash::bs4DashSidebar, shiny::isolate(.g_opts$left_sidebar)),
                                             controlbar = shiny::isolate(.g_opts$right_sidebar),
                                             footer     = shiny::isolate(.g_opts$footer),
                                             freshTheme = create_theme(),
                                             dark       = NULL,
                                             help       = NULL,
                                             preloader  = shiny::isolate(.g_opts$loading_indicator)))))
}

create_theme <- function() {
    bs4dash_vars_keys <- fresh::search_vars_bs4dash()[["variable"]]
    theme_settings    <- load_theme_settings()
    status            <- list()
    sidebar_colors    <- list()
    sidebar_layout    <- list()
    main_colors       <- list()
    colors_contrast   <- list()
    other_variables   <- list()

    # keys
    status_keys         <- c("primary", "secondary", "success", "info",
                              "warning", "danger", "light", "dark")
    sidebar_colors_keys <- c("bg", "hover_bg", "color", "hover_color", "active_color",
                              "submenu_bg", "submenu_color", "submenu_hover_color", "submenu_hover_bg",
                              "submenu_active_color", "submenu_active_bg", "header_color")
    sidebar_layout_keys <- c("sidebar_width", "control_sidebar_width", "sidebar_padding_x", "sidebar_padding_y",
                              "sidebar_mini_width")
    main_colors_keys    <- c("blue", "lightblue", "navy", "cyan", "teal", "olive", "green", "lime", "orange", "yellow",
                              "fuchsia", "purple", "maroon", "red", "black", "gray_x_light", "gray_600", "gray_800",
                              "gray_900", "white")
    contrast_keys       <- c("contrasted_threshold", "text_dark", "text_light")

    if (!is.null(theme_settings) && is.list(theme_settings)) {
        # colors check
        for (color_key in c(status_keys,
                            sidebar_colors_keys,
                            main_colors_keys,
                            c("text_dark", "text_light"))) {
            if ((color_key %in% names(theme_settings)) &&
                !is_valid_color(theme_settings[[color_key]])) {
                warning(color_key, " has invalid color value. Setting default color.")
                theme_settings[color_key] <- NULL
            }
        }

        # measures check
        for (measure_key in c(sidebar_layout_keys, "contrasted_threshold")) {
            measure <- theme_settings[[measure_key]]

            if (!is.null(measure)) {
                if (is.na(as.numeric(measure)) || measure <= 0) {
                    warning(measure, " must be positive value. Setting default value.")
                    theme_settings[[measure_key]] <- NULL
                } else {
                    theme_settings[[measure_key]] <- paste0(measure, "px")
                }
            }
        }

        status          <- theme_settings[status_keys[status_keys %in% names(theme_settings)]]
        sidebar_colors  <- theme_settings[sidebar_colors_keys[sidebar_colors_keys %in% names(theme_settings)]]
        sidebar_layout  <- theme_settings[sidebar_layout_keys[sidebar_layout_keys %in% names(theme_settings)]]
        main_colors     <- theme_settings[main_colors_keys[main_colors_keys %in% names(theme_settings)]]
        colors_contrast <- theme_settings[contrast_keys[contrast_keys %in% names(theme_settings)]]
        other_variables <- theme_settings[bs4dash_vars_keys[bs4dash_vars_keys %in% names(theme_settings)]]

    }

    fresh::create_theme(
        do.call(fresh::bs4dash_status, status),
        do.call(fresh::bs4dash_sidebar_light, sidebar_colors),
        do.call(fresh::bs4dash_layout, sidebar_layout),
        do.call(fresh::bs4dash_color, main_colors),
        do.call(fresh::bs4dash_yiq, colors_contrast),
        do.call(fresh::bs4dash_vars, other_variables)
    )
}

is_valid_color <- function(color) {
    tryCatch({
        grDevices::col2rgb(color)
        TRUE
    },
    error = function(e) {
        FALSE
    })
}


#' load_announcements
#'
#' Reads and parses application announcements configurations in \code{config/announce.yaml}, then display announcements in
#' application header.
#'
#' If announce.yaml does not exist or contains invalid configurations. Nothing will be displayed.
#' Closing announcements is caller application responsibility
#'
#' @param announcements_file_path   The path to announcements configuration file.\cr
#'                                  Use \link[periscope2]{announcementConfigurationsAddin} to generate that file.
#' @param announcement_location_id  Announcement target location div id (default = "announceAlert")
#'
#' @return number of seconds an announcement should be staying in caller application
#'
#' @examples
#'     load_announcements(system.file("fw_templ/announce.yaml", package = "periscope2"))
#'
#' @seealso \link[periscope2:announcementConfigurationsAddin]{periscope2:announcementConfigurationsAddin()}
#' @export
load_announcements <- function(announcements_file_path  = NULL,
                               announcement_location_id = "announceAlert") {
    announce_setup     <- NULL
    auto_close         <- NULL
    announcements_file <- announcements_file_path

    # use configured 'set_app_parameters' parameter 'announcements_file' value
    # to support backward compatibility
    if (is.null(announcements_file)) {
        announcements_file <- shiny::isolate(.g_opts$announcements_file)
    }

    if ((!is.null(announcements_file)) &&
        (file.exists(announcements_file))) {
        tryCatch({
            announce_setup <- yaml::read_yaml(announcements_file)
            if (any(is.null(announce_setup),
                    !is.list(announce_setup),
                    length(announce_setup) < 1)) {
                logwarn(paste("File",
                              announcements_file,
                              "is empty or corrupted. Announcements will be ignored"))
                announce_setup <- NULL
            }
        },
        error = function(e) {
            logwarn(paste("Could not parse", announcements_file, "due to", e$message))
        })

        if (!is.null(announce_setup)) {
            start_date        <- announce_setup[["start_date"]]
            start_date_format <- announce_setup[["start_date_format"]]
            end_date          <- announce_setup[["end_date"]]
            end_date_format   <- announce_setup[["end_date_format"]]
            style             <- announce_setup[["style"]]
            title             <- announce_setup[["title"]]
            text              <- announce_setup[["text"]]
            auto_close        <- announce_setup[["auto_close"]]
            valid             <- TRUE

            if (!is.null(start_date)) {
                if (!is_valid_format(start_date, start_date_format)) {
                    if (is.null(start_date_format)) {
                        start_date_format <- ""
                    }
                    logwarn(paste("Announcement 'start_date' value '",
                                  start_date,
                                  "' could not be converted to a valid date",
                                  " with the given 'start_date_format' value: '",
                                  start_date_format, "' "))
                    valid <- FALSE
                } else {
                    start_date <- lubridate::as_date(start_date, format = start_date_format)
                }
            }

            if (valid && !is.null(end_date)) {
                if (!is_valid_format(end_date, end_date_format)) {
                    if (is.null(end_date_format)) {
                        end_date_format <- ""
                    }
                    logwarn(paste("Announcement 'end_date' value '",
                                  end_date,"' could not be converted to a valid date",
                                  " with the given 'end_date_format' value: '", end_date_format, "' "))
                    valid <- FALSE
                } else {
                    end_date <- lubridate::as_date(end_date, format = end_date_format)
                }
            }

            if (valid && !(any(is.null(start_date), start_date <= Sys.Date()) &&
                           any(is.null(end_date), Sys.Date() <= end_date))) {
                valid <- FALSE
            }

            valid_styles <- c("info", "danger", "success", "warning", "primary")
            style        <- tolower(style)

            if (valid && any(length(style) == 0,
                             style == "",
                             !(tolower(style) %in% valid_styles))) {
                logwarn(paste("Announcement 'style' must be one of ",
                              paste(valid_styles, collapse = ", ")))
                valid <- FALSE
            }

            if (valid && any(is.null(text), text == "")) {
                logwarn("Announcement 'text' value is empty. It must contain non empty text value")
                valid <- FALSE
            }

            if (valid) {
                createPSAlert(id      = announcement_location_id,
                              options = list(title    = title,
                                             status   = style,
                                             closable = TRUE,
                                             content  = text))
                if (is.null(auto_close) ||
                    !is.numeric(auto_close) ||
                    auto_close == 0) {
                    logwarn(paste("Announcement 'auto_close' value '",
                                  auto_close,
                                  "' is invalid. It must contain greater than 0 numeric value.",
                                  "Setting 'auto_close' to default value 'NULL'"))
                    auto_close <- NULL
                } else {
                    auto_close <- auto_close * 1000
                }
            }
        }
    }
    auto_close
}

is_valid_format <- function(x, format = NULL) {
    valid_format <- FALSE

    tryCatch({
        date <- lubridate::as_date(x, format = format)
        if (!is.na(date)) {
            valid_format <- TRUE
        }
    },
    warning = function(w) {
        logerror(paste("Could not convert date: '",
                       x,
                       "' with format: '",
                       format,
                       "' with error: '", w$message,
                       "'"))
    })

    valid_format
}

load_theme_settings <- function() {
    theme_settings <- NULL

    if (file.exists("www/periscope_style.yaml")) {
        tryCatch({
            theme_settings <- yaml::read_yaml("www/periscope_style.yaml")
        },
        error = function(e){
            warning("Could not parse 'periscope_style.yaml' due to: ", e$message)
        })
    }

    theme_settings
}

context("periscope2 - misc functions")

log_directory <- tempdir()

test_that("get_url_parameters - NULL", {
    result <- get_url_parameters(NULL)
    expect_equal(result, list(), "get_url_parameters")
})

test_that("get_url_parameters", {
    fake_session <- list(clientData = list(url_search = "&test1=ABC&test2=123"))
    result       <- get_url_parameters(fake_session)
    expect_equal(result, list(test1 = "ABC", test2 = "123"), "get_url_parameters")
})

test_that("fw_get_user_log", {
    result <- periscope2:::fw_get_user_log()
    expect_equal(class(result)[1], "Logger")
})

test_that("setup_logging", {
    result <- shiny::isolate(.setup_logging(NULL, periscope2:::fw_get_user_log()))
    expect_true(shiny::is.reactive(result))
})

test_that("setup_logging existing log", {
    logger <- periscope2:::fw_get_user_log()
    file.create(paste0(paste(log_directory, logger$name, sep = .Platform$file.sep), ".log"))

    result <- shiny::isolate(.setup_logging(NULL, logger))
    expect_true(shiny::is.reactive(result))
})

test_that("fw_server_setup", {
    local_edition(3)
    expect_snapshot_error(fw_server_setup(input = list(),
                                          output = list(),
                                          session = MockShinySession$new(),
                                          logger = periscope2:::fw_get_user_log()))
})

test_that("is_valid_color", {
    expect_true(is_valid_color("green"))
    expect_false(is_valid_color("not color"))
})

test_that("is_valid_format - valid format", {
    expect_true(periscope2:::is_valid_format("2023-07-07"))
})

# clean up
unlink("log", TRUE)

context("periscope2 - downloadablePlot")

test_that("downloadablePlotUI - default values", {
    plot_ui <- downloadablePlotUI(id = "myid")
    expect_equal(length(plot_ui), 2)
    expect_true(grepl('id="myid-dplotOutputID"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('style="width:100%;height:400px;"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid-dplotButtonDiv"', plot_ui[[2]], fixed = TRUE))
    expect_false(grepl('title="myhovertext"', plot_ui[[2]], fixed = TRUE))
    expect_true(grepl('"myid-dplotButtonID-png"', plot_ui[[2]], fixed = TRUE))
})

test_that("downloadablePlotUI - no downloadable types", {
    plot_ui <- downloadablePlotUI(id            = "myid",
                                  downloadtypes = NULL)
    expect_equal(length(plot_ui), 2)
    expect_true(grepl('id="myid-dplotOutputID"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('style="width:100%;height:400px;"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid-dplotButtonDiv"', plot_ui[[2]], fixed = TRUE))
    expect_false(grepl('title="myhovertext"', plot_ui[[2]], fixed = TRUE))
    expect_false(grepl('"myid-dplotButtonID"', plot_ui[[2]], fixed = TRUE))
})

test_that("downloadablePlotUI btn_overlap=true btn_halign=left btn_valign=bottom", {
    plot_ui <- downloadablePlotUI(id                 = "myid",
                                  downloadtypes      = c("png"),
                                  download_hovertext = "myhovertext",
                                  width              = "80%",
                                  height             = "300px",
                                  btn_halign         = "left",
                                  btn_valign         = "bottom",
                                  btn_overlap        = TRUE,
                                  clickOpts          = NULL,
                                  hoverOpts          = NULL,
                                  brushOpts          = NULL)
    expect_equal(length(plot_ui), 2)
    expect_true(grepl('id="myid-dplotOutputID"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('style="width:80%;height:300px;"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid-dplotButtonDiv"', plot_ui[[2]], fixed = TRUE))
    expect_true(grepl('title="myhovertext"', plot_ui[[2]], fixed = TRUE))
    expect_true(grepl('"myid-dplotButtonID-png"', plot_ui[[2]], fixed = TRUE))
})

test_that("downloadablePlotUI btn_overlap=false btn_halign=center btn_valign=top", {
    plot_ui <- downloadablePlotUI(id                 = "myid",
                                  downloadtypes      = c("png"),
                                  download_hovertext = "myhovertext",
                                  width              = "80%",
                                  height             = "300px",
                                  btn_halign         = "center",
                                  btn_valign         = "top",
                                  btn_overlap        = FALSE,
                                  clickOpts          = NULL,
                                  hoverOpts          = NULL,
                                  brushOpts          = NULL)
    expect_equal(length(plot_ui), 2)
    expect_true(grepl('id="myid-dplotButtonDiv"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('style="display:inherit; padding: 5px;float:none;margin-left:45%;top: -5px"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid-dplotButtonID-png"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('title="myhovertext"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid-dplotOutputID"', plot_ui[[2]], fixed = TRUE))
})

test_that("downloadablePlotUI invalid btn_halign", {
    expect_warning(downloadablePlotUI(id                 = "myid",
                                      downloadtypes      = c("png"),
                                      download_hovertext = "myhovertext",
                                      width              = "80%",
                                      height             = "300px",
                                      btn_halign         = "bottom",
                                      btn_valign         = "top",
                                      btn_overlap        = FALSE,
                                      clickOpts          = NULL,
                                      hoverOpts          = NULL,
                                      brushOpts          = NULL),
                   "bottom  is not a valid btn_halign input - using default value. Valid values: <'left', 'center', 'right'>")
})

test_that("downloadablePlotUI invalid btn_valign", {
    download_warnings <- capture_warnings(plot_ui <- downloadablePlotUI(id                 = "myid",
                                                                        downloadtypes      = c("png"),
                                                                        download_hovertext = "myhovertext",
                                                                        width              = "80%",
                                                                        height             = "300px",
                                                                        btn_halign         = "bottom",
                                                                        btn_valign         = "center",
                                                                        btn_overlap        = FALSE,
                                                                        clickOpts          = NULL,
                                                                        hoverOpts          = NULL,
                                                                        brushOpts          = NULL))
    expect_equal(length(download_warnings), 2)
    expect_equal("bottom  is not a valid btn_halign input - using default value. Valid values: <'left', 'center', 'right'>",
                 download_warnings[1])
    expect_equal("center  is not a valid btn_valign input - using default value. Valid values: <'top', 'bottom'>",
                 download_warnings[2])
    expect_equal(length(plot_ui), 2)
    expect_true(grepl('id="myid-dplotOutputID"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('style="width:80%;height:300px;"', plot_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid-dplotButtonDiv"', plot_ui[[2]], fixed = TRUE))
    expect_true(grepl('title="myhovertext"', plot_ui[[2]], fixed = TRUE))
    expect_true(grepl('id="myid-dplotButtonID-png"', plot_ui[[2]], fixed = TRUE))
})


download_plot <- function() {
    plot <- ggplot2::ggplot(data = mtcars, aes(x = wt, y = mpg)) +
        geom_point(aes(color = cyl)) +
        ggtitle("GGPlot Example w/Hover") +
        xlab("wt") +
        ylab("mpg")
    if (packageVersion("ggplot2") >= "3.5.0") {
        plot <- plot +
            theme(legend.justification   = c(1, 1),
                  legend.position.inside = c(1, 1),
                  legend.title           = element_blank())
    } else {
        plot <- plot +
            theme(legend.justification = c(1, 1),
                  legend.position      = c(1, 1),
                  legend.title         = element_blank())
    }
}

download_data <- function() {
    head(mtcars)
}

test_that("downloadablePlot", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    testServer(downloadablePlot,
               args = list(logger = periscope2:::fw_get_user_log(),
                           filenameroot = "mydownload1",
                           aspectratio  = 2,
                           downloadfxns = list(png  = download_plot,
                                               png2 = download_plot,
                                               tiff = download_plot,
                                               txt  = download_data,
                                               tsv  = download_data),
                           visibleplot  = download_plot),
               expr = {
                   session$setInputs(visibleplot = download_plot)
                   session$setInputs(downloadfxns = list(png  = download_plot,
                                                         png2 = download_plot,
                                                         tiff = download_plot,
                                                         txt  = download_data,
                                                         tsv  = download_data))
                   expect_equal(output$dplotOutputID$width, 600)
               })
})


test_that("downloadablePlot- default values", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    testServer(downloadablePlot,
               args = list(visibleplot = download_plot),
               expr = {
                   session$setInputs(visibleplot = download_plot)
                   expect_equal(output$dplotOutputID$width, 600)
               })
})

context("periscope2 - App reset")

test_that(".appResetButton", {
    reset_btn <- appResetButton("myid")

    expect_true(grepl('<span>Cancel Application Reset</span>', reset_btn, fixed = TRUE))
    expect_true(grepl('id="myid-resetButton"', reset_btn, fixed = TRUE))
})

test_that("app_reset - no reset button", {
    expect_silent(testServer(appReset,
                             args = list(reset_wait = 5000,
                                         logger     = periscope2:::fw_get_user_log()),
                             expr = {session$setInputs(resetPending = NULL)}))
})

test_that("app_reset - reset button - no pending - defaul time", {
    reset_output <- capture_output(testServer(appReset,
                                              args = list(reset_wait     = 5000,
                                                          alert_location = "headerAlert",
                                                          logger         = periscope2:::fw_get_user_log()),
                                              expr = {session$setInputs(resetButton  = TRUE,
                                                                        resetPending = FALSE)}))
    expect_match(reset_output, "Resetting in  5 seconds")
})

test_that("app_reset - reset button - no pending - defaul time", {
    reset_output <- capture_output(testServer(appReset,
                                              args = list(reset_wait     = 5000,
                                                          alert_location = "headerAlert2",
                                                          logger         = periscope2:::fw_get_user_log()),
                                              expr = {session$setInputs(resetButton  = TRUE,
                                                                        resetPending = FALSE)}))
    expect_match(reset_output, "Resetting in  5 seconds")
})

test_that("app_reset - reset button - no pending - custom time", {
    reset_output <- capture_output(testServer(appReset,
                                              args = list(reset_wait = 3000,
                                                          logger     = periscope2:::fw_get_user_log()),
                                              expr = {session$setInputs(resetButton  = TRUE,
                                                                        resetPending = FALSE)}))
    expect_match(reset_output, "Resetting in  3 seconds")
})

test_that("app_reset - reset button - no pending - no passed time", {
    reset_output <- capture_output(testServer(appReset,
                                              args = list(logger = periscope2:::fw_get_user_log()),
                                              expr = {session$setInputs(resetButton  = TRUE,
                                                                        resetPending = FALSE)}))
    expect_match(reset_output, "Resetting in  5 seconds")
})

test_that("app_reset - no reset button - with pending", {
    reset_output <- capture_output(testServer(appReset,
                                              args = list(logger = periscope2:::fw_get_user_log()),
                                              expr = {session$setInputs(resetButton  = FALSE,
                                                                        resetPending = TRUE)}))
    expect_match(reset_output, "Application Reset cancelled by user")

})

test_that("app_reset - reset button - with pending", {
    reset_output <- capture_output(testServer(appReset,
                                              args = list(logger = periscope2:::fw_get_user_log()),
                                              expr = {session$setInputs(resetButton  = TRUE,
                                                                        resetPending = TRUE)}))
    expect_match(reset_output, "WARNING:actions:Application Reset")
})

test_that("app_reset - no reset button - no pending", {
    expect_silent(testServer(appReset,
                             args = list(logger = periscope2:::fw_get_user_log()),
                             expr = {session$setInputs(resetButton  = FALSE,
                                                       resetPending = FALSE)}))
})

# ----------------------------------------
# --       PROGRAM ui_left_sidebar.R         --
# ----------------------------------------
# USE: Create UI elements for the
#      application sidebar (left side on
#      the desktop; contains options) and
#      ATTACH them to the UI by calling
#      add_ui_left_sidebar()
#
# NOTES:
#   - All variables/functions here are
#     not available to the UI or Server
#     scopes - this is isolated
# ----------------------------------------

# -- IMPORTS --



# ----------------------------------------
# --     SIDEBAR ELEMENT CREATION       --
# ----------------------------------------

# -- Left Sidebar Configurations
# Sidebar skin. "dark" or "light"
skin <- "light"

# Sidebar status
status <- "info"

# Sidebar elevation. 4 by default (until 5).
elevation <- 4

# If TRUE, the sidebar will be collapsed on app startup.
collapsed <- FALSE

# Whether to slightly close the sidebar but still show item icons
minified <- FALSE

# Whether to expand the sidebar om hover
expand_on_hover <- FALSE

# Whether to fix the sidebar
fixed <- TRUE

# -- Create Sidebar Elements
# elements to be placed before any menu
sidebar_elements <- NULL

# sidebar menu items
sidebar_menu <-  sidebarMenu(
    sidebarHeader("Periscope2 Features"),
    menuItem(
        "Application Setup",
        tabName = "application_setup",
        icon    = icon("building")
    ),
    menuItem(
        "Periscope2 Modules",
        tabName = "periscope_modules",
        icon    = icon("cubes")
    ),
    menuItem(
        "User Notifications",
        tabName = "user_notifications",
        icon    = icon("comments")
    )
)

# An area at the bottom of the sidebar to contain elements like buttons
custom_area <- NULL

# -- Register Sidebar Elements in the ORDER SHOWN in the UI
add_ui_left_sidebar(sidebar_elements = sidebar_elements,
                    skin             = skin,
                    status           = status,
                    elevation        = elevation,
                    collapsed        = collapsed,
                    minified         = minified,
                    expand_on_hover  = expand_on_hover,
                    fixed            = fixed,
                    sidebar_menu     = sidebar_menu,
                    custom_area      = custom_area)

require(testthat)
require(shiny)
require(bs4Dash)
require(periscope2)
require(ggplot2)

if (interactive()) {
    test_source_path <- "periscope2/R"
    invisible(
        lapply(list.files(test_source_path),
               FUN = function(x) source(file.path(test_source_path, x))))
    rm(test_source_path)
}


reset_g_opts <- function() {
  .g_opts$tt_image           <- "img/tooltip.png"
  .g_opts$tt_height          <- "16px"
  .g_opts$tt_width           <- "16px"
  .g_opts$datetime.fmt       <- "%m-%d-%Y %H:%M"
  .g_opts$log.formatter      <- function(record) { paste0(record$logger, " [", record$timestamp, "] ", record$msg) }
  .g_opts$loglevel           <- "DEBUG"
  .g_opts$app_title          <- "Set using add_ui_header() in program/ui_header.R"
  .g_opts$app_info           <- NULL
  .g_opts$app_version        <- "1.0.0"
  .g_opts$loading_indicator  <- NULL
  .g_opts$announcements_file <- NULL
  .g_opts$data_download_types <- c("csv", "xlsx", "tsv", "txt")
  .g_opts$plot_download_types <- c("png", "jpeg", "tiff", "bmp")
  .g_opts$left_sidebar       <- list(disable = TRUE)
  .g_opts$body_elements      <- c()
  .g_opts$header             <- NULL
  .g_opts$right_sidebar      <- NULL
  .g_opts$footer             <- NULL
}

# -------------------------------------------------
# --               FRAMEWORK ui.R                --
# -------------------------------------------------
# NOTES:                                         --
# Program code goes in "program" directory files --
# -------------------------------------------------
# *****        DO NOT EDIT THIS FILE       ***** --
# -------------------------------------------------

library(bs4Dash)

source(paste("program", "ui_body.R", sep = .Platform$file.sep), local = TRUE)
source(paste("program", "ui_header.R", sep = .Platform$file.sep), local = TRUE)

source(paste("program", "ui_right_sidebar.R", sep = .Platform$file.sep), local = TRUE)
source(paste("program", "ui_footer.R", sep = .Platform$file.sep), local = TRUE)
periscope2:::create_application_dashboard()

# TEMPLATE SETUP FUNCTIONS ------

#' Create a new templated framework application
#'
#' Creates ready-to-use templated application files using the periscope2
#' framework. The application can be created either empty (default) or with a
#' sample/documented example application.\cr \cr
#'
#' @param name          name for the new application and directory
#' @param location      base path for creation of \code{name}
#' @param sample_app    whether to create a sample shiny application
#' @param left_sidebar  whether the left sidebar should be enabled. It can be TRUE/FALSE
#' @param right_sidebar parameter to set the right sidebar. It can be TRUE/FALSE
#'
#' @return no return value, creates application folder structure and files
#'
#' @section Name:
#' The \code{name} directory must not exist in \code{location}.  If the code
#' detects that this directory exists it will abort the creation process with
#' a warning and will not create an application template.
#'
#' Use only filesystem-compatible characters in the name (ideally w/o spaces)
#'
#' @section Directory Structure:
#'
#' \preformatted{
#' name
#'  -- log (log files)
#'  -- program (user application)
#'  -- -- config   (application configuration files)
#'  -- -- data     (user application data)
#'  -- -- fxn      (user application function)
#'  -- -- modules  (application modules files)
#'  -- www (supporting shiny files)
#'  -- -- css  (application css files)
#'  -- -- img  (application image files)
#'  -- -- js   (application js files)
#' }
#'
#' @section File Information:
#'
#' All user application creation and modifications will be done in
#' the \strong{program} directory.  The names & locations
#' of the framework-provided .R files should not be changed or the framework
#' will fail to work as expected. \cr
#' \cr
#' \strong{\emph{name/program/config}} directory :\cr
#' Use this location for configuration files.\cr
#' \cr
#' \strong{\emph{name/program/data}} directory :\cr
#' Use this location for data files.  There is a \strong{.gitignore} file
#' included in this directory to prevent accidental versioning of data\cr
#' \cr
#' \strong{\emph{name/program/fxn}} directory :\cr
#' Use this location for supporting and helper R files.\cr
#' \cr
#' \strong{\emph{name/program/modules}} directory :\cr
#' Use this location for application new modules files.\cr
#' \cr
#' \strong{\emph{name/program}/global.R} :\cr
#' Use this location for code that would have previously resided in global.R
#' and for setting application parameters using
#' \link[periscope2]{set_app_parameters}.  Anything placed in this file will
#' be accessible across all user sessions as well as within the UI context. \cr
#' \cr
#' \strong{\emph{name/program}/server_global.R} :\cr
#' Use this location for code that would have previously resided in server.R
#' above (i.e. outside of) the call to \code{shinyServer(...)}. Anything placed
#' in this file will be accessible across all user sessions. \cr
#' \cr
#' \strong{\emph{name/program}/server_local.R} :\cr
#' Use this location for code that would have previously resided in server.R
#' inside of the call to \code{shinyServer(...)}.  Anything placed in this
#' file will be accessible only within a single user session.\cr
#' \cr
#' \strong{\emph{name/program}/ui_body.R} :\cr
#' Create body UI elements in this file and register them with the
#' framework using a call to \link[periscope2]{add_ui_body} \cr
#' \cr
#' \strong{\emph{name/program}/ui_footer.R} :\cr
#' Create footer UI elements in this file and register them with the
#' framework using a call to \link[periscope2]{add_ui_footer} \cr
#' \cr
#' \strong{\emph{name/program}/ui_header.R} :\cr
#' Create header UI elements in this file and register them with the
#' framework using a call to \link[periscope2]{add_ui_header} \cr
#' \cr
#' \strong{\emph{name/program}/ui_left_sidebar.R} :\cr
#' Create sidebar UI elements in this file and register them with the
#' framework using a call to \link[periscope2]{add_ui_left_sidebar} \cr
#' \cr
#' \strong{\emph{name/program}/ui_right_sidebar.R} :\cr
#' Create right sidebar UI elements in this file and register them with the
#' framework using a call to \link[periscope2]{add_ui_right_sidebar} \cr
#' \cr
#' \strong{\emph{name}/www/css/custom.css} :\cr
#' This is the application custom styling css file. User can update
#' application different parts style using this file.\cr
#' \cr
#' \strong{\emph{name}/www/js/custom.js} :\cr
#' This is the application custom javascript file.\cr
#' \cr
#' \strong{\emph{name}/www/periscope_style.yaml} :\cr
#' This is the application custom styling yaml file. User can update
#' application different parts style using this file.\cr
#' \cr
#' \cr
#' \strong{Do not modify the following files}: \cr
#' \preformatted{
#' name\\global.R
#' name\\server.R
#' name\\ui.R
#' name\\www\\img\\loader.gif
#' name\\www\\img\\tooltip.png
#' }
#'
#' @seealso \link[bs4Dash:dashboardPage]{bs4Dash:dashboardPage()}
#' @seealso \link[waiter:waiter]{waiter:waiter_show()}
#'
#'@examples
#' # sample app named 'mytestapp' created in a temp dir
#' location <- tempdir()
#' create_application(name = 'mytestapp', location = location, sample_app = TRUE)
#' unlink(paste0(location,'/mytestapp'), TRUE)
#'
#' # sample app named 'mytestapp' with a right sidebar using a custom icon created in a temp dir
#' location <- tempdir()
#' create_application(name = 'mytestapp', location = location, sample_app = TRUE, right_sidebar = TRUE)
#' unlink(paste0(location,'/mytestapp'), TRUE)
#'
#' # blank app named 'myblankapp' created in a temp dir
#' location <- tempdir()
#' create_application(name = 'myblankapp', location = location)
#' unlink(paste0(location,'/myblankapp'), TRUE)
#'
#' # blank app named 'myblankapp' without a left sidebar created in a temp dir
#' location <- tempdir()
#' create_application(name = 'myblankapp', location = location, left_sidebar = FALSE)
#' unlink(paste0(location,'/myblankapp'), TRUE)
#'
#' @export
create_application <- function(name,
                               location,
                               sample_app    = FALSE,
                               left_sidebar  = TRUE,
                               right_sidebar = FALSE) {
    assertthat::assert_that(!missing(name),
                            length(name) > 0,
                            !is.na(name),
                            name != "",
                            is.character(name),
                            msg = "Framework creation could not proceed, please provide valid character application name")

    assertthat::assert_that(!missing(location),
                            length(location) > 0,
                            !is.na(location),
                            location != "",
                            is.character(location),
                            msg = "Framework creation could not proceed, please provide valid character application location")
    assertthat::assert_that(dir.exists(location),
                            msg = paste0("Framework creation could not proceed, path=<", location, "> does not exists!"))
    assertthat::assert_that(assertthat::is.writeable(location),
                            msg = paste0("Framework creation could not proceed, path=<", location, "> is not writeable!"))
    if (is.na(sample_app) || !is.logical(sample_app)) {
        warning("'sample_app' must have valid boolean value. Setting 'sample_app' to default value 'FALSE'")
        sample_app <- FALSE
    }

    if (is.na(right_sidebar) || !is.logical(right_sidebar)) {
        warning("'right_sidebar' must have valid boolean value. Setting 'right_sidebar' to default value 'FALSE'")
        right_sidebar <- FALSE
    }

    if (is.na(left_sidebar) || !is.logical(left_sidebar)) {
        warning("'left_sidebar' must have valid boolean value. Setting 'left_sidebar' to default value 'TRUE'")
        left_sidebar <- TRUE
    }

    usersep <- .Platform$file.sep
    newloc  <- paste(location, name, sep = usersep)

    assertthat::assert_that(!dir.exists(newloc),
                            msg = paste0("Framework creation could not proceed, path=<", newloc, "> already exists!"))

    application_created <- FALSE

    tryCatch({
        .create_dirs(newloc  = newloc,
                     usersep = usersep)
        .copy_fw_files(newloc        = newloc,
                       usersep       = usersep,
                       left_sidebar  = left_sidebar,
                       right_sidebar = right_sidebar,
                       sample_app    = sample_app)
        .copy_program_files(newloc        = newloc,
                            usersep       = usersep,
                            sample_app    = sample_app,
                            left_sidebar  = left_sidebar,
                            right_sidebar = right_sidebar)
        application_created <- TRUE
    },
    error = function(e) {
        warning("Framework creation could not proceed due to:", e$message)
    })

    if (application_created) {
        message(paste("periscope2 application", name, "was created successfully at", location))
    }
}


#' .create_dirs
#'   Create new application directories
#'
#' @param newloc  - string represents new application location
#' @param usersep - string represents path separator based on running OS
#'
#' @return nothing
#' @keywords internal
#' @noRd
.create_dirs <- function(newloc, usersep) {
    dir.create(newloc)
    dir.create(paste(newloc, "www", sep = usersep))
    dir.create(paste(newloc, "www", "css", sep = usersep))
    dir.create(paste(newloc, "www", "js", sep = usersep))
    dir.create(paste(newloc, "www", "img", sep = usersep))

    dir.create(paste(newloc, "program", sep = usersep))
    dir.create(paste(newloc, "program", "data", sep = usersep))
    dir.create(paste(newloc, "program", "fxn", sep = usersep))
    dir.create(paste(newloc, "program", "modules", sep = usersep))
    dir.create(paste(newloc, "program", "config", sep = usersep))

    dir.create(paste(newloc, "log", sep = usersep))

    #safety for data directory - create .gitignore
    writeLines(c("*", "*/", "!.gitignore"),
               con = paste(newloc, "program", "data", ".gitignore",
                           sep = usersep))
}


#' .copy_fw_files
#'   Create new application standard files
#'
#' @param newloc        - string represents new application location
#' @param usersep       - string represents path separator based on running OS
#' @param left_sidebar  - boolean to control copying left UI sidebar (default = TRUE)
#' @param right_sidebar - boolean to control copying right UI sidebar (default = FALSE)
#' @param sample_app    - boolean to control copying sample app files (default = FALSE)
#'
#' @return nothing
#' @keywords internal
#' @noRd
.copy_fw_files <- function(newloc,
                           usersep,
                           left_sidebar  = TRUE,
                           right_sidebar = FALSE,
                           sample_app    = FALSE) {
    files <- c("global.R", "server.R", "ui.R")

    for (file in files) {
        file_contents <- readLines(con = system.file("fw_templ", file, package = "periscope2"))

        if (file == "ui.R") {
            if (left_sidebar) {
                file_contents <- append(file_contents,
                                        "source(paste(\"program\", \"ui_left_sidebar.R\", sep = .Platform$file.sep), local = TRUE)")
            }

            if (right_sidebar) {
                file_contents <- append(file_contents,
                                        "source(paste(\"program\", \"ui_right_sidebar.R\", sep = .Platform$file.sep), local = TRUE)")

            }

            file_contents <- append(file_contents, "periscope2:::create_application_dashboard()")
        }

        ui_file <- file(paste(newloc, file, sep = usersep), open = "w+")
        writeLines(file_contents, con = ui_file)
        close(ui_file)
    }

    #subdir copies
    imgs <- c("loader.gif", "tooltip.png")

    for (file in imgs) {
        writeBin(readBin(
            con = system.file("fw_templ", "www", file,
                              package = "periscope2"),
            what = "raw", n = 1e6),
            con = paste(newloc, "www", "img", file, sep = usersep))
    }

    if (sample_app) {
        file.copy(system.file("fw_templ", "p_example", "periscope_style.yaml", package = "periscope2"),
                  paste(newloc, "www", "periscope_style.yaml", sep = usersep))
        file.copy(system.file("fw_templ", "p_example", "custom.css", package = "periscope2"),
                  paste(newloc, "www", "css", "custom.css", sep = usersep))
        file.copy(system.file("fw_templ", "p_example", "custom.js", package = "periscope2"),
                  paste(newloc, "www", "js", "custom.js", sep = usersep))
    } else {
        file.copy(system.file("fw_templ", "www", "periscope_style.yaml", package = "periscope2"),
                  paste(newloc, "www", "periscope_style.yaml", sep = usersep))
        file.copy(system.file("fw_templ", "www", "custom.css", package = "periscope2"),
                  paste(newloc, "www", "css", "custom.css", sep = usersep))
        file.copy(system.file("fw_templ", "www", "custom.js", package = "periscope2"),
                  paste(newloc, "www", "js", "custom.js", sep = usersep))
    }
}


#' .copy_program_files
#'   Copy program folder files
#'
#' @param newloc        - string represents new application location
#' @param usersep       - string represents path separator based on running OS
#' @param sample_app    - boolean to control copying sample app files (default = FALSE)
#' @param left_sidebar  - boolean to control copying left UI sidebar (default = TRUE)
#' @param right_sidebar - boolean to control copying right UI sidebar (default = FALSE)
#'
#' @return nothing
#' @keywords internal
#' @noRd
.copy_program_files <- function(newloc,
                                usersep,
                                sample_app,
                                left_sidebar  = TRUE,
                                right_sidebar = FALSE) {
    file.copy(system.file("fw_templ", "announce.yaml", package = "periscope2"),
              paste(newloc, "program", "config", "announce.yaml", sep = usersep))

    files <- list("global.R"        = "global.R",
                  "server_global.R" = "server_global.R",
                  "server_local.R"  = "server_local.R",
                  "ui_body.R"       = "ui_body.R",
                  "ui_header.R"     = "ui_header.R",
                  "ui_footer.R"     = "ui_footer.R")

    if (left_sidebar) {
        files["ui_left_sidebar.R"] <- "ui_left_sidebar.R"
    }

    if (right_sidebar) {
        files["ui_right_sidebar.R"] <- "ui_right_sidebar.R"
    }

    targetdir <- paste(newloc, "program", sep = usersep)
    sourcedir <- paste("fw_templ",
                       ifelse(sample_app, "p_example", "p_blank"),
                       sep = usersep)

    if (sample_app && !left_sidebar) {
        files["ui_body.R"] <- "ui_body_no_left_sidebar.R"
    }

    for (file in names(files)) {
        file_contents <- readLines(con = system.file(paste(sourcedir, files[[file]], sep = usersep), package = "periscope2"))
        program_file  <- file(paste(targetdir, file, sep = usersep), open = "w+")
        writeLines(file_contents, con = program_file)
        close(program_file)
    }

    #subdir copies for sample_app
    if (sample_app) {
        supporting_files <- list("example.csv"       = "data",
                                 "structure.csv"     = "data",
                                 "struc_indx.csv"    = "data",
                                 "program_helpers.R" = "fxn",
                                 "plots.R"           = "fxn")
        for (file in names(supporting_files)) {
            writeLines(readLines(
                con = system.file(sourcedir, file,
                                  package = "periscope2")),
                con = paste(targetdir,
                            unlist(supporting_files[file], use.names = FALSE),
                            file,
                            sep = usersep))
        }
    }
}

context("periscope2 create new application")

expect_cleanup_create_new_application <- function(full_name,
                                                  sample_app    = FALSE,
                                                  left_sidebar  = TRUE,
                                                  right_sidebar = FALSE) {
    expect_true(dir.exists(full_name))
    expect_true(file.exists(paste0(full_name, "/global.R")))
    expect_true(file.exists(paste0(full_name, "/server.R")))
    expect_true(file.exists(paste0(full_name, "/ui.R")))
    expect_true(dir.exists(paste0(full_name, "/www")))
    expect_true(file.exists(paste0(full_name, "/www/css/custom.css")))
    expect_true(file.exists(paste0(full_name, "/www/js/custom.js")))
    expect_true(file.exists(paste0(full_name, "/www/periscope_style.yaml")))
    expect_true(dir.exists(paste0(full_name, "/www/img")))
    expect_true(file.exists(paste0(full_name, "/www/img/loader.gif")))
    expect_true(file.exists(paste0(full_name, "/www/img/tooltip.png")))
    expect_true(dir.exists(paste0(full_name, "/program")))
    expect_true(file.exists(paste0(full_name, "/program/global.R")))
    expect_true(file.exists(paste0(full_name, "/program/server_global.R")))
    expect_true(file.exists(paste0(full_name, "/program/server_local.R")))
    expect_true(file.exists(paste0(full_name, "/program/ui_body.R")))
    expect_true(file.exists(paste0(full_name, "/program/ui_header.R")))
    expect_true(file.exists(paste0(full_name, "/program/ui_footer.R")))
    expect_true(dir.exists(paste0(full_name, "/program/data")))
    expect_true(dir.exists(paste0(full_name, "/program/fxn")))
    expect_true(file.exists(paste0(full_name, "/program/config/announce.yaml")))
    expect_true(dir.exists(paste0(full_name, "/program/modules")))
    expect_true(dir.exists(paste0(full_name, "/log")))

    if (left_sidebar) {
        expect_true(file.exists(paste0(full_name, "/program/ui_left_sidebar.R")))
    } else {
        expect_true(!file.exists(paste0(full_name, "/program/ui_left_sidebar.R")))
    }

    if (right_sidebar) {
        expect_true(file.exists(paste0(full_name, "/program/ui_right_sidebar.R")))
    } else {
        expect_true(!file.exists(paste0(full_name, "/program/ui_right_sidebar.R")))
    }

    if (sample_app) {
        expect_true(file.exists(paste0(full_name, "/program/data/example.csv")))
        expect_true(file.exists(paste0(full_name, "/program/data/struc_indx.csv")))
        expect_true(file.exists(paste0(full_name, "/program/data/structure.csv")))
        expect_true(file.exists(paste0(full_name, "/program/fxn/program_helpers.R")))
        expect_true(file.exists(paste0(full_name, "/program/fxn/plots.R")))
    }

    # clean up
    unlink(full_name, TRUE)
}

test_that("create_application empty full app", {
    appTemp_dir    <- tempdir()
    appTemp        <- tempfile(pattern = "TestThatApp", tmpdir = appTemp_dir)
    appTemp_name   <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    create_app_msg <- capture_message(create_application(name          = appTemp_name,
                                                        location      = appTemp_dir,
                                                        sample_app    = FALSE,
                                                        right_sidebar = TRUE))$message

    expect_equal(create_app_msg,
                 paste0("periscope2 application ", appTemp_name, " was created successfully at ", appTemp_dir, "\n"))
    expect_cleanup_create_new_application(full_name     = appTemp,
                                          right_sidebar = TRUE)
})

test_that("create_application sample", {
    appTemp_dir    <- tempdir()
    appTemp        <- tempfile(pattern = "TestThatApp", tmpdir = appTemp_dir)
    appTemp_name   <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    create_app_msg <- capture_message(create_application(name       = appTemp_name,
                                                         location   = appTemp_dir,
                                                         sample_app = TRUE))$message

    expect_equal(create_app_msg,
                 paste0("periscope2 application ", appTemp_name, " was created successfully at ", appTemp_dir, "\n"))
    expect_cleanup_create_new_application(full_name  = appTemp,
                                          sample_app = TRUE)
})

test_that("create_application sample full app", {
    appTemp_dir    <- tempdir()
    appTemp        <- tempfile(pattern = "TestThatApp", tmpdir = appTemp_dir)
    appTemp_name   <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    create_app_msg <- capture_message(create_application(name          = appTemp_name,
                                                         location      = appTemp_dir,
                                                         sample_app    = TRUE,
                                                         right_sidebar = TRUE))$message

    expect_equal(create_app_msg,
                   paste0("periscope2 application ", appTemp_name, " was created successfully at ", appTemp_dir, "\n"))
    expect_cleanup_create_new_application(full_name     = appTemp,
                                          sample_app    = TRUE,
                                          right_sidebar = TRUE)
})

test_that("create_application sample right_sidebar without left_sidebar", {
    appTemp_dir    <- tempdir()
    appTemp        <- tempfile(pattern = "TestThatApp", tmpdir = appTemp_dir)
    appTemp_name   <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    create_app_msg <- capture_message(create_application(name          = appTemp_name,
                                                         location      = appTemp_dir,
                                                         sample_app    = TRUE,
                                                         right_sidebar = TRUE,
                                                         left_sidebar  = FALSE))$message

    expect_equal(create_app_msg,
                 paste0("periscope2 application ", appTemp_name, " was created successfully at ", appTemp_dir, "\n"))
    expect_cleanup_create_new_application(full_name     = appTemp,
                                          sample_app    = TRUE,
                                          right_sidebar = TRUE,
                                          left_sidebar  = FALSE)
})

test_that("create_application sample without bars", {
    appTemp_dir    <- tempdir()
    appTemp        <- tempfile(pattern = "TestThatApp", tmpdir = appTemp_dir)
    appTemp_name   <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    create_app_msg <- capture_message(create_application(name          = appTemp_name,
                                                         location      = appTemp_dir,
                                                         sample_app    = TRUE,
                                                         left_sidebar  = FALSE))$message

    expect_equal(create_app_msg,
                 paste0("periscope2 application ", appTemp_name, " was created successfully at ", appTemp_dir, "\n"))
    expect_cleanup_create_new_application(full_name     = appTemp,
                                          sample_app    = TRUE,
                                          left_sidebar  = FALSE)
})

test_that("create_application empty app without bars", {
    appTemp_dir    <- tempdir()
    appTemp        <- tempfile(pattern = "TestThatApp", tmpdir = appTemp_dir)
    appTemp_name   <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    create_app_msg <- capture_message(create_application(name          = appTemp_name,
                                                         location      = appTemp_dir,
                                                         sample_app    = FALSE,
                                                         left_sidebar  = FALSE))$message

    expect_equal(create_app_msg,
                 paste0("periscope2 application ", appTemp_name, " was created successfully at ", appTemp_dir, "\n"))
    expect_cleanup_create_new_application(full_name     = appTemp,
                                          sample_app    = FALSE,
                                          left_sidebar  = FALSE)
})

test_that("create_application invalid name", {
    expect_error(create_application(location = tempfile(), sample_app = FALSE),
                 "Framework creation could not proceed, please provide valid character application name")
    expect_error(create_application(name = NULL, location = tempfile(), sample_app = FALSE),
                   "Framework creation could not proceed, please provide valid character application name")
    expect_error(create_application(name = "", location = tempfile(), sample_app = FALSE),
                 "Framework creation could not proceed, please provide valid character application name")
    expect_error(create_application(name = 123, location = tempfile(), sample_app = FALSE),
                 "Framework creation could not proceed, please provide valid character application name")
    expect_error(create_application(name = NA, location = tempfile(), sample_app = FALSE),
                 "Framework creation could not proceed, please provide valid character application name")
})

test_that("create_application invalid location", {
    expect_error(create_application(name = "Invalid", location = NULL, sample_app = FALSE),
                 "Framework creation could not proceed, please provide valid character application location")
    expect_error(create_application(name = "Invalid", sample_app = FALSE),
                 "Framework creation could not proceed, please provide valid character application location")
    expect_error(create_application(name = "Invalid", location = "", sample_app = FALSE),
                 "Framework creation could not proceed, please provide valid character application location")
    expect_error(create_application(name = "Invalid", location = NA, sample_app = FALSE),
                 "Framework creation could not proceed, please provide valid character application location")
    expect_error(create_application(name = "Invalid", location = 123, sample_app = FALSE),
                 "Framework creation could not proceed, please provide valid character application location")
    expect_error(create_application(name = "Invalid", location = "invalid", sample_app = FALSE),
                 "Framework creation could not proceed, path=<invalid> does not exists!")
})

test_that("create_application existing location", {
    appTemp_dir  <- tempdir()
    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))

    create_application(name = appTemp_name, location = appTemp_dir, sample_app = FALSE)
    create_error_msg <- capture_message(create_application(name = appTemp_name, location = appTemp_dir, sample_app = FALSE))$message

    expect_equal(create_error_msg,
                 paste0("Framework creation could not proceed, path=<", appTemp_dir, .Platform$file.sep, appTemp_name,"> already exists!"))

    expect_cleanup_create_new_application(appTemp)
})

test_that("create_application invalid sample app", {
    appTemp_dir <- tempdir()

    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, sample_app = "FALSE"),
                   "'sample_app' must have valid boolean value. Setting 'sample_app' to default value 'FALSE'")
    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, sample_app = NULL),
                   "'sample_app' must have valid boolean value. Setting 'sample_app' to default value 'FALSE'")

    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, sample_app = NA),
                   "'sample_app' must have valid boolean value. Setting 'sample_app' to default value 'FALSE'")

    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, sample_app = 123),
                   "'sample_app' must have valid boolean value. Setting 'sample_app' to default value 'FALSE'")
    expect_cleanup_create_new_application(appTemp)
})

test_that("create_application invalid left sidebar", {
    appTemp_dir <- tempdir()

    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, left_sidebar = "FALSE"),
                   "'left_sidebar' must have valid boolean value. Setting 'left_sidebar' to default value 'TRUE'")
    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, left_sidebar = NULL),
                   "'left_sidebar' must have valid boolean value. Setting 'left_sidebar' to default value 'TRUE'")

    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, left_sidebar = NA),
                   "'left_sidebar' must have valid boolean value. Setting 'left_sidebar' to default value 'TRUE'")

    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, left_sidebar = 123),
                   "'left_sidebar' must have valid boolean value. Setting 'left_sidebar' to default value 'TRUE'")
    expect_cleanup_create_new_application(appTemp)
})

test_that("create_application invalid right sidebar", {
    appTemp_dir <- tempdir()

    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, right_sidebar = "FALSE"),
                   "'right_sidebar' must have valid boolean value. Setting 'right_sidebar' to default value 'FALSE'")
    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, right_sidebar = NULL),
                   "'right_sidebar' must have valid boolean value. Setting 'right_sidebar' to default value 'FALSE'")

    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, right_sidebar = NA),
                   "'right_sidebar' must have valid boolean value. Setting 'right_sidebar' to default value 'FALSE'")

    appTemp      <- tempfile(pattern = "InvalidApp", tmpdir = appTemp_dir)
    appTemp_name <- gsub('\\\\|/', '', (gsub(appTemp_dir, "", appTemp, fixed = TRUE)))
    expect_warning(create_application(name = appTemp_name, location = appTemp_dir, right_sidebar = 123),
                   "'right_sidebar' must have valid boolean value. Setting 'right_sidebar' to default value 'FALSE'")
    expect_cleanup_create_new_application(appTemp)
})

# -------------------------------------------
# -- Application Downloadable Plot Module --
# -------------------------------------------


#' downloadablePlot module UI function
#'
#' Creates a custom plot output that is paired with a linked downloadFile
#' button.  This module is compatible with ggplot2, grob and lattice
#' produced graphics.
#'
#' downloadFile button will be hidden if \code{downloadablePlot} parameter \code{downloadfxns} or
#' \code{downloadablePlotUI} parameter \code{downloadtypes} is empty
#'
#' @param id character id for the object
#' @param downloadtypes vector of values for download types
#' @param download_hovertext download button tooltip hover text
#' @param width plot width (any valid css size value)
#' @param height plot height (any valid css size value)
#' @param btn_halign horizontal position of the download button ("left", "center", "right")
#' @param btn_valign vertical position of the download button ("top", "bottom")
#' @param btn_overlap whether the button should appear on top of the bottom of
#' the plot area to save on vertical space \emph{(there is often a blank area
#' where a button can be overlayed instead of utilizing an entire horizontal
#' row for the button below the plot area)}
#' @param clickOpts NULL or an object created by the \link[shiny]{clickOpts} function
#' @param hoverOpts NULL or an object created by the \link[shiny]{hoverOpts} function
#' @param brushOpts NULL or an object created by the \link[shiny]{brushOpts} function
#'
#' @return list of downloadFileButton UI and plot object
#'
#' @section Example:
#' \code{downloadablePlotUI("myplotID", c("png", "csv"),
#' "Download Plot or Data", "300px")}
#'
#' @section Notes:
#' When there is nothing to download in any of the linked downloadfxns the
#' button will be hidden as there is nothing to download.
#'
#' This module is NOT compatible with the built-in (base) graphics \emph{(such as
#' basic plot, etc.)} because they cannot be saved into an object and are directly
#' output by the system at the time of creation.
#'
#' @section Shiny Usage:
#' Call this function at the place in ui.R where the plot should be placed.
#'
#' Paired with a call to \code{downloadablePlot(id, ...)}
#' in server.R
#'
#' @seealso \link[periscope2]{downloadablePlot}
#' @seealso \link[periscope2]{downloadFileButton}
#' @seealso \link[shiny]{clickOpts}
#' @seealso \link[shiny]{hoverOpts}
#' @seealso \link[shiny]{brushOpts}
#' @seealso \link[periscope2]{appResetButton}
#' @seealso \link[periscope2]{appReset}
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{downloadableTable}
#' @seealso \link[periscope2]{logViewerOutput}
#' @examples
#' if (interactive()) {
#'   library(shiny)
#'   library(ggplot2)
#'   library(periscope2)
#'   shinyApp(ui = fluidPage(fluidRow(column(width = 12,
#'      downloadablePlotUI("object_id1",
#'                         downloadtypes      = c("png", "csv"),
#'                         download_hovertext = "Download plot and data",
#'                         height             = "500px",
#'                         btn_halign         = "left")))),
#'     server = function(input, output) {
#'       download_plot <- function() {
#'         ggplot(data = mtcars, aes(x = wt, y = mpg)) +
#'         geom_point(aes(color = cyl)) +
#'         theme(legend.justification   = c(1, 1),
#'               legend.position.inside = c(1, 1),
#'               legend.title           = element_blank()) +
#'         ggtitle("GGPlot Example ") +
#'         xlab("wt") +
#'         ylab("mpg")
#'       }
#'       downloadablePlot(id           = "object_id1",
#'                        logger       = "",
#'                        filenameroot = "mydownload1",
#'                        downloadfxns = list(png = download_plot, csv = reactiveVal(mtcars)),
#'                        aspectratio  = 1.33,
#'                        visibleplot  = download_plot)
#'   })
#'}
#'
#' @export
downloadablePlotUI <- function(id,
                               downloadtypes      = c("png"),
                               download_hovertext = NULL,
                               width              = "100%",
                               height             = "400px",
                               btn_halign         = "right",
                               btn_valign         = "bottom",
                               btn_overlap        = TRUE,
                               clickOpts          = NULL,
                               hoverOpts          = NULL,
                               brushOpts          = NULL) {
    ns       <- shiny::NS(id)
    styleval <- "display:inherit; padding: 5px"

    if (btn_halign == "left") {
        styleval <- paste(styleval, "float:left", sep = ";")
    } else if (btn_halign == "center") {
        styleval <- paste(styleval, "float:none", "margin-left:45%", sep = ";")
    } else if (btn_halign != "right") {
        msg <- paste(btn_halign, " is not a valid btn_halign input - using default",
                     "value. Valid values: <'left', 'center', 'right'>")
        warning(msg)
        btn_halign <- "right"
    }

    if (!(btn_valign %in% c("top", "bottom"))) {
        msg <- paste(btn_valign, " is not a valid btn_valign input - using default",
                     "value. Valid values: <'top', 'bottom'>")
        warning(msg)
        btn_valign <- "bottom"
    }

    if (btn_overlap) {
        styleval <- ifelse(btn_valign == "bottom",
                           paste(styleval, "top: -50px", sep = ";"),
                           paste(styleval, paste0("top: -", height), "position:relative", sep = ";"))
    } else {
        styleval <- ifelse(btn_valign == "bottom",
                           paste(styleval, "top: 5px", sep = ";"),
                           paste(styleval, "top: -5px", sep = ";"))

    }

    btn_item <- shiny::conditionalPanel(
        condition = "output.displayButton",
        ns        = ns,
        shiny::span(id    = ns("dplotButtonDiv"),
                    class = "periscope-downloadable-plot-button",
                    style = styleval,
                    periscope2::downloadFileButton(ns("dplotButtonID"),
                                                   downloadtypes,
                                                   download_hovertext)))

    plot_item <- shiny::plotOutput(outputId = ns("dplotOutputID"),
                                   width    = width,
                                   height   = height,
                                   click    = clickOpts,
                                   hover    = hoverOpts,
                                   brush    = brushOpts)
    module_output <- list(plot_item, btn_item)

    if (!btn_overlap && (btn_valign == "top")) {
        module_output <- list(btn_item, plot_item)
    }

    module_output
}

#' downloadablePlot module server function
#'
#' Server-side function for the downloadablePlotUI.  This is a custom
#' plot output paired with a linked downloadFile button.
#'
#' downloadFile button will be hidden if \code{downloadablePlot} parameter \code{downloadfxns} or
#' \code{downloadablePlotUI} parameter \code{downloadtypes} is empty
#'
#' @param id the ID of the Module's UI element
#' @param logger logger to use
#' @param filenameroot the base text used for user-downloaded file - can be
#' either a character string or a reactive expression returning a character
#' string
#' @param aspectratio the downloaded chart image width:height ratio (ex:
#' 1 = square, 1.3 = 4:3, 0.5 = 1:2).  Where not applicable for a download type
#' it is ignored (e.g. data, html downloads)
#' @param downloadfxns a \strong{named} list of functions providing download
#' images or data tables as return values.  The names for the list should be
#' the same names that were used when the plot UI was created.
#' @param visibleplot function or reactive expression providing the plot to
#' display as a return value.  This function should require no input parameters.
#'
#' @return Reactive expression containing the currently selected plot to be available for display and download
#'
#' @section Notes:
#' When there are no values to download in any of the linked downloadfxns the
#' button will be hidden as there is nothing to download.
#'
#' @section Shiny Usage:
#' This function is not called directly by consumers - it is accessed in
#' server.R using the same id provided in \code{downloadablePlotUI}:
#'
#' \strong{\code{downloadablePlot(id, logger, filenameroot,
#' downloadfxns, visibleplot)}}
#'
#' @seealso \link[periscope2]{downloadablePlotUI}
#' @seealso \link[periscope2]{appResetButton}
#' @seealso \link[periscope2]{appReset}
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{downloadableTable}
#' @seealso \link[periscope2]{logViewerOutput}
#' @examples
#' if (interactive()) {
#'   library(shiny)
#'   library(ggplot2)
#'   library(periscope2)
#'   shinyApp(ui = fluidPage(fluidRow(column(width = 12,
#'      downloadablePlotUI("object_id1",
#'                         downloadtypes      = c("png", "csv"),
#'                         download_hovertext = "Download plot and data",
#'                         height             = "500px",
#'                         btn_halign         = "left")))),
#'     server = function(input, output) {
#'       download_plot <- function() {
#'         ggplot(data = mtcars, aes(x = wt, y = mpg)) +
#'         geom_point(aes(color = cyl)) +
#'         theme(legend.justification   = c(1, 1),
#'               legend.position.inside = c(1, 1),
#'               legend.title           = element_blank()) +
#'         ggtitle("GGPlot Example ") +
#'         xlab("wt") +
#'         ylab("mpg")
#'       }
#'       downloadablePlot(id           = "object_id1",
#'                        logger       = "",
#'                        filenameroot = "mydownload1",
#'                        downloadfxns = list(png = download_plot, csv = reactiveVal(mtcars)),
#'                        aspectratio  = 1.33,
#'                        visibleplot  = download_plot)
#'   })
#'}
#'
#' @export
downloadablePlot <- function(id,
                             logger       = NULL,
                             filenameroot = "download",
                             aspectratio  = 1,
                             downloadfxns = NULL,
                             visibleplot) {
    shiny::moduleServer(
        id,
        function(input, output, session) {
            downloadFile("dplotButtonID", logger, filenameroot, downloadfxns, aspectratio)

            shiny::observe({
                output$dplotOutputID <- shiny::renderPlot({
                    plot <- visibleplot()
                    if (inherits(plot, "grob")) {
                        plot <- grid::grid.draw(plot)
                    }
                    plot
                })
            })

            shiny::observe({
                output$displayButton <- shiny::reactive(length(downloadfxns) > 0)
                shiny::outputOptions(output, "displayButton", suspendWhenHidden = FALSE)
            })
        })
}

# -------------------------------------------------
# -- Application Downloadable React Table Module --
# -------------------------------------------------


#' downloadableReactTable module UI function
#'
#' downloadableReactTable module is extending \code{?reactable} package table functions by creating
#' a custom high-functionality table paired with \link[periscope2]{downloadFile} button.
#' The table has the following default functionality:search, highlight functionality, infinite scrolling, sorting by columns and
#' returns a reactive dataset of selected items and table current state.
#'
#' \link[periscope2]{downloadFile} button will be hidden if \code{downloadableReactTableUI} parameter
#' \code{downloadtypes} is empty
#'
#' @param id character id for the object
#' @param downloadtypes vector of values for data download types
#' @param hovertext download button tooltip hover text
#'
#' @return list of downloadFileButton UI and reactable table and hidden inputs for contentHeight option
#'
#' @section Table Features:
#' \itemize{
#'     \item Consistent styling of the table
#'     \item downloadFile module button functionality built-in to the table (it will be shown only if downloadtypes is defined)
#'     \item Ability to show different data from the download data
#'     \item Table is automatically fit to the window size with infinite
#'     y-scrolling
#'     \item Table search functionality including highlighting built-in
#'     \item Multi-select built in, including reactive feedback on which table
#'     items are selected
#' }
#'
#' @section Example:
#' \code{downloadableReactTableUI("mytableID", c("csv", "tsv"),
#' "Click Here")}
#'
#' @section Notes:
#' When there are no rows to download in any of the linked downloaddatafxns the
#' button will be hidden as there is nothing to download.
#'
#' @section Shiny Usage:
#' Call this function at the place in ui.R where the table should be placed.
#'
#' Paired with a call to \code{downloadableReactTable(id, ...)}
#' in server.R
#'
#' @seealso \link[periscope2]{downloadableReactTable}
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{logViewerOutput}
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{downloadablePlot}
#'
#' @examples
#' if (interactive()) {
#'  library(shiny)
#'  library(periscope2)
#'  library(reactable)
#'
#'  shinyApp(
#'      ui = fluidPage(fluidRow(column(
#'          width = 12,
#'          downloadableReactTableUI(
#'              id            = "object_id1",
#'              downloadtypes = c("csv", "tsv"),
#'              hovertext     = "Download the data here!")))),
#'      server = function(input, output) {
#'          table_state <- downloadableReactTable(
#'              id                 = "object_id1",
#'              table_data         = reactiveVal(iris),
#'              download_data_fxns = list(csv = reactiveVal(iris), tsv = reactiveVal(iris)),
#'              selection_mode     = "multiple",
#'              pre_selected_rows  = function() {c(1, 3, 5)},
#'              table_options      = list(columns = list(
#'                  Sepal.Length = colDef(name = "Sepal Length"),
#'                  Sepal.Width  = colDef(filterable = TRUE),
#'                  Petal.Length = colDef(show = FALSE),
#'                  Petal.Width  = colDef(defaultSortOrder = "desc")),
#'                  showSortable = TRUE,
#'                  theme = reactableTheme(
#'                      borderColor = "#dfe2e5",
#'                      stripedColor = "#f6f8fa",
#'                      highlightColor = "#f0f5f9",
#'                      cellPadding = "8px 12px")))
#'
#'         observeEvent(table_state(), { print(table_state()) })
#'     })
#' }
#'
#' @export
downloadableReactTableUI <- function(id,
                                     downloadtypes = NULL,
                                     hovertext     = NULL) {
    ns <- shiny::NS(id)
    list(
        shiny::conditionalPanel(
            condition = "output.displayButton",
            ns        = ns,
            shiny::span(
                id    = ns("reactTableButtonDiv"),
                class = "periscope-downloadable-react-table-button",
                style = ifelse(length(downloadtypes) > 0, "", "display:none"),
                downloadFileButton(ns("reactTableButtonID"),
                                   downloadtypes,
                                   hovertext))),
        reactable::reactableOutput(ns("reactTableOutputID"))
    )
}


#' downloadableReactTable module server function
#'
#' Server-side function for the downloadableReactTableUI.
#'
#'
#' @param id the ID of the Module's UI element
#' @param table_data reactive expression (or parameter-less function) that acts as table data source
#' @param selection_mode to enable row selection, set \code{selection_mode} value to either "single" for single row
#'                       selection or "multiple" for multiple rows selection, case insensitive. Any other value will
#'                       disable row selection. Row selection will be enabled by radio buttons in "single" selection
#'                       and checkboxes in "multiple" selection (default = NULL)
#' @param pre_selected_rows reactive expression (or parameter-less function) provides the rows indices of the rows to
#'                          be selected when the table is rendered. If selection_mode is disabled, this parameter will
#'                          have no effect. If selection_mode is "single" only the first row index will be used (default = NULL)
#' @param file_name_root the base name used for user-downloaded file. It can be either a character string
#'                       a reactive expression or a function returning a character string (default = 'data_file')
#' @param download_data_fxns a \strong{named} list of functions providing the data as return values.
#'                           The names for the list should be the same names as the ones used in the
#'                           downloadableReactTableUI (default = NULL)
#' @param pagination to enable table pagination (default = FALSE)
#' @param table_height max table height in pixels. Vertical scroll will be shown after that height value
#' @param show_rownames enable displaying rownames as a separate column (default = FALSE)
#' @param columns_filter enable the filtering input on each column in the table (default = FALSE)
#' @param global_search enable table global searching input to search and filter in all columns at once
#'                              (default = TRUE)
#' @param row_highlight enable highlighting rows upon mouse hover (default = TRUE)
#' @param row_striping add zebra-striped style to table rows (default = TRUE)
#' @param table_options optional table formatting parameters check \code{?reactable::reactable} for options full list.
#'                      Also see example below to see how to pass options (default = list())
#' @param logger logger to use (default = NULL)
#'
#' @return A named list of two elements:
#'  \itemize{
#'     \item selected_rows: data.frame of current selected rows
#'     \item table_state: a list of the current table state. The list keys are
#'         ("page", "pageSize", "pages", "sorted" and "selected").
#'          Review \code{?reactable::getReactableState} for more info.
#' }
#'
#'
#' @section Shiny Usage:
#' This function is not called directly by consumers - it is accessed in
#' server.R using the same id provided in \code{downloadableReactTableUI}:
#'
#' \strong{\code{downloadableReactTable(id)}}
#'
#' @seealso \link[periscope2]{downloadableReactTableUI}
#' @seealso \link[periscope2]{downloadFileButton}
#' @seealso \link[periscope2]{logViewerOutput}
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{downloadablePlot}
#'
#' @examples
#' if (interactive()) {
#'  library(shiny)
#'  library(periscope2)
#'  library(reactable)
#'
#'  shinyApp(
#'      ui = fluidPage(fluidRow(column(
#'          width = 12,
#'          downloadableReactTableUI(
#'              id            = "object_id1",
#'              downloadtypes = c("csv", "tsv"),
#'              hovertext     = "Download the data here!")))),
#'      server = function(input, output) {
#'          table_state <- downloadableReactTable(
#'              id                 = "object_id1",
#'              table_data         = reactiveVal(iris),
#'              download_data_fxns = list(csv = reactiveVal(iris), tsv = reactiveVal(iris)),
#'              selection_mode     = "multiple",
#'              pre_selected_rows  = function() {c(1, 3, 5)},
#'              table_options      = list(columns = list(
#'                  Sepal.Length = colDef(name = "Sepal Length"),
#'                  Sepal.Width  = colDef(filterable = TRUE),
#'                  Petal.Length = colDef(show = FALSE),
#'                  Petal.Width  = colDef(defaultSortOrder = "desc")),
#'                  theme = reactableTheme(
#'                      borderColor = "#dfe2e5",
#'                      stripedColor = "#f6f8fa",
#'                      highlightColor = "#f0f5f9",
#'                      cellPadding = "8px 12px")))
#'
#'         observeEvent(table_state(), { print(table_state()) })
#'     })
#' }
#'
#' @export
downloadableReactTable <- function(id,
                                   table_data,
                                   selection_mode     = NULL,
                                   pre_selected_rows  = NULL,
                                   file_name_root     = "data_file",
                                   download_data_fxns = NULL,
                                   pagination         = FALSE,
                                   table_height       = 600,
                                   show_rownames      = FALSE,
                                   columns_filter     = FALSE,
                                   global_search      = TRUE,
                                   row_highlight      = TRUE,
                                   row_striping       = TRUE,
                                   table_options      = list(),
                                   logger             = NULL) {
        shiny::moduleServer(id,
             function(input, output, session) {
                 downloadable_module_state <- shiny::reactiveVal(list(selected_rows = NULL, table_state = NULL))
                 is_stale                  <- shiny::reactiveVal(FALSE)

                 if (is.null(table_data) || !is.function(table_data)) {
                     logerror("'table_data' parameter must be a function or reactive expression.", logger = logger)
                     output$reactTableOutputID <- reactable::renderReactable({ NULL })
                 } else {
                     table_react_params <- shiny::reactiveValues(table_data        = NULL,
                                                                 pre_selected_rows = NULL)

                     if (is.null(file_name_root)) {
                         logwarn("'file_name_root' parameter should not be NULL. Setting default value 'data_file'.", logger = logger)
                         file_name_root <- "data_file"
                     }

                     downloadFile(id           = "reactTableButtonID",
                                  logger       = logger,
                                  filenameroot = file_name_root,
                                  datafxns     = download_data_fxns,
                                  row_names    = show_rownames)
                     shiny::observe({
                         output$displayButton <- shiny::reactive(length(download_data_fxns) > 0)
                         shiny::outputOptions(output, "displayButton", suspendWhenHidden = FALSE)
                    })

                     if (!is.null(pre_selected_rows) && !is.function(pre_selected_rows)) {
                         logwarn("'pre_selected_rows' parameter must be a function or reactive expression. Setting default value NULL.", logger = logger)
                         pre_selected_rows <- NULL
                     }

                     shiny::observeEvent(table_data(), {
                         is_stale(TRUE)

                         if (!is.data.frame(table_data())) {
                             table_data <- shiny::reactiveVal(data.frame(table_data()))
                         }

                         table_react_params$table_data <- table_data()
                         output$displayButton <- shiny::reactive((length(download_data_fxns) > 0))

                         if (NROW(table_react_params$table_data) == 0) {
                            output$displayButton <- shiny::reactive(FALSE)

                         }
                         shiny::outputOptions(output, "displayButton", suspendWhenHidden = FALSE)
                     }, priority = 10)

                     shiny::observe({
                         table_react_params$pre_selected_rows <- NULL
                         if ((is.null(selection_mode) || !(tolower(selection_mode) %in% c("single", "multiple"))) &&
                              !is.null(pre_selected_rows)) {
                             logwarn("'selection_mode' parameter must be either 'single' or 'multiple' to use 'pre_selected_rows' param. Setting default value NULL.",
                                     logger = logger)
                         } else if (!is.null(pre_selected_rows) && !is.numeric(pre_selected_rows())) {
                             logwarn("'pre_selected_rows' parameter must be a function or reactive expression that returns numeric vector. Setting default value NULL.",
                                     logger = logger)
                         } else if (!is.null(pre_selected_rows)) {
                             table_react_params$pre_selected_rows <- pre_selected_rows()
                         }

                     })
                     output$reactTableOutputID <- reactable::renderReactable({
                         table_output <- NULL

                         if (!all(is.na(table_react_params$table_data)) &&
                             !is.null(table_react_params$table_data) &&
                             (NCOL(table_react_params$table_data) > 0)) {
                             row_selection_mode <- NULL
                             if (!is.null(selection_mode) && (tolower(selection_mode) %in% c("single", "multiple"))) {
                                 row_selection_mode <- tolower(selection_mode)

                                 if ((row_selection_mode == "single") && (length(table_react_params$pre_selected_rows) > 1)) {
                                     logwarn("when 'selection_mode' is 'single', only first value of 'pre_selected_rows' will be used",
                                             logger = logger)
                                     table_react_params$pre_selected_rows <- table_react_params$pre_selected_rows[1]
                                 }

                                 if (length(table_react_params$pre_selected_rows) > 0) {
                                     excluded_indices <- table_react_params$pre_selected_rows[
                                         sapply(table_react_params$pre_selected_rows,
                                                function(x) {
                                                    x < 1 || x > NROW(table_react_params$table_data)
                                                    })]
                                     if (length(excluded_indices) > 0) {
                                         if (length(excluded_indices) == length(table_react_params$pre_selected_rows)) {
                                             logwarn("All 'pre_selected_rows' values are out of range. Setting default value NULL.",
                                                     logger = logger)
                                             table_react_params$pre_selected_rows <- NULL
                                         } else {
                                             value_msg <- paste("value:", excluded_indices[1] ,"as it is")
                                             if (length(excluded_indices) > 1) {
                                                 value_msg <- paste("values:", paste0(excluded_indices, collapse = ", "), "as they are")
                                             }

                                             logwarn(paste("Excluding 'pre_selected_rows'" , value_msg, "out of range."),
                                                     logger = logger)
                                             table_react_params$pre_selected_rows <- table_react_params$pre_selected_rows[
                                                 !(table_react_params$pre_selected_rows %in% excluded_indices)]
                                         }
                                     }
                                 }
                             }

                             table_arguments <- list(data            = table_react_params$table_data,
                                                     selection       = row_selection_mode,
                                                     defaultSelected = table_react_params$pre_selected_rows,
                                                     pagination      = pagination,
                                                     height          = table_height,
                                                     rownames        = show_rownames,
                                                     filterable      = columns_filter,
                                                     searchable      = global_search,
                                                     highlight       = row_highlight,
                                                     striped         = row_striping)
                             if (length(table_options) > 0) {
                                 all_options       <- methods::formalArgs(reactable::reactable)
                                 unnamed_options   <- append(table_options[names(table_options) == ""],
                                                             table_options[is.null(names(table_options))])
                                 invalid_options <- table_options[!(names(table_options) %in% all_options)]
                                 invalid_options <- invalid_options[!(invalid_options %in% unnamed_options)]
                                 valid_options   <- table_options[names(table_options) %in% all_options]

                                 if (length(unnamed_options) > 0) {
                                     logwarn(paste("Excluding the following unnamed option(s):",
                                                   paste(unnamed_options, collapse = ", ")), logger = logger)
                                 }

                                 if (length(invalid_options) > 0) {
                                     logwarn(paste("Excluding the following invalid option(s):",
                                                   paste(names(invalid_options), collapse = ", ")), logger = logger)
                                 }

                                 table_arguments <- append(table_arguments, valid_options)
                             }

                             table_output <- do.call(reactable::reactable, table_arguments)

                         }
                         table_output
                    })
                 }
                 shiny::observe({

                     table_state   <- reactable::getReactableState("reactTableOutputID")
                     selected_rows <- NULL

                     # data is just re/set and new react state is not ready yet
                     if (is_stale()) {
                         # If the state is NULL or empty, it means the browser just finished resetting.
                         # We can turn off the stale flag.
                         if (is.null(table_state) || is.null(table_state$selected)) {
                             is_stale(FALSE)
                            if (!is.null(table_react_params) && !is.null(table_react_params$pre_selected_rows) && is.data.frame(table_data())) {
                                selected_rows <- table_data()[table_react_params$pre_selected_rows, ]
                            }
                         }

                         downloadable_module_state(list(selected_rows = selected_rows, table_state = NULL))

                     } else { # the table is rendered, get the state directly from react table
                         if (!is.null(table_state)) {
                             if (!is.null(table_state$selected) && is.data.frame(table_data())) {
                                 selected_rows <- table_data()[table_state$selected, ]
                             }
                             downloadable_module_state(list(selected_rows = selected_rows, table_state = table_state))
                         }
                     }
                 })

                 downloadable_module_state
            }
        )
}

context("periscope2 - downloadable react table")
local_edition(3)

test_that("downloadableReactTableUI", {
    table_ui <- downloadableReactTableUI(id            = "myid",
                                         downloadtypes = c("csv"),
                                         hovertext     = "myHoverText")
    expect_equal(length(table_ui), 2)
    expect_true(grepl('id="myid-reactTableButtonDiv"', table_ui[[1]], fixed = TRUE))
    expect_true(grepl('title="myHoverText"', table_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid-reactTableButtonID-csv"', table_ui[[1]], fixed = TRUE))
    expect_false(grepl('style="display:none">', table_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid-reactTableOutputID"', table_ui[[2]], fixed = TRUE))
})


test_that("downloadableReactTableUI - no download types", {
    table_ui <- downloadableReactTableUI(id = "myid2")
    expect_equal(length(table_ui), 2)
    expect_true(grepl('id="myid2-reactTableButtonDiv"', table_ui[[1]], fixed = TRUE))
    expect_true(grepl('style="display:none">', table_ui[[1]], fixed = TRUE))
    expect_true(grepl('id="myid2-reactTableOutputID"', table_ui[[2]], fixed = TRUE))
})


# helper functions
get_mtcars_data <- reactive({
    head(mtcars)
})


test_that("downloadableReactTable - valid data", {
    testServer(downloadableReactTable,
               args = list(table_data = get_mtcars_data),
               expr = {
                   expect_true(grepl(paste0(names(get_mtcars_data()), collapse = "|"), output$reactTableOutputID))
                   expect_false(grepl(paste0(rownames(get_mtcars_data()), collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$mpg, collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$cyl, collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$disp, collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$hp, collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$drat, collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$wt, collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$qsec, collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$vs, collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$am, collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$gear, collapse = "|"), output$reactTableOutputID))
                   expect_true(grepl(paste0(get_mtcars_data()$carb, collapse = "|"), output$reactTableOutputID))
               })
})


test_that("downloadableReactTable - null or empty data.frame", {
    no_table_data_error <- "'table_data' parameter must be a function or reactive expression."
    server_error        <- capture_output(
        testServer(downloadableReactTable,
                   args = list(table_data = NULL),
                   expr = {
                       expect_true(grepl('"x":null', output$reactTableOutputID, fixed = TRUE))
                       }))
    expect_true(grepl(no_table_data_error, server_error))

    server_error <- capture_output(
        testServer(downloadableReactTable,
                   args = list(table_data = data.frame()),
                   expr = {
                       expect_true(grepl('"x":null', output$reactTableOutputID, fixed = TRUE))
                       }))
    expect_true(grepl(no_table_data_error, server_error))

    server_error <- capture_output(
        testServer(downloadableReactTable,
                   args = list(table_data = NA),
                   expr = {
                       expect_true(grepl('"x":null', output$reactTableOutputID, fixed = TRUE))
                       }))
    expect_true(grepl(no_table_data_error, server_error))



    testServer(downloadableReactTable,
               args = list(table_data = function() { NULL }),
               expr = {
                   expect_true(grepl('"x":null', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data = shiny::reactiveVal(NULL)),
               expr = {
                   expect_true(grepl('"x":null', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data = shiny::reactiveVal(NA)),
               expr = {
                   expect_true(grepl('"x":null', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data = function() { data.frame() }),
               expr = {
                   expect_true(grepl('"x":null', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data = shiny::reactiveVal(data.frame())),
               expr = {
                   expect_true(grepl('"x":null', output$reactTableOutputID, fixed = TRUE))
               })
})


test_that("downloadableReactTable - single values", {
    no_table_data_error <- "'table_data' parameter must be a function or reactive expression."
    server_error        <- capture_output(
        testServer(downloadableReactTable,
                   args = list(table_data = 5),
                   expr = {
                       expect_true(grepl('"x":null', output$reactTableOutputID, fixed = TRUE))
                       }))
    expect_true(grepl(no_table_data_error, server_error))

    testServer(downloadableReactTable,
               args = list(table_data = function() {5}),
               expr = {
                   expect_true(grepl('"table_data..":[5]', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data = shiny::reactiveVal(5)),
               expr = {
                   expect_true(grepl('"table_data..":[5]', output$reactTableOutputID, fixed = TRUE))
               })
})


test_that("downloadableReactTable - empty data.frame", {
    no_table_data_error <- "'table_data' parameter must be a function or reactive expression."
    server_error        <- capture_output(
        testServer(downloadableReactTable,
                   args = list(table_data = data.frame(5)),
                   expr = {
                       expect_true(grepl('"x":null', output$reactTableOutputID, fixed = TRUE))
                       }))
    expect_true(grepl(no_table_data_error, server_error))


    testServer(downloadableReactTable,
               args = list(table_data = function() {data.frame(5)}),
               expr = {
                   expect_true(grepl('data":{"X5":[5]}', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data = shiny::reactiveVal(data.frame(5))),
               expr = {
                   expect_true(grepl('data":{"X5":[5]}', output$reactTableOutputID, fixed = TRUE))
               })
})


test_that("downloadableReactTable - selection_mode", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    testServer(downloadableReactTable,
               args = list(table_data     = get_mtcars_data,
                           selection_mode = "Single"),
               expr = {
                   expect_true(grepl('"id":".selection"', output$reactTableOutputID, fixed = TRUE))
                   expect_true(grepl('"selection":"single"', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data     = get_mtcars_data,
                           selection_mode = "multiple"),
               expr = {
                   expect_true(grepl('"id":".selection"', output$reactTableOutputID, fixed = TRUE))
                   expect_true(grepl('"selection":"multiple"', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data     = get_mtcars_data,
                           selection_mode = "not_valid_mode"),
               expr = {
                   expect_false(grepl('"id":".selection"', output$reactTableOutputID, fixed = TRUE))
                   expect_false(grepl('"selection""', output$reactTableOutputID, fixed = TRUE))
               })
})


test_that("downloadableReactTable - pre_selected_rows", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")

    testServer(downloadableReactTable,
               args = list(table_data        = get_mtcars_data,
                           pre_selected_rows = function() {c(1, 3)}),
               expr = {
                   server_error <- capture_output(output$reactTableOutputID, print = TRUE)
                   error_msg    <- "'selection_mode' parameter must be either 'single' or 'multiple' to use 'pre_selected_rows' param. Setting default value NULL"
                   expect_true(grepl(error_msg,  server_error))
               })

    server_error <- capture_output(
        testServer(downloadableReactTable,
                   args = list(table_data        = get_mtcars_data,
                               selection_mode    = "multiple",
                               pre_selected_rows = c(1, 3)),
                   expr = {
                       output$reactTableOutputID
                   }))
    expect_true(grepl("'pre_selected_rows' parameter must be a function or reactive expression. Setting default value NULL.", server_error))

    server_error <- testServer(downloadableReactTable,
                   args = list(table_data        = get_mtcars_data,
                               selection_mode    = "multiple",
                               pre_selected_rows = function() {c(1, 3, "a")}),
                   expr = {
                       server_error <- capture_output(output$reactTableOutputID, print = TRUE)
                       error_msg    <- "'pre_selected_rows' parameter must be a function or reactive expression that returns numeric vector. Setting default value NULL."
                       expect_true(grepl(error_msg,  server_error))
                       })

    testServer(downloadableReactTable,
               args = list(table_data        = get_mtcars_data,
                           selection_mode    = "single",
                           pre_selected_rows = function() {c(1, 3)}),
               expr = {
                   server_error <- capture_output(output$reactTableOutputID, print = TRUE)
                   error_msg    <- "when 'selection_mode' is 'single', only first value of 'pre_selected_rows' will be used"
                   expect_true(grepl(error_msg,  server_error))
                   expect_true(grepl('"defaultSelected":[0]', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data        = get_mtcars_data,
                           selection_mode    = "single",
                           pre_selected_rows = function() {c(1)}),
               expr = {
                   expect_true(grepl('"defaultSelected":[0]', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data        = get_mtcars_data,
                           selection_mode    = "multiple",
                           pre_selected_rows = function() {c(1, 3)}),
               expr = {
                   expect_true(grepl('"defaultSelected":[0,2]', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data        = get_mtcars_data,
                           selection_mode    = "multiple",
                           pre_selected_rows = function() {c(1, 300)}),
               expr = {
                   server_error <- capture_output(output$reactTableOutputID, print = TRUE)
                   error_msg    <- "Excluding 'pre_selected_rows' value: 300 as it is out of range."
                   expect_true(grepl(error_msg,  server_error))
                   expect_true(grepl('"defaultSelected":[0]', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data        = get_mtcars_data,
                           selection_mode    = "multiple",
                           pre_selected_rows = function() {c(-1, 2)}),
               expr = {
                   server_error <- capture_output(output$reactTableOutputID, print = TRUE)
                   error_msg    <- "Excluding 'pre_selected_rows' value: -1 as it is out of range."
                   expect_true(grepl(error_msg,  server_error))
                   expect_true(grepl('"defaultSelected":[1]', output$reactTableOutputID, fixed = TRUE))
               })

    testServer(downloadableReactTable,
               args = list(table_data        = get_mtcars_data,
                           selection_mode    = "multiple",
                           pre_selected_rows = function() {c(-1, 2, 4, 300)}),
               expr = {
                   server_error <- capture_output(output$reactTableOutputID, print = TRUE)
                   error_msg    <- "Excluding 'pre_selected_rows' values: -1, 300 as they are out of range."
                   expect_true(grepl(error_msg,  server_error))
                   expect_true(grepl('"defaultSelected":[1,3]', output$reactTableOutputID, fixed = TRUE))
               })

    server_error <- capture_output(
        testServer(downloadableReactTable,
                   args = list(table_data        = get_mtcars_data,
                               selection_mode    = "multiple",
                               pre_selected_rows = function() {c(-1, 300)}),
                   expr = {
                       expect_false(grepl('"defaultSelected"', output$reactTableOutputID, fixed = TRUE))
                       }), print = TRUE)
    error_msg <- "All 'pre_selected_rows' values are out of range. Setting default value NULL."
    expect_true(grepl(error_msg,  server_error))

    testServer(downloadableReactTable,
                   args = list(table_data        = get_mtcars_data,
                               selection_mode    = "single",
                               pre_selected_rows = function() {c(-1, 300)}),
                   expr = {
                       server_error <- capture_output(output$reactTableOutputID, print = TRUE)
                       error_msg    <- "when 'selection_mode' is 'single', only first value of 'pre_selected_rows' will be used"
                       expect_true(grepl(error_msg,  server_error))
                       expect_false(grepl('"defaultSelected"', output$reactTableOutputID, fixed = TRUE))
               })
})


test_that("downloadableReactTable - file_name_root and download_data_fxns", {
    testServer(downloadableReactTable,
                   args = list(table_data         = get_mtcars_data,
                               download_data_fxns = list(csv = get_mtcars_data),
                               file_name_root     = "test"),
                   expr = {
                       expect_true(grepl(paste0(names(get_mtcars_data()), collapse = "|"), output$reactTableOutputID))
               })

    testServer(downloadableReactTable,
                   args = list(table_data         = get_mtcars_data,
                               download_data_fxns = list(csv = get_mtcars_data),
                               file_name_root     = reactiveVal("test")),
                   expr = {
                       expect_true(grepl(paste0(names(get_mtcars_data()), collapse = "|"), output$reactTableOutputID))
               })

    testServer(downloadableReactTable,
                   args = list(table_data         = get_mtcars_data,
                               download_data_fxns = list(csv = get_mtcars_data),
                               file_name_root     = function() { "test" }),
                   expr = {
                       expect_true(grepl(paste0(names(get_mtcars_data()), collapse = "|"), output$reactTableOutputID))
               })

    server_warning <- capture_output(testServer(downloadableReactTable,
                   args = list(table_data         = function() { "test" },
                               download_data_fxns = list(csv = get_mtcars_data),
                               file_name_root     = NULL),
                   expr = {
                       expect_true(grepl(paste0(names(get_mtcars_data()), collapse = "|"), output$reactTableOutputID))
               }), print = TRUE)

    warn_msg <- "file_name_root' parameter should not be NULL. Setting default value 'data_file'"
    expect_true(grepl(warn_msg, server_warning))
})


test_that("downloadableReactTable - table_options", {
    testServer(downloadableReactTable,
               args = list(table_data    = function() { "test" },
                           table_options = list(showSortable = TRUE)),
               expr = {
                   expect_true(grepl('"showSortable":true', output$reactTableOutputID))
    })

    testServer(downloadableReactTable,
                   args = list(table_data    = function() { "test" },
                               table_options = list("unnamed_option")),
                   expr = {
                       server_warning <- capture_output(output$reactTableOutputID, print = TRUE)
                       warn_msg       <- "Excluding the following unnamed option(s): unnamed_option"
                       expect_true(grepl(warn_msg, server_warning, fixed = TRUE))
    })

    testServer(downloadableReactTable,
                   args = list(table_data    = function() { "test" },
                               table_options = list(not_valid = "option")),
                   expr = {
                       server_warning <- capture_output(output$reactTableOutputID, print = TRUE)
                       warn_msg       <- "Excluding the following invalid option(s): not_valid"
                       expect_true(grepl(warn_msg, server_warning, fixed = TRUE))
    })

    server_warning <- capture_output(testServer(
        downloadableReactTable,
        args = list(table_data    = function() { "test" },
                    table_options = list(invalid      = "option",
                                         showSortable = TRUE,
                                         invalid_2    = "option2",
                                         "unnamed_option")),
        expr = {
            expect_true(grepl('"showSortable":true', output$reactTableOutputID))
    }), print = TRUE)

    warn_msg1 <- "Excluding the following unnamed option(s): unnamed_option"
    warn_msg2 <- "Excluding the following invalid option(s): invalid, invalid_2"

    expect_true(grepl(warn_msg1, server_warning, fixed = TRUE))
    expect_true(grepl(warn_msg2, server_warning, fixed = TRUE))

})


test_that("downloadableReactTable - module return", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")
    local_mocked_bindings(
        getReactableState = function(...) {
         list(showSortable    = TRUE,
              defaultSelected = c(2, 3),
              selected        = c(2, 3))
        },
        .package = "reactable")

    testServer(
        downloadableReactTable,
        args = list(table_data = function() { "test" }),
        expr = {
            result <- session$returned()
            expect_equal(length(result), 2)
            expect_true(all(c("selected_rows", "table_state") %in% names(result)))
            expect_true(is.null(result$selected_rows))
   })

    local_mocked_bindings(
        getReactableState = function(...) NULL,
        .package = "reactable")


    testServer(
        downloadableReactTable,
        args = list(
            table_data        = get_mtcars_data,
            selection_mode    = "multiple",
            pre_selected_rows = reactive(c(2, 3))
        ),
        expr = {
            # Simulate two reactive ticks:
            # Tick 1: table_data changes; reactable hasn't reported state yet
            #         → module falls back to pre_selected_rows
            # Tick 2: reactable state still NULL; assert fallback value survives
            session$flushReact()
            is_stale(TRUE)
            session$flushReact()
            result <- session$returned()
            expect_equal(length(result), 2)
            expect_true(all(c("selected_rows", "table_state") %in% names(result)))
            expect_true(NROW(result$selected_rows) == 2)
            expect_true(all(c("Mazda RX4 Wag", "Datsun 710") %in%  rownames(result$selected_rows)))
  })

})


test_that("downloadableReactTable - returns selected rows from reactable state", {
    skip_if(getRversion() < "4.1.0", "Skipping due to lifecycle warnings in R < 4.1.0")

    local_mocked_bindings(
        getReactableState = function(...) {
            list(showSortable = TRUE,
                 selected     = c(2, 3))
        },
        .package = "reactable")

    testServer(
        downloadableReactTable,
        args = list(
            table_data     = get_mtcars_data,
            selection_mode = "multiple"
        ),
        expr = {
            # Tick 1: observeEvent(table_data) sets is_stale=TRUE.
            #         Final observer runs, sees stale + selected non-NULL,
            #         so it does NOT enter the reset branch and is_stale stays TRUE.
            session$flushReact()

            # Manually flip is_stale to FALSE to simulate the point in time
            # after reactable has rendered and reported its state.
            is_stale(FALSE)
            session$flushReact()

            result <- session$returned()

            expect_equal(length(result), 2)
            expect_true(all(c("selected_rows", "table_state") %in% names(result)))
            expect_false(is.null(result$table_state))
            expect_equal(result$table_state$selected, c(2, 3))
            expect_equal(NROW(result$selected_rows), 2)
            expect_true(all(c("Mazda RX4 Wag", "Datsun 710")
                            %in% rownames(result$selected_rows)))
        })
})

#' Build application theme configuration YAML file
#'
#' Call this as an addin to build valid yaml file that is needed for creating application
#' periscope_style.yaml file. The generated file can be used in periscope2 app by putting it inside generated app www folder.
#'
#' The method can be called directly via `R` console or via RStudio addins menu
#'
#' @return launch gadget window
#'
#' @examples
#' if (interactive()) {
#'    periscope2:::themeConfigurationsAddin()
#' }
#'
#' @seealso \link[periscope2:create_application]{periscope2:create_application()}
#'
#' @export
themeConfigurationsAddin <- function() {
    if (!requireNamespace(c("miniUI", "colourpicker"), quietly = TRUE)) {
        stop("You must have packages 'miniUI' and 'colourpicker' installed to use this add-in",
             call. = FALSE)
    }

    shiny::runGadget(app    = themeBuilder_addin_UI(),
                     server = function(input, output, session){
                         themeBuilder_addin_server()
                      },
                     viewer = shiny::browserViewer())
}


themeBuilder_addin_UI <- function() {
    shiny::addResourcePath(prefix        = "img",
                           directoryPath = system.file("fw_templ/www", package = "periscope2"))
    miniUI::miniPage(
        miniUI::gadgetTitleBar("Theme Configuration YAML File Builder"),
        miniUI::miniTabstripPanel(
            miniUI::miniTabPanel(
                "Status Colors",
                miniUI::miniContentPanel(
                    shiny::p("Specify the colors to used when the corresponding status argument in shiny UI elements such as infoBox, valueBox, cards are set."),
                    stableColumnLayout(
                        colourpicker::colourInput(inputId          = "primary",
                                                  label            = "Primary",
                                                  showColour       = "both",
                                                  value            = "#B221DD",
                                                  allowTransparent = TRUE),
                        colourpicker::colourInput(inputId          = "secondary",
                                                  label            = "Secondary",
                                                  showColour       = "both",
                                                  value            = "#6c757d",
                                                  allowTransparent = TRUE)),
                    stableColumnLayout(
                        colourpicker::colourInput(inputId          = "success",
                                                  label            = "Success",
                                                  showColour       = "both",
                                                  value            = "#2ED610",
                                                  allowTransparent = TRUE),
                        colourpicker::colourInput(inputId          = "info",
                                                  label            = "Info",
                                                  showColour       = "both",
                                                  value            = "#7BDFF2",
                                                  allowTransparent = TRUE)),
                    stableColumnLayout(
                        colourpicker::colourInput(inputId          = "warning",
                                                  label            = "Warning",
                                                  showColour       = "both",
                                                  value            = "#FFF200",
                                                  allowTransparent = TRUE),
                        colourpicker::colourInput(inputId          = "danger",
                                                  label            = "Danger",
                                                  showColour       = "both",
                                                  value            = "#CE0900",
                                                  allowTransparent = TRUE)),
                    stableColumnLayout(
                        colourpicker::colourInput(inputId    = "light",
                                                  label      = "Light",
                                                  showColour = "both",
                                                  value      = "#f8f9fa",
                                                  allowTransparent = TRUE),
                        colourpicker::colourInput(inputId    = "dark",
                                                  label      = "Dark",
                                                  showColour = "both",
                                                  value      = "#343a40",
                                                  allowTransparent = TRUE))
                )
            ),
            miniUI::miniTabPanel(
                "Sidebar Colors",
                miniUI::miniContentPanel(
                    shiny::p("Sidebar colors variables allow you to change sidebars (left and right) related colors"),
                    shiny::tags$i("Use value \"#00000000\" to reset back to original theme default color"),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "bg",
                                                                 label            = "Background Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "hover_bg",
                                                                 label            = "Hover Background Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE)),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "color",
                                                                 label            = "Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "hover_color",
                                                                 label            = "Hover Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE)),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "active_color",
                                                                 label            = "Active Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "submenu_bg",
                                                                 label            = "Submenu Background Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE)),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "submenu_color",
                                                                 label            = "Submenu Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "submenu_hover_color",
                                                                 label            = "Submenu Hover Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE)),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "submenu_hover_bg",
                                                                 label            = "Submenu Hover Background Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "submenu_active_color",
                                                                 label            = "Submenu Active Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE)),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "submenu_active_bg",
                                                                 label            = "Submenu Active Background Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "header_color",
                                                                 label            = "Header Color",
                                                                 showColour       = "both",
                                                                 value            = "#00000000",
                                                                 allowTransparent = TRUE))
                )
            ),
            miniUI::miniTabPanel(
                "Sidebar Layout",
                miniUI::miniContentPanel(
                    shiny::p("Sidebar layout variables allow you to change sidebars (left and right) width, padding, ...",
                             shiny::tags$br(),
                             shiny::tags$i("All units are in pixels")),
                    stableColumnLayout(shiny::numericInput(inputId = "sidebar_width",
                                                           label   = "Left Sidebar Width",
                                                           value   = NA),
                                       shiny::numericInput(inputId = "control_sidebar_width",
                                                           label   = "Right Sidebar Width",
                                                           value   = NA)),
                    stableColumnLayout(shiny::numericInput(inputId = "sidebar_padding_x",
                                                           label   = "Sidebar Horizontal Padding",
                                                           value   = NA),
                                       shiny::numericInput(inputId = "sidebar_padding_y",
                                                           label   = "Sidebar Vertical Padding",
                                                           value   = NA)),
                    stableColumnLayout(shiny::numericInput(inputId = "sidebar_mini_width",
                                                           label   = periscope2::ui_tooltip(id        = "textTip",
                                                                                            label     = "Width for mini sidebar",
                                                                                            text      = paste("Left sidebar can be minimized to show menu icons (and not to be fully collapsed).",
                                                                                                              "That property controls the width of the minimized sidebar.",
                                                                                                              sep = "<br/><br/>"),
                                                                                            placement = "bottom"),
                                                           value   = NA))
                )
            ),
            miniUI::miniTabPanel(
                "Main Colors",
                miniUI::miniContentPanel(
                    shiny::p("Specify the specific hex codes for the following color words that can be referenced in the app."),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "blue",
                                                                 label            = "Blue",
                                                                 showColour       = "both",
                                                                 value            = "#007bff",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "lightblue",
                                                                 label            = "Light Blue",
                                                                 showColour       = "both",
                                                                 value            = "#3c8dbc",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "navy",
                                                                 label            = "Navy",
                                                                 showColour       = "both",
                                                                 value            = "#001f3f",
                                                                 allowTransparent = TRUE)),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "cyan",
                                                                 label            = "Cyan",
                                                                 showColour       = "both",
                                                                 value            = "#17a2b8",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "teal",
                                                                 label            = "Teal",
                                                                 showColour       = "both",
                                                                 value            = "#39cccc",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "olive",
                                                                 label            = "Olive",
                                                                 showColour       = "both",
                                                                 value            = "#3d9970",
                                                                 allowTransparent = TRUE)),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "green",
                                                                 label            = "Green",
                                                                 showColour       = "both",
                                                                 value            = "#28a745",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "lime",
                                                                 label            = "Lime",
                                                                 showColour       = "both",
                                                                 value            = "#01ff70",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "orange",
                                                                 label            = "Orange",
                                                                 showColour       = "both",
                                                                 value            = "#ff851b",
                                                                 allowTransparent = TRUE)),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "yellow",
                                                                 label            = "Yellow",
                                                                 showColour       = "both",
                                                                 value            = "#ffc107",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "fuchsia",
                                                                 label            = "Fuchsia",
                                                                 showColour       = "both",
                                                                 value            = "#f012be",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "purple",
                                                                 label            = "Purple",
                                                                 showColour       = "both",
                                                                 value            = "#605ca8",
                                                                 allowTransparent = TRUE)),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "maroon",
                                                                 label            = "Maroon",
                                                                 showColour       = "both",
                                                                 value            = "#d81b60",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "red",
                                                                 label            = "Red",
                                                                 showColour       = "both",
                                                                 value            = "#dc3545",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "black",
                                                                 label            = "Black",
                                                                 showColour       = "both",
                                                                 value            = "#111",
                                                                 allowTransparent = TRUE)),
                    stableColumnLayout(colourpicker::colourInput(inputId          = "gray_x_light",
                                                                 label            = "Gray X Light",
                                                                 showColour       = "both",
                                                                 value            = "#d2d6de",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "gray_600",
                                                                 label            = "Gray 600",
                                                                 showColour       = "both",
                                                                 value            = "#6c757d",
                                                                 allowTransparent = TRUE),
                                       colourpicker::colourInput(inputId          = "white",
                                                                 label            = "White",
                                                                 showColour       = "both",
                                                                 value            = "#ffffff",
                                                                 allowTransparent = TRUE)),

                )
            ),
            miniUI::miniTabPanel(
                "Color Contrast",
                miniUI::miniContentPanel(
                    miniUI::miniContentPanel(
                        shiny::p("These variables allow to customize color",
                                 " used if contrast between a color and its background is",
                                 " under threshold. For example, it's used to choose text color ",
                                 " written in bs4ValueBox with background defined by a status."),
                        stableColumnLayout(
                            shiny::numericInput(inputId = "contrasted_threshold",
                                                label   = periscope2::ui_tooltip(id    = "contrasted_thresholdTip",
                                                                                 label = "Contrasted Threshold",
                                                                                 text  = paste0("The yiq lightness value",
                                                                                                " that determines when the",
                                                                                                " lightness of color changes",
                                                                                                " from \"dark\" to \"light\"",
                                                                                                " Acceptable values are between 0 and
                                                                                                  255.")),
                                                value   = NA,
                                                min     = 0,
                                                max     = 255),
                            colourpicker::colourInput(inputId          = "text_dark",
                                                      label            = periscope2::ui_tooltip(id    = "text_darkTip",
                                                                                                label = "Text Dark",
                                                                                                text  = "Dark text color"),
                                                      showColour       = "both",
                                                      value            = "#00000000",
                                                      allowTransparent = TRUE),
                            colourpicker::colourInput(inputId          = "text_light",
                                                      label            = periscope2::ui_tooltip(id    = "light_darkTip",
                                                                                                label = "Text Light",
                                                                                                text  = "Light text color"),
                                                      showColour       = "both",
                                                      value            = "#00000000",
                                                      allowTransparent = TRUE)
                        )
                    )
                )
            ),
            miniUI::miniTabPanel(
                "Other Variables",
                miniUI::miniContentPanel(
                    shiny::p("This is an advanced method to add or customize any more AdminLTE",
                             " or Bootstrap 4 variable in periscope2 generated application theme."),
                    shiny::tags$i("Please refer to ",
                                  shiny::tags$a(href   = "https://adminlte.io/docs/3.2/",
                                                target = "_blank",
                                                "AdminLTE"),
                                  " or ",
                                  shiny::tags$a(href   = "https://getbootstrap.com/docs/4.0/getting-started/theming/",
                                                target = "_blank",
                                                "Bootstrap"),
                                  " documentation for variables details info",
                                  shiny::tags$br(),
                                  shiny::tags$br()),
                    stableColumnLayout(shiny::tags$div(style = "margin-bottom: 30px;",
                                                       shiny::actionButton(inputId = "addVariable",
                                                                           label   = "Add Variable"))),
                    stableColumnLayout(shiny::tags$div(id = "variablesPlaceholder"))
                )
            )
        ),
        stableColumnLayout(
            miniUI::miniButtonBlock(
                shiny::downloadButton(outputId = "downloadConfig",
                                      label    = periscope2::ui_tooltip(id    = "downloadTip",
                                                                        label = "Download periscope2 theme",
                                                                        text  = "Download theme configuration file and replace the file in the generated periscope app shiny_app/www folder")))
        )
    )
}


themeBuilder_addin_server <- function(id = NULL) {
    shiny::moduleServer(
        id,
        function(input, output, session) {
            variables       <- fresh::search_vars_bs4dash()
            added_variables <- shiny::reactiveVal(c())
            ids             <- shiny::reactiveVal(c(sample(NROW(variables))))

            shiny::observeEvent(input$addVariable, {
                variable_id      <- ids()[1]
                variable_row_id  <- paste0(variable_id,"-variableRow")
                variable_name_id <- paste0(variable_id,"-variableName")
                variable_val_id  <- paste0(variable_id,"-variableValue")
                remove_btn_id    <- paste0(variable_id,"-removeVariableBtn")

                shiny::insertUI(
                    selector = "#variablesPlaceholder",
                    ui       = shiny::tags$div(id = variable_row_id,
                                               stableColumnLayout(
                                                   shiny::selectizeInput(inputId = variable_name_id,
                                                                         label   = "Select Variable",
                                                                         choices = unique(variables$variable)),
                                                   shiny::textInput(inputId = variable_val_id,
                                                                    label   = "Value"),
                                                   shiny::actionButton(inputId = remove_btn_id,
                                                                       label   = NULL,
                                                                       icon    = shiny::icon("xmark"),
                                                                       style   = "margin-top: 25px;")))
                )

                ids(ids()[-which(ids() == variable_id)])
                added_variables(c(added_variables(), variable_id))
                shiny::observeEvent(input[[remove_btn_id]], {
                    variable_row_id  <- paste0("#", variable_row_id)
                    shiny::removeUI(selector = variable_row_id, immediate = TRUE)
                })
            })

            output$downloadConfig <- shiny::downloadHandler(
                filename = function() {
                    "periscope_style.yaml"
                },
                content = function(theme_file) {
                    lines           <- c()
                    status          <- c()
                    sidebar_colors  <- c()
                    sidebar_layout  <- c()
                    main_colors     <- c()
                    colors_contrast <- c()
                    other_variables <- c()

                    ### Status colors
                    if (input$primary != "#00000000") {
                        status <- c(status, paste0("primary: \"", input$primary, "\""))
                    }

                    if (input$secondary != "#00000000") {
                        status <- c(status, paste0("secondary: \"", input$secondary, "\""))
                    }

                    if (input$success != "#00000000") {
                        status <- c(status, paste0("success: \"", input$success, "\""))
                    }

                    if (input$info != "#00000000") {
                        status <- c(status, paste0("info: \"", input$info, "\""))
                    }

                    if (input$warning != "#00000000") {
                        status <- c(status, paste0("warning: \"", input$warning, "\""))
                    }

                    if (input$danger != "#00000000") {
                        status <- c(status, paste0("danger: \"", input$danger, "\""))
                    }

                    if (input$light != "#00000000") {
                        status <- c(status, paste0("light: \"", input$light, "\""))
                    }

                    if (input$dark != "#00000000") {
                        status <- c(status, paste0("dark: \"", input$dark, "\""))
                    }

                    if (length(status) > 0) {
                        status <- c("# Status Colors",
                                    "## Sets the status colors that affects the color of the header, valueBox, infoBox and box.",
                                    "## Valid values are names of the color or hex-decimal value of the color (i.e,: \"blue\", \"#086A87\").",
                                    "## Blank/empty values will use the default values",
                                    status)
                        lines <- c(lines, status)
                    }

                    ###### Sidebar colors
                    if (input$bg != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Background color"))
                        sidebar_colors <- c(sidebar_colors, paste0("bg: \"", input$bg, "\""))
                    }

                    if (input$hover_bg != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Hover background color"))
                        sidebar_colors <- c(sidebar_colors, paste0("hover_bg: \"", input$hover_bg, "\""))
                    }

                    if (input$color != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Color"))
                        sidebar_colors <- c(sidebar_colors, paste0("color: \"", input$color, "\""))
                    }

                    if (input$hover_color != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Hover color"))
                        sidebar_colors <- c(sidebar_colors, paste0("hover_color: \"", input$hover_color, "\""))
                    }

                    if (input$active_color != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Active color"))
                        sidebar_colors <- c(sidebar_colors, paste0("active_color: \"", input$active_color, "\""))
                    }

                    if (input$submenu_bg != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Submenu background color"))
                        sidebar_colors <- c(sidebar_colors, paste0("submenu_bg: \"", input$submenu_bg, "\""))
                    }

                    if (input$submenu_color != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Submenu color"))
                        sidebar_colors <- c(sidebar_colors, paste0("submenu_color: \"", input$submenu_color, "\""))
                    }

                    if (input$submenu_hover_color != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Submenu hover color"))
                        sidebar_colors <- c(sidebar_colors, paste0("submenu_hover_color: \"", input$submenu_hover_color, "\""))
                    }

                    if (input$submenu_hover_bg != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Submenu hover background color"))
                        sidebar_colors <- c(sidebar_colors, paste0("submenu_hover_bg: \"", input$submenu_hover_bg, "\""))
                    }

                    if (input$submenu_active_color != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Submenu active color"))
                        sidebar_colors <- c(sidebar_colors, paste0("submenu_active_color: \"", input$submenu_active_color, "\""))
                    }

                    if (input$submenu_active_bg != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Submenu active background color"))
                        sidebar_colors <- c(sidebar_colors, paste0("submenu_active_bg: \"", input$submenu_active_bg, "\""))
                    }

                    if (input$header_color != "#00000000") {
                        sidebar_colors <- c(sidebar_colors, paste0("### Header color"))
                        sidebar_colors <- c(sidebar_colors, paste0("header_color: \"", input$header_color, "\""))
                    }

                    if (length(sidebar_colors) > 0) {
                        if (length(lines) > 0) {
                            lines <- c(lines, "\n")
                        }

                        sidebar_colors <- c("# Sidebar Colors",
                                            "## Sidebar colors variables allow you to change sidebars (left and right) related colors",
                                            "## Blank/empty values will use the default values",
                                            sidebar_colors)
                        lines <- c(lines, sidebar_colors)
                    }

                    ###### Sidebar Layout
                    if (!is.na(input$sidebar_width) && (input$sidebar_width != "")) {
                        sidebar_layout <- c(sidebar_layout, paste0("### Left Sidebar width"))
                        sidebar_layout <- c(sidebar_layout, paste0("sidebar_width: \"", input$sidebar_width, "\""))
                    }

                    if (!is.na(input$control_sidebar_width) && (input$control_sidebar_width != "")) {
                        sidebar_layout <- c(sidebar_layout, paste0("### Right Sidebar width"))
                        sidebar_layout <- c(sidebar_layout, paste0("control_sidebar_width: \"", input$control_sidebar_width, "\""))
                    }

                    if (!is.na(input$sidebar_padding_x) && (input$sidebar_padding_x != "")) {
                        sidebar_layout <- c(sidebar_layout, paste0("### Sidebar horizontal padding"))
                        sidebar_layout <- c(sidebar_layout, paste0("sidebar_padding_x: \"", input$sidebar_padding_x, "\""))
                    }

                    if (!is.na(input$sidebar_padding_y) && (input$sidebar_padding_y != "")) {
                        sidebar_layout <- c(sidebar_layout, paste0("### Sidebar vertical padding"))
                        sidebar_layout <- c(sidebar_layout, paste0("sidebar_padding_y: \"", input$sidebar_padding_y, "\""))
                    }

                    if (!is.na(input$sidebar_mini_width) && (input$sidebar_mini_width != "")) {
                        sidebar_layout <- c(sidebar_layout, paste0("### Width for mini sidebar"))
                        sidebar_layout <- c(sidebar_layout, paste0("sidebar_mini_width: \"", input$sidebar_mini_width, "\""))
                    }

                    if (length(sidebar_layout) > 0) {
                        if (length(lines) > 0) {
                            lines <- c(lines, "\n")
                        }

                        sidebar_layout <- c("# Sidebar Layout",
                                            "## Sidebar colors variables allow you to change sidebars (left and right) related colors",
                                            "## Blank/empty values will use the default values",
                                            sidebar_layout)
                        lines <- c(lines, sidebar_layout)
                    }

                    ###### Main colors
                    if (input$blue != "#00000000") {
                        main_colors <- c(main_colors, paste0("blue: \"", input$blue, "\""))
                    }

                    if (input$lightblue != "#00000000") {
                        main_colors <- c(main_colors, paste0("lightblue: \"", input$lightblue, "\""))
                    }

                    if (input$navy != "#00000000") {
                        main_colors <- c(main_colors, paste0("navy: \"", input$navy, "\""))
                    }

                    if (input$cyan != "#00000000") {
                        main_colors <- c(main_colors, paste0("cyan: \"", input$cyan, "\""))
                    }

                    if (input$teal != "#00000000") {
                        main_colors <- c(main_colors, paste0("teal: \"", input$teal, "\""))
                    }

                    if (input$olive != "#00000000") {
                        main_colors <- c(main_colors, paste0("olive: \"", input$olive, "\""))
                    }

                    if (input$green != "#00000000") {
                        main_colors <- c(main_colors, paste0("green: \"", input$green, "\""))
                    }

                    if (input$lime != "#00000000") {
                        main_colors <- c(main_colors, paste0("lime: \"", input$lime, "\""))
                    }

                    if (input$orange != "#00000000") {
                        main_colors <- c(main_colors, paste0("orange: \"", input$orange, "\""))
                    }

                    if (input$yellow != "#00000000") {
                        main_colors <- c(main_colors, paste0("yellow: \"", input$yellow, "\""))
                    }

                    if (input$fuchsia != "#00000000") {
                        main_colors <- c(main_colors, paste0("fuchsia: \"", input$fuchsia, "\""))
                    }

                    if (input$purple != "#00000000") {
                        main_colors <- c(main_colors, paste0("purple: \"", input$purple, "\""))
                    }

                    if (input$maroon != "#00000000") {
                        main_colors <- c(main_colors, paste0("maroon: \"", input$maroon, "\""))
                    }

                    if (input$red != "#00000000") {
                        main_colors <- c(main_colors, paste0("red: \"", input$red, "\""))
                    }

                    if (input$black != "#00000000") {
                        main_colors <- c(main_colors, paste0("black: \"", input$black, "\""))
                    }

                    if (input$gray_x_light != "#00000000") {
                        main_colors <- c(main_colors, paste0("gray_x_light: \"", input$gray_x_light, "\""))
                    }

                    if (input$gray_600 != "#00000000") {
                        main_colors <- c(main_colors, paste0("gray_600: \"", input$gray_600, "\""))
                    }

                    if (input$white != "#00000000") {
                        main_colors <- c(main_colors, paste0("white: \"", input$white, "\""))
                    }

                    if (length(main_colors) > 0) {
                        if (length(lines) > 0) {
                            lines <- c(lines, "\n")
                        }

                        main_colors <- c("# Main Colors",
                                         "## Specify the specific hex codes for the following color words that can be referenced in the app.", main_colors)
                        lines <- c(lines, main_colors)
                    }

                    # colors contrast
                    if (!is.na(input$contrasted_threshold) && (input$contrasted_threshold != "")) {
                        colors_contrast <- c(colors_contrast, paste0("### The yiq lightness value that determines when the",
                                                                     "### lightness of color changes from \"dark\" to \"light\".",
                                                                     "Acceptable values are between 0 and 255"))
                        colors_contrast <- c(colors_contrast, paste0("contrasted_threshold: \"", input$contrasted_threshold, "\""))
                    }

                    if (input$text_dark != "#00000000") {
                        colors_contrast <- c(colors_contrast, paste0("### Dark text color"))
                        colors_contrast <- c(colors_contrast, paste0("text_dark: \"", input$text_dark, "\""))
                    }

                    if (input$text_light != "#00000000") {
                        colors_contrast <- c(colors_contrast, paste0("### Light text color"))
                        colors_contrast <- c(colors_contrast, paste0("text_light: \"", input$text_light, "\""))
                    }

                    if (length(colors_contrast) > 0) {
                        if (length(lines) > 0) {
                            lines <- c(lines, "\n")
                        }

                        colors_contrast <- c("# Colors Contrast",
                                             "## These variables allow to customize color used if contrast between a color",
                                             "## and its background is under threshold. For example, it's used to choose",
                                             "## text color written in bs4ValueBox with background defined by a status.",
                                         colors_contrast)
                        lines <- c(lines, colors_contrast)
                    }

                    # Other variables
                    if (length(added_variables()) > 0) {
                        for (var in added_variables()) {
                            name  <- input[[paste0(var, "-variableName")]]
                            value <- input[[paste0(var, "-variableValue")]]

                            if (!is.na(name) && (name != "") &&
                                !is.na(value) && (value != "")) {
                                other_variables <- c(other_variables,
                                                     paste0("\"", name, "\": ", "\"", value, "\""))
                            }
                        }
                    }

                    if (length(other_variables) > 0) {
                        if (length(lines) > 0) {
                            lines <- c(lines, "\n")
                        }

                        other_variables <- c("# Other Variables",
                                             "## Use any AdminLTE or Bootstrap variables to customize app theme",
                                             other_variables)
                        lines <- c(lines, other_variables)
                    }
                    writeLines(lines, theme_file)
                }
            )

            shiny::observeEvent(input$done, {
                shiny::removeResourcePath(prefix = "img")
                invisible(shiny::stopApp())
            })

            shiny::observeEvent(input$cancel, {
                shiny::removeResourcePath(prefix = "img")
                invisible(shiny::stopApp())
            })
        }
    )
}

# Conversion functions for existing applications.


ui_filename <- "ui.R"


# Checks if the location contains a periscope application by checking mandatory structure
.is_periscope_app <- function(location = ".") {
    all(file.exists(file.path(location, c("server.R", "ui.R", "global.R", "program"))))
}


#' Add the left sidebar to an existing application
#'
#' User can update an existing application that does not have a left side bar and add a new empty one using this function.
#'
#' If conversion is successful, the following message will be returned
#' \emph{"Add left sidebar conversion was successful. File(s) updated: ui.R, ui_left_sidebar.R"}
#'
#' If the function called on an application with an existing left bar, message
#' \emph{"Left sidebar already available, no conversion needed"}
#' will be returned with no conversion
#'
#' If the passed location is invalid, empty, not exist or not a valid periscope2 application, nothing will be added
#' and a related error message will be printed in console
#'
#'
#' @param location path of the existing periscope2 application
#'
#' @return no return value, creates left sidebar related UI R file and updates related source call in ui.R
#'
#' @export
#' @seealso \link[periscope2]{create_right_sidebar}
create_left_sidebar <- function(location) {
    assertthat::assert_that(!is.null(location),
                            location != "",
                            msg = "Add left sidebar conversion could not proceed, location cannot be empty!")
    assertthat::assert_that(dir.exists(location),
                            msg = paste0("Add left sidebar conversion could not proceed, location=<", location, "> does not exist!"))
    assertthat::assert_that(.is_periscope_app(location),
                            msg = paste0("Add left sidebar conversion could not proceed, location=<",
                                         location,
                                         "> does not contain a valid periscope application!"))
    tryCatch({
        left_sidebar_file <- "ui_left_sidebar.R"
        usersep           <- .Platform$file.sep
        files_updated     <- c()
        ui_content        <- readLines(con = paste(location, ui_filename, sep = usersep))

        # update UI if needed
        if (!any(grepl("ui_left_sidebar.R", ui_content))) {
            ui_content <- gsub(pattern     = "periscope2:::create_application_dashboard",
                               replacement = paste("source(paste(\"program\", \"ui_left_sidebar.R\", sep = .Platform$file.sep), local = TRUE)",
                                                   "\nperiscope2:::create_application_dashboard"),
                               x           = ui_content)


            writeLines(ui_content, con = paste(location, ui_filename, sep = usersep))

            # add left_sidebar file
            writeLines(readLines(con = system.file("fw_templ",  "p_blank", left_sidebar_file, package = "periscope2")),
                       con = paste(location, "program", left_sidebar_file, sep = usersep))

            files_updated <- c(files_updated, c(ui_filename, left_sidebar_file))
        }

        if (length(files_updated) > 0) {
            message(paste("Add left sidebar conversion was successful. File(s) updated:",  paste(files_updated, collapse = ", ")))
        } else {
            message("Left sidebar already available, no conversion needed")
        }
    },
    warning = function(w) {
        warning(w$message, call. = FALSE)
    })
}


#' Add the right sidebar to an existing application
#'
#' User can update an existing application that does not have a right side bar and add a new empty one using this function.
#'
#' If conversion is successful, the following message will be returned
#' \emph{"Add right sidebar conversion was successful. File(s) updated: ui.R, ui_right_sidebar.R"}
#'
#' If the function called on an application with an existing right bar, message
#' \emph{"Right sidebar already available, no conversion needed"}
#' will be returned with no conversion
#'
#' If the passed location is invalid, empty, not exist or not a valid periscope2 application, nothing will be added
#' and a related error message will be printed in console
#'
#'
#' @param location path of the existing periscope2 application.
#'
#' @return no return value, creates right sidebar related UI R file and updates related source call in ui.R
#'
#' @export
#' @seealso \link[periscope2]{create_left_sidebar}
create_right_sidebar <- function(location) {
    assertthat::assert_that(!is.null(location),
                            location != "",
                            msg = "Add right sidebar conversion could not proceed, location cannot be empty!")
    assertthat::assert_that(dir.exists(location),
                            msg = paste0("Add right sidebar conversion could not proceed, location=<", location, "> does not exist!"))
    assertthat::assert_that(.is_periscope_app(location),
                            msg = paste0("Add right sidebar conversion could not proceed, location=<",
                                         location,
                                         "> does not contain a valid periscope application!"))
    tryCatch({
        right_sidebar_file <- "ui_right_sidebar.R"
        usersep            <- .Platform$file.sep
        files_updated      <- c()
        ui_content         <- readLines(con = paste(location, ui_filename, sep = usersep))
        # update UI if needed
        if (!any(grepl("ui_right_sidebar.R", ui_content))) {
            ui_content <- gsub(pattern     = "periscope2:::create_application_dashboard",
                               replacement = paste("source(paste(\"program\", \"ui_right_sidebar.R\", sep = .Platform$file.sep), local = TRUE)",
                                                   "\nperiscope2:::create_application_dashboard"),
                               x           = ui_content)
            writeLines(ui_content, con = paste(location, ui_filename, sep = usersep))

            # add right_sidebar file
            writeLines(readLines(con = system.file("fw_templ",  "p_blank", right_sidebar_file, package = "periscope2")),
                       con = paste(location, "program", right_sidebar_file, sep = usersep))

            files_updated <- c(files_updated, c(ui_filename, right_sidebar_file))
        }

        if (length(files_updated) > 0) {
            message(paste("Add right sidebar conversion was successful. File(s) updated:",  paste(files_updated, collapse = ", ")))
        } else {
            message("Right sidebar already available, no conversion needed")
        }
    },
    warning = function(w) {
        warning(w$message, call. = FALSE)
    })
}

context("periscope2 convert existing application")


expect_converted_application <- function(location,
                                         left_sidebar = TRUE,
                                         right_sidebar = FALSE) {
    local_edition(3)
    expect_true(dir.exists(location))
    expect_true(file.exists(file.path(location, "global.R")))
    expect_true(file.exists(file.path(location, "server.R")))
    expect_true(file.exists(file.path(location, "ui.R")))
    expect_true(dir.exists(file.path(location, "program")))
    expect_true(file.exists(file.path(location, "program/ui_header.R")))
    expect_true(file.exists(file.path(location, "program/ui_footer.R")))
    expect_true(file.exists(file.path(location, "program/ui_body.R")))

    ui_content <- readLines(con = paste(location, "ui.R", sep = .Platform$file.sep))
    expect_true(any(grepl("source(paste(\"program\", \"ui_body.R\", sep = .Platform$file.sep), local = TRUE)", ui_content, fixed = TRUE)))
    expect_true(any(grepl("source(paste(\"program\", \"ui_header.R\", sep = .Platform$file.sep), local = TRUE)", ui_content, fixed = TRUE)))
    expect_true(any(grepl("create_application_dashboard()", ui_content, fixed = TRUE)))

    if (right_sidebar) {
        expect_true(any(grepl("source(paste(\"program\", \"ui_right_sidebar.R\", sep = .Platform$file.sep), local = TRUE)", ui_content, fixed = TRUE)))
    } else {
        expect_false(any(grepl("source(paste(\"program\", \"ui_right_sidebar.R\", sep = .Platform$file.sep), local = TRUE)", ui_content, fixed = TRUE)))
    }

    if (left_sidebar) {
        expect_true(any(grepl("source(paste(\"program\", \"ui_left_sidebar.R\", sep = .Platform$file.sep), local = TRUE)", ui_content, fixed = TRUE)))
    } else {
        expect_false(any(grepl("source(paste(\"program\", \"ui_left_sidebar.R\", sep = .Platform$file.sep), local = TRUE)", ui_content, fixed = TRUE)))
    }

    # clean up
    unlink(location, TRUE)
}

# creates a temp directory, copies the sample_app to this directory and returns the path of the temp app
create_app_tmp_dir <- function(left_sidebar = TRUE, right_sidebar = FALSE) {
    app_name     <- "sample_app_both_sidebars"

    if (left_sidebar && !right_sidebar) {
        app_name <- "sample_app_left_sidebar"
    } else if (!left_sidebar && right_sidebar) {
        app_name <- "sample_app_right_sidebar"
    } else if (!left_sidebar && !right_sidebar) {
        app_name <- "sample_app_no_both_sidebars"
    }


    app_temp.dir <- tempdir()
    file.copy(app_name, app_temp.dir, recursive = TRUE)
    file.path(app_temp.dir, app_name)
}

## left_sidebar tests

test_that("create_left_sidebar null location", {
    expect_error(create_left_sidebar(location = NULL),
                 "Add left sidebar conversion could not proceed, location cannot be empty!")
})

test_that("create_left_sidebar empty location", {
    expect_error(create_left_sidebar(location = ""),
                 "Add left sidebar conversion could not proceed, location cannot be empty!")
})

test_that("create_left_sidebar invalid location", {
    expect_error(create_left_sidebar(location = "invalid"),
                 "Add left sidebar conversion could not proceed, location=<invalid> does not exist!")
})

test_that("create_left_sidebar location does not contain an existing application", {
    expect_error(create_left_sidebar(location = "../testthat"),
                 "Add left sidebar conversion could not proceed, location=<../testthat> does not contain a valid periscope application!")
})

test_that("create_left_sidebar to right sidebar, valid location", {
    app_location <- create_app_tmp_dir(left_sidebar = FALSE, right_sidebar = TRUE)

    expect_message(create_left_sidebar(location = app_location), "Add left sidebar conversion was successful. File\\(s\\) updated: ui.R")
    expect_converted_application(location = app_location, left_sidebar = TRUE, right_sidebar = TRUE)
})

test_that("create_left_sidebar valid location", {
    app_location <- create_app_tmp_dir(left_sidebar = FALSE)

    expect_message(create_left_sidebar(location = app_location), "Add left sidebar conversion was successful. File\\(s\\) updated: ui.R")
    expect_converted_application(location = app_location, left_sidebar = TRUE)
})

test_that("create_left_sidebar valid location, added twice", {
    app_location <- create_app_tmp_dir(left_sidebar = FALSE)

    expect_message(create_left_sidebar(location = app_location), "Add left sidebar conversion was successful. File\\(s\\) updated: ui.R")
    expect_message(create_left_sidebar(location = app_location), "Left sidebar already available, no conversion needed")
    expect_converted_application(location = app_location, left_sidebar = TRUE)
})

## create_right_sidebar tests

test_that("create_right_sidebar null location", {
    expect_error(create_right_sidebar(location = NULL),
                 "Add right sidebar conversion could not proceed, location cannot be empty!")
})

test_that("create_right_sidebar empty location", {
    expect_error(create_right_sidebar(location = ""),
                 "Add right sidebar conversion could not proceed, location cannot be empty!")
})

test_that("create_right_sidebar invalid location", {
    expect_error(create_right_sidebar(location = "invalid"),
                 "Add right sidebar conversion could not proceed, location=<invalid> does not exist!")
})

test_that("create_right_sidebar location does not contain an existing application", {
    expect_error(create_right_sidebar(location = "../testthat"),
                 "Add right sidebar conversion could not proceed, location=<../testthat> does not contain a valid periscope application!")
})

test_that("create_right_sidebar to left sidebar, valid location", {
    app_location <- create_app_tmp_dir(right_sidebar = FALSE, left_sidebar = TRUE)

    expect_message(create_right_sidebar(location = app_location), "Add right sidebar conversion was successful. File\\(s\\) updated: ui.R")
    expect_converted_application(location = app_location, right_sidebar = TRUE, left_sidebar = TRUE)
})

test_that("create_right_sidebar, valid location", {
    app_location <- create_app_tmp_dir(right_sidebar = FALSE, left_sidebar = FALSE)

    expect_message(create_right_sidebar(location = app_location), "Add right sidebar conversion was successful. File\\(s\\) updated: ui.R")
    expect_converted_application(location = app_location, right_sidebar = TRUE, left_sidebar = FALSE)
})


test_that("create_right_sidebar valid location, added twice", {
    app_location <- create_app_tmp_dir(right_sidebar = FALSE)

    expect_message(create_right_sidebar(location = app_location), "Add right sidebar conversion was successful. File\\(s\\) updated: ui.R")
    expect_message(create_right_sidebar(location = app_location), "Right sidebar already available, no conversion needed")
    expect_converted_application(location = app_location, right_sidebar = TRUE)
})

# -------------------------------------
# -- Application Reset Button Module --
# -- Reload Application Session      --
# -------------------------------------

#' appResetButton module UI function
#'
#' Creates a toggle button to reset application session. Upon pressing on the
#'  button, its state is flipped to cancel application reload with application
#'  and console warning messages indicating that the application will be
#'  reloaded.
#'
#'  User can either resume reloading application session or cancel reloading
#'  process which will also generate application and console messages to
#'  indicate reloading status and result.
#'
#'
#' @param id character id for the object
#'
#' @return an html div with prettyToggle button
#'
#' @section Button Features:
#' \itemize{
#'     \item Initial state label is "Application Reset" with warning status
#'     \item Reloading state label is "Cancel Application Reset" with danger
#'           status
#' }
#'
#'
#' @section Shiny Usage:
#' Call this function at any place in UI section.
#'
#' It is paired with a call to \code{appReset(id, ...)}
#' in server
#'
#'
#' @examples
#' if (interactive()) {
#'    library(shiny)
#'    library(periscope2)
#'    shinyApp(
#'      ui = fluidPage(fluidRow(column(12, appResetButton(id = "appResetId")))),
#'      server = function(input, output) {
#'        appReset(id = "appResetId", logger = "")
#'    })
#' }
#'
#'
#' @export
#' @seealso \link[periscope2]{appReset}
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{downloadablePlot}
#' @seealso \link[periscope2]{downloadFileButton}
#' @seealso \link[periscope2]{downloadableTable}
#' @seealso \link[periscope2]{logViewerOutput}
appResetButton <- function(id) {
    ns <- shiny::NS(id)

    shiny::div(shinyWidgets::prettyToggle(ns("resetButton"),
                                          label_off  = "Reset Application",
                                          label_on   = "Cancel Application Reset",
                                          shape      = "round",
                                          value      = FALSE,
                                          status_off = "warning",
                                          status_on  = "danger",
                                          width      = "90%",
                                          fill       = TRUE),
               shiny::span(class = "invisible",
                           shinyWidgets::prettyToggle(ns("resetPending"),
                                                      value      = FALSE,
                                                      label_off  = NULL,
                                                      label_on   = NULL))
    )
}


#' appReset module server function
#'
#' Server-side function for the appResetButton  This is a custom
#' high-functionality button for session reload. The server function is used to
#'  provide module configurations.
#'
#' @param id             Character represents the ID of the Module's UI element (the same id used in \code{appResetButton}
#' @param reset_wait     Integer represents the period to wait before session reload in milliseconds (default = 5000)
#' @param alert_location Character represents div ID or selector to display module related messages (default = "bodyAlert")
#' @param logger         logger to use
#'
#' @return nothing, function will display a warning message in the app then reload the whole application
#'
#'
#' @section Shiny Usage:
#' This function is not called directly by consumers - it is accessed in
#' server_local.R (or similar file) using the same id provided in \code{appResetButton}:
#'
#' \strong{\code{appReset(id = "appResetId", logger = ss_userAction.Log)}}
#'
#' @examples
#' if (interactive()) {
#'    library(shiny)
#'    library(periscope2)
#'    shinyApp(
#'      ui = fluidPage(fluidRow(column(12, appResetButton(id = "appResetId")))),
#'      server = function(input, output) {
#'        appReset(id = "appResetId", logger = "")
#'    })
#' }
#'
#' @export
#'
#' @seealso \link[periscope2]{appResetButton}
#' @seealso \link[periscope2]{downloadFile}
#' @seealso \link[periscope2]{downloadFile_ValidateTypes}
#' @seealso \link[periscope2]{downloadFile_AvailableTypes}
#' @seealso \link[periscope2]{downloadablePlot}
#' @seealso \link[periscope2]{downloadFileButton}
#' @seealso \link[periscope2]{downloadableTable}
#' @seealso \link[periscope2]{logViewerOutput}
appReset <- function(id,
                     reset_wait     = 5000,
                     alert_location = "bodyAlert",
                     logger) {
    shiny::moduleServer(
        id,
        function(input, output, session) {
            shiny::observe({
                if (!(alert_location %in% c("bodyAlert", "headerAlert", "footerAlert", "leftAlert", "rightAlert"))) {
                    logwarn("Alert location must be one of standard alert values: 'bodyAlert', 'headerAlert', 'footerAlert', 'leftAlert', 'rightAlert'", logger = logger)
                    logwarn("Setting alert location to 'bodyAlert'")
                }

                pending  <- shiny::isolate(input$resetPending)
                waittime <- reset_wait

                if (!is.null(pending)) {
                    closeResetAlert(alert_location)
                    if (input$resetButton && !(pending)) {
                        # reset initially requested
                        logwarn(paste("Application Reset requested by user. ",
                                      "Resetting in ", (waittime / 1000),
                                      "seconds."),
                                logger = logger)
                        createPSAlert(
                            selector = paste0("#", alert_location),
                            options  = list(status   = "danger",
                                            title    = "Reset Application",
                                            closable = TRUE,
                                            content  = paste("The application will be reset in",
                                                             (waittime / 1000),
                                                             "seconds if you do not cancel below.")))
                        shinyWidgets::updatePrettyToggle(session,
                                                         "resetPending",
                                                         value = TRUE)
                        shiny::invalidateLater(waittime, session)
                    } else if (!input$resetButton && pending) {
                        # reset cancelled by pushing the button again
                        loginfo("Application Reset cancelled by user.",
                                logger = logger)
                        createPSAlert(
                            selector = paste0("#", alert_location),
                            options  = list(status   = "success",
                                            title    = "Reset Application",
                                            closable = TRUE,
                                            content  = "The application reset was canceled."))

                        shinyWidgets::updatePrettyToggle(session,
                                                         "resetPending",
                                                         value = FALSE)
                    } else if (pending) {
                        # reset timed out
                        logwarn("Application Reset", logger = logger)
                        session$reload()
                    }
                }
            })
        })
}

---
title: "Using the downloadableTable Shiny Module"
author: "Dr. Connie Brett"
date: "`r Sys.Date()`"
output: 
    rmarkdown::html_vignette:
        toc: TRUE
        toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Using the downloadableTable Shiny Module}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


# Overview

## Purpose 

This *Shiny Module* provides a consistent and easy-to-use table including a downloadFileButton that is automatically created, linked and managed.

## Features

* Uses the downloadFile Shiny Module functionality to ensure consistent
download functionality and paradigm for the table data.
* Ability to use different data sets for display vs. download if desired
* User selection of multiple rows by default - can be single select also by 
setting the appropriate option
* Returns a reactive expression containing the rows of the table selected by
the user for easier development of code depending on table selections.
* Supports full-table searching including regular expressions with search 
highlighting
* Columns are sort-able in both directions
* Configurable table "window" (viewing area) height with infinite vertical 
scrolling (no paging by default)
* Supports rownames and a table caption
* Requires minimal code (see the Usage section for details)
* **downloadFile** button will be hidden if `downloadableTable` parameter `downloaddatafxn` or
`downloadableTableUI` parameter `downloadtypes` is empty

<br>

# Usage

## Shiny Module Overview

Shiny modules consist of a pair of functions that modularize, or package, a small piece of reusable functionality.  The UI function is called directly by the user to place the UI in the correct location (as with other shiny UI objects). The module server function that is called only once to set it up using the module name as a function inside the server function (i.e. user-local session scope.  The first function argument is a string that represents the module id (the same id used in module UI function). Additional arguments can be supplied by the user based on the specific shiny module that is called.  There can be additional helper functions that are a part of a shiny module.

The **downloadableTable** Shiny Module is a part of the *periscope2* package and 
consists of the following functions:

* **downloadableTableUI** - the UI function to place the table in the application
* **downloadableTable** - the server function to be called inside server_local.R.


## downloadableTableUI

The **downloadableTableUI** function is called from the ui.R (or equivalent) 
file in the location where the table should be placed.  This is similar to other
UI element placement in shiny.

The downloadableTableUI looks like:

<center><img src="figures/downloadableTable-1.jpg" width="500px"></center>

The downloadableTableUI function takes the unique object ID for the UI object.

The next two arguments (downloadtypes and hovertext) are passed to the
downloadFileButton and set the file types the button will allow the user to
request and the downloadFileButton's tooltip text.  

The contentHeight argument is optional but important - it sets the viewable height of the table content which can be any css-recognizable size value.

The downloadableTables are multi-select by default, however if you need a table to only allow single row selections you can set the singleSelect argument to TRUE.


```{r, eval=F}
# Inside ui_body.R or ui_sidebar.R

downloadableTableUI(id            = "object_id1", 
                    downloadtypes = c("csv", "tsv"), 
                    hovertext     = "Download the data here!",
                    contentHeight = "300px",
                    singleSelect  = FALSE)
```

## downloadableTable

The **downloadableTable** function is called directly. The call consists of the following:

* the unique object ID that was provided to downloadableTableUI when creating 
the UI object
* the logging logger to be used
* **filenameroot** is an optional character, function or reactive expression providing downloadable file name
* the root (prefix) of the downloaded file name to be used in the browser as a character string or a reactive expression that returns a character string
* a **named list** of functions or reactive expressions that provide the data to the downloadFileButton (see below).  
  * It is important that the types of files to be downloaded are matched to the correct data function in the list.  
  * The function/reactive expression names are unquoted - they will be called at the time the user initiates a download *(see requirements below)*.
  * a function or reactive expression providing the plot output for the visible plot.  It can be the same,
or different from the ones provided to any of the download functions.  This allows finer control over what the user can view vs. download if desired.  For example you can allow a user to view a smaller subset of data but download an expanded dataset, or perhaps download a redacted version of data, etc.
* This module also supports most of DT table options for further customization. See the example below.


**Data Function or Reactive Expression Requirements**

* If a function is provided it must be parameter-less (require NO parameters).  No parameters will be provided when a function is called to retrieve the plot or data.  Reactive expressions cannot take parameters by definition.
* The function or reactive expression must return an appropriate data format for the file type.  For
instance: csv/tsv/xlsx types require data that is convertible to a tabular type,
tiff formats require a tiff class object, etc.  For more information on conversion of data 
to various download types see the downloadFile module help or vignette.
* For the visible table data the return value must be able to be converted to tabular format
* Since the function or reactive expression is called at the time the user requests the data it is 
recommended that reactive expressions are used to provide dynamic values from the application to create the plot.


**Reactive Return Value**

The server function returns a reactive expression containing the selected rows.  Note that this is the data, not references, rownumbers, etc from the table -- it is the actual, visible, table row data.  This allows the developer to use this more easily to update another table, chart, etc. as desired.

It is acceptable to ignore the return value as well if this functionality is not needed.  Simply do not assign the result to a variable.

**Customization Options**

*downloadableTable* module can be customized using DT arguments, options or format functions (see `?DT::datatable`). These options can be sent as a named options via the server function, see example below. 

*Caveats on DT usage*: 

* `selection` parameter in the server function has different usage than `DT::datatable` `selection` option as it should be a function or reactive expression providing the row_ids of the rows that should be selected. Its default value is `NULL`
* `editable`, `width`, `height` options in `DT::datatable` are not supported

The following is an example of a customized downloadableTable:

<center><img src="figures/downloadableTable-2.jpg" width="500px"></center>

It was generated using the following code:

```{r, eval = F}
# Inside server_local.R
sketch <- htmltools::withTags(
    table(
        class = "display",
        thead(
            tr(
                th(rowspan = 2, "Location"),
                th(colspan = 2, "Statistics")
            ),
            tr(
                th("Change"),
                th("Increase")
            )
        )
    )
)

selectedrows <- downloadableTable(
    id               = "exampleDT1",
    logger           = ss_userAction.Log,
    filenameroot     = "exampletable",
    downloaddatafxns = list(csv = load_data3, tsv = load_data3),
    tabledata        = load_data3,
    colnames         = c("Area", "Delta", "Increase"),
    filter           = "bottom",
    callback         = htmlwidgets::JS("table.order([1, 'asc']).draw();"),
    container        = sketch,
    formatStyle      = list(columns = c("Total.Population.Change"),   
                            color   = DT::styleInterval(0, c("red", "green"))),
    formatStyle      = list(columns = c("Natural.Increase"),   
                            backgroundColor = DT::styleInterval(
                                c(7614, 15914, 34152),
                                c("blue", "lightblue", "#FF7F7F", "red")))
)

# NOTE: selectedrows is the reactive return value, captured for later use
```


## Sample Application

For a complete running shiny example application using the downloadableTable module you can create and run a *periscope2* sample application using:

```{r, eval=F}
library(periscope2)

app_dir = tempdir()
create_new_application(name = 'mysampleapp', location = app_dir, sample_app = TRUE)
runApp(paste(app_dir, 'mysampleapp', sep = .Platform$file.sep))
```

<br>

# Additional Resources


**Vignettes**

* [New Application](new-application.html)
* [downloadFile Module](downloadFile-module.html)
* [downloadablePlot Module](downloadablePlot-module.html)
* [logViewer Module](logViewer-module.html) 
* [applicationReset Module](applicationReset-module.html)
* [announcement Module](announcement-module.html)
* [Announcement Configuration Builder](announcement_addin.html)
* [Theme Configuration Builder](themeBuilder_addin.html)
* [downloadableReactTable Module](downloadableReactTable-module.html)

---
title: "Using the downloadablePlot Shiny Module"
author: "Dr. Connie Brett"
date: "`r Sys.Date()`"
output: 
    rmarkdown::html_vignette:
        toc: TRUE
        toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Using the downloadablePlot Shiny Module}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


# Overview

## Purpose 

This *Shiny Module* provides an easy-to-use downloadFileButton for a plot.  It is that is automatically created, linked and managed.  This module is compatible with ggplot2, grob and lattice graphics -- as well as any superclass of these.


## Features

* Supports ggplot2, grob and lattice graphics (and superclasses) for maximum flexibility
* Uses the downloadFile Shiny Module functionality to ensure consistent
download functionality and paradigm for image or table data
* Ability to link different plot downloads from the displayed plot if 
desired *(for example this might be used to ensure a watermark/copyright
notice on downloaded plots)*
* Ability to add data downloads in addition to image downloads using the same
button
* Supports hover/click/brush functionality
* Configurable plot size in application
* Configurable plot aspect ratio for exported images
* Configurable button placement at the bottom of the plot
* Requires minimal code (see the Usage section for details)
* **downloadFile** button will be hidden if `downloadablePlot` parameter `downloadfxns` or
`downloadablePlotUI` parameter `downloadtypes` is empty

<br>

# Usage

## Shiny Module Overview

Shiny modules consist of a pair of functions that modularize, or package, a small piece of reusable functionality.  The UI function is called directly by the user to place the UI in the correct location (as with other shiny UI objects). The module server function that is called only once to set it up using the module name as a function inside the server function (i.e. user-local session scope.  The first function argument is a string that represents the module id (the same id used in module UI function). Additional arguments can be supplied by the user based on the specific shiny module that is called.  There can be additional helper functions that are a part of a shiny module.


The **downloadablePlot** Shiny Module is a part of the *periscope2* package and 
consists of the following functions:

* **downloadablePlotUI** - the UI function to place the plot in the application
* **downloadablePlot** - the server function to be called inside server_local.R.


## downloadablePlotUI

The **downloadablePlotUI** function is called from the ui.R (or equivalent) file in the location where the plot should be placed.  This is similar to other UI element placement in shiny.

The downloadablePlotUI adds a button to the plot as seen in the bottom right corner below:

<center><img src="figures/downloadablePlot-1.png" width="500px"></center>

The downloadablePlotUI function takes the unique object ID for the UI object.

The downloadtypes and download_hovertext arguments are passed to the downloadFileButton and set the file types the button will allow the user to
request and the downloadFileButton's tooltip text.  

The width and height options set the plot size in the UI and can be any css-recognizable size value.

These options are followed with alignment and overlap options for the button placed on the chart.

The last three options give you the standard ways of interacting with shiny plots - using the functionality available with the clickOpts, hoverOpts and brushOpts functions. 

**Note**: When you create an object using any of these functions shiny will send 
information about the user's interactions with the plot to the server and make 
it available to you via an input$ variable.


```{r, eval=F}
# Inside ui_body.R or ui_sidebar.R

downloadablePlotUI(id                 = "object_id1", 
                   downloadtypes      = c("png", "csv"), 
                   download_hovertext = "Download the plot and data here!",
                   height             = "500px", 
                   btn_halign         = "left")
```


## downloadablePlot

The **downloadablePlot** function is also called directly. The call consists of the following:

* the unique object ID that was provided to downloadablePlotUI when creating the UI object
* the logging logger to be used
* the root (prefix) of the downloaded file name to be used in the browser as a character string or a reactive expression that returns a character string
* the aspect ratio as a numeric value for exported images of supported types.  
  * The values are width:height (width/height which is unitless) for example for a 
1:1 ratio (square) the value would be 1/1 = 1, for a 2:1 ratio (wide rectangle)
the value would be 2/1 = 2, for 4:3 ratio (landscape photo) the value would be
4/3 = 1.33, etc.  
  * This value ensures an exported reasonable shape for a plot 
regardless of the size or shape of the application browser window.
* a **named list** of functions or reactive expressions that provide the images or data to the 
downloadFileButton (see below).  
  * It is important that the types of files to be downloaded are matched to the correct named function in the list.  
  * The function/reactive names are unquoted - they will be called at the time the user initiates a 
download *(see requirements below)*.
* a function or reactive expression providing the plot output for the visible plot.  It can be the same,
or different from the ones provided to any of the download functions.  This allows finer control over what the user can view vs. download if desired.  For example you can allow a user to view a smaller subset of data but download an expanded dataset, or perhaps download a redacted version of data, etc.


** Plot/Download Function or Reactive Expression Requirements**

* If a function is provided it must be parameter-less (require NO parameters).  No parameters will be provided when a function is called to retrieve the plot or data.  Reactive expressions cannot take parameters by definition.
* The function or reactive expression must return an appropriate data format for the file type.  For
instance: csv/tsv/xlsx types require data that is convertible to a tabular type,
tiff formats require a tiff class object, etc.  For more information on conversion of data 
to various download types see the downloadFile module help or vignette.
* For the visible plot the return value must be from the ggplot2, grob or lattice classes (or a superclass therein)
* Since the function or reactive expression is called at the time the user requests the data it is 
recommended that reactive expressions are used to provide dynamic values from the application to create the plot.


```{r, eval = F}
# Inside server_local.R

downloadablePlot(id           = "object_id1",
                 logger       = ss_userAction.Log,
                 filenameroot = "mydownload1",
                 aspectratio  = 1.33,
                 downloadfxns = list(png = myplotfxn,
                                     tsv = mydatafxn),
                 visibleplot  = myplotfxn)
           
```

<br>

# Additional Resources

**Vignettes**

* [New Application](new-application.html)
* [downloadFile Module](downloadFile-module.html)
* [downloadableTable Module](downloadableTable-module.html)
* [logViewer Module](logViewer-module.html)
* [applicationReset Module](applicationReset-module.html)
* [announcement Module](announcement-module.html)
* [Announcement Configuration Builder](announcement_addin.html)
* [Theme Configuration Builder](themeBuilder_addin.html)
* [downloadableReactTable Module](downloadableReactTable-module.html)

---
title: "Using the downloadFile Shiny Module"
author: "Dr. Connie Brett"
date: "`r Sys.Date()`"
output: 
    rmarkdown::html_vignette:
        toc: TRUE
        toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Using the downloadFile Shiny Module}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Overview

## Purpose 

This *Shiny Module* provides a consistent-looking and easy-to-use button that facilitates single or multiple types of file downloads.

## Features

* Auto-adjusts the button image and styling for single or multiple downloads
* Ability to link different data sets to different file types if desired
* Incorporates tooltips and drop-downs automatically
* Converts data to various download types
* Requires minimal code (see the Usage section for details)

<br>

# Usage

## Shiny Module Overview

Shiny modules consist of a pair of functions that modularize, or package, a small piece of reusable functionality.  The UI function is called directly by the user to place the UI in the correct location (as with other shiny UI objects). The module server function that is called only once to set it up using the module name as a function inside the server function (i.e. user-local session scope.  The first function argument is a string that represents the module id (the same id used in module UI function). Additional arguments can be supplied by the user based on the specific shiny module that is called.  There can be additional helper functions that are a part of a shiny module.

The **downloadableTable** Shiny Module is a part of the *periscope2* package and 
consists of the following functions:

The **downloadFile** Shiny Module is a part of the *periscope2* package and  consists of the following functions:

* **downloadFileButton** - the UI function to place the button in the 
application.
* **downloadFile** - the server function to be called inside server_local.R.
* Helper Functions:
  * **downloadFile_ValidateTypes** - will check a given list of file types and warn the caller if the list contains an invalid or unsupported type.
  * **downloadFile_AvailableTypes** - will return a vector of supported file types.

## downloadFileButton

The **downloadFileButton** function is called from the ui.R (or equivalent) file in the location where the button should be placed.  This is similar to other UI element placement in shiny.

The downloadFileButton function takes the unique object ID for the UI object.

The next two arguments (downloadtypes and hovertext) set the file types the button will allow the user to request and the tooltip text.  

If only one download type is given the button will be a single-download type of button
that starts the specified download when the button is pressed.  The button will 
look like:

<center><img src="figures/downloadFile-1.jpg" width="50px"></center>


If more than one type is given there will be a dropdown choice given to the user
when the button is pressed and the download will start when the choose one of 
the types.  The dropdown choices will be in the order given by the call to the
function.  The button and dropdown will look like:

<center>
<img src="figures/downloadFile-2.jpg" width="50px" align="top">
<img src="figures/downloadFile-3.jpg" width="200px">
</center>


```{r, eval=F}
# Inside ui_body.R or ui_sidebar.R

#single download type
downloadFileButton(id            = "object_id1", 
                   downloadtypes = c("csv"), 
                   hovertext     = "Button 1 Tooltip")

#multiple download types
downloadFileButton(id            = "object_id2", 
                   downloadtypes = c("csv", "tsv"), 
                   hovertext     = "Button 2 Tooltip")
```

## downloadFile

The **downloadFile** function is called directly. The call consists of the following:

**Note on Excel file support:** For xlsx files, the module will first attempt to use `openxlsx2` if available, then fall back to `openxlsx`, and finally use `writexl` if neither is available. We recommend `openxlsx2` for best performance with Excel files.


* the unique object ID that was provided to downloadFileButton when creating 
the UI object
* the logging logger to be used
* the root (prefix) of the downloaded file name to be used in the browser as a character
string or reactive expression that returns a character string
* a **named list** of functions or reactive expressions that provide the data (see below).  
  * It is important that the types of files to be downloaded are matched to the correct data function in the list.  
  * The function/reactive expression names are unquoted - they will be called at the time the user initiates a download *(see requirements below)*.

**Data Function or Reactive Expression Requirements**

* If a function is provided it must be parameter-less (require NO parameters).  No parameters will be provided when a function is called to retrieve the plot or data.  Reactive expressions cannot take parameters by definition.
* The function or reactive expression must return an appropriate data format for the file type.  For
instance: csv/tsv/xlsx types require data that is convertible to a tabular type.
* Since the function or reactive expression is called at the time the user requests the data it is 
recommended that reactive expressions are used to provide dynamic values from the application to create the plot.


```{r, eval = F}
# Inside server_local.R

#single download type
downloadFile(id           = "object_id1", 
             logger       = ss_userAction.Log,
             filenameroot = "mydownload1",
             datafxns     = list(csv = mydatafxn1),
             aspectratio  = 1)

#multiple download types
downloadFile(id           = "object_id2",
             logger       = ss_userAction.Log,
             filenameroot = "mytype2",
             datafxns     = list(csv = mydatafxn1, xlsx = mydatafxn2),
             aspectratio  = 1)
```


## Sample Application

For a complete running shiny example application using the downloadFile module you can create and run a *periscope2* sample application using:

```{r, eval=F}
library(periscope2)

app_dir = tempdir()
create_application(name = 'mysampleapp', location = app_dir, sample_app = TRUE)
runApp(paste(app_dir, 'mysampleapp', sep = .Platform$file.sep))
```

<br>

# Additional Resources


**Vignettes**

* [New Application](new-application.html)
* [downloadableTable Module](downloadableTable-module.html)
* [downloadablePlot Module](downloadablePlot-module.html)
* [logViewer Module](logViewer-module.html)
* [applicationReset Module](applicationReset-module.html)
* [announcement Module](announcement-module.html)
* [Announcement Configuration Builder](announcement_addin.html)
* [Theme Configuration Builder](themeBuilder_addin.html)
* [downloadableReactTable Module](downloadableReactTable-module.html)

---
title: "Using the appReset Shiny Module"
author: "Mohammed Ali"
date: "`r Sys.Date()`"
output: 
    rmarkdown::html_vignette:
        toc: TRUE
        toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Using the appReset Shiny Module}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Overview

## Purpose 

This *Shiny Module* allows the user to easily reset application session and logs

## Features

* Resets a user's session
* Resets the session log

# Usage

## Shiny Module Overview

Shiny modules consist of a pair of functions that modularize, or package, a small piece of reusable functionality.  The UI function is called directly by the user to place the UI in the correct location (as with other shiny UI objects). The module server function that is called only once to set it up using the module name as a function inside the server function (i.e. user-local session scope.  The function first arguments is string represents the module id (the same id used in module UI function). Additional arguments can be supplied by the user based on the specific shiny module that is called.  There can be additional helper functions that are a part of a shiny module.

The **appReset** Shiny Module is a part of the *periscope2* package and 
consists of the following functions:

* **appResetButton** - the UI function to place the button in the 
* **appReset** - the UI function to place the button in the 

## appResetButton

<figure>
  <center><img src="figures/appReset-UI-1.jpg"></center>
  <figcaption>App Reset Toggle Button</figcaption>
</figure>

* This is a toggle button can be placed in the UI like any other element

* This button is automatically wired to reset the application to the initial
session state
* The user is given a warning (as an alert on the Advanced tab)
and the reset is delayed (default = 5s) to allow the user to cancel the reset.

<figure>
  <center><img src="figures/appReset-UI-2.jpg" width="600px"></center>
  <figcaption>Reset Warning</figcaption>
</figure>


<figure>
  <center><img src="figures/appReset-UI-3.jpg" width="600px"></center>
  <figcaption>Cancel Reset</figcaption>
</figure>

* Reset requests and cancellations are logged automatically.
<figure>
  <center><img src="figures/appReset_logs.jpg"></center>
  <figcaption>Successful reset request</figcaption>
</figure>

<figure>
  <center><img src="figures/appReset_logs-2.jpg"></center>
  <figcaption>Canceled reset request</figcaption>
</figure>



```{r, eval=F}
# Inside ui_body.R or similar UI file

appResetButton('appResetId')
```

## appReset

The **appReset** function is called directly. The call consists of the following:

* the unique object ID 
* the logging logger to be used

```{r, eval=F}
# Inside server_local.R

appReset(id = 'appResetId', logger = logger)
```


## Sample Application

For a complete running shiny example application using the appReset module you can create and run a *periscope2* sample application using:

```{r, eval=F}
library(periscope2)

app_dir = tempdir()
create_application(name = 'mysampleapp', location = app_dir, sample_app = TRUE)
runApp(paste(app_dir, 'mysampleapp', sep = .Platform$file.sep))
```

<br>

**Vignettes**

* [New Application](new-application.html)
* [downloadableTable Module](downloadableTable-module.html)
* [downloadablePlot Module](downloadablePlot-module.html)
* [downloadFile Module](downloadFile-module.html)
* [logViewer Module](logViewer-module.html)
* [announcement Module](announcement-module.html)
* [Announcement Configuration Builder](announcement_addin.html)
* [Theme Configuration Builder](themeBuilder_addin.html)
* [downloadableReactTable Module](downloadableReactTable-module.html)

---
title: "Using the logViewer Shiny Module"
author: "Mohammed Ali"
date: "`r Sys.Date()`"
output: 
    rmarkdown::html_vignette:
        toc: TRUE
        toc_depth: 3
vignette: >
  %\VignetteIndexEntry{Using the logViewer Shiny Module}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Overview

## Purpose 

This *Shiny Module* displays recorded session logs in tabular format

## Features

* Table contains two columns:
  * action: describe the recorded action
  * time: display action time
* The log files are kept in the /log directory and named 'actions.log'. ONE old copy of the log is kept as 'actions.log.last
* Many actions are automatically logged by the framework and it is easy for developers to add additional items as they see fit.
* Filtering logs by setting log_level argument as desired in set_app_parameters
  * "DEBUG"    will log logdebug, loginfo, logwarn, and logerror messages
  * "INFO"     will log loginfo, logwarn, and logerror messages 
  * "WARNING"  will log logwarn and logerror messages
  * "ERROR"    will log logerror messages
* It is important to note that the log rolls over for each session and is reset if using the appReset module.
  
<center><img src="figures/user_action_log.jpg" width="800px"></center>
<br>

# Usage

## Shiny Module Overview

Shiny modules consist of a pair of functions that modularize, or package, a small piece of reusable functionality.  The UI function is called directly by the user to place the UI in the correct location (as with other shiny UI objects). The module server function that is called only once to set it up using the module name as a function inside the server function (i.e. user-local session scope.  The first function argument is a string that represents the module id (the same id used in module UI function). Additional arguments can be supplied by the user based on the specific shiny module that is called.  There can be additional helper functions that are a part of a shiny module.

The **logViewer** Shiny Module is a part of the *periscope2* package and consists of the following functions:

* **logViewerOutput** - the UI function to place the button in the 

## logViewerOutput

The **logViewerOutput** function is called from the ui_body.R (or equivalent) file in the location where any other UI element (box) can be placed.  

```{r, eval=F}
# Inside ui_body.R

logViewerOutput("logViewerId")
```


## Sample Application

For a complete running shiny example application using the logViewer module you can create and run a *periscope2* sample application using:

```{r, eval=F}
library(periscope2)

app_dir = tempdir()
create_application(name = 'mysampleapp', location = app_dir, sample_app = TRUE)
runApp(paste(app_dir, 'mysampleapp', sep = .Platform$file.sep))
```

<br>

# Additional Resources


**Vignettes**

* [New Application](new-application.html)
* [downloadableTable Module](downloadableTable-module.html)
* [downloadablePlot Module](downloadablePlot-module.html)
* [downloadFile Module](downloadFile-module.html)
* [applicationReset Module](applicationReset-module.html)
* [announcement Module](announcement-module.html)
* [Announcement Configuration Builder](announcement_addin.html)
* [Theme Configuration Builder](themeBuilder_addin.html)
* [downloadableReactTable Module](downloadableReactTable-module.html)

# logViewerOutput

    [[1]]
    <div class="shiny-panel-conditional" data-display-if="output.displayButton" data-ns-prefix="myid-myid-">
      <span id="myid-myid-reactTableButtonDiv" class="periscope-downloadable-react-table-button" style="">
        <span class="btn-group" data-toggle="tooltip" data-placement="top" title="Download application logs">
          <button aria-expanded="false" aria-haspopup="true" class="btn btn-default action-button dropdown-toggle periscope-download-btn" data-toggle="dropdown" id="myid-myid-reactTableButtonID-downloadFileList" type="button action">
            <i class="far fa-copy" role="presentation" aria-label="copy icon"></i>
          </button>
          <ul class="dropdown-menu" id="myid-myid-reactTableButtonID-testList">
            <li>
              <a aria-disabled="true" class="shiny-download-link disabled periscope-download-choice" download href="" id="myid-myid-reactTableButtonID-csv" tabindex="-1" target="_blank">csv</a>
            </li>
            <li>
              <a aria-disabled="true" class="shiny-download-link disabled periscope-download-choice" download href="" id="myid-myid-reactTableButtonID-tsv" tabindex="-1" target="_blank">tsv</a>
            </li>
          </ul>
        </span>
      </span>
    </div>
    
    [[2]]
    <div class="reactable html-widget html-widget-output shiny-report-size html-fill-item" data-reactable-output="myid-myid-reactTableOutputID" id="myid-myid-reactTableOutputID" style="width:auto;height:auto;"></div>
    

# add_ui_left_sidebar no left sidebar

    $disable
    [1] TRUE
    

# add_ui_left_sidebar empty left sidebar

    [[1]]
    [[1]][[1]]
    <div id="sidebarBasicAlert"></div>
    
    [[1]][[2]]
    NULL
    
    
    $skin
    [1] "light"
    
    $status
    [1] "primary"
    
    $elevation
    [1] 4
    
    $collapsed
    [1] FALSE
    
    $minified
    [1] FALSE
    
    $expand_on_hover
    [1] FALSE
    
    $fixed
    [1] TRUE
    
    $custom_area
    NULL
    
    [[10]]
    NULL
    

# add_ui_left_sidebar example left sidebar

    [[1]]
    [[1]][[1]]
    <div id="sidebarBasicAlert"></div>
    
    [[1]][[2]]
    NULL
    
    
    $skin
    [1] "light"
    
    $status
    [1] "primary"
    
    $elevation
    [1] 4
    
    $collapsed
    [1] FALSE
    
    $minified
    [1] FALSE
    
    $expand_on_hover
    [1] FALSE
    
    $fixed
    [1] TRUE
    
    $custom_area
    NULL
    

# add_ui_footer empty footer

    <footer class="main-footer" data-fixed="false">
      <div class="float-right d-none d-sm-inline"></div>
      <div id="footerAlert"></div>
    </footer>

# add_ui_footer example footer

    <footer class="main-footer" data-fixed="false">
      <div class="float-right d-none d-sm-inline">2022</div>
      <div id="footerAlert"></div>
      <a href="https://periscopeapps.org/" target="_blank">periscope2</a>
    </footer>

# add_ui_body empty body

    Code
      shiny::isolate(periscope2:::.g_opts$body_elements)
    Output
      [[1]]
      <div id="bodyAlert"></div>
      
      [[2]]
      <div id="head"></div>
      
      [[3]]
      NULL
      

# add_ui_body example body

    [[1]]
    <div id="bodyAlert"></div>
    
    [[2]]
    <div id="head"></div>
    
    [[3]]
    [[3]][[1]]
    <div class="jumbotron bg-info">
      <h1 class="display-4">periscope2: Enterprise Streamlined 'Shiny' Application Framework</h1>
      <p class="lead">
        <p>
          periscope2 is a scalable and UI-standardized 'shiny' framework including a variety of developer convenience
          functions with the goal of both streamlining robust application development and assisting in creating a consistent
           user experience regardless of application or developer.
        </p>
      </p>
      <hr class="my-4"/>
      <p>
        <dl>
          <dt>Features</dt>
          <ul>
            <li>Predefined but flexible template for new Shiny applications with a default dashboard layout</li>
            <li>Separation by file of functionality that exists in one of the three shiny scopes: global, server-global, and server-local.</li>
            <li>Off-the-shelf and ready to be used modules ('Announcements', 'Table Downloader', 'Plot Downloader', 'File Downloader', 'Application Logger' and 'Reset Application'</li>
            <li>Different methods to notify user and add useful information about application UI and server operations</li>
          </ul>
        </dl>
      </p>
      <a class="btn btn-secondary btn-lg" href="https://periscopeapps.org/" target="_blank" role="button">More</a>
    </div>
    
    

---

    [[1]]
    <div id="bodyAlert"></div>
    
    [[2]]
    <div id="head"></div>
    
    [[3]]
    <div>more elements</div>
    
    [[4]]
    [[4]][[1]]
    <div class="jumbotron bg-info">
      <h1 class="display-4">periscope2: Enterprise Streamlined 'Shiny' Application Framework</h1>
      <p class="lead">
        <p>
          periscope2 is a scalable and UI-standardized 'shiny' framework including a variety of developer convenience
          functions with the goal of both streamlining robust application development and assisting in creating a consistent
           user experience regardless of application or developer.
        </p>
      </p>
      <hr class="my-4"/>
      <p>
        <dl>
          <dt>Features</dt>
          <ul>
            <li>Predefined but flexible template for new Shiny applications with a default dashboard layout</li>
            <li>Separation by file of functionality that exists in one of the three shiny scopes: global, server-global, and server-local.</li>
            <li>Off-the-shelf and ready to be used modules ('Announcements', 'Table Downloader', 'Plot Downloader', 'File Downloader', 'Application Logger' and 'Reset Application'</li>
            <li>Different methods to notify user and add useful information about application UI and server operations</li>
          </ul>
        </dl>
      </p>
      <a class="btn btn-secondary btn-lg" href="https://periscopeapps.org/" target="_blank" role="button">More</a>
    </div>
    
    

# add_ui_body append

    [[1]]
    <div id="bodyAlert"></div>
    
    [[2]]
    <div id="head"></div>
    
    [[3]]
    <div>more elements</div>
    
    [[4]]
    <div>append div</div>
    
    [[5]]
    [[5]][[1]]
    <div class="jumbotron bg-info">
      <h1 class="display-4">periscope2: Enterprise Streamlined 'Shiny' Application Framework</h1>
      <p class="lead">
        <p>
          periscope2 is a scalable and UI-standardized 'shiny' framework including a variety of developer convenience
          functions with the goal of both streamlining robust application development and assisting in creating a consistent
           user experience regardless of application or developer.
        </p>
      </p>
      <hr class="my-4"/>
      <p>
        <dl>
          <dt>Features</dt>
          <ul>
            <li>Predefined but flexible template for new Shiny applications with a default dashboard layout</li>
            <li>Separation by file of functionality that exists in one of the three shiny scopes: global, server-global, and server-local.</li>
            <li>Off-the-shelf and ready to be used modules ('Announcements', 'Table Downloader', 'Plot Downloader', 'File Downloader', 'Application Logger' and 'Reset Application'</li>
            <li>Different methods to notify user and add useful information about application UI and server operations</li>
          </ul>
        </dl>
      </p>
      <a class="btn btn-secondary btn-lg" href="https://periscopeapps.org/" target="_blank" role="button">More</a>
    </div>
    
    

# load_theme_settings - null settings

    Code
      load_theme_settings()
    Output
      NULL

# ui_tooltip

    <span class="periscope-input-label-with-tt">
      mylabel
      <img id="id" src="img/tooltip.png" height="16px" width="16px" data-toggle="tooltip" data-placement="top" title="mytext" data-html="true"/>
    </span>

---

    <span class="periscope-input-label-with-tt">
      mylabel2
      <img id="id2" src="img/tooltip.png" height="16px" width="16px" data-toggle="tooltip" data-placement="left" title="mytext2" data-html="true"/>
    </span>

---

    'arg' should be one of "top", "bottom", "left", "right"

# theme - invalid color

    Code
      theme_warnings
    Output
      [1] "primary has invalid color value. Setting default color."
      [2] "-300 must be positive value. Setting default value."    

# dashboard - create default dashboard

    Code
      periscope2:::create_application_dashboard()
    Output
      [[1]]
      <div class="row">
        <div class="col-sm-12">
          <div id="announceAlert"></div>
        </div>
      </div>
      
      [[2]]
      <div class="row">
        <div class="col-sm-12">
          <div id="headerAlert"></div>
        </div>
      </div>
      
      [[3]]
      <div class="row">
        <div class="col-sm-12">
          <body data-help="0" data-fullscreen="0" data-dark="0" data-scrollToTop="0">
            <div class="wrapper">
              <nav data-fixed="false" class="main-header navbar navbar-expand navbar-white navbar-light">
                <ul class="navbar-nav">
                  <li class="nav-item">
                    <a class="nav-link" data-widget="pushmenu" href="#">
                      <i class="fas fa-bars" role="presentation" aria-label="bars icon"></i>
                    </a>
                  </li>
                </ul>
                <div class="app_header_container">
                  <div class="periscope-busy-ind">
                    Working
                    <img alt="Working..." hspace="5px" src="img/loader.gif"/>
                  </div>
                  <div id="app_header">good title</div>
                </div>
                <ul class="navbar-nav ml-auto navbar-right">
                  <li class="nav-item">
                    <a id="controlbar-toggle" class="nav-link" data-widget="control-sidebar" href="#">
                      <i class="fas fa-table-cells" role="presentation" aria-label="table-cells icon"></i>
                    </a>
                  </li>
                </ul>
              </nav>
              <aside id="sidebarId" data-fixed="true" data-minified="FALSE" data-collapsed="FALSE" data-disable="FALSE" class="main-sidebar sidebar-light-primary elevation-4">
                <div class="sidebar" id="sidebarItemExpanded">
                  <nav class="mt-2" expand_on_hover="FALSE">
                    <div id="sidebarBasicAlert"></div>
                    <ul class="nav nav-pills nav-sidebar flex-column sidebar-menu nav-child-indent" data-widget="treeview" role="menu" data-accordion="true">
                      <li class="nav-header">Periscope2 Features</li>
                      <li class="nav-item">
                        <a class="nav-link" id="tab-application_setup" href="#" data-target="#shiny-tab-application_setup" data-toggle="tab" data-value="application_setup">
                          <i class="far fa-building nav-icon" role="presentation" aria-label="building icon"></i>
                          <p>Application Setup</p>
                        </a>
                      </li>
                      <li class="nav-item">
                        <a class="nav-link" id="tab-periscope_modules" href="#" data-target="#shiny-tab-periscope_modules" data-toggle="tab" data-value="periscope_modules">
                          <i class="fas fa-cubes nav-icon" role="presentation" aria-label="cubes icon"></i>
                          <p>Periscope2 Modules</p>
                        </a>
                      </li>
                      <li class="nav-item">
                        <a class="nav-link" id="tab-user_notifications" href="#" data-target="#shiny-tab-user_notifications" data-toggle="tab" data-value="user_notifications">
                          <i class="far fa-comments nav-icon" role="presentation" aria-label="comments icon"></i>
                          <p>User Notifications</p>
                        </a>
                      </li>
                      <div id="features_id" class="sidebarMenuSelectedTabItem" data-value="null"></div>
                    </ul>
                  </nav>
                </div>
              </aside>
              <div class="content-wrapper">
                <section class="content">
                  <div id="bodyAlert"></div>
                  <div id="head"></div>
                  <div>more elements</div>
                  <div>append div</div>
                  <div class="jumbotron bg-info">
                    <h1 class="display-4">periscope2: Enterprise Streamlined 'Shiny' Application Framework</h1>
                    <p class="lead">
                      <p>
                        periscope2 is a scalable and UI-standardized 'shiny' framework including a variety of developer convenience
                        functions with the goal of both streamlining robust application development and assisting in creating a consistent
                         user experience regardless of application or developer.
                      </p>
                    </p>
                    <hr class="my-4"/>
                    <p>
                      <dl>
                        <dt>Features</dt>
                        <ul>
                          <li>Predefined but flexible template for new Shiny applications with a default dashboard layout</li>
                          <li>Separation by file of functionality that exists in one of the three shiny scopes: global, server-global, and server-local.</li>
                          <li>Off-the-shelf and ready to be used modules ('Announcements', 'Table Downloader', 'Plot Downloader', 'File Downloader', 'Application Logger' and 'Reset Application'</li>
                          <li>Different methods to notify user and add useful information about application UI and server operations</li>
                        </ul>
                      </dl>
                    </p>
                    <a class="btn btn-secondary btn-lg" href="https://periscopeapps.org/" target="_blank" role="button">More</a>
                  </div>
                </section>
              </div>
              <aside class="control-sidebar control-sidebar-light" id="controlbarId" data-collapsed="true" data-overlay="true" data-show="true" data-pin="false" data-slide="true">
                <button id="controlbarPin" class="m-2 p-1 btn btn-xs btn-outline-secondary" type="button">
                  <i class="fas fa-thumbtack" role="presentation" aria-label="thumbtack icon"></i>
                </button>
                <div class="control-sidebar-content" id="controlbarTitle">
                  <div id="sidebarRightAlert"></div>
                  <div style="margin-left:20px">
                    <div class="form-group shiny-input-container">
                      <div class="checkbox">
                        <label>
                          <input id="hideFileOrganization" type="checkbox" class="shiny-input-checkbox"/>
                          <span>Show Files Organization</span>
                        </label>
                      </div>
                    </div>
                  </div>
                </div>
              </aside>
              <footer class="main-footer" data-fixed="false">
                <div class="float-right d-none d-sm-inline">2022</div>
                <div id="footerAlert"></div>
                <a href="https://periscopeapps.org/" target="_blank">periscope2</a>
              </footer>
            </div>
          </body>
        </div>
      </div>
      

# add_ui_header - html title

    Code
      shiny::isolate(periscope2:::.g_opts$app_info)
    Output
      Demonstrate periscope features and generated application layout

---

    Code
      header[[1]]
    Output
      <nav data-fixed="false" class="main-header navbar navbar-expand navbar-white navbar-light">
        <ul class="navbar-nav">
          <li class="nav-item">
            <a class="nav-link" data-widget="pushmenu" href="#">
              <i class="fas fa-bars" role="presentation" aria-label="bars icon"></i>
            </a>
          </li>
        </ul>
        <div class="app_header_container">
          <div class="periscope-busy-ind">
            Working
            <img alt="Working..." hspace="5px" src="img/loader.gif"/>
          </div>
          <div id="app_header">
            <a id="app_info" href="#" class="action-button action-link"><span class="action-label">periscope Example Application</span></a>
          </div>
        </div>
        <ul class="navbar-nav ml-auto navbar-right">
          <li class="nav-item">
            <a id="controlbar-toggle" class="nav-link" data-widget="control-sidebar" href="#">
              <i class="fas fa-table-cells" role="presentation" aria-label="table-cells icon"></i>
            </a>
          </li>
        </ul>
      </nav>

# add_ui_header - url title

    Code
      shiny::isolate(periscope2:::.g_opts$app_info)
    Output
      [1] "https://cran.r-project.org/web/packages/periscope2/index.html"

---

    Code
      header[[1]]
    Output
      <nav data-fixed="false" class="main-header navbar navbar-expand navbar-white navbar-light">
        <ul class="navbar-nav">
          <li class="nav-item">
            <a class="nav-link" data-widget="pushmenu" href="#">
              <i class="fas fa-bars" role="presentation" aria-label="bars icon"></i>
            </a>
          </li>
        </ul>
        <div class="app_header_container">
          <div class="periscope-busy-ind">
            Working
            <img alt="Working..." hspace="5px" src="img/loader.gif"/>
          </div>
          <div id="app_header">
            <a id="app_info" href="https://cran.r-project.org/web/packages/periscope2/index.html" target="_blank">periscope Example Application</a>
          </div>
        </div>
        <ul class="navbar-nav ml-auto navbar-right">
          <li class="nav-item">
            <a id="controlbar-toggle" class="nav-link" data-widget="control-sidebar" href="#">
              <i class="fas fa-table-cells" role="presentation" aria-label="table-cells icon"></i>
            </a>
          </li>
        </ul>
      </nav>

# create alert - id

    

# set_app_parameters update values

    Code
      deprecated_flds_warn
    Output
      [1] "The `announcements_file` argument of `set_app_parameters()` is deprecated as of periscope2 0.2.3.\ni Please use `periscope2::load_announcements` instead"
      [2] "The `title` argument of `set_app_parameters()` is deprecated as of periscope2 0.2.3.\ni Please use `periscope2::add_ui_header(title)` instead"           

---

    Code
      shiny::isolate(periscope2:::.g_opts$app_info)
    Output
      Demonstrate periscope features and generated application layout

---

    Code
      shiny::isolate(periscope2:::.g_opts$loading_indicator)
    Output
      $html
      <div>Loading ...</div>
      

# theme builder addin - server

    Code
      yaml::read_yaml(output$downloadConfig)
    Output
      $primary
      [1] "#B221DD"
      
      $secondary
      [1] "#6C757D"
      
      $success
      [1] "#2ED610"
      
      $info
      [1] "#7BDFF2"
      
      $warning
      [1] "#FFF200"
      
      $danger
      [1] "#CE0900"
      
      $light
      [1] "#F8F9FA"
      
      $dark
      [1] "#343A40"
      
      $bg
      [1] "#D67E7CD1"
      
      $hover_bg
      [1] "#B0B818F7"
      
      $color
      [1] "#CA39D4ED"
      
      $hover_color
      [1] "#000000E4"
      
      $active_color
      [1] "#BF8DF0EE"
      
      $submenu_bg
      [1] "#084D66BF"
      
      $submenu_color
      [1] "#B33634D3"
      
      $submenu_hover_color
      [1] "#0303038F"
      
      $submenu_hover_bg
      [1] "#BE25CFF2"
      
      $submenu_active_color
      [1] "#1FAB9DC2"
      
      $submenu_active_bg
      [1] "#401487E8"
      
      $header_color
      [1] "#D91F1CAF"
      
      $sidebar_width
      [1] "350"
      
      $control_sidebar_width
      [1] "250"
      
      $sidebar_padding_x
      [1] "15"
      
      $sidebar_padding_y
      [1] "20"
      
      $sidebar_mini_width
      [1] "25"
      
      $blue
      [1] "#007BFF"
      
      $lightblue
      [1] "#3C8DBC"
      
      $navy
      [1] "#001F3F"
      
      $cyan
      [1] "#17A2B8"
      
      $teal
      [1] "#39CCCC"
      
      $olive
      [1] "#3D9970"
      
      $green
      [1] "#28A745"
      
      $lime
      [1] "#01FF70"
      
      $orange
      [1] "#FF851B"
      
      $yellow
      [1] "#FFC107"
      
      $fuchsia
      [1] "#F012BE"
      
      $purple
      [1] "#605CA8"
      
      $maroon
      [1] "#D81B60"
      
      $red
      [1] "#DC3545"
      
      $black
      [1] "#111111"
      
      $gray_x_light
      [1] "#D2D6DE"
      
      $gray_600
      [1] "#6C757D"
      
      $white
      [1] "#FFFFFF"
      
      $contrasted_threshold
      [1] "12"
      
      $text_dark
      [1] "#701110D0"
      
      $text_light
      [1] "#2F515EBA"
      
      $`navbar-light-color`
      [1] "#bec5cb"
      

# build_datatable_arguments

    Code
      build_datatable_arguments(table_options)
    Message
      DT option 'width' is not supported. Ignoring it.
      DT option 'height' is not supported. Ignoring it.
      DT option 'editable' is not supported. Ignoring it.
    Output
      $rownames
      [1] FALSE
      
      $class
      [1] "periscope-downloadable-table table-condensed table-striped table-responsive"
      
      $callback
      [1] "table.order([2, 'asc']).draw();"
      
      $caption
      [1] " Very Important Information"
      
      $colnames
      [1] "Area"     "Delta"    "Increase"
      
      $filter
      [1] "bottom"
      
      $extensions
      [1] "Buttons"
      
      $plugins
      [1] "natural"
      
      $options
      $options$order
      $options$order[[1]]
      $options$order[[1]][[1]]
      [1] 2
      
      $options$order[[1]][[2]]
      [1] "asc"
      
      
      $options$order[[2]]
      $options$order[[2]][[1]]
      [1] 3
      
      $options$order[[2]][[2]]
      [1] "desc"
      
      
      
      $options$deferRender
      [1] FALSE
      
      $options$paging
      [1] FALSE
      
      $options$scrollX
      [1] TRUE
      
      $options$dom
      [1] "<\"periscope-downloadable-table-header\"f>tr"
      
      $options$processing
      [1] TRUE
      
      $options$rowId
      [1] 1
      
      $options$searchHighlight
      [1] TRUE
      
      

# format_columns

    Code
      format_columns(DT::datatable(dt), list(formatCurrency = list(columns = c("A",
        "C")), formatPercentage = list(columns = c("D"), 2)))

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{logging-entrypoints}
\alias{logging-entrypoints}
\alias{logdebug}
\alias{loginfo}
\alias{logwarn}
\alias{logerror}
\title{Entry points for logging actions}
\usage{
logdebug(msg, ..., logger = "")

loginfo(msg, ..., logger = "")

logwarn(msg, ..., logger = "")

logerror(msg, ..., logger = "")
}
\arguments{
\item{msg}{the textual message to be output, or the format for the \dots
arguments}

\item{...}{if present, msg is interpreted as a format and the \dots values
are passed to it to form the actual message.}

\item{logger}{the name of the logger to which we pass the record}
}
\value{
no return value, prints log contents into R console and app log file
}
\description{
Generate a log record and pass it to the logging system.\cr
}
\details{
A log record gets timestamped and will be independently formatted by each
of the handlers handling it.\cr

Leading and trailing whitespace is stripped from the final message.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logViewer.R
\name{logViewerOutput}
\alias{logViewerOutput}
\title{Display app logs}
\usage{
logViewerOutput(id = "logViewer")
}
\arguments{
\item{id}{character id for the object(default = "logViewer")}
}
\value{
downloadableReactTableUI instance
}
\description{
Display app log data in downloadableReactTable table containing logged user actions. Table contents are auto updated
whenever a user action is logged. User can search for logs, sort them by time and download them in CSV or TSV format.
The id must match the same id configured in \bold{server.R} file upon calling \code{fw_server_setup} method
}
\section{Table columns}{

\itemize{
\item action - the action that id logged in any place in app
\item time   - action time
}
}

\section{Example}{

\code{logViewerOutput('logViewer')}
}

\section{Shiny Usage}{

Add the log viewer box to your box list

It is paired with a call to \code{fw_server_setup} method in \bold{server.R} file
}

\examples{
# Inside ui_body add the log viewer box to your box list

logViewerOutput('logViewerId')


}
\seealso{
\link[periscope2]{downloadFile}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{downloadablePlot}

\link[periscope2]{downloadFileButton}

\link[periscope2]{downloadableTableUI}

\link[periscope2]{downloadableTable}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downloadFile.R
\name{downloadFile_AvailableTypes}
\alias{downloadFile_AvailableTypes}
\title{downloadFile module list of allowed file types}
\usage{
downloadFile_AvailableTypes()
}
\value{
a vector of all supported types
}
\description{
Returns a list of all supported types
}
\seealso{
\link[periscope2]{downloadFileButton}

\link[periscope2]{downloadFile}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui_helpers.R
\name{add_ui_header}
\alias{add_ui_header}
\title{Add UI elements to dashboard header section}
\usage{
add_ui_header(
  ui_elements = NULL,
  ui_position = "right",
  title = NULL,
  title_position = "center",
  left_menu = NULL,
  right_menu = NULL,
  border = TRUE,
  compact = FALSE,
  right_sidebar_icon = shiny::icon("bars"),
  fixed = FALSE,
  left_sidebar_icon = shiny::icon("th"),
  skin = "light",
  status = "white"
)
}
\arguments{
\item{ui_elements}{It can be any UI element but mostly used for navbarMenu. NULL by default.
Check \code{?bs4Dash::navbarMenu()}}

\item{ui_position}{Location of UI elements in the header. Must be either of 'center', 'left' or 'right'
Default value is 'right'.}

\item{title}{Sets application title. If it is not NULL, it will override "title" value that is set in
\code{?periscope2::set_app_parameters()} (default = NULL)}

\item{title_position}{Location of the title in the header. Must be either of 'center', 'left' or 'right'
Default value is 'Center'. If there are no UI elements, this param will be ignored.}

\item{left_menu}{Left menu. bs4DropdownMenu object (or similar dropdown menu).
Check \code{?bs4Dash::bs4DropdownMenu()}}

\item{right_menu}{Right menu. bs4DropdownMenu object (or similar dropdown menu).
Check \code{?bs4Dash::bs4DropdownMenu()}}

\item{border}{Whether to separate the navbar and body by a border. TRUE by default}

\item{compact}{Whether items should be compacted. FALSE by default}

\item{right_sidebar_icon}{Right sidebar toggle icon}

\item{fixed}{Whether to fix the navbar to the top. FALSE by default}

\item{left_sidebar_icon}{Left sidebar toggle icon}

\item{skin}{Sidebar skin. "dark" or "light"}

\item{status}{Sidebar status. Check \code{?bs4Dash::bs4DashNavbar()} for list of valid values}
}
\value{
list of both shiny UI elements and named header properties
}
\description{
Builds application header with given configurations and elements. It is called within "ui_header.R".
These elements will be displayed in the header beside application title and application busy indicator.
}
\details{
\subsection{Application header consists of three elements:}{
\describe{
\item{busy indicator}{An automatic wait indicator that are shown when the shiny server session is busy}
\item{application title}{Display application title}
\item{heade menu}{Optional header menu to switch between application different tabs}
}
}

Header elements can be arranged via \code{ui_position} and \code{title_position} parameters.
\cr
Header elements look and feel can also be configured in \bold{"www\\css\\custom.css"} file under
\bold{"Application Header"} section.
\cr
Check example application for detailed example
}
\section{Shiny Usage}{

Call this function from \code{program/ui_header.R} to set header parameters
}

\examples{
  library(shiny)
  library(bs4Dash)

  # Inside ui_header.R
  # Custom left UI menu
  left_menu <- tagList(dropdownMenu(badgeStatus = "info",
                                  type        = "notifications",
                                   notificationItem(inputId = "triggerAction2",
                                                    text    = "Error!",
                                                    status  = "danger")),
                      dropdownMenu(badgeStatus = "info",
                                   type        = "tasks",
                                   taskItem(inputId = "triggerAction3",
                                            text    = "My progress",
                                            color   = "orange",
                                            value   = 10)))

  # Custom right UI menu
  right_menu <- dropdownMenu(badgeStatus = "danger",
                          type        = "messages",
                          messageItem(inputId = "triggerAction1",
                                      message = "message 1",
                                      from    = "Divad Nojnarg",
                                      time    = "today",
                                      color   = "lime"))
  # -- Register Header Elements in the ORDER SHOWN in the UI
  add_ui_header(left_menu = left_menu, right_menu = right_menu)

}
\seealso{
\link[bs4Dash:bs4DashNavbar]{bs4Dash:bs4DashNavbar()}

\link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}

\link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}

\link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}

\link[periscope2:add_ui_body]{periscope2:add_ui_body()}

\link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}

\link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}

\link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/appReset.R
\name{appReset}
\alias{appReset}
\title{appReset module server function}
\usage{
appReset(id, reset_wait = 5000, alert_location = "bodyAlert", logger)
}
\arguments{
\item{id}{Character represents the ID of the Module's UI element (the same id used in \code{appResetButton}}

\item{reset_wait}{Integer represents the period to wait before session reload in milliseconds (default = 5000)}

\item{alert_location}{Character represents div ID or selector to display module related messages (default = "bodyAlert")}

\item{logger}{logger to use}
}
\value{
nothing, function will display a warning message in the app then reload the whole application
}
\description{
Server-side function for the appResetButton  This is a custom
high-functionality button for session reload. The server function is used to
provide module configurations.
}
\section{Shiny Usage}{

This function is not called directly by consumers - it is accessed in
server_local.R (or similar file) using the same id provided in \code{appResetButton}:

\strong{\code{appReset(id = "appResetId", logger = ss_userAction.Log)}}
}

\examples{
if (interactive()) {
   library(shiny)
   library(periscope2)
   shinyApp(
     ui = fluidPage(fluidRow(column(12, appResetButton(id = "appResetId")))),
     server = function(input, output) {
       appReset(id = "appResetId", logger = "")
   })
}

}
\seealso{
\link[periscope2]{appResetButton}

\link[periscope2]{downloadFile}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{downloadablePlot}

\link[periscope2]{downloadFileButton}

\link[periscope2]{downloadableTable}

\link[periscope2]{logViewerOutput}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downloadablePlot.R
\name{downloadablePlot}
\alias{downloadablePlot}
\title{downloadablePlot module server function}
\usage{
downloadablePlot(
  id,
  logger = NULL,
  filenameroot = "download",
  aspectratio = 1,
  downloadfxns = NULL,
  visibleplot
)
}
\arguments{
\item{id}{the ID of the Module's UI element}

\item{logger}{logger to use}

\item{filenameroot}{the base text used for user-downloaded file - can be
either a character string or a reactive expression returning a character
string}

\item{aspectratio}{the downloaded chart image width:height ratio (ex:
1 = square, 1.3 = 4:3, 0.5 = 1:2).  Where not applicable for a download type
it is ignored (e.g. data, html downloads)}

\item{downloadfxns}{a \strong{named} list of functions providing download
images or data tables as return values.  The names for the list should be
the same names that were used when the plot UI was created.}

\item{visibleplot}{function or reactive expression providing the plot to
display as a return value.  This function should require no input parameters.}
}
\value{
Reactive expression containing the currently selected plot to be available for display and download
}
\description{
Server-side function for the downloadablePlotUI.  This is a custom
plot output paired with a linked downloadFile button.
}
\details{
downloadFile button will be hidden if \code{downloadablePlot} parameter \code{downloadfxns} or
\code{downloadablePlotUI} parameter \code{downloadtypes} is empty
}
\section{Notes}{

When there are no values to download in any of the linked downloadfxns the
button will be hidden as there is nothing to download.
}

\section{Shiny Usage}{

This function is not called directly by consumers - it is accessed in
server.R using the same id provided in \code{downloadablePlotUI}:

\strong{\code{downloadablePlot(id, logger, filenameroot,
downloadfxns, visibleplot)}}
}

\examples{
if (interactive()) {
  library(shiny)
  library(ggplot2)
  library(periscope2)
  shinyApp(ui = fluidPage(fluidRow(column(width = 12,
     downloadablePlotUI("object_id1",
                        downloadtypes      = c("png", "csv"),
                        download_hovertext = "Download plot and data",
                        height             = "500px",
                        btn_halign         = "left")))),
    server = function(input, output) {
      download_plot <- function() {
        ggplot(data = mtcars, aes(x = wt, y = mpg)) +
        geom_point(aes(color = cyl)) +
        theme(legend.justification   = c(1, 1),
              legend.position.inside = c(1, 1),
              legend.title           = element_blank()) +
        ggtitle("GGPlot Example ") +
        xlab("wt") +
        ylab("mpg")
      }
      downloadablePlot(id           = "object_id1",
                       logger       = "",
                       filenameroot = "mydownload1",
                       downloadfxns = list(png = download_plot, csv = reactiveVal(mtcars)),
                       aspectratio  = 1.33,
                       visibleplot  = download_plot)
  })
}

}
\seealso{
\link[periscope2]{downloadablePlotUI}

\link[periscope2]{appResetButton}

\link[periscope2]{appReset}

\link[periscope2]{downloadFile}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{downloadableTable}

\link[periscope2]{logViewerOutput}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui_helpers.R
\name{ui_tooltip}
\alias{ui_tooltip}
\title{Add tooltip icon and text to UI elements labels}
\usage{
ui_tooltip(id, label = "", text = "", placement = "top")
}
\arguments{
\item{id}{The id for the tooltip object}

\item{label}{Text label to appear to the left of the tooltip image}

\item{text}{Tooltip text shown when the user hovers over the image}

\item{placement}{Where to display tooltip label. Available places are "top", "bottom", "left", "right" (default is "top")}
}
\value{
html span with the label, tooltip image and tooltip text
}
\description{
This function inserts a standardized tooltip image, label (optional),
and hovertext into the application UI
}
\section{Shiny Usage}{

Call this function from \code{program/ui_body.R} to set tooltip parameters
}

\examples{
  library(shiny)
  library(periscope2)

  # Inside ui_body.R or similar UI file
   ui_tooltip(id   = "top_tip",
             label = "Top Tooltips",
             text  = "Top tooltip")

}
\seealso{
\link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}

\link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}

\link[periscope2:add_ui_header]{periscope2:add_ui_header()}

\link[periscope2:add_ui_body]{periscope2:add_ui_body()}

\link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}

\link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}

\link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}

\link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_template.R
\name{create_left_sidebar}
\alias{create_left_sidebar}
\title{Add the left sidebar to an existing application}
\usage{
create_left_sidebar(location)
}
\arguments{
\item{location}{path of the existing periscope2 application}
}
\value{
no return value, creates left sidebar related UI R file and updates related source call in ui.R
}
\description{
User can update an existing application that does not have a left side bar and add a new empty one using this function.
}
\details{
If conversion is successful, the following message will be returned
\emph{"Add left sidebar conversion was successful. File(s) updated: ui.R, ui_left_sidebar.R"}

If the function called on an application with an existing left bar, message
\emph{"Left sidebar already available, no conversion needed"}
will be returned with no conversion

If the passed location is invalid, empty, not exist or not a valid periscope2 application, nothing will be added
and a related error message will be printed in console
}
\seealso{
\link[periscope2]{create_right_sidebar}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/themeConfigurationsAddin.R
\name{themeConfigurationsAddin}
\alias{themeConfigurationsAddin}
\title{Build application theme configuration YAML file}
\usage{
themeConfigurationsAddin()
}
\value{
launch gadget window
}
\description{
Call this as an addin to build valid yaml file that is needed for creating application
periscope_style.yaml file. The generated file can be used in periscope2 app by putting it inside generated app www folder.
}
\details{
The method can be called directly via \code{R} console or via RStudio addins menu
}
\examples{
if (interactive()) {
   periscope2:::themeConfigurationsAddin()
}

}
\seealso{
\link[periscope2:create_application]{periscope2:create_application()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui_helpers.R
\name{add_ui_left_sidebar}
\alias{add_ui_left_sidebar}
\title{Add UI elements to dashboard left sidebar section}
\usage{
add_ui_left_sidebar(
  sidebar_elements = NULL,
  sidebar_menu = NULL,
  collapsed = FALSE,
  custom_area = NULL,
  elevation = 4,
  expand_on_hover = TRUE,
  fixed = TRUE,
  minified = FALSE,
  status = "primary",
  skin = "light"
)
}
\arguments{
\item{sidebar_elements}{List of regular shiny UI elements (inputText, textArea, etc..)}

\item{sidebar_menu}{\code{?bs4Dash::bs4SidebarMenu()} object to created a menu inside left sidebar}

\item{collapsed}{If TRUE, the sidebar will be collapsed on app start up}

\item{custom_area}{List of regular shiny UI elements but for sidebar bottom space area only.
Only works if sidebar is fixed}

\item{elevation}{A number between 0 and 5, which applies a shadow to the sidebar to add a shadow effect.}

\item{expand_on_hover}{When \code{minified} is TRUE, if this property is TRUE, the sidebar opens when hovering but re-collapses as soon as the focus is lost (default = TRUE)}

\item{fixed}{Whether to see all menus at once without scrolling up and down.(default = TRUE)}

\item{minified}{Whether to slightly close the sidebar but still show item icons (default = FALSE)}

\item{status}{Determines which color menu items (if exist) will have Check \code{?bs4Dash::dashboardSidebar()} for list of valid values}

\item{skin}{Sidebar skin. "dark" or "light" (default = "light")}
}
\value{
list of both shiny UI elements and named left sidebar properties
}
\description{
This function adds left sidebar configurations and UI elements. It is called within "ui_left_sidebar.R".
Check example application for detailed example
}
\section{Shiny Usage}{

Call this function from \code{program/ui_left_sidebar.R} to set left sidebar parameters
}

\examples{
  library(shiny)
  library(bs4Dash)
  # Inside ui_left_sidebar.R
  # sidebar menu items
  sidebar_elements <- textInput("text_id", "Test", "Test Data")
  sidebar_menu     <- sidebarMenu(sidebarHeader("Main Menu"),
                                  menuItem("menu item 1",
                                           tabName = "item_1 page"),
                                  menuItem("menu item 2",
                                           tabName = "item_2 page"))
  add_ui_left_sidebar(sidebar_elements = sidebar_elements,
                      sidebar_menu     = sidebar_menu)
}
\seealso{
\link[bs4Dash:bs4DashSidebar]{bs4Dash:bs4DashSidebar()}

\link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}

\link[periscope2:add_ui_header]{periscope2:add_ui_header()}

\link[periscope2:add_ui_body]{periscope2:add_ui_body()}

\link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}

\link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}

\link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/generate_template.R
\name{create_application}
\alias{create_application}
\title{Create a new templated framework application}
\usage{
create_application(
  name,
  location,
  sample_app = FALSE,
  left_sidebar = TRUE,
  right_sidebar = FALSE
)
}
\arguments{
\item{name}{name for the new application and directory}

\item{location}{base path for creation of \code{name}}

\item{sample_app}{whether to create a sample shiny application}

\item{left_sidebar}{whether the left sidebar should be enabled. It can be TRUE/FALSE}

\item{right_sidebar}{parameter to set the right sidebar. It can be TRUE/FALSE}
}
\value{
no return value, creates application folder structure and files
}
\description{
Creates ready-to-use templated application files using the periscope2
framework. The application can be created either empty (default) or with a
sample/documented example application.\cr \cr
}
\section{Name}{

The \code{name} directory must not exist in \code{location}.  If the code
detects that this directory exists it will abort the creation process with
a warning and will not create an application template.

Use only filesystem-compatible characters in the name (ideally w/o spaces)
}

\section{Directory Structure}{


\preformatted{
name
 -- log (log files)
 -- program (user application)
 -- -- config   (application configuration files)
 -- -- data     (user application data)
 -- -- fxn      (user application function)
 -- -- modules  (application modules files)
 -- www (supporting shiny files)
 -- -- css  (application css files)
 -- -- img  (application image files)
 -- -- js   (application js files)
}
}

\section{File Information}{


All user application creation and modifications will be done in
the \strong{program} directory.  The names & locations
of the framework-provided .R files should not be changed or the framework
will fail to work as expected. \cr
\cr
\strong{\emph{name/program/config}} directory :\cr
Use this location for configuration files.\cr
\cr
\strong{\emph{name/program/data}} directory :\cr
Use this location for data files.  There is a \strong{.gitignore} file
included in this directory to prevent accidental versioning of data\cr
\cr
\strong{\emph{name/program/fxn}} directory :\cr
Use this location for supporting and helper R files.\cr
\cr
\strong{\emph{name/program/modules}} directory :\cr
Use this location for application new modules files.\cr
\cr
\strong{\emph{name/program}/global.R} :\cr
Use this location for code that would have previously resided in global.R
and for setting application parameters using
\link[periscope2]{set_app_parameters}.  Anything placed in this file will
be accessible across all user sessions as well as within the UI context. \cr
\cr
\strong{\emph{name/program}/server_global.R} :\cr
Use this location for code that would have previously resided in server.R
above (i.e. outside of) the call to \code{shinyServer(...)}. Anything placed
in this file will be accessible across all user sessions. \cr
\cr
\strong{\emph{name/program}/server_local.R} :\cr
Use this location for code that would have previously resided in server.R
inside of the call to \code{shinyServer(...)}.  Anything placed in this
file will be accessible only within a single user session.\cr
\cr
\strong{\emph{name/program}/ui_body.R} :\cr
Create body UI elements in this file and register them with the
framework using a call to \link[periscope2]{add_ui_body} \cr
\cr
\strong{\emph{name/program}/ui_footer.R} :\cr
Create footer UI elements in this file and register them with the
framework using a call to \link[periscope2]{add_ui_footer} \cr
\cr
\strong{\emph{name/program}/ui_header.R} :\cr
Create header UI elements in this file and register them with the
framework using a call to \link[periscope2]{add_ui_header} \cr
\cr
\strong{\emph{name/program}/ui_left_sidebar.R} :\cr
Create sidebar UI elements in this file and register them with the
framework using a call to \link[periscope2]{add_ui_left_sidebar} \cr
\cr
\strong{\emph{name/program}/ui_right_sidebar.R} :\cr
Create right sidebar UI elements in this file and register them with the
framework using a call to \link[periscope2]{add_ui_right_sidebar} \cr
\cr
\strong{\emph{name}/www/css/custom.css} :\cr
This is the application custom styling css file. User can update
application different parts style using this file.\cr
\cr
\strong{\emph{name}/www/js/custom.js} :\cr
This is the application custom javascript file.\cr
\cr
\strong{\emph{name}/www/periscope_style.yaml} :\cr
This is the application custom styling yaml file. User can update
application different parts style using this file.\cr
\cr
\cr
\strong{Do not modify the following files}: \cr
\preformatted{
name\\global.R
name\\server.R
name\\ui.R
name\\www\\img\\loader.gif
name\\www\\img\\tooltip.png
}
}

\examples{
# sample app named 'mytestapp' created in a temp dir
location <- tempdir()
create_application(name = 'mytestapp', location = location, sample_app = TRUE)
unlink(paste0(location,'/mytestapp'), TRUE)

# sample app named 'mytestapp' with a right sidebar using a custom icon created in a temp dir
location <- tempdir()
create_application(name = 'mytestapp', location = location, sample_app = TRUE, right_sidebar = TRUE)
unlink(paste0(location,'/mytestapp'), TRUE)

# blank app named 'myblankapp' created in a temp dir
location <- tempdir()
create_application(name = 'myblankapp', location = location)
unlink(paste0(location,'/myblankapp'), TRUE)

# blank app named 'myblankapp' without a left sidebar created in a temp dir
location <- tempdir()
create_application(name = 'myblankapp', location = location, left_sidebar = FALSE)
unlink(paste0(location,'/myblankapp'), TRUE)

}
\seealso{
\link[bs4Dash:dashboardPage]{bs4Dash:dashboardPage()}

\link[waiter:waiter]{waiter:waiter_show()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui_helpers.R
\name{createPSAlert}
\alias{createPSAlert}
\title{Display alert panel at specified location}
\usage{
createPSAlert(
  session = shiny::getDefaultReactiveDomain(),
  id = NULL,
  selector = NULL,
  options
)
}
\arguments{
\item{session}{Shiny session object}

\item{id}{Anchor id (either id or selector only should be set)}

\item{selector}{Character vector represents jQuery selector to add the alert to is
(i.e ".alertClass", div.badge-danger.navbar-badge). If 'id' is specified, this parameter will be neglected}

\item{options}{List of options to pass to the alert}
}
\value{
html div and inserts it in the app DOM
}
\description{
Create an alert panel in server code to be displayed in the specified UI selector location
}
\section{Shiny Usage}{

Call this function from \code{program/server_local.R} or any other server file to setup the needed alert
}

\examples{
  library(shiny)
  library(bs4Dash)

  # Inside server_local.R
  createPSAlert(id       = "sidebarRightAlert",
                options  = list(title    = "Right Side",
                                status   = "success",
                                closable = TRUE,
                                content  = "Example Basic Sidebar Alert"))
  # Test se
  ## a div with class "badge-danger.navbar-badge" must be exist in UI to display alert
  selector <- "div.badge-danger.navbar-badge"
  createPSAlert(selector = selector,
                options  = list(title    = "Selector Title",
                                status   = "danger",
                                closable = TRUE,
                                content  = "Selector Alert"))


}
\seealso{
\link[bs4Dash:closeAlert]{bs4Dash:closeAlert()}

\link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}

\link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}

\link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/periscope2.R
\docType{package}
\name{periscope2}
\alias{periscope2-package}
\alias{periscope2}
\title{Periscope2 Shiny Application Framework}
\description{
Periscope2 is the next-generation package following the paradigm of the 'periscope' package to
support a UI-standardized and rail-guarded enterprise quality application environment.  This package
also includes a variety of convenience functions for 'shiny' applications in a more modernized way. Base
reusable functionality as well as UI paradigms are included to ensure a consistent user experience
regardless of application or developer.
}
\details{
'periscope2' differs from the 'periscope' package as follows:
\itemize{
\item Upgraded dependency on bootstrap v4 instead of bootstrap v3
\item New user modules (i.e. announcements)
\item More functionality and finer control over existing modules such as
\link[periscope2:createPSAlert]{alert} and \link[periscope2:appResetButton]{reset}
\item More control over customizing different application parts (header, footer, left sidebar, right sidebar and body)
\item Enhanced file structure to organize application UI, shiny modules, app configuration, .. etc
}

A gallery of 'periscope' and 'periscope2' example apps is hosted at \href{http://periscopeapps.org:3838}{http://periscopeapps.org}
}
\section{Function Overview}{


\emph{Create a new framework application instance:\cr}
\link[periscope2]{create_application}\cr

\emph{Set application parameters in program/global.R:\cr}
\link[periscope2]{set_app_parameters}\cr

\emph{Get any url parameters passed to the application:\cr}
\link[periscope2]{get_url_parameters}\cr

\emph{Update an existing application with a needed sidebar:\cr}
\link[periscope2]{create_left_sidebar}\cr
\link[periscope2]{create_right_sidebar}\cr

\emph{Register user-created UI objects to the requisite application locations:\cr}
\link[periscope2]{add_ui_body}\cr
\link[periscope2]{add_ui_footer}\cr
\link[periscope2]{add_ui_header}\cr
\link[periscope2]{add_ui_left_sidebar}\cr
\link[periscope2]{add_ui_right_sidebar}

\emph{Included shiny modules with a customized UI:\cr}
\link[periscope2]{downloadFileButton}\cr
\link[periscope2]{downloadableTableUI}\cr
\link[periscope2]{downloadablePlotUI}\cr
\link[periscope2]{appResetButton}\cr
\link[periscope2]{logViewerOutput}\cr

\emph{High-functionality standardized tooltips:\cr}
\link[periscope2]{ui_tooltip}
}

\section{More Information}{

\code{browseVignettes(package = 'periscope2')}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/Aggregate-Genius/periscope2}
  \item \url{http://periscopeapps.org:3838}
  \item Report bugs at \url{https://github.com/Aggregate-Genius/periscope2/issues}
}

}
\author{
\strong{Maintainer}: Mohammed Ali \email{mohammed@aggregate-genius.com}

Other contributors:
\itemize{
  \item Constance Brett [contributor]
  \item Aggregate Genius Inc [sponsor]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/appReset.R
\name{appResetButton}
\alias{appResetButton}
\title{appResetButton module UI function}
\usage{
appResetButton(id)
}
\arguments{
\item{id}{character id for the object}
}
\value{
an html div with prettyToggle button
}
\description{
Creates a toggle button to reset application session. Upon pressing on the
button, its state is flipped to cancel application reload with application
and console warning messages indicating that the application will be
reloaded.
}
\details{
User can either resume reloading application session or cancel reloading
process which will also generate application and console messages to
indicate reloading status and result.
}
\section{Button Features}{

\itemize{
\item Initial state label is "Application Reset" with warning status
\item Reloading state label is "Cancel Application Reset" with danger
status
}
}

\section{Shiny Usage}{

Call this function at any place in UI section.

It is paired with a call to \code{appReset(id, ...)}
in server
}

\examples{
if (interactive()) {
   library(shiny)
   library(periscope2)
   shinyApp(
     ui = fluidPage(fluidRow(column(12, appResetButton(id = "appResetId")))),
     server = function(input, output) {
       appReset(id = "appResetId", logger = "")
   })
}


}
\seealso{
\link[periscope2]{appReset}

\link[periscope2]{downloadFile}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{downloadablePlot}

\link[periscope2]{downloadFileButton}

\link[periscope2]{downloadableTable}

\link[periscope2]{logViewerOutput}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downloadableReactTable.R
\name{downloadableReactTable}
\alias{downloadableReactTable}
\title{downloadableReactTable module server function}
\usage{
downloadableReactTable(
  id,
  table_data,
  selection_mode = NULL,
  pre_selected_rows = NULL,
  file_name_root = "data_file",
  download_data_fxns = NULL,
  pagination = FALSE,
  table_height = 600,
  show_rownames = FALSE,
  columns_filter = FALSE,
  global_search = TRUE,
  row_highlight = TRUE,
  row_striping = TRUE,
  table_options = list(),
  logger = NULL
)
}
\arguments{
\item{id}{the ID of the Module's UI element}

\item{table_data}{reactive expression (or parameter-less function) that acts as table data source}

\item{selection_mode}{to enable row selection, set \code{selection_mode} value to either "single" for single row
selection or "multiple" for multiple rows selection, case insensitive. Any other value will
disable row selection. Row selection will be enabled by radio buttons in "single" selection
and checkboxes in "multiple" selection (default = NULL)}

\item{pre_selected_rows}{reactive expression (or parameter-less function) provides the rows indices of the rows to
be selected when the table is rendered. If selection_mode is disabled, this parameter will
have no effect. If selection_mode is "single" only the first row index will be used (default = NULL)}

\item{file_name_root}{the base name used for user-downloaded file. It can be either a character string
a reactive expression or a function returning a character string (default = 'data_file')}

\item{download_data_fxns}{a \strong{named} list of functions providing the data as return values.
The names for the list should be the same names as the ones used in the
downloadableReactTableUI (default = NULL)}

\item{pagination}{to enable table pagination (default = FALSE)}

\item{table_height}{max table height in pixels. Vertical scroll will be shown after that height value}

\item{show_rownames}{enable displaying rownames as a separate column (default = FALSE)}

\item{columns_filter}{enable the filtering input on each column in the table (default = FALSE)}

\item{global_search}{enable table global searching input to search and filter in all columns at once
(default = TRUE)}

\item{row_highlight}{enable highlighting rows upon mouse hover (default = TRUE)}

\item{row_striping}{add zebra-striped style to table rows (default = TRUE)}

\item{table_options}{optional table formatting parameters check \code{?reactable::reactable} for options full list.
Also see example below to see how to pass options (default = list())}

\item{logger}{logger to use (default = NULL)}
}
\value{
A named list of two elements:
\itemize{
\item selected_rows: data.frame of current selected rows
\item table_state: a list of the current table state. The list keys are
("page", "pageSize", "pages", "sorted" and "selected").
Review \code{?reactable::getReactableState} for more info.
}
}
\description{
Server-side function for the downloadableReactTableUI.
}
\section{Shiny Usage}{

This function is not called directly by consumers - it is accessed in
server.R using the same id provided in \code{downloadableReactTableUI}:

\strong{\code{downloadableReactTable(id)}}
}

\examples{
if (interactive()) {
 library(shiny)
 library(periscope2)
 library(reactable)

 shinyApp(
     ui = fluidPage(fluidRow(column(
         width = 12,
         downloadableReactTableUI(
             id            = "object_id1",
             downloadtypes = c("csv", "tsv"),
             hovertext     = "Download the data here!")))),
     server = function(input, output) {
         table_state <- downloadableReactTable(
             id                 = "object_id1",
             table_data         = reactiveVal(iris),
             download_data_fxns = list(csv = reactiveVal(iris), tsv = reactiveVal(iris)),
             selection_mode     = "multiple",
             pre_selected_rows  = function() {c(1, 3, 5)},
             table_options      = list(columns = list(
                 Sepal.Length = colDef(name = "Sepal Length"),
                 Sepal.Width  = colDef(filterable = TRUE),
                 Petal.Length = colDef(show = FALSE),
                 Petal.Width  = colDef(defaultSortOrder = "desc")),
                 theme = reactableTheme(
                     borderColor = "#dfe2e5",
                     stripedColor = "#f6f8fa",
                     highlightColor = "#f0f5f9",
                     cellPadding = "8px 12px")))

        observeEvent(table_state(), { print(table_state()) })
    })
}

}
\seealso{
\link[periscope2]{downloadableReactTableUI}

\link[periscope2]{downloadFileButton}

\link[periscope2]{logViewerOutput}

\link[periscope2]{downloadFile}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{downloadablePlot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui_helpers.R
\name{add_ui_right_sidebar}
\alias{add_ui_right_sidebar}
\title{Add UI elements to dashboard right sidebar section}
\usage{
add_ui_right_sidebar(
  sidebar_elements = NULL,
  sidebar_menu = NULL,
  collapsed = TRUE,
  overlay = TRUE,
  pinned = FALSE,
  skin = "light"
)
}
\arguments{
\item{sidebar_elements}{List of regular shiny UI elements (inputText, textArea, etc..)}

\item{sidebar_menu}{\code{?bs4Dash::controlbarMenu()} object to created a menu inside right sidebar}

\item{collapsed}{If TRUE, the sidebar will be collapsed on app startup (default = TRUE)}

\item{overlay}{Whether the sidebar covers the content when expanded (default = TRUE)}

\item{pinned}{If TRUE, allows right sidebar to remain open even after a click outside (default = FALSE)}

\item{skin}{Sidebar skin. "dark" or "light" (default = "light")}
}
\value{
list of both shiny UI elements and named right sidebar properties
}
\description{
Builds application right sidebar with given configurations and elements. It is called within "ui_right_sidebar.R".
Check example application for detailed example
}
\section{Shiny Usage}{

Call this function from \code{program/ui_right_sidebar.R} to set right sidebar parameters
}

\examples{
  library(shiny)
  library(bs4Dash)

  # Inside ui_right_sidebar.R
  sidebar_elements <- list(div(checkboxInput("checkMe", "Example Check")))
  sidebar_menu     <- controlbarMenu(id = "controlbarmenu",
                                     controlbarItem("Item 2", "Simple text"))
 # -- Register Right Sidebar Elements in the ORDER SHOWN in the UI
  add_ui_right_sidebar(sidebar_elements = sidebar_elements,
                       sidebar_menu     = sidebar_menu)

}
\seealso{
\link[bs4Dash:bs4DashControlbar]{bs4Dash:bs4DashControlbar()}

\link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}

\link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}

\link[periscope2:add_ui_header]{periscope2:add_ui_header()}

\link[periscope2:add_ui_body]{periscope2:add_ui_body()}

\link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}

\link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}

\link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui_helpers.R
\name{add_ui_footer}
\alias{add_ui_footer}
\title{Add UI elements to dashboard footer section}
\usage{
add_ui_footer(left = NULL, right = NULL, fixed = FALSE)
}
\arguments{
\item{left}{Left side UI elements}

\item{right}{Right side UI elements}

\item{fixed}{Always show footer at page bottom regardless page scroll location (default = FALSE).}
}
\value{
list of both shiny UI elements and named footer properties
}
\description{
Builds application footer with given configurations and elements. It is called within "ui_footer.R".
Check example application for detailed example
}
\section{Shiny Usage}{

Call this function from \code{program/ui_footer.R} to set footer parameters
}

\examples{
  library(shiny)
  library(bs4Dash)

  # Inside ui_footer.R
  # Left text
  left <- a(href   = "https://periscopeapps.org/",
            target = "_blank",
            "periscope2")
  # Right text
  right <- "2022"

  # -- Register Elements in the ORDER SHOWN in the UI
  add_ui_footer(left, right)

}
\seealso{
\link[bs4Dash:bs4DashFooter]{bs4Dash:bs4DashFooter()}

\link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}

\link[periscope2:add_ui_header]{periscope2:add_ui_header()}

\link[periscope2:add_ui_body]{periscope2:add_ui_body()}

\link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}

\link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}

\link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}

\link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downloadFile.R
\name{downloadFile_ValidateTypes}
\alias{downloadFile_ValidateTypes}
\title{Check passed file types against downloadFile module allowed file types list}
\usage{
downloadFile_ValidateTypes(types)
}
\arguments{
\item{types}{list of types to test}
}
\value{
the list input given in types
}
\description{
It is a downloadFile module helper to return periscope2 defined file types list and warns user if an invalid type is included
}
\examples{
  #inside console
  ## Check valid types
  result <- periscope2::downloadFile_AvailableTypes()
  identical(result, c("csv", "xlsx", "tsv", "txt", "png", "jpeg", "tiff", "bmp"))

  ## check invalid type
  testthat::expect_warning(downloadFile_ValidateTypes(types = "csv_invalid"),
                           "file download list contains an invalid type <csv_invalid>")


}
\seealso{
\link[periscope2]{downloadFileButton}

\link[periscope2]{downloadFile}

\link[periscope2]{logViewerOutput}

\link[periscope2]{downloadablePlot}

\link[periscope2]{downloadableTableUI}

\link[periscope2]{downloadableTable}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downloadableReactTable.R
\name{downloadableReactTableUI}
\alias{downloadableReactTableUI}
\title{downloadableReactTable module UI function}
\usage{
downloadableReactTableUI(id, downloadtypes = NULL, hovertext = NULL)
}
\arguments{
\item{id}{character id for the object}

\item{downloadtypes}{vector of values for data download types}

\item{hovertext}{download button tooltip hover text}
}
\value{
list of downloadFileButton UI and reactable table and hidden inputs for contentHeight option
}
\description{
downloadableReactTable module is extending \code{?reactable} package table functions by creating
a custom high-functionality table paired with \link[periscope2]{downloadFile} button.
The table has the following default functionality:search, highlight functionality, infinite scrolling, sorting by columns and
returns a reactive dataset of selected items and table current state.
}
\details{
\link[periscope2]{downloadFile} button will be hidden if \code{downloadableReactTableUI} parameter
\code{downloadtypes} is empty
}
\section{Table Features}{

\itemize{
\item Consistent styling of the table
\item downloadFile module button functionality built-in to the table (it will be shown only if downloadtypes is defined)
\item Ability to show different data from the download data
\item Table is automatically fit to the window size with infinite
y-scrolling
\item Table search functionality including highlighting built-in
\item Multi-select built in, including reactive feedback on which table
items are selected
}
}

\section{Example}{

\code{downloadableReactTableUI("mytableID", c("csv", "tsv"),
"Click Here")}
}

\section{Notes}{

When there are no rows to download in any of the linked downloaddatafxns the
button will be hidden as there is nothing to download.
}

\section{Shiny Usage}{

Call this function at the place in ui.R where the table should be placed.

Paired with a call to \code{downloadableReactTable(id, ...)}
in server.R
}

\examples{
if (interactive()) {
 library(shiny)
 library(periscope2)
 library(reactable)

 shinyApp(
     ui = fluidPage(fluidRow(column(
         width = 12,
         downloadableReactTableUI(
             id            = "object_id1",
             downloadtypes = c("csv", "tsv"),
             hovertext     = "Download the data here!")))),
     server = function(input, output) {
         table_state <- downloadableReactTable(
             id                 = "object_id1",
             table_data         = reactiveVal(iris),
             download_data_fxns = list(csv = reactiveVal(iris), tsv = reactiveVal(iris)),
             selection_mode     = "multiple",
             pre_selected_rows  = function() {c(1, 3, 5)},
             table_options      = list(columns = list(
                 Sepal.Length = colDef(name = "Sepal Length"),
                 Sepal.Width  = colDef(filterable = TRUE),
                 Petal.Length = colDef(show = FALSE),
                 Petal.Width  = colDef(defaultSortOrder = "desc")),
                 showSortable = TRUE,
                 theme = reactableTheme(
                     borderColor = "#dfe2e5",
                     stripedColor = "#f6f8fa",
                     highlightColor = "#f0f5f9",
                     cellPadding = "8px 12px")))

        observeEvent(table_state(), { print(table_state()) })
    })
}

}
\seealso{
\link[periscope2]{downloadableReactTable}

\link[periscope2]{downloadFile}

\link[periscope2]{logViewerOutput}

\link[periscope2]{downloadFile}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{downloadablePlot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downloadFile.R
\name{downloadFileButton}
\alias{downloadFileButton}
\title{downloadFileButton module UI function}
\usage{
downloadFileButton(id, downloadtypes = c("csv"), hovertext = NULL)
}
\arguments{
\item{id}{character id for the object}

\item{downloadtypes}{vector of values for data download types}

\item{hovertext}{tooltip hover text}
}
\value{
html span with tooltip and either shiny downloadButton in case of single download or shiny actionButton otherwise
}
\description{
Creates a custom high-functionality button for file downloads with two
states - single download type or multiple-download types.  The button image
and pop-up menu (if needed) are set accordingly.  A tooltip can also be set
for the button.
}
\section{Button Features}{

\itemize{
\item Consistent styling of the button, including a hover tooltip
\item Single or multiple types of downloads
\item Ability to download different data for each type of download
}
}

\section{Example}{

\code{downloadFileUI("mybuttonID1", c("csv", "tsv"), "Click Here")}
\code{downloadFileUI("mybuttonID2", "csv", "Click to download")}
}

\section{Shiny Usage}{

Call this function at the place in ui.R where the button should be placed.

It is paired with a call to \code{downloadFile(id, ...)}
in server.R
}

\examples{
if (interactive()) {
   library(shiny)
   library(periscope2)
   shinyApp(ui = fluidPage(fluidRow(column(width = 6,
     # single download type
     downloadFileButton("object_id1",
                        downloadtypes = c("csv"),
                        hovertext     = "Button 1 Tooltip")),
      column(width = 6,
      # multiple download types
      downloadFileButton("object_id2",
                         downloadtypes = c("csv", "tsv"),
                         hovertext     = "Button 2 Tooltip")))),
     server = function(input, output) {
       # single download type
       downloadFile(id           = "object_id1",
                    logger       = "",
                    filenameroot = "mydownload1",
                    datafxns     = list(csv = reactiveVal(iris)),
                    row_names    = FALSE)
       # multiple download types
       downloadFile(id           = "object_id2",
                    logger       = "",
                    filenameroot = "mydownload2",
                    datafxns     = list(csv = reactiveVal(mtcars),
                    tsv = reactiveVal(mtcars)))
   })
}

}
\seealso{
\link[periscope2]{downloadFile}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{logViewerOutput}

\link[periscope2]{downloadablePlot}

\link[periscope2]{downloadableTableUI}

\link[periscope2]{downloadableTable}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fw_helpers_external.R
\name{load_announcements}
\alias{load_announcements}
\title{load_announcements}
\usage{
load_announcements(
  announcements_file_path = NULL,
  announcement_location_id = "announceAlert"
)
}
\arguments{
\item{announcements_file_path}{The path to announcements configuration file.\cr
Use \link[periscope2]{announcementConfigurationsAddin} to generate that file.}

\item{announcement_location_id}{Announcement target location div id (default = "announceAlert")}
}
\value{
number of seconds an announcement should be staying in caller application
}
\description{
Reads and parses application announcements configurations in \code{config/announce.yaml}, then display announcements in
application header.
}
\details{
If announce.yaml does not exist or contains invalid configurations. Nothing will be displayed.
Closing announcements is caller application responsibility
}
\examples{
    load_announcements(system.file("fw_templ/announce.yaml", package = "periscope2"))

}
\seealso{
\link[periscope2:announcementConfigurationsAddin]{periscope2:announcementConfigurationsAddin()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downloadFile.R
\name{downloadFile}
\alias{downloadFile}
\title{downloadFile module server function}
\usage{
downloadFile(
  id,
  logger = NULL,
  filenameroot = "download",
  datafxns = NULL,
  aspectratio = 1,
  row_names = TRUE
)
}
\arguments{
\item{id}{ID of the Module's UI element}

\item{logger}{logger to use}

\item{filenameroot}{the base text used for user-downloaded file - can be
either a character string or a reactive expression that returns a character
string}

\item{datafxns}{a \strong{named} list of functions providing the data as
return values.  The names for the list should be the same names that were used
when the button UI was created.}

\item{aspectratio}{the downloaded chart image width:height ratio (ex:
1 = square, 1.3 = 4:3, 0.5 = 1:2). Where not applicable for a download type
it is ignored (e.g. data downloads).}

\item{row_names}{logical value indicating whether row names are to be written
for tabular data. Where not applicable for a download type it is ignored.}
}
\value{
no return value, called for downloading selected file type
}
\description{
Server-side function for the downloadFileButton.  This is a custom
high-functionality button for file downloads supporting single or multiple
download types.  The server function is used to provide the data for download.
}
\section{Shiny Usage}{

This function is not called directly by consumers - it is accessed in
server.R using the same id provided in \code{downloadFileButton}:

\strong{\code{downloadFile(id, logger, filenameroot, datafxns)}}
}

\examples{
if (interactive()) {
   library(shiny)
   library(periscope2)
   shinyApp(ui = fluidPage(fluidRow(column(width = 6,
     # single download type
     downloadFileButton("object_id1",
                        downloadtypes = c("csv"),
                        hovertext     = "Button 1 Tooltip")),
      column(width = 6,
      # multiple download types
      downloadFileButton("object_id2",
                         downloadtypes = c("csv", "tsv"),
                         hovertext     = "Button 2 Tooltip")))),
     server = function(input, output) {
       # single download type
       downloadFile(id           = "object_id1",
                    logger       = "",
                    filenameroot = "mydownload1",
                    datafxns     = list(csv = reactiveVal(iris)),
                    row_names    = FALSE)
       # multiple download types
       downloadFile(id           = "object_id2",
                    logger       = "",
                    filenameroot = "mydownload2",
                    datafxns     = list(csv = reactiveVal(mtcars),
                    tsv = reactiveVal(mtcars)))
   })
}

}
\seealso{
\link[periscope2]{downloadFileButton}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{logViewerOutput}

\link[periscope2]{downloadablePlot}

\link[periscope2]{downloadableTableUI}

\link[periscope2]{downloadableTable}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui_helpers.R
\name{add_ui_body}
\alias{add_ui_body}
\title{Add UI elements to dashboard body section}
\usage{
add_ui_body(body_elements = NULL, append = FALSE)
}
\arguments{
\item{body_elements}{List of UI elements to be displayed in application body}

\item{append}{Add elements to current body elements or remove previous body elements (default = FALSE)}
}
\value{
list of both shiny UI elements and html div tags for alert and linking app JS and CSS files
}
\description{
Builds application body with given configurations and elements. It is called within "ui_body.R".
Check example application for detailed example
}
\section{Shiny Usage}{

Call this function from \code{program/ui_body.R} to set body parameters
}

\examples{
  library(shiny)
  library(bs4Dash)
  # Inside ui_body.R
  about_box <- jumbotron(title  = "periscope2: Test Example",
                         lead   = p("periscope2 is a scalable and UI-standardized 'shiny' framework
                         	       including a variety of developer convenience functions"),
                         status = "info",
                         href   = "https://periscopeapps.org/")
  # -- Register Elements in the ORDER SHOWN in the UI
  add_ui_body(list(about_box))

}
\seealso{
\link[bs4Dash:bs4DashBody]{bs4Dash:bs4DashBody()}

\link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}

\link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}

\link[periscope2:add_ui_header]{periscope2:add_ui_header()}

\link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}

\link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}

\link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downloadableTable.R
\name{downloadableTable}
\alias{downloadableTable}
\title{downloadableTable module server function}
\usage{
downloadableTable(
  id,
  logger = NULL,
  filenameroot = "download",
  downloaddatafxns = NULL,
  tabledata,
  selection = NULL,
  table_options = list()
)
}
\arguments{
\item{id}{the ID of the Module's UI element}

\item{logger}{logger to use}

\item{filenameroot}{the text used for user-downloaded file - can be
either a character string, a reactive expression or a function returning a character
string}

\item{downloaddatafxns}{a \strong{named} list of functions providing the data as
return values.  The names for the list should be the same names that were used
when the table UI was created.}

\item{tabledata}{function or reactive expression providing the table display
data as a return value. This function should require no input parameters.}

\item{selection}{function or reactive expression providing the row_ids of the
rows that should be selected}

\item{table_options}{optional table formatting parameters check \code{?DT::datatable} for options full list.
Also see example below to see how to pass options}
}
\value{
Reactive expression containing the currently selected rows in the
display table
}
\description{
Server-side function for the downloadableTableUI.  This is a custom
high-functionality table paired with a linked downloadFile
button.
}
\details{
downloadFile button will be hidden if \code{downloadableTable} parameter \code{downloaddatafxn} or
\code{downloadableTableUI} parameter \code{downloadtypes} is empty

Generated table can highly customized using function \code{?DT::datatable} same arguments
except for \code{options} and \code{selection} parameters.

For \code{options} user can pass the same \code{?DT::datatable} options using the same names and
values one by one separated by comma.

For \code{selection} parameter it can be either a function or reactive expression providing the row_ids of the
rows that should be selected.

Also, user can apply the same provided \code{?DT::formatCurrency} columns formats on passed
dataset using format functions names as keys and their options as a list.
}
\section{Notes}{

\itemize{
\item When there are no rows to download in any of the linked downloaddatafxns
the button will be hidden as there is nothing to download.
\item \code{selection} parameter has different usage than DT::datatable \code{selection} option.
See parameters usage section.
\item DT::datatable options \code{editable}, \code{width} and \code{height} are not supported
}
}

\section{Shiny Usage}{

This function is not called directly by consumers - it is accessed in
server.R using the same id provided in \code{downloadableTableUI}:

\strong{\code{downloadableTable(id, logger, filenameroot,
downloaddatafxns, tabledata, rownames, caption, selection)}}

\emph{Note}: calling module server returns the reactive expression containing the
currently selected rows in the display table.
}

\examples{
if (interactive()) {
 library(shiny)
 library(periscope2)
 shinyApp(ui = fluidPage(fluidRow(column(width = 12,
   downloadableTableUI("object_id1",
                       downloadtypes = c("csv", "tsv"),
                       hovertext     = "Download the data here!",
                       contentHeight = "300px",
                       singleSelect  = FALSE)))),
   server = function(input, output) {
     mydataRowIds <- function(){
       rownames(head(mtcars))[c(2, 5)]
     }
     selectedrows <- downloadableTable(
       id               = "object_id1",
       logger           = "",
       filenameroot     = "mydownload1",
       downloaddatafxns = list(csv = reactiveVal(mtcars), tsv = reactiveVal(mtcars)),
       tabledata        = reactiveVal(mtcars),
       selection        = mydataRowIds,
       table_options    = list(rownames = TRUE,
                            caption  = "This is a great table!"))
     observeEvent(selectedrows(), {
       print(selectedrows())
     })})
}

}
\seealso{
\link[periscope2]{downloadableTableUI}

\link[periscope2]{downloadFileButton}

\link[periscope2]{logViewerOutput}

\link[periscope2]{downloadFile}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{downloadablePlot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downloadablePlot.R
\name{downloadablePlotUI}
\alias{downloadablePlotUI}
\title{downloadablePlot module UI function}
\usage{
downloadablePlotUI(
  id,
  downloadtypes = c("png"),
  download_hovertext = NULL,
  width = "100\%",
  height = "400px",
  btn_halign = "right",
  btn_valign = "bottom",
  btn_overlap = TRUE,
  clickOpts = NULL,
  hoverOpts = NULL,
  brushOpts = NULL
)
}
\arguments{
\item{id}{character id for the object}

\item{downloadtypes}{vector of values for download types}

\item{download_hovertext}{download button tooltip hover text}

\item{width}{plot width (any valid css size value)}

\item{height}{plot height (any valid css size value)}

\item{btn_halign}{horizontal position of the download button ("left", "center", "right")}

\item{btn_valign}{vertical position of the download button ("top", "bottom")}

\item{btn_overlap}{whether the button should appear on top of the bottom of
the plot area to save on vertical space \emph{(there is often a blank area
where a button can be overlayed instead of utilizing an entire horizontal
row for the button below the plot area)}}

\item{clickOpts}{NULL or an object created by the \link[shiny]{clickOpts} function}

\item{hoverOpts}{NULL or an object created by the \link[shiny]{hoverOpts} function}

\item{brushOpts}{NULL or an object created by the \link[shiny]{brushOpts} function}
}
\value{
list of downloadFileButton UI and plot object
}
\description{
Creates a custom plot output that is paired with a linked downloadFile
button.  This module is compatible with ggplot2, grob and lattice
produced graphics.
}
\details{
downloadFile button will be hidden if \code{downloadablePlot} parameter \code{downloadfxns} or
\code{downloadablePlotUI} parameter \code{downloadtypes} is empty
}
\section{Example}{

\code{downloadablePlotUI("myplotID", c("png", "csv"),
"Download Plot or Data", "300px")}
}

\section{Notes}{

When there is nothing to download in any of the linked downloadfxns the
button will be hidden as there is nothing to download.

This module is NOT compatible with the built-in (base) graphics \emph{(such as
basic plot, etc.)} because they cannot be saved into an object and are directly
output by the system at the time of creation.
}

\section{Shiny Usage}{

Call this function at the place in ui.R where the plot should be placed.

Paired with a call to \code{downloadablePlot(id, ...)}
in server.R
}

\examples{
if (interactive()) {
  library(shiny)
  library(ggplot2)
  library(periscope2)
  shinyApp(ui = fluidPage(fluidRow(column(width = 12,
     downloadablePlotUI("object_id1",
                        downloadtypes      = c("png", "csv"),
                        download_hovertext = "Download plot and data",
                        height             = "500px",
                        btn_halign         = "left")))),
    server = function(input, output) {
      download_plot <- function() {
        ggplot(data = mtcars, aes(x = wt, y = mpg)) +
        geom_point(aes(color = cyl)) +
        theme(legend.justification   = c(1, 1),
              legend.position.inside = c(1, 1),
              legend.title           = element_blank()) +
        ggtitle("GGPlot Example ") +
        xlab("wt") +
        ylab("mpg")
      }
      downloadablePlot(id           = "object_id1",
                       logger       = "",
                       filenameroot = "mydownload1",
                       downloadfxns = list(png = download_plot, csv = reactiveVal(mtcars)),
                       aspectratio  = 1.33,
                       visibleplot  = download_plot)
  })
}

}
\seealso{
\link[periscope2]{downloadablePlot}

\link[periscope2]{downloadFileButton}

\link[shiny]{clickOpts}

\link[shiny]{hoverOpts}

\link[shiny]{brushOpts}

\link[periscope2]{appResetButton}

\link[periscope2]{appReset}

\link[periscope2]{downloadFile}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{downloadableTable}

\link[periscope2]{logViewerOutput}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui_helpers.R
\name{set_app_parameters}
\alias{set_app_parameters}
\title{Set Application Parameters}
\usage{
set_app_parameters(
  title = NULL,
  app_info = NULL,
  log_level = "DEBUG",
  app_version = "1.0.0",
  loading_indicator = NULL,
  announcements_file = NULL
)
}
\arguments{
\item{title}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Use \link[periscope2]{add_ui_header} to configure application title text}

\item{app_info}{Application detailed information. It can be character string, HTML value or NULL
\itemize{
\item{A \strong{character} string will be used to set a link target.
This means the user will be able to click on the application title and be
redirected in a new window to whatever value is given in the string.
Any valid URL, File, or other script functionality that would normally be
accepted in an \code{<a href=...></a>}  tag is allowed.}
\item{An \strong{HTML} value will be used to as the HTML content for a modal pop-up
window that will appear on-top of the application when the user clicks on the
application title.}
\item{Supplying \strong{NULL} will disable the title link functionality.}
}}

\item{log_level}{Designating the log level to use for the user log as 'DEBUG','INFO', 'WARNING' or 'ERROR' (default = 'DEBUG')}

\item{app_version}{Character string designating the application version (default = '1.0.0')}

\item{loading_indicator}{It uses waiter (see https://waiter.john-coene.com/#/).\cr
Pass a list like list(html = spin_1(), color = "#333e48") to \cr configure
waiterShowOnLoad (refer to the package help for all styles).}

\item{announcements_file}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}. Use \link[periscope2]{load_announcements} to configure announcement.}
}
\value{
no return value, called for setting new application global properties
}
\description{
This function sets global parameters customizing the shiny application.
}
\section{Shiny Usage}{

Call this function from \code{program/global.R} to set the application
parameters.
}

\examples{
  library(shiny)
  library(waiter)
  library(periscope2)

  # Inside program/global.R
  set_app_parameters(app_info          = HTML("Example info"),
                     log_level         = "DEBUG",
                     app_version       = "1.0.0",
                     loading_indicator = list(html = tagList(spin_1(), "Loading ...")))


}
\seealso{
\link[periscope2:announcementConfigurationsAddin]{periscope2:announcementConfigurationsAddin()}

\link[periscope2:load_announcements]{periscope2:load_announcements()}

\link[waiter:waiter]{waiter:waiter_show()}

\link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}

\link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}

\link[periscope2:add_ui_header]{periscope2:add_ui_header()}

\link[periscope2:add_ui_body]{periscope2:add_ui_body()}

\link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}

\link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}

\link[periscope2:get_url_parameters]{periscope2:get_url_parameters()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/announcementConfigurationsAddin.R
\name{announcementConfigurationsAddin}
\alias{announcementConfigurationsAddin}
\title{Build Announcement Module Configuration YAML File}
\usage{
announcementConfigurationsAddin()
}
\value{
launch gadget window
}
\description{
Call this as an addin to build valid yaml file that is needed for running
announcements module. The generated file can be used in periscope2 app using \link[periscope2]{load_announcements}.
}
\details{
The method can be called directly via \code{R} console or via RStudio addins menu
}
\examples{
if (interactive()) {
   periscope2:::announcementConfigurationsAddin()
}

}
\seealso{
\link[periscope2:load_announcements]{periscope2:load_announcements()}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/downloadableTable.R
\name{downloadableTableUI}
\alias{downloadableTableUI}
\title{downloadableTable module UI function}
\usage{
downloadableTableUI(
  id,
  downloadtypes = NULL,
  hovertext = NULL,
  contentHeight = "200px",
  singleSelect = FALSE
)
}
\arguments{
\item{id}{character id for the object}

\item{downloadtypes}{vector of values for data download types}

\item{hovertext}{download button tooltip hover text}

\item{contentHeight}{viewable height of the table (any valid css size value)}

\item{singleSelect}{whether the table should only allow a single row to be
selected at a time (FALSE by default allows multi-select).}
}
\value{
list of downloadFileButton UI and DT datatable
}
\description{
Creates a custom high-functionality table paired with a linked downloadFile
button.  The table has search and highlight functionality, infinite scrolling,
sorting by columns and returns a reactive dataset of selected items.
}
\details{
downloadFile button will be hidden if \code{downloadableTable} parameter \code{downloaddatafxn} or
\code{downloadableTableUI} parameter \code{downloadtypes} is empty
}
\section{Table Features}{

\itemize{
\item Consistent styling of the table
\item downloadFile module button functionality built-in to the table (it will be shown only if downloadtypes is defined)
\item Ability to show different data from the download data
\item Table is automatically fit to the window size with infinite
y-scrolling
\item Table search functionality including highlighting built-in
\item Multi-select built in, including reactive feedback on which table
items are selected
}
}

\section{Example}{

\code{downloadableTableUI("mytableID", c("csv", "tsv"),
"Click Here", "300px")}
}

\section{Notes}{

When there are no rows to download in any of the linked downloaddatafxns the
button will be hidden as there is nothing to download.
}

\section{Shiny Usage}{

Call this function at the place in ui.R where the table should be placed.

Paired with a call to \code{downloadableTable(id, ...)}
in server.R
}

\examples{
if (interactive()) {
 library(shiny)
 library(periscope2)
 shinyApp(ui = fluidPage(fluidRow(column(width = 12,
   downloadableTableUI("object_id1",
                       downloadtypes = c("csv", "tsv"),
                       hovertext     = "Download the data here!",
                       contentHeight = "300px",
                       singleSelect  = FALSE)))),
   server = function(input, output) {
     mydataRowIds <- function(){
       rownames(head(mtcars))[c(2, 5)]
     }
     selectedrows <- downloadableTable(
       id               = "object_id1",
       logger           = "",
       filenameroot     = "mydownload1",
       downloaddatafxns = list(csv = reactiveVal(mtcars), tsv = reactiveVal(mtcars)),
       tabledata        = reactiveVal(mtcars),
       selection        = mydataRowIds,
       table_options    = list(rownames = TRUE,
                            caption  = "This is a great table!"))
     observeEvent(selectedrows(), {
       print(selectedrows())
     })})
}

}
\seealso{
\link[periscope2]{downloadableTable}

\link[periscope2]{downloadFileButton}

\link[periscope2]{logViewerOutput}

\link[periscope2]{downloadFile}

\link[periscope2]{downloadFile_ValidateTypes}

\link[periscope2]{downloadFile_AvailableTypes}

\link[periscope2]{downloadablePlot}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ui_helpers.R
\name{get_url_parameters}
\alias{get_url_parameters}
\title{Parse application passed URL parameters}
\usage{
get_url_parameters(session)
}
\arguments{
\item{session}{shiny session object}
}
\value{
named list of url parameters and values.  List may be empty if
no URL parameters were passed when the application instance was launched
}
\description{
This function returns any url parameters passed to the application as
a named list.  Keep in mind url parameters are always user-session scoped
}
\section{Shiny Usage}{

Call this function from \code{program/server_local.R} or any other server file
}

\examples{
  library(shiny)
  library(periscope2)

  # Display application info
  observeEvent(input$app_info, {
               url_params <- get_url_parameters(session)
               show_alert(html                = TRUE,
                          showCloseButton     = FALSE,
                          animation           = "slide-from-top",
                          closeOnClickOutside = TRUE,
                          text                = url_params[["passed_paramter"]],
                          title               = "alert title")
  })


}
\seealso{
\link[periscope2:set_app_parameters]{periscope2:set_app_parameters()}

\link[periscope2:add_ui_footer]{periscope2:add_ui_footer()}

\link[periscope2:add_ui_left_sidebar]{periscope2:add_ui_left_sidebar()}

\link[periscope2:add_ui_header]{periscope2:add_ui_header()}

\link[periscope2:add_ui_body]{periscope2:add_ui_body()}

\link[periscope2:add_ui_right_sidebar]{periscope2:add_ui_right_sidebar()}

\link[periscope2:ui_tooltip]{periscope2:ui_tooltip()}
}
